package bean;

import utils.Utils;

import java.beans.*;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Set;

/**
 * Created by IntelliJ IDEA.
 * User: David Huo
 * Date: Sep 19, 2006
 * Time: 9:56:18 PM
 */
public class PropertyChangeMultiCaster extends HashMap<Class, List<PropertyChangeListener>> {
    private static PropertyChangeMultiCaster _instance = new PropertyChangeMultiCaster();

    private PropertyChangeMultiCaster() {
    }

    public static PropertyChangeMultiCaster getInstance() {
        return _instance;
    }

    public void addListener(Class beanType, PropertyChangeListener p) {
        List<PropertyChangeListener> list = getListenerList(beanType);
        if (!list.contains(p)) {
            list.add(p);
        }
    }

    public void removeListener(Class type, PropertyChangeListener p) {
        if (containsKey(type)) {
            getListenerList(type).remove(p);
        }
    }

    public void set(Object bean, String propertyName, Object value) throws Exception {
        PropertyDescriptor pd = getPropertyDescriptor(bean, propertyName);
        if (null != pd) {
            set(bean, pd, value);
        } else {
            throw new IllegalArgumentException(bean.getClass() + " doesn't have a property called: " + propertyName);
        }
    }

    private PropertyDescriptor getPropertyDescriptor(Object bean, String propertyName) throws IntrospectionException {
        BeanInfo bi = Introspector.getBeanInfo(bean.getClass());
        PropertyDescriptor[] pds = bi.getPropertyDescriptors();
        for (PropertyDescriptor pd : pds) {
            if (pd.getName().equals(propertyName)) {
                return pd;
            }
        }
        return null;
    }

    public void set(Object bean, PropertyDescriptor pd, Object value) throws Exception {
        Method rm = pd.getReadMethod();
        Object oldValue = null;
        if (null != rm) {
            oldValue = rm.invoke(bean);
        }
        Method wm = pd.getWriteMethod();
        if (wm != null) {
            if ( value == null || !value.equals(oldValue) ) {
                wm.invoke(bean, value);
                PropertyChangeEvent pce = new PropertyChangeEvent(bean, pd.getName(), oldValue, value);
                fireChangeEvent(pce);
            }
        } else {
            Utils.debug("There is no write method for property: "
                    + pd.getName() + " of bean " + bean);
        }
    }

    public void fireChangeEvent(Object bean, String propertyName) {
//            Utils.debug("bean = " + bean + ", propertyName = " + propertyName);
        Object value;
        try {
            PropertyDescriptor pd = getPropertyDescriptor(bean, propertyName);
            value = null;
            if (null != pd) {
                Method rm = pd.getReadMethod();
                if (rm != null) {
                    value = rm.invoke(bean);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            return;
        }
        fireChangeEvent(new PropertyChangeEvent(bean, propertyName, value, value));
    }

    public void fireChangeEvent(PropertyChangeEvent pce) {
        Class<? extends Object> sourceType = pce.getSource().getClass();
        Utils.debug("sourceType = " + sourceType);
        try {
            Set<Class> classes = keySet();
            for (Class aClass : classes) {
                if (aClass.isAssignableFrom(sourceType)) {
                    Utils.debug("fire class type: " + aClass);
                    List<PropertyChangeListener> list = getListenerList(aClass);
                    for (PropertyChangeListener l : list) {
                        l.propertyChange(pce);
                    }
                }
            }
        } catch (Exception e) {
            Utils.error(e);
        }
    }

    private List<PropertyChangeListener> getListenerList(Class beanType) {
        List<PropertyChangeListener> list = get(beanType);
        if (null == list) {
            list = new ArrayList<PropertyChangeListener>();
            put(beanType, list);
        }
        return list;
    }

}
