package bean.editor;

import bean.annotation.Property;
import bean.annotation.AnnotationUtil;
import utils.Utils;

import javax.swing.*;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableModel;
import java.awt.*;
import java.awt.event.*;
import java.beans.PropertyDescriptor;
import java.beans.PropertyEditor;
import java.beans.PropertyEditorManager;
import java.lang.reflect.Method;

/**
 * User: huojun
 * Date: 7-Sep-2006
 * Time: 1:32:58 PM
 */
public class AnnotationPropertyEditor extends JTable {
    protected BeanEditorCellEditor _myEditor = new BeanEditorCellEditor();
    protected BeanEditorCellRenderer _renderer = new BeanEditorCellRenderer();
    protected PasswordCellRenderer _passwordRenderer = new PasswordCellRenderer();
    protected PasswordCellEditor _passwordEditor = new PasswordCellEditor();
    protected PropertyDescriptorRenderer _pdRenderer = new PropertyDescriptorRenderer();

    private JComboBox _tagsCombox = new JComboBox();
    private DropdownCellEditor _tagsEditor = new DropdownCellEditor(_tagsCombox);
    private CategoryRenderer _categoryRenderer = new CategoryRenderer();
    protected BeanPropertyModel _model = new BeanPropertyModel();
    protected BeanPropertySorter _sorter = new BeanPropertySorter(_model);
    protected CategorizedModel _categorizer = new CategorizedModel(_model);

    public AnnotationPropertyEditor() {
        setModel(_sorter);
        setRowHeight(20);
        getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        setDefaultRenderer(PropertyDescriptor.class, _pdRenderer);
        getTableHeader().setReorderingAllowed(false);
        addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                int r = rowAtPoint(e.getPoint());
                if (r >= 0 && getValueAt(r, 0) instanceof CategorizedModel.CategorizedRow) {
                    toggleCategoryExpand(r);
                }
            }

        });
    }

    public void sortableView() {
        setModel(_sorter);
    }

    public void categorizedView() {
        setModel(_categorizer);
    }

    public void setReadOnly(boolean b) {
        _model.setReadOnly(b);
    }

    public void setModel(TableModel dataModel) {
        super.setModel(dataModel);
        if (dataModel instanceof ITableHeaderMonitor) {
            ITableHeaderMonitor monitor = (ITableHeaderMonitor) dataModel;
            monitor.setTableHeader(getTableHeader());
        }
    }

    private void toggleCategoryExpand(int categoryRow) {
        TableModel m = getModel();
        if (m instanceof CategorizedModel) {
            CategorizedModel c = (CategorizedModel) m;
            c.toggleCategoryExpand(categoryRow);
        }

    }

    public TableCellRenderer getCellRenderer(int row, int column) {
        TableModel model = getModel();
        Object valueAt = model.getValueAt(row, column);
        if (valueAt instanceof CategorizedModel.CategorizedRow) {
            return _categoryRenderer;
        }
        if (column == 1) {
            if (model instanceof IBeanPropertyModel) {
                IBeanPropertyModel m = (IBeanPropertyModel) model;
                try {
                    PropertyDescriptor pd = m.getPropertyDescriptor(row);
                    Class type = pd.getPropertyType();
                    if (Boolean.class.equals(type) || boolean.class.equals(type)) {
                        return super.getDefaultRenderer(Boolean.class);
                    } else if( AnnotationUtil.isPassword(pd) ) {
                        return _passwordRenderer;
                    }
                    PropertyEditor pe = getPropertyEditor(m, row);
                    if (pe != null) {
                        _renderer.setEditor(pe);
                        return _renderer;
                    }
                } catch (Exception e) {
                    Utils.debug("AnnotationPropertyEditor.getCellRenderer: " + e);
                }
            }
        }
        return super.getCellRenderer(row, column);
    }

    public TableCellEditor getCellEditor(int row, int column) {
        if (row >= 0) {
            TableModel model = getModel();
            if (model instanceof IBeanPropertyModel) {
                IBeanPropertyModel m = (IBeanPropertyModel) model;
                try {
                    PropertyDescriptor pd = m.getPropertyDescriptor(row);
                    PropertyEditor pe = getPropertyEditor(m, row);
                    Class type = pd.getPropertyType();
                    if (pe != null) {
                        if (Boolean.class.equals(type) || boolean.class.equals(type)) {
                            return super.getDefaultEditor(Boolean.class);
                        } else if (pe.getTags() != null) {
                            _tagsEditor.setEditor(pe);
                            _tagsCombox.setModel(new DefaultComboBoxModel(pe.getTags()));
                            return _tagsEditor;
                        } else if( AnnotationUtil.isPassword(pd) ) {
                            return _passwordEditor;
                        } else {
                            _myEditor.setEditor(pe);
                            return _myEditor;
                        }
                    }
                } catch (Exception e) {
                    Utils.debug("AnnotationPropertyEditor.getCellEditor: " + e);
                }
            }
        }
        return super.getCellEditor(row, column);
    }

    protected PropertyEditor getPropertyEditor(IBeanPropertyModel m, int row) {
        PropertyDescriptor pd = m.getPropertyDescriptor(row);
        Class<?> type = pd.getPropertyType();
        String className = AnnotationUtil.getPropertyEditorClass(pd);
        PropertyEditor pe = null;
        if (Utils.hasValue(className)) {
            // try to find the editor based on Property annotation editor field
            try {
                pe = (PropertyEditor) Class.forName(className).newInstance();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        if (null == pe) {
            if (null != type) {
                // if it is an array type, we use array editor, otherwise try to find
                // a registered editor
                pe = type.isArray() ? new ArrayEditor() : PropertyEditorManager.findEditor(type);
            }
        }
        if (null != pe) {
            if (pe instanceof AbstractEditor) {
                AbstractEditor editor = (AbstractEditor) pe;
                editor.setBean(m.getBean());
                editor.setPropertyDescriptor(pd);
            }
            Object valueAt = getValueAt(row, 1);
            pe.setValue(valueAt);
        }
        return pe;
    }

    public boolean accept() {
        return _myEditor.stopCellEditing();
    }

    public void setBean(Object bean) {
        _model.setBean(bean);
    }

    private class CategoryRenderer extends JLabel implements TableCellRenderer {
        private class CategoryIcon implements Icon, SwingConstants {
            private boolean _expanded = false;
            private static final int SIZE = 8;

            public boolean isExpanded() {
                return _expanded;
            }

            public void setExpanded(boolean expanded) {
                _expanded = expanded;
            }

            public int getIconHeight() {
                return SIZE;
            }

            public int getIconWidth() {
                return SIZE;
            }

            public void paintIcon(Component c, Graphics g, int x, int y) {
                int dy = (c.getHeight() - SIZE) / 2;
                g.setColor(Color.white);
                g.fillRect(0, dy, SIZE, SIZE);
                g.setColor(Color.black);
                g.drawRect(0, dy, SIZE, SIZE);
                g.drawLine(2, dy + SIZE / 2, SIZE - 2, dy + SIZE / 2);
                if (!isExpanded()) {
                    g.drawLine(SIZE / 2, dy + 2, SIZE / 2, dy + SIZE - 2);
                }
            }
        }

        private Color RENDERER_COLOR = new Color(218, 218, 218);
        private CategoryIcon _icon = new CategoryIcon();

        public CategoryRenderer() {
            setOpaque(true);
            setBackground(RENDERER_COLOR);
        }

        public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
            CategorizedModel.CategorizedRow r = (CategorizedModel.CategorizedRow) value;
            if (column == 0) {
                _icon.setExpanded(r.isExpanded());
                setText(r.name);
                setIcon(_icon);
            } else {
                setIcon(null);
                setText("");
            }
            return this;
        }
    }

    private static class PropertyDescriptorRenderer extends DefaultTableCellRenderer {
        public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
            PropertyDescriptor pd = (PropertyDescriptor) value;
            String aName = getPropertyAnnotationName(pd);
            String name = Utils.hasValue(aName) ? aName : ( pd == null ? "" : pd.getName());
            return super.getTableCellRendererComponent(table, name, isSelected, hasFocus, row, column);
        }

        private String getPropertyAnnotationName(PropertyDescriptor pd) {
            if ( null != pd ) {
                Method rm = pd.getReadMethod();
                if( null != rm ) {
                    Property a = rm.getAnnotation(Property.class);
                    return a == null ? null : a.name();
                }
            }
            return null;
        }
    }
}
