package bean.editor;

import bean.annotation.Property;
import dbhelp.db.IDBObject;
import dbhelp.plugin.ui.QuickCloseDialog;

import javax.swing.*;
import javax.swing.FocusManager;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.*;
import static java.awt.GridBagConstraints.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Method;

/**
 * User: huojun
 * Date: 11-Sep-2006
 * Time: 11:54:39 AM
 */
public class AnnotationPropertyEditorPane extends JPanel {
    private static QuickCloseDialog _dialog;
    protected AnnotationPropertyEditor _editor = new AnnotationPropertyEditor();
    protected static AnnotationPropertyEditorPane _staticEditor;
    private boolean _readOnly;
    private JEditorPane _notesArea;

    public static void main(String[] args) {
        AnnotationPropertyEditorPane.edit(new JButton("test"));
    }

    public static boolean edit(Object bean) {
        return edit(bean, false);
    }

    public static boolean edit(Object bean, boolean editable) {
        if( null == _dialog ) {
            Window aw = FocusManager.getCurrentManager().getActiveWindow();
            _dialog = aw instanceof Frame ? new QuickCloseDialog((Frame)aw)
                    : new QuickCloseDialog((Dialog)aw);
            Container cp = _dialog.getContentPane();
            cp.setLayout( new GridBagLayout() );
            GridBagConstraints c = new GridBagConstraints(0,0,2,1,1.0,1.0, CENTER, BOTH, new Insets(2,2,2,2), 1, 1);
            _staticEditor = new AnnotationPropertyEditorPane();
            cp.add(_staticEditor, c );
            c = new GridBagConstraints(1,1,1,1,0.0,0.0, EAST, NONE, new Insets(2,2,2,2), 1, 1);
            JButton jbOK = new JButton("OK");
            cp.add( jbOK, c);
            _dialog.setDefaultButton(jbOK);
            jbOK.setVisible(editable);
            jbOK.setMnemonic('O');
            c = new GridBagConstraints(0,1,1,1,0.0,0.0, WEST, NONE, new Insets(2,2,2,2), 1, 1);
            JButton jbClose = new JButton("Cancel");
            cp.add( jbClose, c);
            jbClose.setMnemonic('C');
            jbClose.addActionListener( new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    _dialog.setVisible(false);
                }
            });
            _dialog.setModal(true);
            _dialog.setSize(640, 480);
            _dialog.setLocationRelativeTo( null );
            _dialog.setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
            jbOK.addActionListener( new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    _dialog.setVisible(false);
                    _dialog.setConfirmed( true );
                }
            });
        }
        _dialog.setTitle("Properties: " + bean);
        _staticEditor.setReadOnly(bean instanceof IDBObject);
        _staticEditor.setBean(bean);
        _dialog.setConfirmed(false);
        _dialog.setVisible(true);
        return _dialog.isConfirmed();
    }

    public void setBean(Object bean) {
        _editor.setBean(bean);
    }

    public boolean isReadOnly() {
        return _readOnly;
    }

    public void setReadOnly(boolean readOnly) {
        _readOnly = readOnly;
        getBeanEditorModel().setReadOnly(_readOnly);
    }

    public Object getBean() {
        return getBeanEditorModel().getBean();
    }

    private IBeanPropertyModel getBeanEditorModel() {
        return (IBeanPropertyModel) _editor.getModel();
    }

    public AnnotationPropertyEditorPane() {
        setLayout(new BorderLayout() );
        JPanel gridAndNotes = new JPanel(new GridBagLayout());
        JScrollPane grid = new JScrollPane(_editor);
        grid.getViewport().setBackground(Color.white);
        GridBagConstraints c;
        c = new GridBagConstraints(0,0,1,1, 1,1,NORTH, BOTH, new Insets(2,2,2,2), 1, 1);
        gridAndNotes.add( grid, c );
        c = new GridBagConstraints(0,1,1,1, 1,.1,NORTH, BOTH, new Insets(2,2,2,2), 1, 1);
        _notesArea = new JEditorPane();
        _notesArea.setBorder( BorderFactory.createLineBorder(Color.black));
        _notesArea.setEditable(false);
        _notesArea.setFont( getFont().deriveFont(Font.BOLD, 11) );
        gridAndNotes.add(_notesArea, c);
        add(gridAndNotes, BorderLayout.CENTER);
        JToolBar toolbar = new JToolBar(JToolBar.HORIZONTAL);
        add( toolbar, BorderLayout.PAGE_START );
        ToolbarButton cbSort = new ToolbarButton(3, new ActionSortView());
        toolbar.add(cbSort);
        toolbar.add(Box.createRigidArea(new Dimension(4,4)));
        ToolbarButton cbCategorized = new ToolbarButton( 16, new ActionCategorizedView());
        toolbar.add(cbCategorized);

        ToolbarButton cbEdit = new ToolbarButton(15, new ActionEdit());
        toolbar.add(Box.createRigidArea(new Dimension(8,8)));
        toolbar.add(cbEdit);
        setPreferredSize(new Dimension(300, 200));
        _editor.getSelectionModel().addListSelectionListener( new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                if( !e.getValueIsAdjusting() ) {
                    int row = _editor.getSelectedRow();
                    if( row >= 0 ) {
                        Object o = _editor.getValueAt(row, 0);
                        if ( o instanceof PropertyDescriptor ) {
                            PropertyDescriptor pd = (PropertyDescriptor) o;
                            _notesArea.setText( getDescription(pd) );
                        }
                    }
                }
            }

            private String getDescription(PropertyDescriptor pd) {
                Method rm = pd.getReadMethod();
                if( null != rm ) {
                    Property a = rm.getAnnotation(Property.class);
                    if( a != null ) {
                        return a.desc();
                    }
                }
                return "";
            }
        });
    }

    public AnnotationPropertyEditor getEditor() {
        return _editor;
    }

    private class ActionSortView extends AbstractAction {
        public ActionSortView() {
            super("Sortable View");
        }

        public void actionPerformed(ActionEvent e) {
            _editor.sortableView();
        }
    }

    private class ActionCategorizedView extends AbstractAction {
        public ActionCategorizedView() {
            super("Categorized View");
        }

        public void actionPerformed(ActionEvent e) {
            _editor.categorizedView();
        }
    }

    private class ActionEdit extends AbstractAction {
        public ActionEdit() {
            super("Edit");
        }

        public void actionPerformed(ActionEvent e) {
            if( !(getBean() instanceof IDBObject ) ) {
                setReadOnly(false);
            }
        }
    }
}
