package bean.editor;

import dbhelp.db.Attribute;
import utils.Utils;

import javax.swing.*;
import java.awt.*;
import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Array;
import java.util.Date;

/**
 * Created by IntelliJ IDEA.
 * User: huojun
 * Date: 16-Aug-2005
 * Time: 5:22:42 PM
 */
public class ArrayEditor extends AbstractEditor {
    private IArrayCustomEditor _uiEditor = null;
    private Class _componentType;

    public static void main(String[] args) throws IntrospectionException {
//        Attribute[] a = new Attribute[]{};
//        String[] lines = new String[0];
//        a = editArray(a, Attribute.class);
//        for (Attribute s : a) {
//            Utils.debug("s = " + s);
//        }
        FileHolder[] a = new FileHolder[0];
        Object[] lines = editArray(a, FileHolder.class);
        for (int i = 0; i < lines.length; i++) {
            System.out.println("line = " + lines[i]);
        }
        System.exit(0);
    }

    public Component getCustomEditor() {
        boolean firstCall = null == _uiEditor;
        ICustomEditor ce = getUIEditor();
        if (firstCall) {
            setCustomEditorValue(getValue());
        }
        ce.setReadOnly(isReadOnly());
        return (Component) ce;
    }

    private IArrayCustomEditor getUIEditor() {
        if (null == _uiEditor) {
            boolean b = isSimpleType();
            if (b || simpleEditable()) {
                ArrayPropertyPanel app = new ArrayPropertyPanel();
                app.setPackTable( false );
                app.getTable().setAutoResizeMode(JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS);
                if( b ) {
                    app.getTable().setShowGrid(false);
                }
                _uiEditor = app;
            } else {
                _uiEditor = new ArrayEditorPanel();
            }
        }
        return _uiEditor;
    }

    private boolean simpleEditable() {
        return findComponentType().equals(Attribute.class);
    }

    private boolean isSimpleType() {
        Class ct = findComponentType();
        return isSimpleType(ct);
    }

    public static boolean isSimpleType(Class ct) {
        return ct.isPrimitive()
                || ct.equals(Integer.class)
                || ct.equals(Double.class)
                || ct.equals(Long.class)
                || ct.equals(Short.class)
                || ct.equals(Character.class)
                || ct.equals(Date.class)
                || ct.equals(FileHolder.class)
                || ct.equals(Boolean.class)
                || ct.equals(Float.class)
                || ct.equals(String.class);
    }

    public boolean supportsCustomEditor() {
        return true;
    }

    private void setCustomEditorValue(Object value) {
        Class<?> type = findComponentType();
        IArrayCustomEditor uiEditor = _uiEditor;
        uiEditor.setComponentType(type);
        uiEditor.setValue(value);
    }

    public void editConfirm() {
        setValue(_uiEditor.getValue());
    }

    private Class<?> findComponentType() {
        Class<?> type = getType().getComponentType();
        type = type.equals(Object.class) ? (_componentType == null ? type : _componentType) : type;
        return type;
    }

    public Class getComponentType() {
        return _componentType;
    }

    public void setComponentType(Class componentType) {
        _componentType = componentType;
    }

    public String getAsText() {
        StringBuilder sb = new StringBuilder();
        Object v = getValue();
        if (null != v) {
            int length = Array.getLength(v);
            for (int i = 0; i < length; i++) {
                Object o = Array.get(v, i);
                if (sb.length() > 0) {
                    sb.append(", ");
                }
                sb.append(o);
            }
            return sb.toString();
        } else {
            return "";
        }
    }

    public void setAsText(String text) throws IllegalArgumentException {
        if (null == text || text.length() == 0) {
            setValue(null);
        }
    }

    public static <T> T[] editArray(T[] a, Class componentType) throws IntrospectionException {
        EditorDialog ed = new EditorDialog((Frame) Utils.getFrame());
        ArrayEditor ae = new ArrayEditor();
        ArrayHolder<T> holder = new ArrayHolder<T>(a);
        ae.setBean(holder);
        BeanInfo beanInfo = Introspector.getBeanInfo(ArrayHolder.class);
        PropertyDescriptor[] pds = beanInfo.getPropertyDescriptors();
        for (PropertyDescriptor pd : pds) {
            if (pd.getName().equals("array")) {
                ae.setPropertyDescriptor(pd);
                ae.setBean(holder);
                ae.setComponentType(componentType);
                ae.setValue(holder.getArray());
                ed.setEditor(ae);
                ed.setLocationRelativeTo(null);
                ed.setVisible(true);
                return (T[]) ae.getValue();
            }
        }
        throw new RuntimeException("array is not defined property name!");
    }
}
