package bean.editor;


import bean.PropertyChangeMultiCaster;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.lang.reflect.Array;

public class ArrayEditorPanel extends JPanel implements PropertyChangeListener, IArrayCustomEditor {

    private JList jlItems;
    private MyListModel _model = new MyListModel();
    private AnnotationPropertyEditorPane _propertyPane = new AnnotationPropertyEditorPane();
    private JButton _jbAdd;
    private JButton _jbRemove;
    private JButton _jbUp;
    private JButton _jbDown;
    private Class _componentType;

    public ArrayEditorPanel() {
        initComponents();
    }

    public Class getComponentType() {
        return _componentType;
    }

    public void setComponentType(Class componentType) {
        _componentType = componentType;
    }

    public void setReadOnly(boolean b) {
        _propertyPane.setReadOnly(b);
        _jbRemove.setEnabled(!b);
        _jbAdd.setEnabled(!b);
        _jbUp.setEnabled(!b);
        _jbDown.setEnabled(!b);
    }

    public void setValue(Object o) {
        _model = new MyListModel();
        if (null != o) {
            int length = Array.getLength(o);
            for (int i = 0; i < length; i++) {
                Object e = Array.get(o, i);
                _model.addElement(e);
            }
        }
        jlItems.setModel(_model);
        if (_model.size() > 0) {
            jlItems.setSelectedIndex(0);
        }
        PropertyChangeMultiCaster.getInstance().addListener(getComponentType(), this);
    }


    public Object getValue() {
        int size = _model.size();
        Class type = getComponentType();
        Object a = Array.newInstance(type, size);
        for (int i = 0; i < size; i++) {
            Array.set(a, i, _model.getElementAt(i));
        }
        return a;
    }

    public void accept() {
    }

    private void initComponents() {
        JSplitPane jSplitPane = new JSplitPane();
        JPanel jpLeft = new JPanel(new BorderLayout());
        jlItems = new JList();
        JScrollPane jsp = new JScrollPane(jlItems);

        _jbAdd = new ToolbarButton(18, new ActionAdd());
        _jbRemove = new ToolbarButton(19, new ActionRemove());
        _jbUp = new ToolbarButton(30, new ActionUp());
        _jbDown = new ToolbarButton(31, new ActionDown());
        JToolBar toolbar = new JToolBar();
        toolbar.add( _jbAdd );
        toolbar.add( _jbRemove );
        toolbar.addSeparator( new Dimension(10,10) );
        toolbar.add( _jbUp );
        toolbar.add( _jbDown );

        jpLeft.add(toolbar, BorderLayout.PAGE_START);

        setLayout(new BorderLayout());

        jpLeft.add(jsp, BorderLayout.CENTER);


        jpLeft.setMinimumSize(new Dimension(0, 0));

        jSplitPane.setLeftComponent(jpLeft);

        _propertyPane.setMinimumSize( new Dimension(0,0) );
        jSplitPane.setRightComponent(_propertyPane);
        add(jSplitPane, BorderLayout.CENTER);
        jSplitPane.setDividerLocation(300);
        setSize(640, 300);
        validate();
        jSplitPane.resetToPreferredSizes();

        jlItems.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                if (!e.getValueIsAdjusting() && jlItems.getSelectedIndex() >= 0) {
                    _propertyPane.setBean(jlItems.getSelectedValue());
                }
            }
        });
    }

    public void propertyChange(PropertyChangeEvent evt) {
        if( isVisible() ) {
            MyListModel model = (MyListModel) jlItems.getModel();
            for( int i=0; i<model.getSize(); i++ ) {
                if( model.getElementAt(i).equals(evt.getSource()) ) {
                    model.fireContentsChanged(this, i, i);
                    return;
                }
            }
        }
    }

    private static class MyListModel extends DefaultListModel {
        public void fireContentsChanged(Object source, int index0, int index1) {
            super.fireContentsChanged(source, index0, index1);
        }
    }

    private class ActionAdd extends AbstractAction {

        public ActionAdd() {
            super("Add");
        }

        public void actionPerformed(ActionEvent e) {
            try {
                Object obj = null;
                Class aClass = getComponentType();
                if (!aClass.isInterface()) {
                    obj = aClass.newInstance();
                }
                if (null != obj) {
                    _model.addElement(obj);
                    jlItems.setSelectedValue(obj, true);
                    MyListModel m = (MyListModel) jlItems.getModel();
                    int index = m.indexOf(obj);
                    m.fireContentsChanged(this, index, index);
                }
            } catch (Exception e1) {
                e1.printStackTrace();
            }
        }
    }

    private class ActionUp extends AbstractAction {

        public ActionUp() {
            super("Move Up");
        }

        public void actionPerformed(ActionEvent e) {
            int index = jlItems.getSelectedIndex();
            if (index > 0) {
                Object o = _model.getElementAt(index - 1);
                Object o1 = jlItems.getSelectedValue();
                _model.setElementAt(o1, index - 1);
                _model.setElementAt(o, index);
                jlItems.setSelectedIndex(index - 1);
            }
        }
    }

    private class ActionDown extends AbstractAction {

        public ActionDown() {
            super("Move Down");
        }

        public void actionPerformed(ActionEvent e) {
            int index = jlItems.getSelectedIndex();
            if (index >=0 && index + 1 < _model.size()) {
                Object o = jlItems.getSelectedValue();
                Object o1 = _model.getElementAt(index + 1);
                _model.setElementAt(o1, index);
                _model.setElementAt(o, index + 1);
                jlItems.setSelectedIndex(index + 1);
            }
        }
    }

    private class ActionRemove extends AbstractAction {

        public ActionRemove() {
            super("Remove");
        }

        public void actionPerformed(ActionEvent e) {
            int index = jlItems.getSelectedIndex();
            if (index >= 0) {
                Object o = _model.getElementAt(index);
                int ret = JOptionPane.showConfirmDialog(ArrayEditorPanel.this, "Are you sure to delete " + o, "Confirm", JOptionPane.YES_NO_OPTION);
                if (ret == 0) {
                    _model.removeElementAt(index);
                }
            }
        }
    }
}
