package bean.editor;

import bean.annotation.AnnotationUtil;
import dbhelp.plugin.ui.QuickEditTable;
import utils.Utils;

import javax.swing.*;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableModel;
import java.beans.PropertyDescriptor;
import java.beans.PropertyEditor;
import java.beans.PropertyEditorManager;

/**
 * User: huojun
 * Date: 3-Nov-2006
 * Time: 11:52:27 AM
 */
public class ArrayPropertyEditTable extends QuickEditTable {
    protected BeanEditorCellEditor _myEditor = new BeanEditorCellEditor();
    protected BeanEditorCellRenderer _renderer = new BeanEditorCellRenderer();
    protected PasswordCellRenderer _passwordRenderer = new PasswordCellRenderer();
    protected PasswordCellEditor _passwordEditor = new PasswordCellEditor();
    protected JComboBox _tagsCombox = new JComboBox();
    protected DropdownCellEditor _tagsEditor = new DropdownCellEditor(_tagsCombox);

    public TableCellRenderer getCellRenderer(int row, int column) {
        TableModel model = getDataModel();
        if (model instanceof BeanArrayModel) {
            BeanArrayModel m = (BeanArrayModel) model;
            try {
                PropertyEditor pe = getPropertyEditor(m, row, column);
                PropertyDescriptor pd = getPropertyDescriptor(m, column);
                Utils.debug("pd.getName() = " + pd.getName() + ", pe = " + pe);
                Class type = pd.getPropertyType();
                if (Boolean.class.equals(type) || boolean.class.equals(type)) {
                    return super.getDefaultRenderer(Boolean.class);
                } else if (AnnotationUtil.isPassword(pd)) {
                    return _passwordRenderer;
                }
                if (pe != null) {
                    _renderer.setEditor(pe);
                    return _renderer;
                }
            } catch (Exception e) {
                Utils.debug("ArrayPropertyEditTable.getCellRenderer: " + e);
            }
        }
        return super.getCellRenderer(row, column);
    }

    private PropertyDescriptor getPropertyDescriptor(BeanArrayModel m, int column) {
        return m.getPropertyDescriptor(getFilterColumnModel().getModelIndex(column));
    }

    private PropertyEditor getPropertyEditor(BeanArrayModel m, int row, int column) throws Exception {
        PropertyDescriptor pd = getPropertyDescriptor(m, column);
        PropertyEditor pe;
        if ( !ArrayEditor.isSimpleType(m.getComponentType())) {
            String editorClass = AnnotationUtil.getPropertyEditorClass(pd);
            pe = Utils.hasValue(editorClass) ? (PropertyEditor) Class.forName(editorClass).newInstance()
                    : PropertyEditorManager.findEditor(pd.getPropertyType());
        } else {
            pe = PropertyEditorManager.findEditor(m.getComponentType());
        }
        if (null != pe) {
            Object value = getValueAt(row, column);
            if (pe instanceof AbstractEditor) {
                AbstractEditor editor = (AbstractEditor) pe;
                editor.setBean(value);
                editor.setPropertyDescriptor(pd);
            }
            pe.setValue(value);
        }
        return pe;
    }

    public TableCellEditor getCellEditor(int row, int column) {
        if (row >= 0) {
            TableModel model = getDataModel();
            if (model instanceof BeanArrayModel) {
                BeanArrayModel m = (BeanArrayModel) model;
                try {
                    PropertyEditor pe = getPropertyEditor(m, row, column);
                    PropertyDescriptor pd = getPropertyDescriptor(m, column);
                    Class type = pd.getPropertyType();
                    if (pe != null) {
                        if (Boolean.class.equals(type) || boolean.class.equals(type)) {
                            return super.getDefaultEditor(Boolean.class);
                        } else if (pe.getTags() != null) {
                            _tagsEditor.setEditor(pe);
                            _tagsCombox.setModel(new DefaultComboBoxModel(pe.getTags()));
                            return _tagsEditor;
                        } else if (AnnotationUtil.isPassword(pd)) {
                            return _passwordEditor;
                        } else {
                            _myEditor.setEditor(pe);
                            return _myEditor;
                        }
                    }
                } catch (Exception e) {
                    Utils.debug("ArrayPropertyEditTable.getCellEditor: " + e);
                }
            }
        }
        return super.getCellEditor(row, column);
    }
}
