package bean.editor;

import dbhelp.plugin.ui.QuickEditTable;
import utils.Utils;
import utils.ui.TableFilterDialog;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;

/**
 * User: huojun
* Date: 3-Nov-2006
* Time: 3:40:52 PM
*/
public class ArrayPropertyPanel extends JPanel implements IArrayCustomEditor {
    protected BeanArrayModel _arrayModel;
    protected JLabel _status = new JLabel();
    protected ArrayPropertyEditTable _table = new ArrayPropertyEditTable();
    protected boolean _updateOnly;
    protected ToolbarButton _buttonAdd;
    protected ToolbarButton _buttonDelete;
    protected ToolbarButton _buttonUp;
    protected ToolbarButton _buttonDown;

    public ArrayPropertyPanel() {
        setLayout(new BorderLayout());
        _arrayModel = new BeanArrayModel();
        JScrollPane pane = new JScrollPane(_table);
        add(pane, BorderLayout.CENTER);
        JToolBar toolbar = new JToolBar();
        add(toolbar, BorderLayout.PAGE_START);
        toolbar.add(new ToolbarButton(3, new ActionFilter(_table)));
        toolbar.addSeparator( new Dimension(18,18) );
        _buttonAdd = new ToolbarButton(18, new ActionAdd(this));
        toolbar.add(_buttonAdd);
        _buttonDelete = new ToolbarButton(19, new ActionDelete(this));
        toolbar.add(_buttonDelete);
        toolbar.addSeparator( new Dimension(18,18) );
        _buttonUp = new ToolbarButton(30, new ActionUp(this));
        toolbar.add(_buttonUp);
        _buttonDown = new ToolbarButton(31, new ActionDown(this));
        toolbar.add(_buttonDown);
        _table.setModel(_arrayModel);
        add( _status, BorderLayout.PAGE_END );
        setSize( new Dimension(400, 300) );
    }


    public void setPackTable(boolean packTable) {
        _table.setPackTable(packTable);
    }

    public Class getComponentType() {
        return _arrayModel.getComponentType();
    }

    public void setComponentType(Class componentType) {
        _arrayModel.setComponentType(componentType);
    }

    public ArrayPropertyEditTable getTable() {
        return _table;
    }

    public Object getValue() {
        return _arrayModel.getBeanArray();
    }

    public void accept() {
    }

    public void setValue(Object value) {
        _arrayModel.setBeanArray((Object[]) value);
        _status.setText("Total: " + _arrayModel.getRowCount() + " item(s)");
    }

    public void setReadOnly(boolean b) {
        _arrayModel.setEditable(!b);
        toggleButtons(!b);
    }

    private void toggleButtons(boolean b) {
        _buttonAdd.setEnabled(b);
        _buttonDelete.setEnabled(b);
        _buttonDown.setEnabled(b);
        _buttonUp.setEnabled(b);
    }

    public Object getSelectedValue() {
        int row = getTable().getSelectedRow();
        return row >=0 ? _arrayModel.getBeanArray()[row] : null;
    }


    public boolean isUpdateOnly() {
        return _updateOnly;
    }

    public void setUpdateOnly(boolean b) {
        _updateOnly = b;
        toggleButtons(!b);
    }

    private static class ActionFilter extends AbstractAction {
        private QuickEditTable _table;

        public ActionFilter(QuickEditTable table) {
            super("Filter columns");
            _table = table;
        }

        public void actionPerformed(ActionEvent e) {
            TableFilterDialog.showFilterDialog(_table, "Filter Columns");
        }
    }

    private static class ActionAdd extends AbstractAction {
        private ArrayPropertyPanel _tab;
        public ActionAdd(ArrayPropertyPanel tab) {
            super("New");
            _tab = tab;
        }

        public void actionPerformed(ActionEvent e) {
            ArrayPropertyEditTable table = _tab.getTable();
            BeanArrayModel m = (BeanArrayModel) table.getDataModel();
            Class<?> type = _tab.getComponentType();
            if ( m.isEditable() ) {
                try {
                    Object o = type.newInstance();
                    int row = m.getRowCount();
                    m.insert(row, o);
                    _tab.setText("Total: " + m.getRowCount() + " item(s)");
                    table.getSelectionModel().setSelectionInterval(row, row);
                    table.scrollRectToVisible(table.getCellRect(row, 0, true));
                } catch (Exception e1) {
                    Utils.error(e1);
                }
            } else {
                Utils.message("This view is read only!");
            }
        }
    }

    private void setText(String s) {
        _status.setText(s);
    }

    private static class ActionDelete extends AbstractAction {
        private ArrayPropertyPanel _tab;
        public ActionDelete(ArrayPropertyPanel tab) {
            super("Delete");
            _tab = tab;
        }

        public void actionPerformed(ActionEvent e) {
            ArrayPropertyEditTable table = _tab.getTable();
            BeanArrayModel m = (BeanArrayModel) table.getDataModel();
            if ( m.isEditable() ) {
                int index = table.getSelectedRow();
                if( index >= 0 ) {
                    int ret = JOptionPane.showConfirmDialog(_tab, "Are you sure to delete: " + m.getBeanArray()[index], "Confirm", JOptionPane.YES_NO_OPTION);
                    if (ret == 0) {
                        m.remove(index);
                        _tab.setText("Total: " + m.getRowCount() + " item(s)");
                        if( index - 1 >= 0 ) {
                            int row = index - 1;
                            table.getSelectionModel().setSelectionInterval(row, row);
                        }
                    }
                }
            } else {
                Utils.message("This view is read only!");
            }
        }
    }

    private static class ActionUp extends AbstractAction {
        private ArrayPropertyPanel _tab;
        public ActionUp(ArrayPropertyPanel tab) {
            super("Up");
            _tab = tab;
        }

        public void actionPerformed(ActionEvent e) {
            ArrayPropertyEditTable table = _tab.getTable();
            BeanArrayModel m = (BeanArrayModel) table.getDataModel();
            if (m.isEditable()) {
                int index = table.getSelectedRow();
                if( index >= 0 ) {
                    m.up(index);
                    if ( index - 1 >= 0 ) {
                        table.getSelectionModel().setSelectionInterval(index-1, index-1);
                    }
                }
            } else {
                Utils.message("This view is read only!");
            }
        }
    }

    private static class ActionDown extends AbstractAction {
        private ArrayPropertyPanel _tab;
        public ActionDown(ArrayPropertyPanel tab) {
            super("Down");
            _tab = tab;
        }

        public void actionPerformed(ActionEvent e) {
            ArrayPropertyEditTable table = _tab.getTable();
            BeanArrayModel m = (BeanArrayModel) table.getDataModel();
            if ( m.isEditable() ) {
                int index = table.getSelectedRow();
                if( index >= 0 && index + 1 < table.getRowCount()) {
                    m.down(index);
                    if ( index + 1 < table.getRowCount() ) {
                        table.getSelectionModel().setSelectionInterval(index+1, index+1);
                    }
                }
            } else {
                Utils.message("This view is read only!");
            }
        }
    }
}
