package bean.editor;

import bean.annotation.AnnotationUtil;
import dbhelp.db.Column;
import dbhelp.db.Database;
import dbhelp.db.Table;
import dbhelp.plugin.ui.QuickEditTable;
import utils.Config;
import utils.ui.TableFilterDialog;

import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import java.awt.*;
import static java.awt.GridBagConstraints.NONE;
import static java.awt.GridBagConstraints.WEST;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.*;
import java.lang.reflect.Method;
import java.util.*;
import java.util.List;

/**
 * User: huojun
 * Date: 2-Nov-2006
 * Time: 1:27:09 PM
 */
public class BeanArrayModel extends AbstractTableModel {
    protected Object[] _beanArray;
    protected Class _componentType;
    protected Accessor _accessor = new Accessor();
    protected List<PropertyDescriptor> _columns = new ArrayList<PropertyDescriptor>();
    protected boolean _editable = true;

    public static void main(String[] args) throws Exception {
        try {
            Config.restore();
        } catch (Exception e) {
            e.printStackTrace();
        }
        JFrame f = new JFrame();
        Container cp = f.getContentPane();
        cp.setLayout(new BorderLayout());
        f.setSize(800, 600);
        f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        BeanArrayModel am = new BeanArrayModel();
        final QuickEditTable jTable = new ArrayPropertyEditTable();
        jTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        cp.add(new JScrollPane(jTable), BorderLayout.CENTER);
        JPanel buttons = new JPanel(new GridBagLayout());
        cp.add(buttons, BorderLayout.SOUTH);
        GridBagConstraints g;
        g = new GridBagConstraints(1, 1, 1, 1, 1, 0, WEST, NONE, new Insets(0, 0, 0, 0), 1, 1);
        JButton jbFilter = new JButton("Filter");
        buttons.add(jbFilter, g);
        jbFilter.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                TableFilterDialog.showFilterDialog(new TableFilterDialog.DefaultFilterModel(jTable.getFilterColumnModel()), "Set Column Filter", jTable);
            }
        });
        Database db = Config.getConfig().getDatabases()[1];
        System.out.println("db = " + db);
        db.connect();
        List<Table> list = db.getMetaData().getTables(null, null, null, "customer");
        Table table = list.get(0);
        System.out.println("table = " + table);
        am.setBeanArray(table.getColumns());
        am.setComponentType(Column.class);
        jTable.setModel(am);
        f.setVisible(true);
    }

    public Object[] getBeanArray() {
        return _beanArray;
    }

    public void setBeanArray(Object[] beanArray) {
        _beanArray = beanArray;
    }

    public Class getComponentType() {
        return _componentType;
    }

    public void up(int index) {
        if( null != _beanArray && index - 1 >= 0 && index < _beanArray.length ) {
            Object o = _beanArray[index];
            _beanArray[index] = _beanArray[index - 1];
            _beanArray[index-1] = o;
            fireTableRowsUpdated(index-1, index);
        }
    }

    public void down(int index) {
        if(null != _beanArray && index + 1 < _beanArray.length && index >= 0) {
            Object o = _beanArray[index + 1];
            _beanArray[index + 1] = _beanArray[index];
            _beanArray[index] = o;
            fireTableRowsUpdated(index, index+1);
        }
    }

    public void insert(int index, Object e) {
        List<Object> list = new ArrayList<Object>();
        if( null != _beanArray ) {
            list.addAll(Arrays.asList(_beanArray));
        }
        list.add(index, e);
        _beanArray = list.toArray();
        fireTableRowsInserted(index, index);
    }

    public void remove(int index) {
        if( null != _beanArray ) {
            List<Object> list = new ArrayList<Object>(Arrays.asList(_beanArray));
            list.remove(index);
            _beanArray = list.toArray();
            fireTableRowsDeleted(index, index);
        }
    }

    public void remove(Object e) {
        for (int i = 0; null != _beanArray && i < _beanArray.length; i++) {
            Object o = _beanArray[i];
            if( o.equals(e) ) {
                remove(i);
                return;
            }
        }
    }

    public boolean isEditable() {
        return _editable;
    }

    public void setEditable(boolean editable) {
        _editable = editable;
    }

    public void setComponentType(Class componentType) {
        _componentType = componentType;
        getMetaData();
    }

    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return isEditable() && _columns.get(columnIndex).getWriteMethod() != null;
    }

    public int getRowCount() {
        return _beanArray == null ? 0 : _beanArray.length;
    }

    public int getColumnCount() {
        return _columns.size();
    }

    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        if ( rowIndex >= 0 && rowIndex < getRowCount() && columnIndex >=0 && columnIndex < getColumnCount() ) {
            if (!ArrayEditor.isSimpleType(getComponentType())) {
                PropertyDescriptor pd = _columns.get(columnIndex);
                try {
                    Object bean = _beanArray[rowIndex];
//                    PropertyChangeMultiCaster.getInstance().set(bean, pd, aValue);
                    pd.getWriteMethod().invoke(bean, aValue);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            } else {
                _beanArray[rowIndex] = aValue;
            }
        }
    }

    public Class<?> getColumnClass(int columnIndex) {
        Class<?> aClass = _columns.get(columnIndex).getPropertyType();
        if( aClass.equals(boolean.class) ) {
            return Boolean.class;
        } else if( aClass.equals(int.class) ) {
            return Integer.class;
        } else if( aClass.equals(long.class) ) { 
            return Long.class;
        } else if( aClass.equals(double.class) ) {
            return Double.class;
        } else if( aClass.equals(float.class) ) {
            return Float.class;
        } else if( aClass.equals(short.class) ) {
            return Short.class;
        } else if( aClass.equals(char.class) ) {
            return Character.class;
        } else {
            return aClass;
        }
    }

    private void getMetaData() {
        _columns.clear();
        try {
            if (!ArrayEditor.isSimpleType(getComponentType())) {
                BeanInfo info = Introspector.getBeanInfo(getComponentType());
                PropertyDescriptor[] pds = info.getPropertyDescriptors();
                for (int i = 0; null != pds && i < pds.length; i++) {
                    PropertyDescriptor pd = pds[i];
                    if (hasEditor(pd)) {
                        Method rm = pd.getReadMethod();
                        Boolean b = AnnotationUtil.getBoolean(rm, "expose");
                        if (null == b || b) {
                            _columns.add(pd);
                        }
                    }
                }
                Collections.sort(_columns, new Comparator<PropertyDescriptor>() {
                    public int compare(PropertyDescriptor o1, PropertyDescriptor o2) {
                        return getOrdinal(o1) - getOrdinal(o2);
                    }
                });
            } else {
                _columns.add(_accessor.getPropertyDescriptor());
            }
        } catch (IntrospectionException e) {
            e.printStackTrace();
        }
        fireTableStructureChanged();
    }

    private boolean hasEditor(PropertyDescriptor pd) {
        PropertyEditor editor = PropertyEditorManager.findEditor(pd.getPropertyType());
        return editor != null || hasAnnotatedEditor(pd);
    }

    private boolean hasAnnotatedEditor(PropertyDescriptor pd) {
        String s = AnnotationUtil.getString(pd.getWriteMethod(), "editor");
        return s != null && s.length() > 0;
    }


    public String getColumnName(int column) {
        PropertyDescriptor pd = _columns.get(column);
        return getPropertyName(pd);
    }

    private String getPropertyName(PropertyDescriptor pd) {
        if (null != pd) {
            String aName = AnnotationUtil.getString(pd.getReadMethod(), "name");
            return aName != null && aName.length() > 0 ? aName : pd.getName();
        }
        return null;
    }

    private int getOrdinal(PropertyDescriptor o) {
        Integer ra = AnnotationUtil.getInt(o.getReadMethod(), "ordinal");
        if (null != ra) {
            return ra < 0 ? 10000 : ra;
        }
        ra = AnnotationUtil.getInt(o.getWriteMethod(), "ordinal");
        return ra == null ? 10000 : ( ra < 0 ? 10000 : ra);
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        Object bean = _beanArray[rowIndex];
        if ( !ArrayEditor.isSimpleType(getComponentType())) {
            PropertyDescriptor pd = _columns.get(columnIndex);
            try {
                return pd.getReadMethod().invoke(bean);
            } catch (Exception e) {
                e.printStackTrace();
            }
        } else {
            return bean;
        }
        return null;
    }

    public PropertyDescriptor getPropertyDescriptor(int column) {
        return _columns.get(column);
    }

    public static class Accessor {
        private Object _value;

        public PropertyDescriptor getPropertyDescriptor() throws IntrospectionException {
            return new PropertyDescriptor("value", Accessor.class);
        }

        public Object getValue() {
            return _value;
        }

        public void setValue(Object value) {
            _value = value;
        }
    }
}
