package bean.editor;

import utils.Utils;

import javax.swing.*;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableModel;
import java.util.Vector;
import java.util.EventObject;
import java.beans.PropertyEditor;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;

/**
 * User: huojun
* Date: 3-Nov-2006
* Time: 12:12:19 PM
*/
public class BeanEditorCellEditor extends JPanel implements TableCellEditor {
    private int _row;
    private int _column;
    private Vector<CellEditorListener> _listeners = new Vector<CellEditorListener>();
    private JTextField _text = new JTextField();
    private PropertyEditor _editor;
    private JButton _button;
    private TableModel _tableModel;


    public BeanEditorCellEditor() {
        setLayout(new GridBagLayout());
        GridBagConstraints c;
        c = new GridBagConstraints(1, 0, 1, 1, 0, 0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(1, 1, 1, 1), 1, 1);
        _button = new JButton("...");
        _button.setMinimumSize(new Dimension(16, 16));
        add(_button, c);
        c = new GridBagConstraints(0, 0, 1, 1, 1, 1, GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0);
        add(_text, c);
        _button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                _editor.setAsText(_text.getText());
                EditorDialog ed = getEditorDialog();
                if (_editor instanceof AbstractEditor) {
                    AbstractEditor ae = (AbstractEditor) _editor;
//                        ae.setReadonly(_model.isReadOnly());
                    ed.setTitle("Property: " + ae.getPropertyDescriptor().getName());
                }
                ed.setEditor(_editor);
                ed.setVisible(true);
                _text.setText(_editor.getAsText());
                stopCellEditing();
            }

            private EditorDialog getEditorDialog() {
                Window owner = getActiveWindow();
                if (owner instanceof Frame) {
                    Frame frame = (Frame) owner;
                    return new EditorDialog(frame);
                } else {
                    return new EditorDialog((Dialog) owner);
                }
            }
        });
        _text.addFocusListener(new FocusAdapter() {
            public void focusGained(FocusEvent e) {
                _text.selectAll();
                _button.repaint();
            }
        });
        _text.setBorder(BorderFactory.createLineBorder(Color.black));
    }

    private Window getActiveWindow() {
        return KeyboardFocusManager.getCurrentKeyboardFocusManager().getActiveWindow();
    }

    public void requestFocus() {
        _text.requestFocus();
    }

    public PropertyEditor getEditor() {
        return _editor;
    }

    public void setEditor(PropertyEditor editor) {
        _editor = editor;
        _button.setVisible(_editor.supportsCustomEditor());
    }


    public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
        _tableModel = table.getModel();
        _editor.setValue(value);
        _text.setText(null == value ? null : _editor.getAsText());
        _row = row;
        _column = column;
        return this;
    }

    public void cancelCellEditing() {
        Object a = _tableModel.getValueAt(_row, _column);
        _editor.setValue(a);
        _text.setText(_editor.getAsText());
    }

    public boolean stopCellEditing() {
        try {
            _editor.setAsText(_text.getText());
        } catch (Exception e) {
            Utils.debug("Error: %1$s: %2$s", _tableModel.getValueAt(_row, 0), e.getMessage());
        }
        notifyStopped(new ChangeEvent(this));
        return true;
    }

    public Object getCellEditorValue() {
        return _editor.getValue();
    }

    public boolean isCellEditable(EventObject anEvent) {
        return true;
    }

    public boolean shouldSelectCell(EventObject anEvent) {
        return true;
    }

    public void addCellEditorListener(CellEditorListener l) {
        if (!_listeners.contains(l)) {
            _listeners.add(l);
        }
    }

    protected void notifyCancelled(ChangeEvent e) {
        CellEditorListener[] v = _listeners.toArray(new CellEditorListener[_listeners.size()]);
        for (CellEditorListener l : v) {
            l.editingCanceled(e);
        }
    }

    protected void notifyStopped(ChangeEvent e) {
        CellEditorListener[] v = _listeners.toArray(new CellEditorListener[_listeners.size()]);
        for (CellEditorListener l : v) {
            l.editingStopped(e);
        }
    }

    public void removeCellEditorListener(CellEditorListener l) {
        _listeners.remove(l);
    }
}
