package bean.editor;


import javax.swing.event.TableModelListener;
import javax.swing.event.TableModelEvent;
import javax.swing.table.AbstractTableModel;
import java.util.*;
import java.beans.PropertyDescriptor;

/**
 * Created by IntelliJ IDEA.
 * User: huojun
 * Date: 17-Aug-2005
 * Time: 11:07:36 AM
 */
public class CategorizedModel extends AbstractTableModel implements IBeanPropertyModel {
    protected ArrayList<CategorizedRow> _view = new ArrayList<CategorizedRow>();
    protected ICategorizableTableModel _delegate;
    private MyTableModelListener _modelListener = new MyTableModelListener();

    public CategorizedModel(ICategorizableTableModel delegate) {
        super();
        _delegate = delegate;
        updateModel(delegate);
    }

    public boolean isReadOnly() {
        return _delegate.isReadOnly();
    }

    public Object getBean() {
        return _delegate.getBean();
    }

    public PropertyDescriptor getPropertyDescriptor(int row) {
        return _delegate.getPropertyDescriptor(mapRow(row));
    }

    public void setBean(Object bean) {
        _delegate.setBean(bean);
        updateModel(_delegate);
        fireTableDataChanged();
    }

    protected void updateModel(ICategorizableTableModel m) {
        if( _delegate != null ) {
            _delegate.removeTableModelListener(_modelListener);
        }
        if ( null != m ) {
            _delegate = m;
            _delegate.addTableModelListener(_modelListener);
            categorize();
        }
    }

    private void categorize() {
        PropertyCategory pc = new PropertyCategory();
        for (int i = 0; i < _delegate.getRowCount(); i++) {
            String cat = _delegate.getCategory(i);
            pc.addItem(new CategorizedRow(-1, i), cat == null ? "General" : cat);
        }
        categorize(pc);
        fireTableDataChanged();
    }

    public int mapRow(int row) {
        if( row >=0 && row < _view.size() ) {
            CategorizedRow r = _view.get(row);
            return r.index;
        }
        return row;
    }

    protected void categorize(PropertyCategory pc) {
        int viewIndex = 0;
        _view.clear();
        Object[] categories = pc.getCategories();
        for (Object cat : categories) {
            List<CategorizedRow> items = pc.getCategorizedItems(cat);
            CategorizedRow cr = new CategorizedRow(viewIndex, cat.toString());
            _view.add(cr);
            viewIndex++;
            for (CategorizedRow r : items) {
                r.viewIndex = viewIndex++;
                _view.add(r);
            }
        }
    }


    public void toggleCategoryExpand(int row) {
        CategorizedRow r = _view.get(row);
        r.setExpanded(!r.isExpanded());
    }

    public Class getColumnClass(int columnIndex) {
        return _delegate.getColumnClass(columnIndex);
    }

    public String getColumnName(int column) {
        return _delegate.getColumnName(column);
    }

    public int getColumnCount() {
        return _delegate.getColumnCount();
    }


    public boolean isCellEditable(int rowIndex, int columnIndex) {
        CategorizedRow r = _view.get(rowIndex);
        if (r.isCategory()) {
            return false;
        } else {
            return _delegate.isCellEditable(r.index, columnIndex);
        }
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        CategorizedRow r = _view.get(rowIndex);
        if (!r.isCategory()) {
            return r.index < _delegate.getRowCount() ? _delegate.getValueAt(r.index, columnIndex) : null;
        } else {
            return r;
        }
    }

    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        CategorizedRow r = _view.get(rowIndex);
        if ( r.index < _delegate.getRowCount() ) {
            _delegate.setValueAt(aValue, r.index, columnIndex);
        }
    }

    public int getRowCount() {
        return _view.size();
    }

    public void setReadOnly(boolean readOnly) {
        _delegate.setReadOnly(readOnly);
    }

    public class PropertyCategory extends HashMap<Object, ArrayList<CategorizedRow>> {
        public void addItem(CategorizedRow pd, Object cat) {
            ArrayList<CategorizedRow> a = get(cat);
            if( a == null ) {
                a = new ArrayList<CategorizedRow>();
                put( cat,a );
            }
            a.add(pd);
        }

        public Object[] getCategories() {
            return this.keySet().toArray();
        }

        public List<CategorizedRow> getCategorizedItems(Object cat) {
            return get(cat);
        }
    }

    public class CategorizedRow {
        public int viewIndex;
        public int index;
        public String name;
        private boolean expanded = true;
        private ArrayList<CategorizedRow> _buffer = new ArrayList<CategorizedRow>();

        public CategorizedRow(int viewIndex, String name) {
            this.viewIndex = viewIndex;
            this.name = name;
            this.index = -1;
        }

        public boolean isExpanded() {
            return expanded;
        }

        public void setExpanded(boolean expanded) {
            this.expanded = expanded;
            if (expanded) {
                _view.addAll(viewIndex + 1, _buffer);
            } else {
                _buffer.clear();
                int i;
                for (i = viewIndex + 1; i < _view.size(); i++) {
                    CategorizedRow o = _view.get(i);
                    if (!o.isCategory()) {
                        _buffer.add(o);
                    } else {
                        break;
                    }
                }
                _view.removeAll(_buffer);
            }
            for (int i = 0; i < _view.size(); i++) {
                CategorizedRow o = _view.get(i);
                o.viewIndex = i;
            }
            fireTableDataChanged();
        }

        public CategorizedRow(int viewIndex, int index) {
            this.viewIndex = viewIndex;
            this.index = index;
        }

        public boolean isCategory() {
            return index == -1;
        }
    }

    private class MyTableModelListener implements TableModelListener {
        public void tableChanged(TableModelEvent e) {
            if( e.getColumn() == TableModelEvent.ALL_COLUMNS ) {
                categorize();
            }
        }
    }
}
