package bean.editor;

import dbhelp.plugin.ui.QuickCloseDialog;
import utils.Utils;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyEditor;

/**
 * Created by IntelliJ IDEA.
 * User: David Huo
 * Date: Sep 22, 2006
 * Time: 11:01:58 PM
 */
public class EditorDialog extends QuickCloseDialog {
    private PropertyEditor _editor;
    private JButton _cbConfirm = new JButton("OK");

    public EditorDialog(Frame owner) throws HeadlessException {
        super(owner);
        initComponents();
    }

    public EditorDialog(Dialog owner) throws HeadlessException {
        super(owner);
        initComponents();
    }

    private void initComponents() {
        setTitle("Property Editor");
        getContentPane().setLayout(new GridBagLayout());
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        _cbConfirm.setMnemonic('O');
        setDefaultButton(_cbConfirm);
        _cbConfirm.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (_editor instanceof AbstractEditor ) {
                    AbstractEditor editor = (AbstractEditor) _editor;
                    if (editor.getCustomEditor() instanceof ICustomEditor) {
                        ICustomEditor ce = (ICustomEditor) editor.getCustomEditor();
                        ce.accept();
                    }
                    editor.editConfirm();
                }
                dispose();
            }
        });
        setModal(true);
        Utils.addEscClose(this);
    }

    public void setEditor(PropertyEditor editor) {
        if (null != _editor) {
            getContentPane().remove(_editor.getCustomEditor());
        }
        _editor = editor;
        GridBagConstraints c;
        c = new GridBagConstraints(0, 0, 2, 1, 1, 1, GridBagConstraints.NORTH, GridBagConstraints.BOTH, new Insets(2, 2, 2, 2), 1, 1);
        Component customEditor = _editor.getCustomEditor();
        getContentPane().add(customEditor, c);
        c = new GridBagConstraints(1, 1, 1, 1, 0, 0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 1, 1);
        getContentPane().add(_cbConfirm, c);
        c = new GridBagConstraints(0, 1, 1, 1, 0, 0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 1, 1);
        JButton jbClose = new JButton("Cancel");
        getContentPane().add(jbClose, c);
        jbClose.setMnemonic('C');
        jbClose.addActionListener( new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dispose();
            }
        });
        Dimension size = customEditor.getSize();
        setSize(size.width, size.height + 40);
        setLocationRelativeTo(null);
    }
}
