package bean.editor;

import bean.annotation.AnnotationUtil;
import dbhelp.db.Database;
import dbhelp.db.Procedure;
import dbhelp.db.SQLCommander;
import dbhelp.db.Table;
import dbhelp.db.model.ISQLResultListener;
import dbhelp.db.model.SQLResultHolder;
import dbhelp.plugin.action.portable.ViewTableData;
import dbhelp.plugin.ui.ResultTablePanel;
import org.apache.velocity.VelocityContext;
import template.TemplateManager;
import utils.text.editor.TokenEditor;

import javax.swing.*;
import static javax.swing.KeyStroke.getKeyStroke;
import java.awt.*;
import static java.awt.Event.CTRL_MASK;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.beans.BeanInfo;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;

/**
 * User: huojun
 * Date: 2-Nov-2006
 * Time: 1:41:54 PM
 */
public class TabbedBeanEditorPane extends JTabbedPane {
    protected Object _bean;
    protected AnnotationPropertyEditorPane _propertyEditorPane = new AnnotationPropertyEditorPane();
    protected TokenEditor _sqlView = new TokenEditor();
    protected PrimaryInfoPane _primaryInfoPane = new PrimaryInfoPane();


    public TabbedBeanEditorPane() {
        this(null);
    }

    public TabbedBeanEditorPane(Object bean) {
        addTab("Properties", _primaryInfoPane);
        setBean(bean);
        setTabPlacement(JTabbedPane.BOTTOM);
    }

    public Object getBean() {
        return _bean;
    }

    public void setBean(Object bean) {
        _bean = bean;
        _primaryInfoPane.setBean(bean);
        removeArrayPropertyTabs();
        try {
            addArrayPropertyTabs(_bean);
            if ( _bean instanceof Table ) {
                addDataTab((Table)_bean);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void addDataTab(Table table) {
        final ResultTablePanel panel = new ResultTablePanel();
        addTab("Data", panel);
        ViewTableData.viewTableData(table, new ISQLResultListener() {
            public void notifyResult(SQLResultHolder results) {
                panel.addResults(results);
            }
        });

    }

    private void addArrayPropertyTabs(Object bean) throws Exception {
        if (null != bean) {
            BeanInfo info = Introspector.getBeanInfo(bean.getClass());
            PropertyDescriptor[] pds = info.getPropertyDescriptors();
            for (int i = 0; null != pds && i < pds.length; i++) {
                PropertyDescriptor pd = pds[i];
                Class<?> pType = pd.getPropertyType();
                if (AnnotationUtil.exposed(pd) && pType.isArray()) {
                    ArrayPropertyPanel aTab = new ArrayPropertyPanel();
                    Object array = pd.getReadMethod().invoke(bean);
                    aTab.setComponentType(pType.getComponentType());
                    aTab.setValue(array);
                    // todo: remove the read only when add/delete features are ready!
                    aTab.setReadOnly(true);
                    addTab(pd.getName(), aTab);
                }
            }
        }
    }

    private void removeArrayPropertyTabs() {
        Component[] components = getComponents();
        for (int i = 0; null != components && i < components.length; i++) {
            Component component = components[i];
            if (!(component instanceof PrimaryInfoPane)) {
                remove(component);
            }
        }
    }

    private class PrimaryInfoPane extends JPanel {
        protected JLabel _databaseLabel = new JLabel();

        public PrimaryInfoPane() {
            setLayout(new BorderLayout());
            JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
            add(splitPane, BorderLayout.CENTER);
            Dimension zeroSize = new Dimension(0, 0);
            _propertyEditorPane.setMinimumSize(zeroSize);
            _propertyEditorPane.setPreferredSize(new Dimension(400, 200));
            splitPane.setOneTouchExpandable(true);
            splitPane.setLeftComponent(_propertyEditorPane);
            JPanel editorPane = new JPanel(new BorderLayout());
            JScrollPane editorView = new JScrollPane(_sqlView);
            editorPane.setMinimumSize(zeroSize);
            editorPane.setPreferredSize(new Dimension(600, 200));
            editorPane.add(editorView, BorderLayout.CENTER);
            JToolBar toolbar = new JToolBar();
            toolbar.add(_databaseLabel);
            ActionRun actionRun = new ActionRun();
            toolbar.addSeparator(new Dimension(20,20));
            toolbar.add(new ToolbarButton(28, actionRun));
            toolbar.addSeparator(new Dimension(20,20));
            toolbar.add(new JLabel("Ctrl+SPACE to complete"));
            editorPane.add(toolbar, BorderLayout.PAGE_START);
            splitPane.setRightComponent(editorPane);
            _sqlView.addHotKey("run-sql", getKeyStroke(KeyEvent.VK_ENTER, CTRL_MASK), actionRun);
        }

        public void setBean(Object bean) {
            _propertyEditorPane.setBean(bean);
            if( bean instanceof Table ) {
                Table t = (Table) bean;
                _databaseLabel.setText("Database: " + t.getDatabase());
                _sqlView.setDatabase(t.getDatabase());
                VelocityContext c = new VelocityContext();
                c.put("target", bean);
                String output = TemplateManager.getMgr().runTemplate("CREATE TABLE Statement", c);
                _sqlView.setText(output);
            } else if( bean instanceof Procedure ) {
                Procedure t = (Procedure) bean;
                _databaseLabel.setText("Database: " + t.getDatabase());
                Database db = t.getDatabase();
                _sqlView.setDatabase(db);
                _sqlView.setText(Procedure.getProcedureSyntax(t));
            }
        }

        private class ActionRun extends AbstractAction {

            public ActionRun() {
                super("Run script (Ctrl+Enter)");
            }

            public void actionPerformed(ActionEvent e) {
                Database db = _sqlView.getDatabase();
                if (null != db) {
//                    OutputPane out = SingletonProvider.getProvider().getOutputPane();
                    SQLCommander.runSQLCommand(db, _sqlView.getText(), (getBean() instanceof Table ? (Table) getBean() : null), null);
                }
            }
        }
    }
}
