package bean.editor;

import utils.Icons;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.net.URL;

/**
 * Created by IntelliJ IDEA.
 * User: David Huo
 * Date: Jan 31, 2006
 * Time: 12:15:16 AM
 */
public class ToolbarButton extends JButton {

    private final Insets margins = new Insets(-1, -1, -1, -1);
    private static ActionListener _action = new MappedActionListener();

    public ToolbarButton(Icon icon) {
        super(icon);
        setMargin(margins);
        setVerticalTextPosition(BOTTOM);
        setHorizontalTextPosition(CENTER);
    }

    public ToolbarButton(int iconIndex, AbstractAction action) {
        this(Icons.getInstance().get(iconIndex), action);
    }

    public ToolbarButton(int iconIndex, String tip, String actionId) {
        this(Icons.getInstance().get(iconIndex));
        setToolTipText(tip);
        setActionCommand(actionId);
        addActionListener(_action);
    }

    public ToolbarButton(Icon icon, AbstractAction action) {
        this(icon);
        addActionListener(action);
        setToolTipText("" + action.getValue("Name"));
    }

    public ToolbarButton(String imageFile) {
        URL resource = getClass().getResource(imageFile);
        if (null != resource) {
            setIcon(new ImageIcon(resource));
        }
    }

    public ToolbarButton(String imageFile, AbstractAction action) {
        this(imageFile, action, "" + action.getValue("Name"));
    }

    public ToolbarButton(String imageFile, ActionListener action, String tip) {
        this(imageFile);
        if (null != action) {
            addActionListener(action);
        }
        setToolTipText(tip);
    }

    public ToolbarButton(String imageFile, String text) {
        this(new ImageIcon(imageFile));
        setText(text);
    }

    private static class MappedActionListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            ToolbarButton b = (ToolbarButton) e.getSource();
            JComponent c = (JComponent) b.getParent();
            Action a = c.getActionMap().get(b.getActionCommand());
            if( a != null ) {
                a.actionPerformed(e);
            }
        }
    }
}
