package bean.serialization;

import bean.annotation.Property;
import utils.Converter;
import utils.Utils;

import javax.xml.stream.*;
import static javax.xml.stream.events.XMLEvent.END_ELEMENT;
import static javax.xml.stream.events.XMLEvent.START_ELEMENT;
import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.io.Reader;
import java.io.Writer;
import java.lang.reflect.Array;
import java.lang.reflect.Method;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;


/**
 * User: huojun
 * Date: 26-Sep-2006
 * Time: 1:42:26 PM
 */
public class BeanXmlSerializer {
//    private static final String C_TEMP_BEAN_XML = "C:\\Documents and Settings\\huojun\\dbhelp\\temporary\\jdbc_odbc_dsn_ASA_9_0_Sample.dia";
//    private static final String C_TEMP_BEAN_XML = "c:\\temp\\bean.xml";
    private static final String XML_STREAM_INPUT_FACTORY = "com.sun.xml.stream.ZephyrParserFactory";
    private static final String XML_STREAM_OUTPUT_FACTORY = "com.sun.xml.stream.ZephyrWriterFactory";

//    public static void main(String[] args) throws Exception {
//        JFrame f = new JFrame();
//        Container cp = f.getContentPane();
//        cp.setLayout(new BorderLayout());
//        DatabaseDiagram diagram = new IDEADiagram();
//        cp.add(diagram, BorderLayout.CENTER);
//        JPanel panel = new JPanel();
//        cp.add(panel, BorderLayout.SOUTH);
//        panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
//        panel.add(new JButton(new ActionSave(diagram)), BoxLayout.X_AXIS);
//        panel.add(new JButton(new ActionRead(diagram)), BoxLayout.X_AXIS);
//        f.setSize(800, 600);
//        f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
//        f.setVisible(true);
//
//        Config.restore();
//        Database db = Config.getConfig().getDatabases()[0];
//        db.connect();
//        java.util.List<Table> t1 = db.getMetaData().getTables(null, null, "TABLE", null);
//        for (Table table : t1) {
//            diagram.addTable(table);
//        }
//    }

    public static Object fromXml(Reader xmlIn) throws Exception {
        XMLInputFactory input = (XMLInputFactory) Class.forName(XML_STREAM_INPUT_FACTORY).newInstance();
        XMLStreamReader r = input.createXMLStreamReader(xmlIn);
        return consumeObject(r);
    }

    private static Object consumeObject(XMLStreamReader r) throws Exception {
        Object o = null;
        while (r.hasNext()) {
            int et = r.next();
            switch (et) {
                case START_ELEMENT:
//                    Utils.debug("consumeObject Start Element = " + r.getLocalName());
                    String type = r.getAttributeValue(null, "type");
                    if (null != type) {
                        Class t = getClassFromString(type);
                        o = t.newInstance();
                        consumeProperties(o, r);
                        return o;
                    }
                    break;
                case END_ELEMENT:
//                    Utils.debug("consumeObject End Element = " + r.getLocalName());
                    return o;
            }
        }
        return null;
    }

    private static Object consumeProperties(Object parent, XMLStreamReader r) throws Exception {
        while (r.hasNext()) {
            int et = r.next();
            switch (et) {
                case START_ELEMENT:
//                    Utils.debug("consumeProperties Start Element = " + r.getLocalName());
                    String type = r.getAttributeValue(null, "type");
                    Class t = getClassFromString(type);
                    String isArray = r.getAttributeValue(null, "isArray");
                    if ("true".equalsIgnoreCase(isArray)) {
                        consumeArray(type, r, parent);
                    } else if (isSimpleType(t)) {
                        consumeProperty(parent, r);

                    } else {
                        String isNull = r.getAttributeValue(null, "isNull");
                        if (!"true".equals(isNull)) {
                            Object o = t.newInstance();
                            consumeProperties(o, r);
                            PropertyDescriptor pd = getPropertyDescriptor(parent, r.getLocalName());
                            pd.getWriteMethod().invoke(parent, o);
                        } else {
                            consumeToEndofElement(r);
                        }

                    }
                    break;
                case END_ELEMENT:
//                    Utils.debug("consumeProperties End Element = " + r.getLocalName());
                    return parent;
            }
        }
        return parent;
    }

    private static Class<?> getClassFromString(String type) throws ClassNotFoundException {
        if (type.equals(int.class.toString())) {
            return int.class;
        } else if (type.equals(boolean.class.toString())) {
            return boolean.class;
        } else if (type.equals(short.class.toString())) {
            return short.class;
        } else if (type.equals(double.class.toString())) {
            return double.class;
        } else if (type.equals(byte.class.toString())) {
            return byte.class;
        } else if (type.equals(long.class.toString())) {
            return long.class;
        } else if (type.equals(float.class.toString())) {
            return float.class;
        } else if (type.equals(char.class.toString())) {
            return char.class;
        }
        return Class.forName(type);
    }

    private static Object consumeArray(String componentType, XMLStreamReader r, Object parent) throws Exception {
        Class<?> t = getClassFromString(componentType);
        String size = r.getAttributeValue(null, "size");
        String propertyName = r.getLocalName();
        if (null != size && size.length() > 0) {
            int len = Integer.parseInt(size);
            if (len > 0) {
                Object array = Array.newInstance(t, len);
                for (int i = 0; i < len; i++) {
                    Array.set(array, i, consumeObject(r));
                }

                PropertyDescriptor pd = getPropertyDescriptor(parent, propertyName);
                pd.getWriteMethod().invoke(parent, array);
            }
        }
        consumeToEndofElement(r);
        return parent;
    }

    private static PropertyDescriptor getPropertyDescriptor(Object parent, String propertyName) throws IntrospectionException {
        BeanInfo beanInfo = Introspector.getBeanInfo(parent.getClass());
        for (PropertyDescriptor pd : beanInfo.getPropertyDescriptors()) {
            if (pd.getName().equals(propertyName)) {
                return pd;
            }
        }
        Utils.debug("Can't find property: " + propertyName + " in " + parent.getClass());
        return null;
    }

    private static Object consumeProperty(Object parent, XMLStreamReader r) throws Exception {
        String propertyName = r.getLocalName();
        String isNull = r.getAttributeValue(null, "isNull");
        if (!"true".equals(isNull)) {
            String value = r.getAttributeValue(null, "value");
            value = null == value ? null : value.replaceAll("&zwnj;", "\n");
            String type = r.getAttributeValue(null, "type");
            PropertyDescriptor pd = getPropertyDescriptor(parent, propertyName);
            Object v = Converter.fromString(value, getClassFromString(type));
//            Utils.debug("type = " + type + ", pd: " + pd.getPropertyType());
            pd.getWriteMethod().invoke(parent, v);
        }
        consumeToEndofElement(r);
        return parent;
    }

    private static void consumeToEndofElement(XMLStreamReader r) throws XMLStreamException {
        while (r.hasNext()) {
            if (r.next() == END_ELEMENT) {
//                Utils.debug("End Element: " + r.getLocalName());
                break;
            }
        }
    }

    public static void toXml(Object bean, Writer out) throws Exception {
        XMLOutputFactory f = (XMLOutputFactory) Class.forName(XML_STREAM_OUTPUT_FACTORY).newInstance();
        XMLStreamWriter xw = f.createXMLStreamWriter(out);
        xw.writeStartDocument();
        xw.writeStartElement("BeanXml");
        serialize(bean, null, xw);
        xw.writeEndElement();
        xw.writeEndDocument();
    }

    private static void serialize(Object bean, String propertyName, XMLStreamWriter out) throws Exception {
        if (null != bean) {
            try {
                Class type = bean.getClass();
                out.writeStartElement(propertyName == null ? type.getSimpleName() : propertyName);
                out.writeAttribute("type", type.getName());
                Collection<PropertyDescriptor> pds = getSerializableProperties(type);
                for (PropertyDescriptor pd : pds) {
                    Class<?> pt = pd.getPropertyType();
                    try {
                        Object value = pd.getReadMethod().invoke(bean);
//                        String name = pd.getName();
//                        Utils.debug("pd = " + pd.getName() + ", value: " + value);
                        if (isSimpleType(pt)) {
                            writeProperty(pd, bean, out);
                        } else if (pt.isArray()) {
                            out.writeStartElement(pd.getName());
                            out.writeAttribute("type", pt.getComponentType().getName());
                            out.writeAttribute("isArray", "true");
                            out.writeAttribute("isNull", value == null ? "true" : "false");
                            if (null != value) {
                                int len = Array.getLength(value);
                                out.writeAttribute("size", String.valueOf(len));
                                for (int i = 0; i < len; i++) {
                                    Object e = Array.get(value, i);
                                    if (null != e) {
//                                        Utils.debug("Complex type element of array " + pd.getName() + ", value: " + e);
                                        serialize(e, pt.getComponentType().getName(), out);
                                    } else {
                                        // should do something
                                    }
                                }
                            }
                            out.writeEndElement();
                        } else {
                            if (null != value) {
//                                Class<? extends Object> valueType = value.getClass();
                                serialize(value, pd.getName(), out);
//                                if (!pt.equals(valueType) && !pt.isAssignableFrom(valueType)) {
//                                    serialize(value, out);
//                                } else {
//                                    Utils.debug(name + " has a recursive type definition, ignored.");
//                                }
                            } else {
                                writeProperty(pd, bean, out);
                            }
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
                out.writeEndElement();
            } catch (IntrospectionException e) {
                throw new RuntimeException(e);
            }
        } else {
            throw new IllegalArgumentException("Bean can't be null!");
        }
    }

    private static void writeProperty(PropertyDescriptor pd, Object bean, XMLStreamWriter out) throws Exception {
        Class<?> pt = pd.getPropertyType();
        Object value = pd.getReadMethod().invoke(bean);
        out.writeStartElement(pd.getName());
        if (null != value) {
            String s = value.toString();
//            if (isCDATA(pd)) {
//                out.writeAttribute("value", "<![CDATA[" + s + "]]>");
//            } else {
//                out.writeAttribute("value", s);
//            }
            out.writeAttribute("value", s.replaceAll("\\n", "&zwnj;"));
        }
        out.writeAttribute("isNull", null == value ? "true" : "false");
        out.writeAttribute("type", pt.getName());
        out.writeEndElement();
    }

//    private static boolean isCDATA(PropertyDescriptor pd) {
//        Property a = pd.getReadMethod().getAnnotation(Property.class);
//        Property a1 = pd.getWriteMethod().getAnnotation(Property.class);
//        return ( null != a && a.xmlCDATA() ) || ( null != a1 && a1.xmlCDATA() );
//    }

    private static boolean isSimpleType(Class<?> ct) {
        return ct.isPrimitive()
                || ct.equals(Integer.class)
                || ct.equals(Double.class)
                || ct.equals(Long.class)
                || ct.equals(Short.class)
                || ct.equals(Character.class)
                || ct.equals(Date.class)
                || ct.equals(Time.class)
                || ct.equals(Timestamp.class)
                || ct.equals(Boolean.class)
                || ct.equals(Float.class)
                || ct.equals(String.class);
    }

    private static Collection<PropertyDescriptor> getSerializableProperties(Class aClass) throws IntrospectionException {
        Collection<PropertyDescriptor> list = new ArrayList<PropertyDescriptor>();
        BeanInfo beanInfo = Introspector.getBeanInfo(aClass);
        PropertyDescriptor[] pds = beanInfo.getPropertyDescriptors();
        for (PropertyDescriptor pd : pds) {
            Class<?> propertyType = pd.getPropertyType();
            if (null != propertyType) {
                String name = propertyType.getName();
                if (!name.startsWith("java.awt.") && !name.startsWith("javax.swing.")) {
                    Method wm = pd.getWriteMethod();
                    Method rm = pd.getReadMethod();
                    if (null != wm && rm != null) {
                        Property a = wm.getAnnotation(Property.class);
                        Property a1 = rm.getAnnotation(Property.class);
                        if (annotated(a) && annotated(a1)) {
                            list.add(pd);
                        }
                    }
                }
            }
        }
        return list;
    }

    private static boolean annotated(Property a) {
        return null == a || a.xmlAttr();
    }
//
//    private static class ActionSave extends AbstractAction {
//        private DatabaseDiagram _holder;
//
//        public ActionSave(DatabaseDiagram ah) {
//            super("Save");
//            _holder = ah;
//        }
//
//        public void actionPerformed(ActionEvent e) {
//            Writer sw = new StringWriter();
//            try {
//                ArrayHolder<AbstractShape> ah = new ArrayHolder<AbstractShape>();
//                AbstractShape[] shapes = _holder.getDiagram().getShapes();
//                ah.setArray(shapes);
//                BeanXmlSerializer.toXml(ah, sw);
//                FileWriter fw = new FileWriter(C_TEMP_BEAN_XML);
//                fw.write(sw.toString());
//                fw.close();
//            } catch (Exception e1) {
//                e1.printStackTrace();
//            }
//        }
//    }
//
//    private static class ActionRead extends AbstractAction {
//        private DatabaseDiagram _diagram;
//
//        public ActionRead(DatabaseDiagram ah) {
//            super("Load");
//            _diagram = ah;
//        }
//
//        public void actionPerformed(ActionEvent e) {
//            try {
//                FileReader xmlIn = new FileReader(C_TEMP_BEAN_XML);
//                ArrayHolder<Object> ah = (ArrayHolder<Object>) BeanXmlSerializer.fromXml(xmlIn);
//                _diagram.getDiagram().removeAllShapes();
//                for (Object o : ah.getArray()) {
//                    Utils.debug("abstractShape.getObject() = " + o);
//                    _diagram.getDiagram().addShape((AbstractShape) o);
//                }
//                _diagram.fixReferences();
//                xmlIn.close();
//            } catch (Exception e1) {
//                e1.printStackTrace();
//            }
//        }
//    }

}
