package dbhelp.db;

import bean.annotation.Property;
import utils.Utils;

import java.lang.reflect.Field;
import java.sql.Types;

/**
 * User: huojun
 * Date: 5-Sep-2006
 * Time: 3:26:20 PM
 */
public class Column implements INamedDBObject {
//TABLE_CAT String => table catalog (may be null)

    private String _catalog;
//TABLE_SCHEM String => table schema (may be null)

    private String _schema;
//TABLE_NAME String => table name

    private String _tableName;
//COLUMN_NAME String => column name

    private String _name;
//DATA_TYPE int => SQL type from java.sql.Types

    private int _type;
//TYPE_NAME String => Data source dependent type name, for a UDT the type name is fully qualified

    private String _typeName;
//COLUMN_SIZE int => column size. For char or date types this is the maximum number of characters, for numeric or decimal types this is precision.

    private int _columnSize;
//BUFFER_LENGTH is not used.

//DECIMAL_DIGITS int => the number of fractional digits

    private int _decimalDigits;
//NUM_PREC_RADIX int => Radix (typically either 10 or 2)

    private int _radix;
//NULLABLE int => is NULL allowed.

    private int _nullable;
//REMARKS String => comment describing column (may be null)

    private String _remarks;
//COLUMN_DEF String => default value (may be null)

    private String _columnDef;
//SQL_DATA_TYPE int => unused

//SQL_DATETIME_SUB int => unused

//CHAR_OCTET_LENGTH int => for char types the maximum number of bytes in the column

    private int _charOctetLength;
//ORDINAL_POSITION int => index of column in table (starting at 1)

    private int _ordinalPosition;
//IS_NULLABLE String => "NO" means column definitely does not allow NULL values; "YES" means the column might allow NULL values. An empty string means nobody knows.

    private String _isNullable;
//SCOPE_CATLOG String => catalog of table that is the scope of a reference attribute (null if DATA_TYPE isn't REF)

    private String _scopeCatlog;
//SCOPE_SCHEMA String => schema of table that is the scope of a reference attribute (null if the DATA_TYPE isn't REF)

    private String _scopeSchema;
//SCOPE_TABLE String => table name that this the scope of a reference attribure (null if the DATA_TYPE isn't REF)

    private String _scopeTable;
//SOURCE_DATA_TYPE short => source type of a distinct type or user-generated Ref type, SQL type from java.sql.Types (null if DATA_TYPE isn't DISTINCT or user-generated REF)

    private boolean _autoIncrement;

    private String _constraint;

    private short _sourceDateType;

    private PrimaryKey _primaryKey;

    private ForeignKey _foreignKey;

    private Table _table;

    public Column() {
        _name = "New Column";
    }

    public Column(String name, int type) {
        setName(name);
        setType(type);
    }

    public Column(String name, int type, int columnSize) {
        this(name, type);
        setColumnSize( columnSize );
    }

    public String getCatalog() {
        return _catalog;
    }

    @Property(expose = false)
    public String getSimpleName() {
        return getName();
    }

    public boolean isAutoIncrement() {
        return _autoIncrement;
    }

    @Property(expose = false)
    public void setAutoIncrement(boolean autoIncrement) {
        _autoIncrement = autoIncrement;
    }

    @Property(ordinal = 1, category = "Name")
    public void setCatalog(String catalog) {
        _catalog = catalog;
    }

    public String getSchema() {
        return _schema;
    }

    @Property(expose = false, xmlAttr = false)
    public Table getTable() {
        return _table;
    }

    public void setTable(Table table) {
        _table = table;
    }

    @Property(ordinal = 2, category = "Name")
    public void setSchema(String schema) {
        _schema = schema;
    }

    public String getTableName() {
        return _tableName;
    }

    @Property(ordinal = 3, category = "Name")
    public void setTableName(String tableName) {
        _tableName = tableName;
    }

    @Property(desc = "Column constraints")
    public String getConstraint() {
        return _constraint;
    }

    @Property(editor = "bean.editor.SQLEditor")
    public void setConstraint(String constraint) {
        _constraint = constraint;
    }

    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        final Column column = (Column) o;

        if (_type != column._type) return false;
        if (_catalog != null ? !_catalog.equals(column._catalog) : column._catalog != null) return false;
        if (!_name.equals(column._name)) return false;
        if (_schema != null ? !_schema.equals(column._schema) : column._schema != null) return false;
        return _tableName.equals(column._tableName);

    }

    public int hashCode() {
        int result;
        result = (_catalog != null ? _catalog.hashCode() : 0);
        result = 29 * result + (_schema != null ? _schema.hashCode() : 0);
        result = 29 * result + _tableName.hashCode();
        result = 29 * result + _name.hashCode();
        result = 29 * result + _type;
        return result;
    }

    public String getName() {
        return _name;
    }

    @Property(ordinal = 4, category = "Name")
    public void setName(String name) {
        _name = name;
    }

    @Property(desc = "SQL type from java.sql.Types")
    public int getType() {
        return _type;
    }

    @Property(ordinal = 5, editor = "bean.editor.SQLTypeEditor" , category = "Data Type")
    public void setType(int type) {
        _type = type;
    }

    @Property( desc = "Data source dependent type name, for a UDT the type name is fully qualified")
    public String getTypeName() {
        if( null == _typeName ) {
            _typeName = makeTypeName();
        }
        return _typeName;
    }

    private String makeTypeName() {
        Field[] fields = Types.class.getDeclaredFields();
        for (Field f : fields) {
            try {
                if (f.get(null).equals(getType())) {
                    return f.getName();
                }
            } catch (IllegalAccessException e) {
                Utils.debug("Column.makeTypeName: " + e);
            }
        }
        return null;
    }

    @Property(ordinal = 6, category = "Data Type")
    public void setTypeName(String typeName) {
        _typeName = typeName;
    }

    @Property( desc = "column size. For char or date types this is the maximum number of characters, for numeric or decimal types this is precision." )
    public int getColumnSize() {
        return _columnSize;
    }

    @Property(ordinal = 7, category = "Data Type")
    public void setColumnSize(int columnSize) {
        _columnSize = columnSize;
    }

    @Property( desc = "the number of fractional digits")
    public int getDecimalDigits() {
        return _decimalDigits;
    }

    @Property(ordinal = 9, category = "Data Type")
    public void setDecimalDigits(int decimalDigits) {
        _decimalDigits = decimalDigits;
    }

    @Property(desc = "Radix (typically either 10 or 2)")
    public int getRadix() {
        return _radix;
    }

    @Property(ordinal = 10, category = "Data Type")
    public void setRadix(int radix) {
        _radix = radix;
    }

    @Property(desc = "comment describing column (may be null)")
    public int getNullable() {
        return _nullable;
    }

    @Property(ordinal = 11, category = "Data Type")
    public void setNullable(int nullable) {
        _nullable = nullable;
    }

    public String getRemarks() {
        return _remarks;
    }

    @Property(ordinal = 12)
    public void setRemarks(String remarks) {
        _remarks = remarks;
    }

    @Property(desc = "default value (may be null)")
    public String getColumnDef() {
        return _columnDef;
    }

    @Property(ordinal = 13)
    public void setColumnDef(String columnDef) {
        _columnDef = columnDef;
    }

    @Property( desc = "for char types the maximum number of bytes in the column")
    public int getCharOctetLength() {
        return _charOctetLength;
    }

    @Property(ordinal = 16, category = "Data Type")
    public void setCharOctetLength(int charOctetLength) {
        _charOctetLength = charOctetLength;
    }

    @Property(desc = "index of column in table (starting at 1)")
    public int getOrdinalPosition() {
        return _ordinalPosition;
    }

    @Property(ordinal = 17)
    public void setOrdinalPosition(int ordinalPosition) {
        _ordinalPosition = ordinalPosition;
    }

    @Property( desc = "\"NO\" means column definitely does not allow NULL values; \"YES\" means the column might allow NULL values. An empty string means nobody knows.")
    public String getIsNullable() {
        return _isNullable;
    }

    @Property(ordinal = 18, category = "Data Type")
    public void setIsNullable(String nullable) {
        _isNullable = nullable;
    }

    public String getScopeCatlog() {
        return _scopeCatlog;
    }

    @Property(ordinal = 19)
    public void setScopeCatlog(String scopeCatlog) {
        _scopeCatlog = scopeCatlog;
    }

    public String getScopeSchema() {
        return _scopeSchema;
    }

    @Property(ordinal = 20)
    public void setScopeSchema(String scopeSchema) {
        _scopeSchema = scopeSchema;
    }

    public String getScopeTable() {
        return _scopeTable;
    }

    @Property(ordinal = 21)
    public void setScopeTable(String scopeTable) {
        _scopeTable = scopeTable;
    }

    public short getSourceDateType() {
        return _sourceDateType;
    }

    @Property(ordinal = 22)
    public void setSourceDateType(short sourceDateType) {
        _sourceDateType = sourceDateType;
    }

    public boolean isPrimaryKey() {
        return null != _primaryKey;
    }

    public boolean isForeignKey() {
        return null != _foreignKey;
    }

    public PrimaryKey getPrimaryKeyInfo() {
        return _primaryKey;
    }

    @Property( expose = false )
    public void setPrimaryKeyInfo(PrimaryKey primaryKey) {
        _primaryKey = primaryKey;
    }

    public ForeignKey getForeignKeyInfo() {
        return _foreignKey;
    }

    @Property( expose = false )
    public void setForeignKeyInfo(ForeignKey foreignKey) {
        _foreignKey = foreignKey;
    }

    public String toString() {
        StringBuffer sb = new StringBuffer(_name);
        sb.append(" ").append( getTypeString());
        sb.append( isPrimaryKey() ? " (pk)" : "");
        sb.append( isForeignKey() ? " (fk)" : "");
        return sb.toString();
    }

    public String getTypeString() {
        StringBuffer sb = new StringBuffer();
        sb.append(getTypeName());
        int type = getType();
        if (type == Types.CHAR
                || type == Types.VARCHAR
                || type == Types.DECIMAL
                || type == Types.NUMERIC
                || type == Types.VARBINARY
                || type == Types.BINARY) {
            sb.append('(');
            sb.append(getColumnSize());
            if (type == Types.NUMERIC || type == Types.DECIMAL) {
                sb.append(',').append(getDecimalDigits());
            }
            sb.append(')');
        }
        return sb.toString();
    }
}
