package dbhelp.db;

import utils.Utils;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.Map;
import java.util.HashMap;

/**
 * Created by IntelliJ IDEA.
 * User: David Huo
 * Date: Sep 27, 2006
 * Time: 10:54:45 PM
 */
public class ConnectionManager {
    private static Map<Thread, ConnectionCollector> _connectionRecycles = new HashMap<Thread, ConnectionCollector>();

    public static Connection getMetaConnection(Database db) throws Exception {
        Thread thread = Thread.currentThread();
        if (thread.getClass().toString().indexOf("EventDispatchThread") >= 0) {
            // swing thread, use interall connection
            return db.getInterConn();
        } else {
            // else we give a new connection
            synchronized (db) {
                if (_connectionRecycles.containsKey(thread)) {
                    ConnectionCollector cr = _connectionRecycles.get(thread);
                    return cr.getConn();
                } else {
                    ConnectionCollector target = new ConnectionCollector(db.getConnection());
                    _connectionRecycles.put(thread, target);
                    Thread t = new Thread(target);
                    t.start();
                    return target.getConn();
                }
            }
        }
    }

    private static class ConnectionCollector implements Runnable {
        private Connection _conn;
        private Thread _thread;

        public ConnectionCollector(Connection conn) {
            _conn = conn;
            _thread = Thread.currentThread();
            Utils.debug(_conn + " allocated for thread: " + _thread.getId());
        }

        public Connection getConn() {
            return _conn;
        }

        public Thread getThread() {
            return _thread;
        }

        public void run() {
            try {
                _thread.join();
            } catch (InterruptedException e) {
                e.printStackTrace();
            } finally {
                if (null != _conn) {
                    try {
                        _conn.close();
                        Utils.debug(_conn + " recycled for thread: " + _thread.getId());
                    } catch (SQLException e) {
                        //
                    }
                }
            }
        }
    }
}
