package dbhelp.db;

import bean.annotation.Property;
import bean.editor.FileHolder;
import bean.editor.IJarProvider;
import bean.PropertyChangeMultiCaster;
import dbhelp.plugin.complete.ISQLResolver;
import dbhelp.db.model.IMetaProvider;
import utils.Utils;
import utils.Config;

import javax.swing.*;
import java.sql.Connection;
import java.sql.Driver;
import java.sql.SQLException;
import java.sql.ResultSet;
import java.util.*;
import java.text.SimpleDateFormat;

/**
 * User: huojun
 * Date: 7-Sep-2006
 * Time: 10:20:32 AM
 */
public class Database implements IJarProvider {
    private String _name;
    private String _url;
    private String _driver;
    private String _user;
    private String _password;
    private Attribute[] _properties;
    private FileHolder[] _jars;
    private String[] _keywords;
    IDbMeta _metaData;
    private Connection _interConn;
    private boolean _autoCommit = true;
    private boolean _connected;
    private String _metaProvider;
    private ISQLResolver _resolver;
    private String _resolverClass = "dbhelp.plugin.complete.DefaultSQLResolver";
    private SQLCommander _commander;
    private static Map<String, Driver> _loadedDrivers = new HashMap<String, Driver>();
    private String _procedureSyntaxQuery;
    private String _databaseProductName;
    private String _databaseProductVersion;
    private String _quoteString;
    private boolean _useFullObjectName = true;
    private Set<String> _keywordSet;
    protected static final String DEFAULT_DATE_FORMAT = "yyyy-MM-dd";
    protected static final String DEFAULT_TIME_FORMAT = "HH:mm:ss:SSS";
    private String[] _catalogFilter;


    private static String[] SQL92_KEYWORDS = new String[]{"ADA", "ALL", "AND", "ANY", "AS", "ASC", "AT",
            "AVG", "BETWEEN", "BIT", "BIT_LENGTH", "BOTH", "BY",
            "CASE", "CAST", "CHAR", "CHARACTER", "CHAR_LENGTH", "CHARACTER_LENGTH", "COALESCE",
            "COLLATE", "CONVERT", "CORRESPONDING", "COUNT", "CROSS", "CURRENT", "CURRENT_DATE",
            "CURRENT_TIME", "CURRENT_TIMESTAMP", "CURRENT_USER", "DATE", "DAY", "DEC", "DECIMAL",
            "DEFAULT", "DELETE", "DESC", "DISTINCT", "DOUBLE", "ELSE", "END", "ESCAPE", "EXCEPT",
            "EXISTS", "EXTRACT", "FALSE", "FLOAT", "FOR", "FROM", "FULL", "GLOBAL", "GROUP", "HAVING",
            "HOUR", "IN", "INDICATOR", "INNER", "INSERT", "INT", "INTEGER", "INTERSECT", "INTERVAL",
            "INTO", "IS", "JOIN", "LEADING", "LEFT", "LIKE", "LOCAL", "LOWER", "MATCH", "MAX", "MIN",
            "MINUTE", "MODULE", "MONTH", "NATIONAL", "NATURAL", "NCHAR", "NOT", "NULL", "NULLIF", "NUMERIC",
            "OCTET_LENGTH", "OF", "ON", "ONLY", "OR", "ORDER", "OUTER", "OVERLAPS", "PARTIAL", "POSITION",
            "PRECISION", "READ", "REAL", "RIGHT", "SECOND", "SELECT", "SESSION_USER", "SET", "SMALLINT",
            "SOME", "SUBSTRING", "SUM", "SYSTEM_USER", "TABLE", "THEN", "TIME", "TIMESTAMP", "TIMEZONE_HOUR",
            "TIMEZONE_MINUTE", "TO", "TRAILING", "TRANSLATE", "TRIM", "TRUE", "UNION", "UNIQUE", "UNKNOWN",
            "UPDATE", "UPPER", "USER", "USING", "VALUE", "VALUES", "VARCHAR", "VARYING", "WHEN", "WHERE",
            "WITH", "YEAR", "ZONE",
            // none ansi keywords but very common
            "IF", "CREATE", "PROCEDURE", "FUNCTION", "BEGIN", "COMMIT", "ROLLBACK", "EXEC", "EXECUTE"};
    public static Set<String> SQL92_KEYWORD_LIST = new HashSet<String>(Arrays.asList(SQL92_KEYWORDS));
    private int _maximumRowsToRetrieve = 3000;
    protected SimpleDateFormat _dateFormatter = new SimpleDateFormat(DEFAULT_DATE_FORMAT);
    protected SimpleDateFormat _timeFormatter = new SimpleDateFormat(DEFAULT_TIME_FORMAT);
    protected SimpleDateFormat _dateTimeFormatter;

    public Database() {
        this("sun.jdbc.odbc.JdbcOdbcDriver", "jdbc:odbc:[dsn]", "user", "password");
    }

    public Database(String driver, String url, String user, String password) {
        _url = url;
        _driver = driver;
        _user = user;
        _password = password;
    }

    @Property(expose = false, xmlAttr = false)
    public SQLCommander getCommander() {
        if (null == _commander) {
            _commander = new SQLCommander(this);
        }
        return _commander;
    }

    public String quote(INamedDBObject item) {
        return quote(item.getCatalog(), item.getSchema(), item.getSimpleName());
    }

    @Property(category = "format",
            desc = "The time format to convert strings to SQL time. Follow java SimpleDateFormat pattern")
    public String getTimeFormat() {
        return _timeFormatter.toPattern();
    }

    @Property(category = "format")
    public String getDateTimeFormat() {
        return getDateTimeFormatter().toPattern();
    }

    public void setTimeFormat(String timeFormat) {
        String fmt = Utils.hasValue(timeFormat) ? timeFormat : DEFAULT_TIME_FORMAT;
        _timeFormatter = new SimpleDateFormat(fmt);
        _dateTimeFormatter = null;
    }

    public void setDateFormat(String dateFormat) {
        String fmt = Utils.hasValue(dateFormat) ? dateFormat : DEFAULT_DATE_FORMAT;
        _dateFormatter = new SimpleDateFormat(fmt);
        _dateTimeFormatter = null;
    }

    @Property(category = "format",
            desc = "The date format to convert strings to SQL date. Follow java SimpleDateFormat pattern")
    public String getDateFormat() {
        return _dateFormatter.toPattern();
    }

    @Property(expose = false)
    public SimpleDateFormat getDateFormatter() {
        return _dateFormatter;
    }

    @Property(expose = false)
    public SimpleDateFormat getTimeFormatter() {
        return _timeFormatter;
    }

    @Property(expose = false)
    public SimpleDateFormat getDateTimeFormatter() {
        if( null == _dateTimeFormatter ) {
            _dateTimeFormatter = new SimpleDateFormat(getDateFormat() + " " + getTimeFormat());
        }
        return _dateTimeFormatter;
    }

    public String getName() {
        return _name == null ? _url : _name;
    }

    public void setName(String name) {
        _name = name;
    }

    @Property( desc = "The catalogs that the user wants to work against. Usually catalog is a database. The user can " +
            "use this option to filter out other databases which he/she doesn't have the access!")
    public String[] getCatalogFilter() {
        return _catalogFilter;
    }

    @Property( editor = "dbhelp.db.model.CatalogFilterEditor")
    public void setCatalogFilter(String[] catalogFilter) {
        _catalogFilter = catalogFilter;
    }

    private Set<String> getKeywordSet() {
        if (null == _keywordSet) {
            _keywordSet = new HashSet<String>(Arrays.asList(getKeywords()));
        }
        return _keywordSet;
    }

    @Property(expose = false, xmlAttr = false)
    Connection getInterConn() {
        return _interConn;
    }

    @Property(category = "Connection")
    public String getUrl() {
        return _url;
    }

    public Attribute[] getProperties() {
        return _properties;
    }

    @Property(category = "Connection")
    public void setProperties(Attribute[] properties) {
        _properties = properties;
    }

    public void setUrl(String url) {
        _url = url;
        if (null == _name || _name.length() == 0) {
            _name = url;
        }
    }

    @Property(category = "Connection", xmlAttr = false)
    public boolean isConnected() {
        return _connected;
    }

    public ISQLResolver getResolver() throws Exception {
        if (_resolver == null) {
            _resolver = (ISQLResolver) Class.forName(getResolverClass()).newInstance();
        }
        return _resolver;
    }

    public String getResolverClass() {
        return _resolverClass;
    }

    @Property(editor = "bean.editor.ClassNameEditor"
            , className = "dbhelp.plugin.complete.ISQLResolver"
            , defaultClassNames = "dbhelp.plugin.complete.DefaultSQLResolver"
    )
    public void setResolverClass(String resolverClass) {
        if (!_resolverClass.equals(resolverClass)) {
            _resolverClass = resolverClass;
            _resolver = null;
        }
    }

    @Property(desc = "Whether DBHelper uses catalog.schema.object or object as table/stored procdure name references" +
            "Some database doesn't support using catalog in SQL statememnt or you defined the connection URL with database name, " +
            "you can checkRead this option to turn if the full name reference")
    public boolean isUseFullObjectName() {
        return _useFullObjectName;
    }

    public void setUseFullObjectName(boolean useFullObjectName) {
        _useFullObjectName = useFullObjectName;
    }

    public void connect() throws Exception {
        if (isConnected()) {
            disconnect();
        }
        _interConn = getConnection();
        _connected = true;
        getResolver().refresh();
        _keywords = createKeywordList();
    }

    @Property(category = "Connection", xmlAttr = false)
    public boolean isAutoCommit() {
        return _autoCommit;
    }

    public void setAutoCommit(boolean autoCommit) {
        _autoCommit = autoCommit;
    }

    public void disconnect() {
        getCommander().close();
        if (null != _interConn) {
            try {
                _interConn.close();
                _interConn = null;
            } catch (SQLException e) {
                //
            }
        }
        _connected = false;
        _databaseProductName = null;
        _databaseProductVersion = null;
        _quoteString = null;
        PropertyChangeMultiCaster.getInstance().fireChangeEvent(this, "connected");
    }

    public String getDriver() {
        return _driver;
    }

    @Property(desc = "Use $name as the stored procedure name")
    public String getProcedureSyntaxQuery() {
        return _procedureSyntaxQuery;
    }

    @Property(editor = "bean.editor.SQLEditor")
    public void setProcedureSyntaxQuery(String procedureSyntaxQuery) {
        _procedureSyntaxQuery = procedureSyntaxQuery;
    }

    @Property(editor = "bean.editor.ClassNameEditor", category = "Connection",
            className = "java.sql.Driver",
            defaultClassNames = "sun.jdbc.odbc.JdbcOdbcDriver"
    )
    public void setDriver(String driver) {
        _driver = driver;
    }

    public String getUser() {
        return _user;
    }

    @Property(category = "Connection")
    public void setUser(String user) {
        _user = user;
    }

    public String getPassword() {
        return _password;
    }

    @Property(category = "Connection", password = true)
    public void setPassword(String password) {
        _password = password;
    }

    @Property(expose = false, xmlAttr = false)
    public IDbMeta getMetaData() {
        if (null == _metaData) {
            _metaData = new DbMetaImpl(this);
        }
        return _metaData;
    }

    public FileHolder[] getJars() {
        return _jars;
    }

    @Property(category = "Connection")
    public void setJars(FileHolder[] jars) {
        _jars = jars;
    }

    public String toString() {
        return getName();
    }

    public void reportMetaError(final Exception e) {
        PropertyChangeMultiCaster.getInstance().fireChangeEvent(this, "error");
        Runnable runnable = new Runnable() {
            public void run() {
                Utils.error(e);
            }
        };
        SwingUtilities.invokeLater(runnable);
        try {
            // test connection
            ResultSet types = getInterConn().getMetaData().getTableTypes();
            types.close();
        } catch (Exception e1) {
            Utils.debug("test connection exception: " + e1);
            disconnect();
        }
    }

    @Property(expose = false, xmlAttr = false)
    public Connection getConnection() throws Exception {
        Properties info = new Properties();
        info.put("user", getUser());
        info.put("password", getPassword());
        Attribute[] properties = getProperties();
        for (int i = 0; null != properties && i < properties.length; i++) {
            Attribute property = properties[i];
            String name = property.getName();
            if (name.length() > 0) {
                info.put(name, property.getValue());
            }
        }

        Driver driver;
        if (_loadedDrivers.containsKey(getDriver())) {
            driver = _loadedDrivers.get(getDriver());
        } else {
            FileHolder[] jarList = Utils.getJarList(getJars(), Config.getConfig().getExtensionJars());
            driver = (java.sql.Driver) Utils.activate(getDriver(), jarList);
            _loadedDrivers.put(getDriver(), driver);
        }
        Connection connection = driver.connect(getUrl(), info);
        if (null == connection) {
            throw new SQLException("Connection failed!");
        }
        return connection;
    }

    public String[] getKeywords() {
        if (null == _keywords) {
            _keywords = createKeywordList();
        }
        return _keywords;
    }

    @Property(xmlAttr = false)
    public void setKeywords(String[] keywords) {
        _keywords = keywords;
    }

    private String[] createKeywordList() {
        Set<String> a = new HashSet<String>();
        a.addAll(SQL92_KEYWORD_LIST);
        if (isConnected()) {
            IDbMeta md = getMetaData();
            try {
                String[] keywords = md.getKeywords();
                for (int i = 0; null != keywords && i < keywords.length; i++) {
                    a.add(keywords[i].trim().toUpperCase());
                }
            } catch (Exception e) {
                Utils.debug("e = " + e);
            }
        }
        return a.toArray(new String[a.size()]);
    }

    public void setMaximumRowsToRetrieve(int value) {
        _maximumRowsToRetrieve = value;
    }

    @Property(category = "Advanced", xmlAttr = false)
    public int getMaximumRowsToRetrieve() {
        return _maximumRowsToRetrieve;
    }

    public boolean isSameDatabase(Database db) {
        return db != null && getName().equals(db.getName());
    }

    public String getIdentifierQuoteString() {
        if (_quoteString == null) {
            try {
                _quoteString = getMetaData().getIdentifierQuoteString();
            } catch (Exception e) {
                // ignore
            }
        }
        return _quoteString == null ? "" : _quoteString;
    }

    public String unQuote(String token) {
        String q = getIdentifierQuoteString();
        int len = q.length();
        if (token.startsWith(q)) {
            token = token.substring(len, token.length());
        }
        if (token.endsWith(q)) {
            token = token.substring(0, token.length() - len);
        }
        return token;
    }

    public String quote(String o) {
        String q = getIdentifierQuoteString();
        if (o != null) {
            if (o.length() > 0) {
                if (!o.startsWith(q) && (o.indexOf(' ') >= 0 || getKeywordSet().contains(o.toUpperCase()))) {
                    return q + o + q;
                } else {
                    return o;
                }
            }
        }
        return null;
    }

    @Property(category = "Version info", xmlAttr = false)
    public String getDatabaseProductName() {
        if (_databaseProductName == null) {
            try {
                _databaseProductName = getMetaData().getDatabaseProductName();
            } catch (Exception e) {
                // ignore
            }
        }
        return _databaseProductName;
    }

    @Property(category = "Version info", xmlAttr = false)
    public String getDatabaseProductVersion() {
        if (_databaseProductVersion == null)
            try {
                _databaseProductVersion = getMetaData().getDatabaseProductVersion();
            } catch (Exception e) {
                //ignore
            }
        return _databaseProductVersion;
    }

    public String getMetaProvider() {
        return _metaProvider;
    }

    public IMetaProvider getMetaProviderInstance() {
        if (null != _metaProvider) {
            try {
                FileHolder[] jarList = Utils.getJarList(getJars(), Config.getConfig().getExtensionJars());
                return (IMetaProvider) Utils.activate(_metaProvider, jarList);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return null;
    }

    @Property(editor = "bean.editor.ClassNameEditor", category = "Advanced",
            className = "dbhelp.db.model.IMetaProvider", defaultClassNames = "None"
    )
    public void setMetaProvider(String metaProvider) {
        _metaProvider = "None".equals(metaProvider) ? null : metaProvider;
    }

    public SQLCommander spawnCommander() {
        if (null != _commander) {
            _commander.setDiscard(true);
        }
        _commander = new SQLCommander(this);
        return _commander;
    }

    public String quote(String catalog, String schema, String name) {
        if (isUseFullObjectName()) {
            StringBuffer sb = new StringBuffer();
            String cat = quote(catalog);
            sb.append(cat == null ? "" : cat);
            if (cat != null) {
                sb.append('.');
            }
            String s = quote(schema);
            sb.append(s == null ? "" : s);
            if (s != null) {
                sb.append('.');
            }
            sb.append(quote(name));
            return sb.toString();
        } else {
            return quote(name);
        }
    }

    public boolean isKeyword(String s) {
        return null != s && getKeywordSet().contains(s.toUpperCase());
    }
}
