package dbhelp.db;

import bean.annotation.Property;
import bean.annotation.LabelTag;

/**
 * User: huojun
 * Date: 12-Sep-2006
 * Time: 2:05:22 PM
 */
public class ForeignKey implements IDBObject {
//    PKTABLE_CAT String => primary key table catalog being imported (may be null)

    private String _pkTableCatalog;
//PKTABLE_SCHEM String => primary key table schema being imported (may be null)

    private String _pkTableSchema;
//PKTABLE_NAME String => primary key table name being imported

    private String _pkTableName;
//PKCOLUMN_NAME String => primary key column name being imported

    private String _pkColumnName;

//FKTABLE_CAT String => foreign key table catalog (may be null)

    private String _fkTableCatalog;

//FKTABLE_SCHEM String => foreign key table schema (may be null)

    private String _fkTableSchema;

//FKTABLE_NAME String => foreign key table name

    private String _fkTableName;

//FKCOLUMN_NAME String => foreign key column name

    private String _fkColumnName;

//KEY_SEQ short => sequence number within a foreign key

    private short _keySeq;

//UPDATE_RULE short => What happens to a foreign key when the primary key is updated:

    private short _updateRule;
//importedNoAction - do not allow update of primary key if it has been imported
//importedKeyCascade - change imported key to agree with primary key update
//importedKeySetNull - change imported key to NULL if its primary key has been updated
//importedKeySetDefault - change imported key to default values if its primary key has been updated
//importedKeyRestrict - same as importedKeyNoAction (for ODBC 2.x compatibility)
//DELETE_RULE short => What happens to the foreign key when primary is deleted.

    private short _deleteRule;
//importedKeyNoAction - do not allow delete of primary key if it has been imported
//importedKeyCascade - delete rows that import a deleted key
//importedKeySetNull - change imported key to NULL if its primary key has been deleted
//importedKeyRestrict - same as importedKeyNoAction (for ODBC 2.x compatibility)
//importedKeySetDefault - change imported key to default if its primary key has been deleted
//FK_NAME String => foreign key name (may be null)

    private String _fkName;
//PK_NAME String => primary key name (may be null)

    private String _pkName;
//DEFERRABILITY short => can the evaluation of foreign key constraints be deferred until commit

    private short _deferrability;
//importedKeyInitiallyDeferred - see SQL92 for definition
//importedKeyInitiallyImmediate - see SQL92 for definition
//importedKeyNotDeferrable - see SQL92 for definition

    public ForeignKey() {
    }

    public String getPkTableCatalog() {
        return _pkTableCatalog;
    }

    @Property(ordinal = 1, category = "Names")
    public void setPkTableCatalog(String pkTableCatalog) {
        _pkTableCatalog = pkTableCatalog;
    }

    public String getPkTableSchema() {
        return _pkTableSchema;
    }

    @Property(ordinal = 2, category = "Names")
    public void setPkTableSchema(String pkTableSchema) {
        _pkTableSchema = pkTableSchema;
    }

    public String getPkTableName() {
        return _pkTableName;
    }

    @Property(ordinal = 3, category = "Names")
    public void setPkTableName(String pkTableName) {
        _pkTableName = pkTableName;
    }

    public String getPkColumnName() {
        return _pkColumnName;
    }

    @Property(ordinal = 4, category = "Names")
    public void setPkColumnName(String pkColumnName) {
        _pkColumnName = pkColumnName;
    }

    public String getFkTableCatalog() {
        return _fkTableCatalog;
    }

    @Property(ordinal = 5, category = "Names")
    public void setFkTableCatalog(String fkTableCatalog) {
        _fkTableCatalog = fkTableCatalog;
    }

    public String getFkTableSchema() {
        return _fkTableSchema;
    }

    @Property(ordinal = 6, category = "Names")
    public void setFkTableSchema(String fkTableSchema) {
        _fkTableSchema = fkTableSchema;
    }

    public String getFkTableName() {
        return _fkTableName;
    }

    @Property(ordinal = 7, category = "Names")
    public void setFkTableName(String fkTableName) {
        _fkTableName = fkTableName;
    }

    public String getFkColumnName() {
        return _fkColumnName;
    }

    @Property(ordinal = 8, category = "Names")
    public void setFkColumnName(String fkColumnName) {
        _fkColumnName = fkColumnName;
    }

    public short getKeySeq() {
        return _keySeq;
    }

    @Property(ordinal = 9)
    public void setKeySeq(short keySeq) {
        _keySeq = keySeq;
    }

    public short getUpdateRule() {
        return _updateRule;
    }

    @Property(ordinal = 10, category = "Rules", editor = "bean.editor.CodeLabelEditor",
            tags = {
            @LabelTag(id = 0, label = "Cascade"), @LabelTag(id = 1, label = "Restrict"),
            @LabelTag(id = 2, label = "SetNull"), @LabelTag(id = 3, label = "NoAction"),
            @LabelTag(id = 4, label = "SetDefault")
                    }
    )
    public void setUpdateRule(short updateRule) {
        _updateRule = updateRule;
    }

    public short getDeleteRule() {
        return _deleteRule;
    }

    @Property(ordinal = 11, category = "Rules", editor = "bean.editor.CodeLabelEditor",
            tags = {
            @LabelTag(id = 0, label = "Cascade"), @LabelTag(id = 1, label = "Restrict"),
            @LabelTag(id = 2, label = "SetNull"), @LabelTag(id = 3, label = "NoAction"),
            @LabelTag(id = 4, label = "SetDefault")
                    }
    )
    public void setDeleteRule(short deleteRule) {
        _deleteRule = deleteRule;
    }

    @Property(expose = false)
    public String getDeleteRuleString() {
        return ruleTypeToString(_deleteRule);
    }

    @Property(expose = false)
    public String getUpdateRuleString() {
        return ruleTypeToString(_updateRule);
    }

    private String ruleTypeToString(short ruleType) {
        switch( ruleType ) {
            case 0: return "CASCADE";
            case 1: return "RESTRICT";
            case 2: return "SETNULL";
            case 3: return "NOACTION";
            default: return "SETDEFAULT";
        }
    }

    public String getFkName() {
        return _fkName;
    }

    @Property(ordinal = 12, category = "Names")
    public void setFkName(String fkName) {
        _fkName = fkName;
    }

    public String getPkName() {
        return _pkName;
    }

    @Property(ordinal = 13, category = "Names")
    public void setPkName(String pkName) {
        _pkName = pkName;
    }

    public short getDeferrability() {
        return _deferrability;
    }

    @Property(ordinal = 14, editor = "bean.editor.CodeLabelEditor",
            tags = {
            @LabelTag(id = 5, label = "Initially Deferred"), @LabelTag(id = 6, label = "Initially Immediate"),
            @LabelTag(id = 7, label = "Not Deferrable"), @LabelTag(id = 0, label = "Undefined")
                    }
    )
    public void setDeferrability(short deferrability) {
        _deferrability = deferrability;
    }

    public String toString() {
        return _fkColumnName + " --> " + _pkTableName + "." + _pkColumnName;
    }

    @Property(expose = false)
    public String getSimpleName() {
        return toString();
    }
}
