package dbhelp.db;

import bean.annotation.Property;
import bean.annotation.LabelTag;

import java.sql.DatabaseMetaData;

/**
 * User: huojun
 * Date: 20-Sep-2006
 * Time: 1:01:15 PM
 */
public class Index implements IDBObject, INamedDBObject {
//    TABLE_CAT String => table catalog (may be null)
    private String _catalog;
//    TABLE_SCHEM String => table schema (may be null)
    private String _schema;
//    TABLE_NAME String => table name
    private String _name;
//    NON_UNIQUE boolean => Can index values be non-unique. false when TYPE is tableIndexStatistic
    private boolean nonUnique;
//    INDEX_QUALIFIER String => index catalog (may be null); null when TYPE is tableIndexStatistic
    private String _qualifier;
//    INDEX_NAME String => index name; null when TYPE is tableIndexStatistic
    private String _indexName;
//    TYPE short => index type:
    private short _indexType;
//    tableIndexStatistic - this identifies table statistics that are returned in conjuction with a table's index descriptions
//    tableIndexClustered - this is a clustered index
//    tableIndexHashed - this is a hashed index
//    tableIndexOther - this is some other style of index
//    ORDINAL_POSITION short => column sequence number within index; zero when TYPE is tableIndexStatistic
    private short _ordinalPosition;
//    COLUMN_NAME String => column name; null when TYPE is tableIndexStatistic
    private String _columnName;
//    ASC_OR_DESC String => column sort sequence, "A" => ascending, "D" => descending, may be null if sort sequence is not supported; null when TYPE is tableIndexStatistic
    private String _ascOrDesc;
//    CARDINALITY int => When TYPE is tableIndexStatistic, then this is the number of rows in the table; otherwise, it is the number of unique values in the index.
    private int _cardinality;
//    PAGES int => When TYPE is tableIndexStatisic then this is the number of pages used for the table, otherwise it is the number of pages used for the current index.
    private int _pages;
//    FILTER_CONDITION String => Filter condition, if any. (may be null)
    private String _filterCondition;

    public Index() {
    }

    public String getCatalog() {
        return _catalog;
    }

    @Property(ordinal = 1)
    public void setCatalog(String catalog) {
        _catalog = catalog;
    }

    public String getSchema() {
        return _schema;
    }

    @Property(ordinal = 2)
    public void setSchema(String schema) {
        _schema = schema;
    }

    public String getName() {
        return _name;
    }

    @Property(ordinal = 3)
    public void setName(String name) {
        _name = name;
    }

    public boolean isNonUnique() {
        return nonUnique;
    }

    @Property(ordinal = 4)
    public void setNonUnique(boolean nonUnique) {
        this.nonUnique = nonUnique;
    }

    public String getQualifier() {
        return _qualifier;
    }

    @Property(ordinal = 5)
    public void setQualifier(String qualifier) {
        _qualifier = qualifier;
    }

    public String getIndexName() {
        return _indexName;
    }

    @Property(ordinal = 6)
    public void setIndexName(String indexName) {
        _indexName = indexName;
    }

    public short getIndexType() {
        return _indexType;
    }

    @Property(ordinal = 7, editor = "bean.editor.CodeLabelEditor",
            tags = {
            @LabelTag(id = 0, label = "Table Index Statistic"), @LabelTag(id = 1, label = "Table Index Clustered"),
            @LabelTag(id = 2, label = "Table Index Hashed"), @LabelTag(id = 3, label = "Table Index Other")
                    }
    )
    public void setIndexType(short indexType) {
        _indexType = indexType;
    }

    public short getOrdinalPosition() {
        return _ordinalPosition;
    }

    @Property(ordinal = 8)
    public void setOrdinalPosition(short ordinalPosition) {
        _ordinalPosition = ordinalPosition;
    }

    public String getColumnName() {
        return _columnName;
    }

    @Property(ordinal = 9)
    public void setColumnName(String columnName) {
        _columnName = columnName;
    }

    public String getAscOrDesc() {
        return _ascOrDesc;
    }

    @Property(ordinal = 10)
    public void setAscOrDesc(String ascOrDesc) {
        _ascOrDesc = ascOrDesc;
    }

    public int getCardinality() {
        return _cardinality;
    }

    @Property(ordinal = 11)
    public void setCardinality(int cardinality) {
        _cardinality = cardinality;
    }

    public int getPages() {
        return _pages;
    }

    @Property(ordinal = 12)
    public void setPages(int pages) {
        _pages = pages;
    }

    public String getFilterCondition() {
        return _filterCondition;
    }

    @Property(ordinal = 13)
    public void setFilterCondition(String filterCondition) {
        _filterCondition = filterCondition;
    }

    public String toString() {
        return _indexType == DatabaseMetaData.tableIndexStatistic ? "Table Index" : _indexName;
    }

    @Property(expose = false)
    public String getSimpleName() {
        return getName();
    }
}
