package dbhelp.db;

import java.util.HashMap;
import java.util.Map;

/**
 * Created by IntelliJ IDEA.
 * User: David Huo
 * Date: Oct 28, 2006
 * Time: 11:53:03 AM
 */
public class JDBCDriver {
    private String _name;
    private String[] _urlPattern;
    private String _className;


    public JDBCDriver() {
    }

    public JDBCDriver(String name, String className, String... urlPattern) {
        _name = name;
        _urlPattern = urlPattern;
        _className = className;
    }

    public String getName() {
        return _name;
    }

    public String[] getUrlPattern() {
        return _urlPattern;
    }

    public String getClassName() {
        return _className;
    }

    public String toString() {
        return getName() + "(" + getClassName() + ")";
    }

    public static void addDriver(JDBCDriver d) {
        _driverList.put(d.getClassName(), d);
    }

    public static JDBCDriver getDriver(String className) {
        return _driverList.get(className);
    }

    private static Map<String, JDBCDriver> _driverList = new HashMap<String, JDBCDriver>();

    static {


        addDriver(new JDBCDriver("IBM DB2",
                "COM.ibm.db2.jdbc.app.DB2Driver", "jdbc:db2://<HOST>:<PORT>/<DB>"
        ));


        addDriver(new JDBCDriver("JDBC-ODBC Bridge",
                "sun.jdbc.odbc.JdbcOdbcDriver", "jdbc:odbc:<ODBC DSN>"
        ));

        addDriver(new JDBCDriver("Microsoft SQL Server",
                "weblogic.jdbc.mssqlserver4.Driver", "jdbc:weblogic:mssqlserver4:<DB>@<HOST>:<PORT>"
        ));

        addDriver(new JDBCDriver("Oracle Thin",
                "oracle.jdbc.driver.OracleDriver", "jdbc:oracle:thin:@<HOST>:<PORT>:<SID>"
        ));

        addDriver(new JDBCDriver("PointBase Embedded Server",
                "com.pointbase.jdbc.jdbcUniversalDriver", "jdbc:pointbase://embedded[:<PORT>]/<DB>"
        ));

        addDriver(new JDBCDriver("Cloudscape",
                "COM.cloudscape.core.JDBCDriver", "jdbc:cloudscape:<DB>"
        ));

        addDriver(new JDBCDriver("Cloudscape RMI",
                "RmiJdbc.RJDriver", "jdbc:rmi://<HOST>:<PORT>/jdbc:cloudscape:<DB>"
        ));

        addDriver(new JDBCDriver("Firebird (JCA/JDBC Driver)",
                "org.firebirdsql.jdbc.FBDriver", "jdbc:firebirdsql:[//<HOST>[:<PORT>]/]<DB>"
        ));

        addDriver(new JDBCDriver("IDS Server",
                "ids.sql.IDSDriver", "jdbc:ids://<HOST>:<PORT>/conn?dsn='<ODBC_DSN_NAME>'"
        ));

        addDriver(new JDBCDriver("Informix Dynamic Server",
                "com.informix.jdbc.IfxDriver", "jdbc:informix-sqli://<HOST>:<PORT>/<DB>:INFORMIXSERVER=<SERVER_NAME>"
        ));

        addDriver(new JDBCDriver("InstantDB (v3.13 and earlier)",
                "jdbc.idbDriver", "jdbc:idb:<DB>"
        ));

        addDriver(new JDBCDriver("InstantDB (v3.14 and later)",
                "org.enhydra.instantdb.jdbc.idbDriver", "jdbc:idb:<DB>"
        ));

        addDriver(new JDBCDriver("Interbase (InterClient Driver)",
                "interbase.interclient.Driver", "jdbc:interbase://<HOST>/<DB>"
        ));

        addDriver(new JDBCDriver("Hypersonic SQL (v1.2 and earlier)",
                "hSql.hDriver", "jdbc:HypersonicSQL:<DB>"
        ));

        addDriver(new JDBCDriver("Hypersonic SQL (v1.3 and later)",
                "org.hsql.jdbcDriver", "jdbc:HypersonicSQL:<DB>"
        ));

        addDriver(new JDBCDriver("Microsoft SQL JDBCDriver(JTurbo Driver)",
                "com.ashna.jturbo.driver.Driver", "jdbc:JTurbo://<HOST>:<PORT>/<DB>"
        ));

        addDriver(new JDBCDriver("Microsoft SQL Server 2000 (Microsoft Driver)",
                "com.microsoft.jdbc.sqlserver.SQLServerDriver", "jdbc:microsoft:sqlserver://<HOST>:<PORT>[;DatabaseName=<DB>]"
        ));

        addDriver(new JDBCDriver("MySQL (MM.MySQL Driver)",
                "org.gjt.mm.mysql.Driver", "jdbc:mysql://<HOST>:<PORT>/[<DB>]"
        ));

        addDriver(new JDBCDriver("MySQL 5.0 (mysql-connector-java)",
                "com.mysql.jdbc.Driver", "jdbc:mysql://<HOST>:<PORT>/<DB>"
        ));

        addDriver(new JDBCDriver("Oracle OCI 9i",
                "oracle.jdbc.driver.OracleDriver"
                , "jdbc:oracle:oci:@<SID>"
                , "jdbc:oracle:thin:@<host>:[port]:<SID>"
        ));

        addDriver(new JDBCDriver("PostgreSQL (v6.5 and earlier)",
                "postgresql.Driver", "jdbc:postgresql://<HOST>:<PORT>/<DB>"
        ));

        addDriver(new JDBCDriver("PostgreSQL (v7.0 and later)",
                "org.postgresql.Driver", "jdbc:postgresql://<HOST>:<PORT>/<DB>"
        ));

        addDriver(new JDBCDriver("Sybase (jConnect 5.x)",
                "com.sybase.jdbc2.jdbc.SybDriver", "jdbc:sybase:Tds:<HOST>:<PORT>"
        ));

        addDriver(new JDBCDriver("Sybase (jConnect 6.x)",
                "com.sybase.jdbc3.jdbc.SybDriver", "jdbc:sybase:Tds:<HOST>:<PORT>"
        ));

        addDriver(new JDBCDriver("Sybase SQL Anywhere JODBC",
                "ianywhere.ml.jdbcodbc.IDriver", "jdbc:odbc:dsn=<ODBC DSN>"
        ));

        addDriver(new JDBCDriver("H2 Database Engine",
                "org.h2.Driver", "jdbc:h2:<Database Name>"
        ));
    }

}
