package dbhelp.db;

import bean.annotation.LabelTag;
import bean.annotation.Property;
import org.apache.velocity.VelocityContext;
import template.TemplateManager;
import utils.Utils;

import java.io.StringReader;
import java.io.StringWriter;
import static java.sql.DatabaseMetaData.*;
import java.util.ArrayList;
import java.util.List;

/**
 * User: huojun
 * Date: 15-Sep-2006
 * Time: 3:01:23 PM
 */
public class Procedure implements INamedDBObject {
//PROCEDURE_CAT String => procedure catalog (may be null)
    private String _catalog;
//PROCEDURE_SCHEM String => procedure schema (may be null)
    private String _schema;
//PROCEDURE_NAME String => procedure name
    private String _name;
//reserved for future use
//reserved for future use
//reserved for future use
//REMARKS String => explanatory comment on the procedure
    private String _remarks;
//PROCEDURE_TYPE short => kind of procedure:
    private short _procedureType;
//procedureResultUnknown - May return a result
//procedureNoResult - Does not return a result
//procedureReturnsResult - Returns a result
    private ProcedureColumn[] _parameters;
    private ProcedureColumn[] _resultset;
    private ProcedureColumn _return;

    private Database _database;

    public Procedure() {
    }

    public Procedure(String name) {
        setName(name);
    }

    @Property(expose = false)
    public Database getDatabase() {
        return _database;
    }

    public ProcedureColumn getReturn() {
        return _return;
    }

    public void setReturn(ProcedureColumn aReturn) {
        _return = aReturn;
    }

    public void setDatabase(Database database) {
        _database = database;
    }

    public String getCatalog() {
        return _catalog;
    }

    @Property(ordinal = 1)
    public void setCatalog(String catalog) {
        _catalog = catalog;
    }

    public String getSchema() {
        return _schema;
    }

    @Property(ordinal = 2)
    public void setSchema(String schema) {
        _schema = schema;
    }

    public String getName() {
        return _name;
    }

    @Property(ordinal = 3)
    public void setName(String name) {
        _name = name;
    }

    public String getRemarks() {
        return _remarks;
    }

    @Property(ordinal = 7)
    public void setRemarks(String remarks) {
        _remarks = remarks;
    }

    public short getProcedureType() {
        return _procedureType;
    }

    @Property(ordinal = 8, editor = "bean.editor.CodeLabelEditor",
            tags = {
            @LabelTag(id = 2, label = "Returns Result"), @LabelTag(id = 0, label = "Result Unknown"),
            @LabelTag(id = 1, label = "NoResult")
                    })
    public void setProcedureType(short procedureType) {
        _procedureType = procedureType;
    }

    public ProcedureColumn[] getParameters() {
        if( null == _parameters ) {
            getColumnInfo();
        }
        return _parameters;
    }

    private void getColumnInfo() {
        if( _database != null && _database.isConnected() ) {
            try {
                List<ProcedureColumn> columns = _database.getMetaData().getProcedureColumns(this);
                List<ProcedureColumn> params = new ArrayList<ProcedureColumn>();
                List<ProcedureColumn> results = new ArrayList<ProcedureColumn>();
                for (ProcedureColumn c : columns) {
                    short ct = c.getColumnType();
                    if( ct == procedureColumnIn
                            || ct == procedureColumnInOut || ct == procedureColumnOut ) {
                        params.add(c);
                    } else if ( ct == procedureColumnResult ) {
                        results.add(c);
                    } else if( ct == procedureColumnReturn ) {
                        _return = c;
                    }
                }
                _resultset = results.toArray(new ProcedureColumn[results.size()]);
                _parameters = params.toArray(new ProcedureColumn[params.size()]);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    public void setParameters(ProcedureColumn[] parameters) {
        _parameters = parameters;
    }

    public ProcedureColumn[] getResultset() {
        if( null == _resultset ) {
            getColumnInfo();
        }
        return _resultset;
    }

    public void setResultset(ProcedureColumn[] resultset) {
        _resultset = resultset;
    }

    public String getQuotedName() {
        return null == getDatabase() ? toString() : getDatabase().quote(this);
    }

    public String toString() {
        String cat = null == _catalog ? "" : _catalog + ".";
        String schema = null == _schema ? "" : _schema + ".";
//        return cat + schema + (_name.indexOf(' ') >= 0 ? "\"" + _name + "\"" : _name);
        return cat + schema + _name;
    }

    @Property(expose = false)
    public String getSimpleName() {
        return getName();
    }

    public static String getProcedureSyntax(Procedure p) {
        try {
            StringWriter out = new StringWriter();
            Database db = p.getDatabase();
            if ( null != db ) {
                String query = db.getProcedureSyntaxQuery();
                if ( query != null && query.trim().length() > 0 ) {
                    StringReader in = new StringReader(query);
                    VelocityContext c = new VelocityContext();
                    c.put("name", p);
                    c.put("fullName", p);
                    if (TemplateManager.getMgr().runTemplate(c, out, in)) {
                        String queryCmd = out.toString();
                        return db.getMetaData().getProcedureSyntax(queryCmd);
                    }
                } else {
                    Utils.debug("Stored procedure query is not defined in the database profile!");
                }
            } else {
                Utils.debug("Database is not defined for this procedure: " + p);
            }
        } catch (Exception e) {
            Utils.debug("Procedure.getProcedureSyntax: " + e);
            e.printStackTrace();
        }
        return null;
    }
}
