package dbhelp.db;

import bean.annotation.Property;

import java.util.List;
import java.util.ArrayList;

/**
 * User: huojun
 * Date: 5-Sep-2006
 * Time: 3:08:54 PM
 */
public class Table implements INamedDBObject {
//    TABLE_CAT String => table catalog (may be null)

    private String _catalog;
//TABLE_SCHEM String => table schema (may be null)

    private String _schema;
//TABLE_NAME String => table name

    private String _name;
//TABLE_TYPE String => table type. Typical types are "TABLE", "VIEW", "SYSTEM TABLE", "GLOBAL TEMPORARY", "LOCAL TEMPORARY", "ALIAS", "SYNONYM".

    private String _tableType;
//REMARKS String => explanatory comment on the table

    private String _remarks;
//TYPE_CAT String => the types catalog (may be null)

    private String _typeCatalog;
//TYPE_SCHEM String => the types schema (may be null)

    private String _typeSchema;
//TYPE_NAME String => type name (may be null)

    private String _typeName;
//SELF_REFERENCING_COL_NAME String => name of the designated "identifier" column of a typed table (may be null)

    private String _selfReferencingColName;
//REF_GENERATION String => specifies how values in SELF_REFERENCING_COL_NAME are created. Values are "SYSTEM", "USER", "DERIVED". (may be null)

    private String _refGeneration;

    private String _constraint;

    private Column[] _columns = new Column[] {};

    private Database _database;

    private PrimaryKey[] _primaryKeys;

    private ForeignKey[] _foreignKeys;

    private Index[] _indexes;

    private Table[] _referencedTables;

    private ForeignKey[] _exportedKeys;

    private Attribute[] _attributes;


    public Table() {
        this("Table New");
    }

    public Table(String catalog, String schema, String name) {
        _catalog = catalog;
        _schema = schema;
        _name = name;
    }

    public Table(String name) {
        _name = name;
    }

    public String getCatalog() {
        return _catalog;
    }

    @Property( ordinal = 1 )
    public void setCatalog(String catalog) {
        _catalog = catalog;
    }

    public String getSchema() {
        return _schema;
    }

    @Property( ordinal = 2 )
    public void setSchema(String schema) {
        _schema = schema;
    }

    public String getName() {
        return _name;
    }

    @Property( ordinal = 3 )
    public void setName(String name) {
        _name = name;
    }

    @Property(desc = "table type. Typical types are \"TABLE\", \"VIEW\", \"SYSTEM TABLE\", \"GLOBAL TEMPORARY\", \"LOCAL TEMPORARY\", \"ALIAS\", \"SYNONYM\".")
    public String getTableType() {
        return _tableType;
    }

    @Property( ordinal = 4 )
    public void setTableType(String tableType) {
        _tableType = tableType;
    }

    public String getRemarks() {
        return _remarks;
    }

    @Property( ordinal = 5 )
    public void setRemarks(String remarks) {
        _remarks = remarks;
    }

    public String getTypeCatalog() {
        return _typeCatalog;
    }

    public String getQuotedName() {
        return getDatabase() == null ? toString() : getDatabase().quote(this);
    }

    @Property( ordinal = 6 )
    public void setTypeCatalog(String typeCatalog) {
        _typeCatalog = typeCatalog;
    }

    public String getTypeSchema() {
        return _typeSchema;
    }

    public Column[] getColumns() {
        if( null == _columns || _columns.length == 0 ) {
            if( _database != null ) {
                try {
                    List<Column> columns = _database.getMetaData().getColumns(this);
                    _columns = columns.toArray(new Column[columns.size()]);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
        return _columns;
    }

    public void setColumns(Column[] columns) {
        _columns = columns;
    }

    public Index[] getIndexes() {
        if( _indexes == null ) {
            if( null != _database && _database.isConnected() ) {
                try {
                    List<Index> indexes = _database.getMetaData().getIndexes(this);
                    _indexes = indexes.toArray( new Index[indexes.size()]);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
        return _indexes;
    }

    public void setIndexes(Index[] indexes) {
        _indexes = indexes;
    }

    @Property( ordinal = 7 )
    public void setTypeSchema(String typeSchema) {
        _typeSchema = typeSchema;
    }


    public String getTypeName() {
        return _typeName;
    }

    @Property( ordinal = 8 )
    public void setTypeName(String typeName) {
        _typeName = typeName;
    }

    @Property( desc = "name of the designated \"identifier\" column of a typed table (may be null)")
    public String getSelfReferencingColName() {
        return _selfReferencingColName;
    }

    @Property( ordinal = 9 )
    public void setSelfReferencingColName(String selfReferencingColName) {
        _selfReferencingColName = selfReferencingColName;
    }

    @Property( desc = "specifies how values in SELF_REFERENCING_COL_NAME are created. Values are \"SYSTEM\", \"USER\", \"DERIVED\". (may be null)")
    public String getRefGeneration() {
        return _refGeneration;
    }

    @Property( ordinal = 10 )
    public void setRefGeneration(String refGeneration) {
        _refGeneration = refGeneration;
    }

    @Property(expose = false)
    public Database getDatabase() {
        return _database;
    }

    public void setDatabase(Database database) {
        _database = database;
    }

    @Property(desc = "Specify the table level constraints")
    public String getConstraint() {
        return _constraint;
    }

    @Property(editor = "bean.editor.SQLEditor")
    public void setConstraint(String constraint) {
        _constraint = constraint;
    }

    @Property(desc = "Extra database specific table definition goes here")
    public Attribute[] getAttributes() {
        return _attributes;
    }

    public void setAttributes(Attribute[] attributes) {
        _attributes = attributes;
    }

    public String toString() {
        String cat = null == _catalog ? "" : _catalog + ".";
        String s = null == _schema ? "" : _schema + ".";
        return cat + s + _name;
    }

    public void setPrimaryKeys(PrimaryKey[] primaryKeys) {
        _primaryKeys = primaryKeys;
    }

    public PrimaryKey[] getPrimaryKeys() {
        if( null == _primaryKeys ) {
            List<PrimaryKey> list = new ArrayList<PrimaryKey>();
            Column[] columns = getColumns();
            for( int i=0; null != columns && i<columns.length; i++ ) {
                PrimaryKey primaryKeyInfo = columns[i].getPrimaryKeyInfo();
                if( primaryKeyInfo != null ) {
                    list.add(primaryKeyInfo);
                }
            }
            _primaryKeys = list.toArray(new PrimaryKey[list.size()]);
        }
        return _primaryKeys;
    }

    public ForeignKey[] getForeignKeys() {
        if( null == _foreignKeys ) {
            List<ForeignKey> list = new ArrayList<ForeignKey>();
            Column[] columns = getColumns();
            for( int i=0; null != columns && i<columns.length; i++ ) {
                ForeignKey fKeyInfo = columns[i].getForeignKeyInfo();
                if( fKeyInfo != null ) {
                    list.add(fKeyInfo);
                }
            }
            _foreignKeys = list.toArray(new ForeignKey[list.size()]);
        }
        return _foreignKeys;
    }

    public void setForeignKeys(ForeignKey[] foreignKeys) {
        _foreignKeys = foreignKeys;
    }

    @Property(expose = false)
    public ForeignKey[] getExportedKeys() {
        if( null == _exportedKeys ) {
            if( _database != null && _database.isConnected() ) {
                try {
                    List<ForeignKey> exportedKeys = _database.getMetaData().getExportedKeys(this);
                    _exportedKeys = exportedKeys.toArray(new ForeignKey[exportedKeys.size()]);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
        return _exportedKeys;
    }

    @Property( expose = false )
    public Table[] getReferencedTables() {
        if( _referencedTables == null ) {
            try {
                IDbMeta m = getDatabase().getMetaData();
                List<Table> list = new ArrayList<Table>();
                for( ForeignKey fk : getExportedKeys() ) {
                    List<Table> tables = m.getTables(fk.getFkTableCatalog(), fk.getFkTableSchema(), null, fk.getFkTableName());
                    if( tables.size() > 0 ) {
                        list.add(tables.get(0));
                    }
                }
                _referencedTables = list.toArray( new Table[list.size()]);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return _referencedTables;
    }

    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        final Table table = (Table) o;
        return !(_catalog != null ? !_catalog.equals(table._catalog) : table._catalog != null)
                && _name.equals(table._name) && !(_schema != null ? !_schema.equals(table._schema) : table._schema != null);
    }

    public int hashCode() {
        int result;
        result = (_catalog != null ? _catalog.hashCode() : 0);
        result = 29 * result + (_schema != null ? _schema.hashCode() : 0);
        result = 29 * result + _name.hashCode();
        return result;
    }

    @Property(expose = false)
    public String getSimpleName() {
        return getName();
    }
}
