package dbhelp.db.model;

import bean.editor.AbstractEditor;
import bean.editor.ICustomEditor;
import bean.editor.utils.CheckList;
import bean.editor.utils.CheckListModel;
import bean.editor.utils.Item;
import dbhelp.db.Catalog;
import dbhelp.db.Database;
import dbhelp.db.IDbMeta;
import utils.Utils;

import javax.swing.*;
import java.awt.*;
import java.util.*;

/**
 * User: huojun
 * Date: 6-Oct-2006
 * Time: 10:42:47 PM
 */
public class CatalogFilterEditor extends AbstractEditor {
    private MyEditor _uiEditor;

    public boolean supportsCustomEditor() {
        return true;
    }

    public Component getCustomEditor() {
        if( null == _uiEditor ) {
            Database f = (Database) getBean();
            MyModel model = new MyModel(f);
            model.setValue((Object[]) getValue());
            _uiEditor = new MyEditor(model);
        }
        return _uiEditor;
    }

    public void editConfirm() {
        Object o = _uiEditor.getValue();
        setValue(o);
    }

    public String getAsText() {
        Object[] value = (Object[]) getValue();
        return Utils.arrayToString(value);
    }

    public void setAsText(String text) throws IllegalArgumentException {
        if( text.length() == 0 ) {
            setValue( null );
        }
    }

    private static class MyModel extends CheckListModel<String> {

        public MyModel(Database database) {
            java.util.List<String> _objects = new ArrayList<String>();
            if( database.isConnected() ) {
                try {
                    IDbMeta md = database.getMetaData();
                    java.util.List<Catalog> catalogs = md.getAllCatalogs();
                    for(Catalog c : catalogs) {
                        _objects.add(c.getName());
                    }
                } catch (Exception e) {
                    Utils.debug("FilterEditor$MyModel.MyModel: " + e);
                }
            }
            setData(_objects);
        }

        public void setValue(Object[] objects) {
            for (int i = 0; null != objects && i < objects.length; i++) {
                Object object = objects[i];
                for(Item item : _list ) {
                    if(item.getValue().toString().equals(object.toString())) {
                        item.setSelected(true);
                        break;
                    }
                }
            }
            fireContentsChanged(this, 0, _list.size() - 1);
        }
    }

    private static class MyEditor extends JPanel implements ICustomEditor {
        private MyModel _model;
        public MyEditor(MyModel model) {
            super( new BorderLayout() );
            _model = model;
            CheckList table = new CheckList(model);
            add( new JScrollPane(table), BorderLayout.CENTER );
            setSize(320, 250);
        }

        public Object getValue() {
            java.util.List<String> list = _model.getData();
            return list.toArray(new String[list.size()]);
        }

        public void accept() {
        }

        public void setValue(Object value) {
            _model.setValue((Object[]) value);
        }

        public void setReadOnly(boolean b) {
        }
    }
}
