package dbhelp.db.model;

import bean.editor.AbstractEditor;
import bean.editor.ICustomEditor;
import dbhelp.db.Catalog;
import dbhelp.db.Database;
import dbhelp.db.IDbMeta;
import dbhelp.db.Schema;
import bean.editor.utils.CheckListModel;
import bean.editor.utils.Item;
import bean.editor.utils.CheckList;
import utils.Utils;

import javax.swing.*;
import java.awt.*;
import java.util.ArrayList;
import java.util.List;

/**
 * User: huojun
 * Date: 6-Oct-2006
 * Time: 10:42:47 PM
 */
public class DBFilterEditor extends AbstractEditor {
    private MyEditor _uiEditor;

    public boolean supportsCustomEditor() {
        return true;
    }

    public Component getCustomEditor() {
        if( null == _uiEditor ) {
            DBFilterObject f = (DBFilterObject) getBean();
            MyModel model = new MyModel(f.getDatabase());
            model.setValue((Object[]) getValue());
            _uiEditor = new MyEditor(model);
        }
        return _uiEditor;
    }

    public void editConfirm() {
        setValue(_uiEditor.getValue());
    }

    public String getAsText() {
        Object[] value = (Object[]) getValue();
        return Utils.arrayToString(value);
    }

    public void setAsText(String text) throws IllegalArgumentException {
        if( text.length() == 0 ) {
            setValue( null );
        }
    }

    private static class MyModel extends CheckListModel<Object> {

        public MyModel(Database database) {
            List<Object> _objects = new ArrayList<Object>();
            if( database.isConnected() ) {
                try {
                    IDbMeta md = database.getMetaData();
                    List<Catalog> catalogs = md.getCatalogs();
                    for(Catalog c : catalogs) {
                        _objects.add(c);
                    }
                    for( Schema s : md.getSchemas() ) {
                        _objects.add(s);
                    }
                } catch (Exception e) {
                    Utils.debug("FilterEditor$MyModel.MyModel: " + e);
                }
            }
            setData(_objects);
        }

        public void setValue(Object[] objects) {
            for (int i = 0; null != objects && i < objects.length; i++) {
                Object object = objects[i];
                for(Item item : _list ) {
                    if(item.getValue().toString().equals(object.toString())) {
                        item.setSelected(true);
                        break;
                    }
                }
            }
            fireContentsChanged(this, 0, _list.size() - 1);
        }
    }
    
    private static class MyEditor extends JPanel implements ICustomEditor {
        private MyModel _model;
        public MyEditor(MyModel model) {
            super( new BorderLayout() );
            _model = model;
            CheckList table = new CheckList(model);
            add( new JScrollPane(table), BorderLayout.CENTER );
            setSize(320, 250);
        }

        public Object getValue() {
            return _model.getData().toArray();
        }

        public void accept() {
        }

        public void setValue(Object value) {
            _model.setValue((Object[]) value);
        }

        public void setReadOnly(boolean b) {
        }
    }
}
