package dbhelp.db.model;

import javax.swing.table.TableModel;
import javax.swing.table.AbstractTableModel;
import javax.swing.event.TableModelListener;
import javax.swing.event.TableModelEvent;
import java.util.List;
import java.util.ArrayList;

/**
 * User: huojun
 * Date: 25-Oct-2006
 * Time: 11:10:13 AM
 */
public class FilterColumnModel extends AbstractTableModel implements TableModelListener {
    protected TableModel _delegate;
    protected List<Boolean> _visibles = new ArrayList<Boolean>();


    public FilterColumnModel() {
    }

    public FilterColumnModel(TableModel delegate) {
        setDelegate(delegate);
    }

    public TableModel getDelegate() {
        return _delegate;
    }

    public void setDelegate(TableModel delegate) {
        if( null != _delegate ) {
            _delegate.removeTableModelListener(this);
        }
        _delegate = delegate;
        _visibles.clear();
        if( _delegate != null ) {
            _delegate.addTableModelListener(this);
            for( int i=0; i<_delegate.getColumnCount(); i++ ) {
                _visibles.add(true);
            }
        }
        fireTableStructureChanged();
    }

    public int getRowCount() {
        return null == _delegate ? 0 : _delegate.getRowCount();
    }

    public int getColumnCount() {
        if( null == _delegate ) {
            return 0;
        } else {
            int n=0;
            for(boolean b : _visibles) {
                if( b ) {
                    n++;
                }
            }
            return n;
        }
    }

    public int getModelIndex(int viewIndex) {
        int index = 0;
        for( int n=0; n<_visibles.size(); n++ ) {
            if( _visibles.get(n) ) {
                if( index == viewIndex ) {
                    return n;
                }
                index++;
            }
        }
        return -1;
    }

    public void setVisible(int columnIndex, boolean b) {
        _visibles.set(columnIndex, b);
        fireTableStructureChanged();
    }

    public String getColumnName(int columnIndex) {
        return _delegate.getColumnName(getModelIndex(columnIndex));
    }

    public Class<?> getColumnClass(int columnIndex) {
        return _delegate.getColumnClass(getModelIndex(columnIndex));
    }

    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return _delegate.isCellEditable(rowIndex, getModelIndex(columnIndex));
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        return _delegate.getValueAt(rowIndex, getModelIndex(columnIndex));
    }

    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        _delegate.setValueAt(aValue, rowIndex, getModelIndex(columnIndex));
    }

    public void tableChanged(TableModelEvent e) {
        if( e.getColumn() == TableModelEvent.ALL_COLUMNS ) {
            setDelegate( _delegate );
        }
        fireTableChanged(e);
    }

    public boolean isColumnVisible(int i) {
        return _visibles.get(i);
    }
}
