package dbhelp.db.model;

import utils.Utils;

import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;
import javax.swing.event.TableModelListener;
import javax.swing.event.TableModelEvent;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

/**
 * Created by IntelliJ IDEA.
 * User: huojun
 * Date: 30-Jan-2006
 * Time: 3:26:15 PM
 */
public class FilteredTableModel extends AbstractTableModel implements TableModelListener {
    private TableModel _model;
    private String _filter;
    private int _filteredColumn = 0;
    private boolean _caseSensitive;
    private boolean _regExp;
    private List<Integer> _indexes = new ArrayList<Integer>();
    private Pattern _pattern;

    public FilteredTableModel(TableModel model) {
        setModel(model);
    }

    public void setModel(TableModel model) {
        if( _model != null ) {
            _model.removeTableModelListener(this);
        }
        if (null != model) {
            _model = model;
            _model.addTableModelListener( this );
        } else {
            throw new IllegalArgumentException("Table model is null!");
        }
        _indexes.clear();
        _filter = null;
        fireTableStructureChanged();
    }

    public int getRowCount() {
        if (_indexes.size() == 0) {
            if (null == _filter) {
                return _model.getRowCount();
            } else {
                return 0;
            }
        } else {
            return _indexes.size();
        }
    }

    public boolean isCaseSensitive() {
        return _caseSensitive;
    }

    public void setCaseSensitive(boolean caseSensitive) {
        _caseSensitive = caseSensitive;
    }

    public String getFilter() {
        return _filter;
    }

    public void setFilter(String filter) {
        _filter = filter;
        _indexes.clear();
        try {
            if (_regExp) {
                _pattern = Pattern.compile(filter);
            }
            for (int i = 0; null != _filter && i < _model.getRowCount(); i++) {
                Object v = _model.getValueAt(i, _filteredColumn);
                if (null != v) {
                    String text = _caseSensitive ? v.toString() : v.toString().toLowerCase();
                    if (accept(text, _filter)) {
                        _indexes.add(i);
                    }
                }
            }
        } catch (PatternSyntaxException e) {
            Utils.debug("Error: " + e);
        } catch (Exception e) {
//            Utils.debug("Regular Expression: " + e);
        }
        fireTableDataChanged();
    }

    private boolean accept(String text, String filter) {
        return _regExp ? _pattern.matcher(text).matches() : text.startsWith(_caseSensitive ? filter : filter.toLowerCase());
//        return text.startsWith(_filter);
    }

    public TableModel getModel() {
        return _model;
    }

    public int getFilteredColumn() {
        return _filteredColumn;
    }

    public void setFilteredColumn(int filteredColumn) {
        _filteredColumn = filteredColumn;
    }

    public int getColumnCount() {
        return _model.getColumnCount();
    }

    public String getColumnName(int columnIndex) {
        return _model.getColumnName(columnIndex);
    }

    public Class getColumnClass(int columnIndex) {
        return _model.getColumnClass(columnIndex);
    }

    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return _model.isCellEditable(mapRow(rowIndex), columnIndex);
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        return _model.getValueAt(mapRow(rowIndex), columnIndex);
    }

    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        _model.setValueAt(aValue, mapRow(rowIndex), columnIndex);
    }

    public int mapRow(int rowIndex) {
        if (_indexes.size() > 0) {
            return _indexes.get(rowIndex);
        } else {
            return rowIndex;
        }
    }

    public void tableChanged(TableModelEvent e) {
        fireTableStructureChanged();
    }

    public void reguarExpSearch(boolean selected) {
        _regExp = selected;
    }
}
