package dbhelp.db.model;

import dbhelp.db.Database;
import dbhelp.db.ForeignKey;
import dbhelp.db.Table;
import dbhelp.plugin.ui.AbortException;
import dbhelp.plugin.ui.DatabaseDiagram;
import dbhelp.plugin.ui.IProgressReporter;
import dbhelp.plugin.ui.IProgressor;
import diagram.DiagramViewer;
import diagram.IPositioner;
import diagram.shape.database.TableBox;
import utils.Utils;

import java.awt.*;
import java.util.*;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: huojun
 * Date: 20-Feb-2006
 * Time: 3:45:39 PM
 */
public class TablePositioner implements IPositioner, IProgressor {
    private Table[] _tables;
    private TablesHolder<Integer, Table> _holder = new TablesHolder<Integer, Table>();
    private TablesHolder<Integer, TableBox> _shapeholder = new TablesHolder<Integer, TableBox>();
    private static final int ROW_INTERVAL = 100;
    private static final int TABLE_INTERVAL = 60;
    private IProgressReporter _irp;
    private int _value = 0;
    private int _level = 0;
    private boolean _addRefTables = true;
    protected Database _database;

    public TablePositioner(Table[] tables) {
        // sort tables based on how many refereneced tables
        if (null != tables && tables.length > 0) {
            _tables = tables.clone();
            Arrays.sort(_tables, new Comparator<Table>() {
                public int compare(Table t1, Table t2) {
                    // populate columns and key information
                    t1.getColumns();
                    t2.getColumns();
                    int c1 = t1.getForeignKeys() == null ? 0 : t1.getForeignKeys().length;
                    int c2 = t2.getForeignKeys() == null ? 0 : t2.getForeignKeys().length;
                    int r1 = t2.getReferencedTables() == null ? 0 : t2.getReferencedTables().length;
                    int r2 = t2.getReferencedTables() == null ? 0 : t2.getReferencedTables().length;
                    return (c2 + r2) - (c1 + r1);
                }
            });
        }
    }

    public void setProgressReporter(IProgressReporter p) {
        _irp = p;
    }


    public boolean isAddRefTables() {
        return _addRefTables;
    }

    public void setAddRefTables(boolean addRefTables) {
        _addRefTables = addRefTables;
    }

    public void layoutShapes(final DatabaseDiagram d) {
        _database = d.getDatabase();
        _irp.setMaxValue(_tables.length * 3);
        _irp.setMinValue(0);
        _irp.setValue(0);
        groupTables();
        _irp.setText("Layouting shapes in diagram...");
        doLayout(d);
        _irp.setText("finished.");
        _irp.done(false);
    }

    private void doLayout(DatabaseDiagram d) {
        int height = 0;
        int width = 0;
        List<Dimension> rowDimensions = new ArrayList<Dimension>();
        List<Table> selectedTables = Arrays.asList(_tables);
        for (int i = 0; i < _holder.size(); i++) {
            List<Table> row = _holder.getRow(i);
            int rowWidth = 0;
            int rowHeight = 0;
            for (Object aRow : row) {
                Table t = (Table) aRow;
                if (isAddRefTables() || (!isAddRefTables() && selectedTables.contains(t))) {
                    TableBox tb = (TableBox) d.addTable(t);
                    List<TableBox> r = _shapeholder.getRow(i);
                    r.add(tb);
                    Dimension size = tb.calulateSize();
                    rowWidth = rowWidth + size.width + TABLE_INTERVAL;
                    rowHeight = rowHeight < size.height ? size.height : rowHeight;
                }
                updateProgress();
            }
            height = height + rowHeight + ROW_INTERVAL;
            rowDimensions.add(new Dimension(rowWidth - TABLE_INTERVAL, rowHeight));
            width = width < rowWidth - TABLE_INTERVAL ? rowWidth - TABLE_INTERVAL : width;
        }
        int x = (DiagramViewer.DIAGRAM_WIDTH - width) / 2;
        int y = (DiagramViewer.DIAGRAM_HEIGHT - height + ROW_INTERVAL) / 2;
        int ry = y;
        for (int i = 0; i < _shapeholder.size(); i++) {
            List<TableBox> r = _shapeholder.getRow(i);
            Dimension rowDimension = rowDimensions.get(i);
            int rx = x + (width - rowDimension.width) / 2;
            for (Object aR : r) {
                TableBox tb = (TableBox) aR;
                tb.setLocation(rx, ry);
                rx = rx + tb.getWidth() + TABLE_INTERVAL;
                tb.fireShapeChangeEvent();
                updateProgress();
            }
            ry = ry + rowDimension.height + ROW_INTERVAL;
        }
        d.getDiagram().scrollRectToVisible(new Rectangle(x, y, width, height));
    }

    private void updateProgress() {
        _irp.setValue(_value++);
        if (_irp.isCancelled()) {
            throw new AbortException("User cancelled!");
        }
    }

    private void groupTables() {
        ArrayList<Table> tableList = new ArrayList<Table>();
        int row = 0;
        int refCount;
        _level = 0;
        for (int i = 0; null != _tables && i < _tables.length; i++) {
            Table table = _tables[i];
            if (!tableList.contains(table)) {
                refCount = addTable(row, table, tableList);
                if (i + 1 < _tables.length && refCount >= 0) {
                    if (refCount != getRefCount(_tables[i + 1])) {
                        row++;
                    }
                }
            }
        }
    }

    private int addTable(int row, Table table, ArrayList<Table> tableList) {
        int refCount;
        int fkId, refId;

        _level++;
        if (_level > 4) {
            _level--;
            return -1;
        }
        addTableToRow(row, table, tableList);
        ForeignKey[] foreignKeys = table.getForeignKeys();
        for (fkId = 0; null != foreignKeys && fkId < foreignKeys.length; fkId++) {
            ForeignKey foreignKey = foreignKeys[fkId];
            Table refTable = findFkRefTable(foreignKey);
            if (null != refTable && addTableToRow(row + 1, refTable, tableList)) {
                addTable(row + 1, refTable, tableList);
            }
        }
        Table[] refTables = table.getReferencedTables();
        for (refId = 0; null != refTables && refId < refTables.length; refId++) {
            Table refTable = refTables[refId];
            // populate columns
            refTable.getColumns();
            if (addTableToRow(row + 1, refTable, tableList)) {
                addTable(row + 1, refTable, tableList);
            }
        }
        refCount = fkId + refId;
        _level--;
        return refCount;
    }

    private Table findFkRefTable(ForeignKey fk) {
        for (int i = 0; null != _tables && i < _tables.length; i++) {
            Table table = _tables[i];
            if (table.getName().equals(fk.getPkTableName())) {
                return table;
            }
        }
        try {
            List<Table> list = _database.getMetaData().getTables(fk.getPkTableCatalog(), fk.getPkTableSchema(), "TABLE", fk.getPkTableName());
            return list.size() > 0 ? list.get(0) : null;
        } catch (Exception e) {
            Utils.debug("TablePositioner.findFkRefTable: " + e);
        }
        return null;
    }

    private boolean addTableToRow(int row, Table table, ArrayList<Table> tableList) {
        if (!tableList.contains(table)) {
//            Utils.debug("Add Table " + table.getName() + " to row " + row);
            List<Table> currentRow = _holder.getRow(row);
            currentRow.add(table);
            tableList.add(table);
            updateProgress();
            _irp.setText("Adding table: " + table.toString());
            return true;
        }
        return false;
    }

    private int getRefCount(Table table) {
        ForeignKey[] fkeys = table.getForeignKeys();
        Table[] referencedTables = table.getReferencedTables();
        int fkCount = null == fkeys ? 0 : fkeys.length;
        int refConnt = null == referencedTables ? 0 : referencedTables.length;
        return fkCount + refConnt;
    }

    private static class TablesHolder<Integer, T> extends HashMap<Integer, List<T>> {

        public List<T> getRow(Integer row) {
            if (!containsKey(row)) {
                put(row, new ArrayList<T>());
            }
            return get(row);
        }
    }
}
