package dbhelp.db.model.search;

import dbhelp.db.Database;
import dbhelp.db.Procedure;
import dbhelp.db.Table;
import utils.Utils;
import utils.text.ITokenEnumeration;
import utils.text.IWordEnumeration;
import utils.text.WordScanner;

import java.util.ArrayList;
import java.util.List;


/**
 * User: huojun
 * Date: 12-Oct-2006
 * Time: 3:36:30 PM
 */
public class ProcedureSearcher implements ISearchable {
    private List<Procedure> _procedures;


    public ProcedureSearcher(List<Procedure> procedures) {
        _procedures = procedures;
    }

    public List<SearchResult> find(Object o) {
        List<SearchResult> list = new ArrayList<SearchResult>();
        if (o instanceof Table || o instanceof Procedure) {
            SearchInfo si = new SearchInfo(o);
            Database db = si.getDatabase();
            String name = si.getName();
            for (Procedure p : _procedures) {
                String text = Procedure.getProcedureSyntax(p);
                if (Utils.hasValue(text)) {
                    ITokenEnumeration tokens = WordScanner.tokenScan(text, 0);
                    while (tokens.hasMoreElements()) {
                        CharSequence token = tokens.nextElement();
                        IWordEnumeration.TYPE type = tokens.getType();
                        if (type.equals(IWordEnumeration.TYPE.WORD) || type.equals(IWordEnumeration.TYPE.OBJECT)) {
                            String x = token.toString().toLowerCase();
                            if (type.equals(IWordEnumeration.TYPE.WORD)) {
                                if (x.equalsIgnoreCase(name)) {
                                    int line = tokens.getLine();
                                    SearchResult.TextSearchResult sr = new SearchResult.TextSearchResult(p, line);
                                    sr.setDescription(getLineText(text, line));
                                    sr.setFullText(text);
                                    list.add(sr);
                                }
                            } else {
                                if (x.equalsIgnoreCase("\"" + name + "\"")) {
                                    int line = tokens.getLine();
                                    SearchResult.TextSearchResult sr = new SearchResult.TextSearchResult(p, line);
                                    sr.setDescription(getLineText(text, line));
                                    sr.setFullText(text);
                                    list.add(sr);
                                }
                            }
                        }
                    }
                }
            }
        }
        return list;
    }

    private String getLineText(String text, int line) {
        String[] lines = text.split("\\n");
        return line > lines.length ? text : lines[line];
    }

    public String toString() {
        return "Search usages in stored procedures";
    }

    private static class SearchInfo {
        protected Object _object;

        public SearchInfo(Object object) {
            _object = object;
        }

        public Database getDatabase() {
            return _object instanceof Table ? ((Table) _object).getDatabase() : ((Procedure) _object).getDatabase();
        }

        public String getName() {
            return _object instanceof Table ? ((Table) _object).getName() : ((Procedure) _object).getName();
        }
    }
}
