package dbhelp.db.ui;


import bean.PropertyChangeMultiCaster;
import dbhelp.db.Database;
import dbhelp.db.model.AbstractDBObject;
import dbhelp.db.model.DBFilterObject;
import dbhelp.db.model.DBTreeModel;
import dbhelp.db.model.IDBNode;
import dbhelp.plugin.action.portable.PopupMouseAdapter;
import utils.Config;
import utils.Icons;
import utils.Utils;

import javax.swing.*;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeWillExpandListener;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreePath;
import java.awt.*;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.lang.reflect.Array;

/**
 * User: huojun
 * Date: 6-Sep-2006
 * Time: 11:16:29 AM
 */
public class DBTree extends JTree {

    protected static Icons _icons = Icons.getInstance();

    public DBTree() {
        this(new AbstractDBObject.ProfilesNode());
    }

    public DBTree(IDBNode node) {
        super(new DBTreeModel(node));
        setCellRenderer(new MyCellRenderer());
        addTreeWillExpandListener(new MyTreeWillExpandListener(this));
        addMouseListener(new MyPopupMouseAdapter(this));
        DbListener listener = new DbListener(this);
        PropertyChangeMultiCaster pcmc = PropertyChangeMultiCaster.getInstance();
        pcmc.addListener(Database.class, listener);
        pcmc.addListener(DBFilterObject.class, listener);
        pcmc.addListener(Config.class, listener);
    }

    private static class MyCellRenderer extends DefaultTreeCellRenderer {
        public Component getTreeCellRendererComponent(JTree tree, Object value, boolean sel, boolean expanded, boolean leaf, int row, boolean hasFocus) {
            super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf, row, hasFocus);
            if (value instanceof IDBNode) {
                IDBNode viewNode = (IDBNode) value;
                int iconIndex = viewNode.getIconIndex();
                if (iconIndex >= 0 && iconIndex < _icons.size()) {
                    Icon icon = _icons.get(iconIndex);
                    setIcon(icon);
                }
            }
            return this;
        }
    }

    private static class MyTreeWillExpandListener implements TreeWillExpandListener {
        private DBTree _tree;

        public MyTreeWillExpandListener(DBTree tree) {
            _tree = tree;
        }

        public void treeWillExpand(TreeExpansionEvent event) {
            final TreePath path = event.getPath();
            Object c = path.getLastPathComponent();
            if (c instanceof IDBNode) {
                final IDBNode cc = (IDBNode) c;
                final DBTreeModel m = (DBTreeModel) _tree.getModel();
                m.fireTreeNodeChange(new TreeModelEvent(this, path));
//                SwingUtilities.invokeLater(new Runnable() {
//                    public void run() {
//                        populateChildren(cc, m, path);
//                    }
//                });
                populateChildren(cc, m, path);
            }
        }

        private void populateChildren(IDBNode cc, DBTreeModel m, TreePath path) {
            try {
                cc.getChildren();
                m.fireTreeNodeChange(new TreeModelEvent(this, path));
            } catch (Exception e) {
                Utils.debug("DBTree$MyTreeWillExpandListener.populateChildren: " + e);
            }
        }

        public void treeWillCollapse(TreeExpansionEvent event) {
        }
    }

    private static class MyPopupMouseAdapter extends PopupMouseAdapter {
        private DBTree _tree;

        public MyPopupMouseAdapter(DBTree tree) {
            _tree = tree;
        }

        protected Set<Object> getActionKeys(MouseEvent e) {
            Set<Object> actions = new HashSet<Object>();
            TreePath p = _tree.getClosestPathForLocation(e.getX(), e.getY());
            TreePath[] paths = _tree.getSelectionPaths();
            if (null != paths && paths.length > 1) {
                // multiple selection
                int n = paths[0].getPathCount();
                List<Object> values = new ArrayList<Object>();
                Object userObject = null;
                for (TreePath tp : paths) {
                    if (tp.getPathCount() != n) {
                        // the user selected multiple different level paths
                        return null;
                    }
                    IDBNode dbObject = (IDBNode) tp.getLastPathComponent();
                    userObject = dbObject.getUserObject();
                    values.add(userObject);
                }
                if (null != userObject) {
                    actions.add(Utils.collectionToTypeArray(values, userObject.getClass()));
                }
            } else if (null != p) {
                _tree.setSelectionPath(p);
                IDBNode c = (IDBNode) p.getLastPathComponent();
                actions.add(c.getUserObject());
                // add as one element array
                Object o = Array.newInstance(c.getUserObject().getClass(), 1);
                Array.set(o,0, c.getUserObject());
                actions.add(o);
                List<IDBNode> children = c.getChildren();
                if (null != children && children.size() > 0) {
                    List<Object> list = new ArrayList<Object>();
                    Class type = null;
                    for (IDBNode idbNode : children) {
                        Object uo = idbNode.getUserObject();
                        if (type == null || type.equals(uo.getClass())) {
                            list.add(uo);
                        } else {
                            list.clear();
                            break;
                        }
                        type = uo.getClass();
                    }
                    if (list.size() > 0) {
                        actions.add(Utils.collectionToTypeArray(list, children.get(0).getUserObject().getClass()));
                    }
                }
                if (!_tree.getModel().isLeaf(c)) {
                    actions.add(_tree);
                }
            }
            return actions;
        }
    }

    private static class DbListener implements PropertyChangeListener {
        private DBTree _tree;

        public DbListener(DBTree tree) {
            _tree = tree;
        }

        public void propertyChange(PropertyChangeEvent evt) {
            Object source = evt.getSource();
            String propertyName = evt.getPropertyName();
            final DBTreeModel m = (DBTreeModel) _tree.getModel();
            if (source instanceof Database) {
                Database db = (Database) source;
                if (propertyName.equals("connected")) {
                    refreshDatabaseNode(db, m);
                } else if( "name".equals(propertyName) ) {
                    int n = m.getChildCount(m.getRoot());
                    for( int i=0; i<n; i++ ) {
                        IDBNode dbNode = (IDBNode) m.getChild(m.getRoot(), i);
                        if( dbNode.getUserObject().equals(db) ) {
                            m.fireTreeNodeChange(new TreeModelEvent(this, AbstractDBObject.getNodePath(dbNode)));
                        }
                    }
                } else if( "catalogFilter".equals(propertyName) ) {
                    refreshDatabaseNode(db, m);
                }
            } else if (source instanceof DBFilterObject) {
                DBFilterObject filterObject = (DBFilterObject) source;
                IDBNode node = filterObject.getNode();
                node.refresh();
                Object[] path = AbstractDBObject.getNodePath(node);

                m.fireTreeNodeChange(new TreeModelEvent(this, path));
            } else if (source instanceof Config) {
                if ("databases".equals(propertyName)) {
                    IDBNode root = (IDBNode) m.getRoot();
                    root.refresh();
                    m.fireTreeNodeChange(new TreeModelEvent(this, new Object[]{root}));
                    _tree.setSelectionPath( new TreePath(_tree.getModel().getRoot()) );
                }
            }
        }

        private void refreshDatabaseNode(Database db, DBTreeModel m) {
            IDBNode root = (IDBNode) _tree.getModel().getRoot();
            for (IDBNode n : root.getChildren()) {
                if (n.getUserObject().equals(db)) {
                    n.refresh();
                    TreePath match = new TreePath( new Object[] {root, n});
                    m.fireTreeNodeChange(new TreeModelEvent(this, match));
                    if (db.isConnected()) {
                        _tree.expandPath(match);
                    }
                    return;
                }
            }
        }
    }
}
