package dbhelp.db.ui;


import bean.PropertyChangeMultiCaster;
import bean.editor.AnnotationPropertyEditorPane;
import dbhelp.db.Database;
import dbhelp.db.INamedDBObject;
import dbhelp.db.Catalog;
import dbhelp.db.model.DBFilterObject;
import dbhelp.db.model.IDBNode;
import dbhelp.db.model.AbstractDBObject;
import dbhelp.plugin.action.portable.PopupMouseAdapter;
import dbhelp.plugin.action.portable.SetFilterAction;
import dbhelp.plugin.ui.SearchListPanel;
import utils.Config;
import utils.Utils;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;
import java.awt.*;
import static java.awt.GridBagConstraints.*;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * Created by IntelliJ IDEA.
 * User: huojun
 * Date: 30-Jan-2006
 * Time: 10:58:09 AM
 */
public class LocalView extends JPanel {
    private JComboBox _combCategories = new JComboBox();
    private JComboBox _combSubCats = new JComboBox();
    private JComboBox _combDatabases;
    private MyTableModel _listModel = new MyTableModel();
    private SearchListPanel _searchPanel = new SearchListPanel();
    private AnnotationPropertyEditorPane _propertyEditor = new AnnotationPropertyEditorPane();
    private IDBNode _root;
    private JLabel _jlCategoryName;


    public static void main(String[] args) throws Exception {
        JFrame f = new JFrame();
        Config.restore();
        Container cp = f.getContentPane();
        cp.setLayout(new BorderLayout());
        cp.add(new LocalView());
        f.setSize(320, 580);
        f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        f.setVisible(true);
    }

    public LocalView() {
        initComponent();
        PropertyChangeMultiCaster caster = PropertyChangeMultiCaster.getInstance();
        caster.addListener(Database.class, new ErrorListener());
        caster.addListener(DBFilterObject.class, new FilterListener());
    }

    private void initComponent() {
        setLayout(new BorderLayout());
        Dimension zeroSize = new Dimension(0, 0);
        JPanel view = new JPanel(new GridBagLayout());
        view.setMinimumSize(zeroSize);
        view.setPreferredSize(new Dimension(300, 400));
        _combDatabases = new JComboBox();

        GridBagConstraints g;
        g = new GridBagConstraints(0, 0, 1, 1, 0, 0, WEST, HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0);
        JButton jbDatabases = new JButton(new ActionRefreshDatabases());
        jbDatabases.setMnemonic('D');
        view.add(jbDatabases, g);
        jbDatabases.setToolTipText("Refresh Databases");
        g = new GridBagConstraints(1, 0, 1, 1, 1, 0, WEST, HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0);
        view.add(_combDatabases, g);
        g = new GridBagConstraints(0, 2, 1, 1, 0, 0, WEST, HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0);
        JButton jlCategory = new JButton(new ActionSetFilter());
        jlCategory.setToolTipText("Set Filter");
        jlCategory.setMnemonic('e');
        view.add(jlCategory, g);
        g = new GridBagConstraints(1, 1, 1, 1, 1, 0, WEST, HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0);
        view.add(_combCategories, g);
        g = new GridBagConstraints(1, 2, 1, 1, 1, 0, WEST, HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0);
        view.add(_combSubCats, g);
        g = new GridBagConstraints(0, 1, 1, 1, 0, 0, WEST, HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0);
        _jlCategoryName = new JLabel("Catalogs: ");
        view.add(_jlCategoryName, g);
        _jlCategoryName.setDisplayedMnemonic('j');
        _jlCategoryName.setLabelFor(_combSubCats);
//        _jcbPrefix = new JCheckBox("Hide catalog and schema prefix");
//        _jcbPrefix.setMnemonic('H');
//        g = new GridBagConstraints(0, 2, 2, 1, 1, 0, WEST, HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0);
//        view.add(_jcbPrefix, g);

        g = new GridBagConstraints(0, 3, 2, 1, 1, 1, NORTH, BOTH, new Insets(2, 2, 2, 2), 0, 0);
        view.add(_searchPanel, g);
        _searchPanel.setModel(_listModel);

        _propertyEditor.setMinimumSize(zeroSize);
        _propertyEditor.setPreferredSize(new Dimension(300, 100));
        _propertyEditor.setReadOnly(true);

        JSplitPane jsp = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
        jsp.setResizeWeight(.6);
        jsp.setOneTouchExpandable(true);

        jsp.setTopComponent(view);
        jsp.setBottomComponent(_propertyEditor);

        add(jsp, BorderLayout.CENTER);

        _combDatabases.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                if (e.getStateChange() == ItemEvent.SELECTED) {
                    refreshCategory();
                }
            }
        });
        _combCategories.addItemListener(new MyItemListener());
        _combSubCats.addItemListener( new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                refreshList();
            }
        });
        final JTable searchResult = _searchPanel.getTable();
        searchResult.getSelectionModel().addListSelectionListener(new MyListSelectionListener(searchResult));
        searchResult.addMouseListener(new MyPopupMouseAdapter(searchResult));
        PropertyChangeMultiCaster caster = PropertyChangeMultiCaster.getInstance();
        caster.addListener(Database.class, new DBConnectWatcher());
        caster.addListener(Config.class, new DBListWatcher());
    }

    private class MyItemListener implements ItemListener {
        public void itemStateChanged(ItemEvent e) {
            refreshSubCat();
            _searchPanel.setSearchText("");
        }
    }

    public void refresh() {
        setModel(_root);
    }

    public void setModel(IDBNode root) {
        _root = root;
        _combDatabases.removeAllItems();
        if (null != root) {
            List<IDBNode> list = root.getChildren();
            DefaultComboBoxModel cbm = list == null
                    ? new DefaultComboBoxModel() : new DefaultComboBoxModel(list.toArray());
            _combDatabases.setModel(cbm);
        }
        refreshCategory();
    }

    private void refreshCategory() {
        _combCategories.removeAllItems();
        IDBNode selectedItem = (IDBNode) _combDatabases.getSelectedItem();
        if (null != selectedItem) {
            List<IDBNode> children = selectedItem.getChildren();
            if (null != children && children.size() > 0 ) {
                IDBNode idbNode = children.get(0);
                if( idbNode instanceof AbstractDBObject.CatalogNode ) {
                    Catalog c = (Catalog) idbNode.getUserObject();
                    _jlCategoryName.setText(c.getVendorTerm() + ": ");
                    _combCategories.setModel(new DefaultComboBoxModel(children.toArray()));
                    _combCategories.setEnabled(true);
                    refreshSubCat();
                    return;
                } else {
                    _combCategories.setEnabled(false);
                    _combSubCats.setModel( new DefaultComboBoxModel(children.toArray()) );
                    refreshList();
                    return;
                }
            }
        }
        refreshSubCat();
    }

    private void refreshSubCat() {
        _combSubCats.removeAllItems();
        IDBNode selectedItem = (IDBNode) _combCategories.getSelectedItem();
        if (null != selectedItem) {
            List<IDBNode> children = selectedItem.getChildren();
            if (null != children) {
                _combSubCats.setModel( new DefaultComboBoxModel(children.toArray()) );
            }
        }
        refreshList();
    }

    private void refreshList() {
        _listModel.setData(null);
        IDBNode node = (IDBNode) _combSubCats.getSelectedItem();
        if (null != node) {
            _listModel.setData(node.getChildren());
        }
        _propertyEditor.setBean(null);
    }


    private class ActionSetFilter extends AbstractAction {
        public ActionSetFilter() {
            super("Set Filter:");
        }

        public void actionPerformed(ActionEvent e) {
//            refreshCategory();
            IDBNode node = (IDBNode) _combSubCats.getSelectedItem();
            if ( null != node ) {
                Object o = node.getUserObject();
                if( o instanceof DBFilterObject ) {
                    SetFilterAction.editFilter((DBFilterObject) o);
                    _combSubCats.repaint();
                } else {
                    Utils.message("This category doesn't support filter!");
                }
            }
        }
    }


    private class ActionRefreshDatabases extends AbstractAction {
        public ActionRefreshDatabases() {
            super("Databases:");

        }

        public void actionPerformed(ActionEvent e) {
            refresh();
        }
    }

    private class MyListSelectionListener implements ListSelectionListener {
        private final JTable _searchResult;

        public MyListSelectionListener(JTable searchResult) {
            _searchResult = searchResult;
        }

        public void valueChanged(ListSelectionEvent e) {
            if (!e.getValueIsAdjusting()) {
                int index = _searchResult.getSelectedRow();
                if (index >= 0) {
                    IDBNode valueAt = (IDBNode) _searchResult.getModel().getValueAt(index, 0);
                    _propertyEditor.setBean(valueAt.getUserObject());
                }
            }
        }
    }

    private static class MyPopupMouseAdapter extends PopupMouseAdapter {
        private final JTable _searchResult;

        public MyPopupMouseAdapter(JTable searchResult) {
            _searchResult = searchResult;
        }

        protected Set<Object> getActionKeys(MouseEvent e) {
            int row = _searchResult.getSelectedRow();
            Set<Object> list = new HashSet<Object>();
            if (row >= 0) {
                TableModel model = _searchResult.getModel();
                Object valueAt = model.getValueAt(row, 0);
                IDBNode dbObject = (IDBNode) valueAt;
                list.add(dbObject.getUserObject());
                List<Object> selectedItems = new ArrayList<Object>();
                int[] selectedRows = _searchResult.getSelectedRows();
                for (int selectedRow : selectedRows) {
                    IDBNode x = (IDBNode) model.getValueAt(selectedRow, 0);
                    selectedItems.add(x.getUserObject());
                }
                list.add(Utils.collectionToTypeArray(selectedItems, dbObject.getUserObject().getClass()));
                list.add(valueAt);
            }
            return list;
        }
    }

    private static class MyTableModel extends AbstractTableModel {
        private java.util.List<IDBNode> _data = new ArrayList<IDBNode>();

        public void setData(java.util.List<IDBNode> data) {
            _data.clear();
            if (null != data) {
                for (IDBNode n : data) {
                    _data.add(new NamedIDBNode(n));
                }
            }
            fireTableStructureChanged();
        }

        public String getColumnName(int column) {
            return column == 0 ? "Object(s)" : "Catalog/Schema";
        }

        public int getRowCount() {
            return _data.size();
        }

        public int getColumnCount() {
            return 2;
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            IDBNode o = _data.get(rowIndex);
            Object uo = o.getUserObject();
            if (columnIndex == 0) {
                return o;
            } else {
                if (uo instanceof INamedDBObject) {
                    INamedDBObject named = (INamedDBObject) uo;
                    String catalog = named.getCatalog();
                    String schema = named.getSchema();
                    return null == catalog ? (schema == null ? "" : schema) : catalog + (schema == null ? "" : "." + schema);
                } else {
                    return "n/a";
                }
            }
        }
    }

    public static class NamedIDBNode implements IDBNode {
        private IDBNode _delegate;

        public NamedIDBNode(IDBNode delegate) {
            _delegate = delegate;
        }

        public IDBNode getDelegate() {
            return _delegate;
        }

        public IDBNode getParent() {
            return _delegate.getParent();
        }

        public List<IDBNode> getChildren() {
            return _delegate.getChildren();
        }

        public void refresh() {
            _delegate.refresh();
        }

        public Object getUserObject() {
            return _delegate.getUserObject();
        }

        public int getIconIndex() {
            return _delegate.getIconIndex();
        }

        public String toString() {
            Object uo = getUserObject();
            if (uo instanceof INamedDBObject) {
                INamedDBObject named = (INamedDBObject) uo;
                return named.getSimpleName();
            } else {
                return uo.toString();
            }
        }
    }

    private class DBConnectWatcher implements PropertyChangeListener {
        public void propertyChange(PropertyChangeEvent evt) {
            if (evt.getPropertyName().equals("connected")) {
                Database db = (Database) evt.getSource();
                IDBNode item = (IDBNode) _combDatabases.getSelectedItem();
                if (null != item) {
                    if (db.equals(item.getUserObject())) {
                        refreshCategory();
                    }
                }
            }
        }
    }

    private class DBListWatcher implements PropertyChangeListener {
        public void propertyChange(PropertyChangeEvent evt) {
            if( evt.getPropertyName().equals("databases") ) {
                refresh();
            }
        }
    }

    private class ErrorListener implements PropertyChangeListener {
        public void propertyChange(PropertyChangeEvent evt) {
            if ("error".equals(evt.getPropertyName())) {
                _propertyEditor.setBean(null);
            }
        }
    }
    private class FilterListener implements PropertyChangeListener {
        public void propertyChange(PropertyChangeEvent evt) {
            if ("filterObjects".equals(evt.getPropertyName())) {
                refreshList();
            }
        }
    }
}
