package dbhelp.db.ui;

import bean.editor.ArrayPropertyEditTable;
import bean.editor.ArrayPropertyPanel;
import dbhelp.db.Database;
import dbhelp.db.SQLParam;
import utils.Utils;
import utils.text.IWordEnumeration;
import utils.text.WordScanner;
import utils.text.editor.TokenEditor;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.text.BadLocationException;
import java.awt.*;
import static java.awt.GridBagConstraints.*;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * User: huojun
 * Date: 20-Nov-2006
 * Time: 1:46:21 PM
 */
public class SQLParamEditor extends JDialog {
    protected Map<SQLParam, Integer> _placeHolders = new LinkedHashMap<SQLParam, Integer>();
    protected TokenEditor _editor;
    protected ArrayPropertyPanel _paramEditor;
    protected String _sqlCmd;
    protected boolean _confirmed;
    protected Database _database;
    protected WordScanner _scanner = new WordScanner();

    public static void main(String[] args) {
        String cmd = "? ? SELECT * from employee \n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n where emp_id > ? and emp_firstName like ?";
        SQLParamEditor editor = editParameters(new Database(), cmd);
        editor.setVisible(true);
        if ( editor.isConfirmed() || !editor.hasParam() ) {
            SQLParam[] sqlParams = editor.getParams();
            for (int i = 0; null != sqlParams && i < sqlParams.length; i++) {
                SQLParam sqlParam = sqlParams[i];
                System.out.println("sqlParam = " + sqlParam);
            }
        }
    }

    public static SQLParamEditor editParameters(Database db, String sqlCmd) {
        Component frame = Utils.getFrame();
        SQLParamEditor e = frame instanceof Dialog ? new SQLParamEditor((Dialog) frame) : new SQLParamEditor((Frame) frame);
        e.setDatabase(db);
        e.setSqlCmd(sqlCmd);
        return e;
    }


    public Database getDatabase() {
        return _database;
    }

    public void setDatabase(Database database) {
        _database = database;
    }

    public boolean hasParam() {
        return _placeHolders.size() > 0;
    }


    public boolean isConfirmed() {
        return _confirmed;
    }

    public SQLParamEditor(Frame owner) throws HeadlessException {
        super(owner);
        initComponents();
    }

    public SQLParamEditor(Dialog owner) throws HeadlessException {
        super(owner);
        initComponents();
    }

    public String getSqlCmd() {
        return _sqlCmd;
    }

    public void setSqlCmd(String sqlCmd) {
        _sqlCmd = sqlCmd;
        _editor.setText(_sqlCmd);
        
        _scanner.setText(_sqlCmd);
        IWordEnumeration tokens = _scanner.scan(0);
        int id=1;
        while (tokens.hasMoreElements()) {
            CharSequence t = tokens.nextElement();
            if ("?".equals(t)) {
                _placeHolders.put(new SQLParam(id++), tokens.getOffset());
            }
        }
        _paramEditor.setValue(_placeHolders.keySet().toArray());
    }

    protected void initComponents() {
        _paramEditor = new ArrayPropertyPanel();
        _paramEditor.setComponentType(SQLParam.class);
        _paramEditor.setUpdateOnly(true);
        Container c = getContentPane();
        c.setLayout(new GridBagLayout());
        JSplitPane jsp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        _editor = new TokenEditor();
        Dimension zeroSize = new Dimension(0, 0);
        _paramEditor.setPreferredSize( new Dimension(320, 600));
        _paramEditor.setMinimumSize(zeroSize);
        GridBagConstraints g;
        g = new GridBagConstraints(0, 0, 2, 1, 1, 1, NORTH, BOTH, new Insets(2, 2, 2, 2), 1, 1);
        c.add(jsp, g);
        _paramEditor.setBorder(BorderFactory.createTitledBorder("Values for the placeholders"));
        jsp.setTopComponent(_paramEditor);
        JScrollPane editorJsp = new JScrollPane(_editor);
        editorJsp.setMinimumSize(zeroSize);
        jsp.setBottomComponent(editorJsp);
        setSize(800, 400);
        g = new GridBagConstraints(1, 1, 1, 1, 1, 0, EAST, NONE, new Insets(2, 2, 2, 2), 1, 1);
        c.add(new JButton(new ActionRun()), g);
        g = new GridBagConstraints(0, 1, 1, 1, 1, 0, WEST, NONE, new Insets(2, 2, 2, 2), 1, 1);
        c.add(new JButton(new ActionCancel()), g);
        final ArrayPropertyEditTable table = _paramEditor.getTable();
        table.setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN);
        TableColumnModel columnModel = table.getColumnModel();
        // id column
        TableColumn tableColumn = columnModel.getColumn(0);
        tableColumn.setPreferredWidth(30);
        tableColumn.setMaxWidth(50);
        // is null
        tableColumn = columnModel.getColumn(2);
        tableColumn.setPreferredWidth(30);
        tableColumn.setMaxWidth(50);
        ListSelectionModel model = table.getSelectionModel();
        model.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        model.addListSelectionListener(new MyListSelectionListener());
        setTitle("Query Parameter Editor");
        setLocationRelativeTo(null);
        _editor.setEditable(false);
        setModal(true);
    }

    public SQLParam[] getParams() {
        return _placeHolders.keySet().toArray(new SQLParam[_placeHolders.size()]);
    }

    private class MyListSelectionListener implements ListSelectionListener {
        public void valueChanged(ListSelectionEvent e) {
            if (!e.getValueIsAdjusting()) {
                SQLParam o = (SQLParam) _paramEditor.getSelectedValue();
                if (null != o) {
                    Integer offset = _placeHolders.get(o);
                    _editor.select(offset, offset + 1);
                    _editor.getCaret().setSelectionVisible(true);
                    try {
                        Rectangle rect = _editor.modelToView(offset);
                        _editor.scrollRectToVisible(rect);
                    } catch (BadLocationException e1) {
                        //
                    }
                }
            }
        }
    }

    private class ActionRun extends AbstractAction {

        public ActionRun() {
            super("Run SQL");
            putValue(AbstractAction.MNEMONIC_KEY, KeyEvent.VK_R);
        }

        public void actionPerformed(ActionEvent e) {
            SQLParam[] sqlParams = getParams();
            for (int i = 0; null != sqlParams && i < sqlParams.length; i++) {
                SQLParam sqlParam = sqlParams[i];
                try {
                    sqlParam.getObject(getDatabase());
                } catch (Exception e1) {
                    Utils.error(e1);
                    _paramEditor.getTable().getSelectionModel().setSelectionInterval(i, i);
                    return;
                }
            }
            _confirmed = true;
            dispose();
        }
    }

    private class ActionCancel extends AbstractAction {

        public ActionCancel() {
            super("Cancel");
            putValue(AbstractAction.MNEMONIC_KEY, KeyEvent.VK_C);
        }

        public void actionPerformed(ActionEvent e) {
            dispose();
        }
    }
}
