package dbhelp.plugin;

import bean.PropertyChangeMultiCaster;
import bean.editor.AnnotationPropertyEditorPane;
import dbhelp.db.Database;
import dbhelp.db.IDBObject;
import dbhelp.db.model.AbstractDBObject;
import dbhelp.db.model.IDBNode;
import dbhelp.db.ui.DBTree;
import dbhelp.db.ui.LocalView;
import dbhelp.plugin.ui.ReminderTab;
import utils.Config;
import utils.SingletonProvider;
import utils.Utils;

import javax.swing.*;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.TreePath;
import java.awt.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

/**
 * User: huojun
 * Date: 12-Sep-2006
 * Time: 11:54:53 AM
 */
public class DBBrowserPane extends JPanel {
    private JTabbedPane _tabs = new JTabbedPane(JTabbedPane.TOP);
    private AnnotationPropertyEditorPane _propertyEditor;
    public static final String BROWSER_WINDOW_ID = "DBHelper";
    private DBTree _tree;
    private JSplitPane _treeTab;

    public static void main(String[] args) {
        try {
            Config.restore();
        } catch (Exception e) {
            e.printStackTrace();
        }
        JFrame f = new JFrame();
        Container cp = f.getContentPane();
        cp.setLayout(new BorderLayout());
        f.setSize(800,900);
        f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        cp.add(DBBrowserPane.getInstance());
        f.setVisible(true);
    }

    public static DBBrowserPane getInstance() {
        return SingletonProvider.getProvider().getDbBrowserPane();
    }

    public DBBrowserPane() {
        setLayout(new BorderLayout());
        add( _tabs, BorderLayout.CENTER );
        _treeTab = createTreeTab();
        _tabs.addTab("DB Tree", _treeTab);
        LocalView localView = new LocalView();
        _tabs.addTab("Category View", localView);
        _tabs.addTab("Settings", new ConfigPane());
        PropertyChangeMultiCaster.getInstance().addListener(Database.class, new DBErrorListener());
        localView.setModel((IDBNode) _tree.getModel().getRoot());
        _tabs.addTab("Getting Started!", new ReminderTab());
    }

    private JSplitPane createTreeTab() {
        JSplitPane jsp = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
        _tree = new DBTree();
        JScrollPane treeView = new JScrollPane(_tree);
        Dimension minimumSize = new Dimension(0, 0);
        treeView.setMinimumSize(minimumSize);
        jsp.setTopComponent( treeView );
        treeView.setPreferredSize(new Dimension(300, 200));
        _propertyEditor = new AnnotationPropertyEditorPane();
        _propertyEditor.setReadOnly(true);
        _propertyEditor.setMinimumSize(minimumSize);
        jsp.setBottomComponent(_propertyEditor);
        jsp.setOneTouchExpandable(true);
        jsp.setResizeWeight(.9);
        _tree.addTreeSelectionListener(new TreeSelectionListener() {
            public void valueChanged(TreeSelectionEvent e) {
                TreePath path = e.getPath();
                if( path != null ) {
                    IDBNode node = (IDBNode) path.getLastPathComponent();
                    Object userObject = node.getUserObject();
                    if( null != userObject && !ignoredType(userObject.getClass())) {
                        _propertyEditor.setReadOnly( userObject instanceof IDBObject );
                        _propertyEditor.setBean(userObject);
                    }
                }
            }

            private boolean ignoredType(Class<? extends Object> aClass) {
                return aClass.equals(String.class) || aClass.isPrimitive();
            }
        });
        return jsp;
    }

    public void viewTreeNode(IDBNode node) {
        IDBNode parent = node.getParent();
        Object[] parentPath = AbstractDBObject.getNodePath(parent);
        if ( null != parentPath ) {
            TreePath tp = new TreePath(parentPath);
            Utils.debug("tp = " + tp);
            _tree.expandPath(tp);
            TreePath path = new TreePath(AbstractDBObject.getNodePath(node));
            Utils.debug("path = " + path);
            _tree.scrollPathToVisible(path);
            _tree.setSelectionPath(path);
            _tabs.setSelectedComponent( _treeTab );
        }
    }

    private class DBErrorListener implements PropertyChangeListener {
        public void propertyChange(PropertyChangeEvent evt) {
            if( "error".equals(evt.getPropertyName()) ) {
                _propertyEditor.setBean( null );                
            } else if( _propertyEditor.getBean() instanceof Database ) {
                _propertyEditor.repaint();
            }
        }
    }
}
