package dbhelp.plugin;

import bean.PropertyChangeMultiCaster;
import dbhelp.db.model.ISQLResultListener;
import dbhelp.db.model.SQLResultHolder;
import dbhelp.db.model.search.ISearchResultListener;
import dbhelp.db.model.search.SearchResults;
import dbhelp.plugin.action.portable.PopupMouseAdapter;
import dbhelp.plugin.action.portable.TabClose;
import dbhelp.plugin.ui.QueryManagerPane;
import dbhelp.plugin.ui.ResultTablePanel;
import dbhelp.plugin.ui.SearchResultPane;
import dbhelp.plugin.ui.IStayTab;
import utils.SingletonProvider;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseEvent;
import java.io.PrintWriter;
import java.util.HashSet;
import java.util.Set;

/**
 * Created by IntelliJ IDEA.
 * User: David Huo
 * Date: Sep 15, 2006
 * Time: 10:37:46 PM
 */
public class OutputPane extends JTabbedPane implements ISQLResultListener, ISearchResultListener {
    private JTextArea _text = new JTextArea();
    private LogConsole _console = new LogConsole(_text);
    private PrintWriter _logWriter = new PrintWriter(_console);
    public static final String OUTPUT_WINDOW_ID = "DBOutput";
    private QueryManagerPane _queryMgrPane = new QueryManagerPane();
    private SearchResultPane _searchResultPane = new SearchResultPane();

    public static OutputPane getOutputPane() {
        return SingletonProvider.getProvider().getOutputPane();
    }

    public OutputPane() {
        _text.setFont(new Font("Monospaced", Font.PLAIN, 11));
        JScrollPane output = new MyScrollPane(_text);
        _text.setBackground(Color.white);
        addTab("Output", output);
        addMouseListener(new PopupMouseAdapter() {
            protected Set<Object> getActionKeys(MouseEvent e) {
                Set<Object> list = new HashSet<Object>();
                list.add(getOutputPane());
                return list;
            }

            protected void handleEvent(MouseEvent e) {
                if( e.getButton() == MouseEvent.BUTTON2 ) {
                    JTabbedPane tab = (JTabbedPane) e.getSource();
                    Component c = tab.getSelectedComponent();
                    if( !TabClose.shouldKeep(c) ) {
                        tab.remove(c);
                    }
                }
            }
        });
        addTab("Saved Queries", _queryMgrPane);
    }

    public QueryManagerPane getQueryMgr() {
        return _queryMgrPane;
    }

    public PrintWriter getLogWriter() {
        return _logWriter;
    }

    public void clearConsole() {
        _text.setText("");
    }

    public void addResultTabs(SQLResultHolder resultModels) {
        SwingUtilities.invokeLater(new tabRunner(resultModels));

    }

    public void showSearchResult(final SearchResults searchResult) {
        Runnable runnable = new Runnable() {
            public void run() {
                String title = "Find Usages: " + searchResult.getTarget();
                if (!isAncestorOf(_searchResultPane)) {
                    addTab(title, _searchResultPane);
                    setSelectedComponent(_searchResultPane);
                } else {
                    setSelectedComponent(_searchResultPane);
                    setTitleAt(getSelectedIndex(), title);
                }
                _searchResultPane.setSearchResult(searchResult.getResults());
            }
        };
        SwingUtilities.invokeLater(runnable);
    }

    public void notifyResult(SQLResultHolder results) {
        addResultTabs(results);
        PropertyChangeMultiCaster.getInstance().fireChangeEvent(this, "visible");
    }

    public void notifyResult(SearchResults result) {
        showSearchResult(result);
        PropertyChangeMultiCaster.getInstance().fireChangeEvent(this, "visible");        
    }

    public static class MyScrollPane extends JScrollPane implements IStayTab {
        public MyScrollPane(Component view) {
            super(view);
        }
    }

    private class tabRunner implements Runnable {
        private final SQLResultHolder _resultModels;

        public tabRunner(SQLResultHolder resultModels) {
            _resultModels = resultModels;
        }

        public void run() {
            ResultTablePanel tablePane = new ResultTablePanel();
            tablePane.addResults(_resultModels);
            addTab(tablePane.getTitle(), tablePane);
            setSelectedComponent(tablePane);
        }
    }
}
