package dbhelp.plugin.action.portable;

import bean.editor.utils.CheckList;
import bean.editor.utils.CheckListModel;
import dbhelp.db.model.search.ISearchResultListener;
import dbhelp.db.model.search.ISearchable;
import dbhelp.db.model.search.SearchResult;
import dbhelp.db.model.search.SearchResults;
import dbhelp.plugin.OutputPane;
import utils.Config;
import utils.DataCollector;
import utils.SingletonProvider;
import utils.Utils;

import javax.swing.*;
import java.awt.*;
import static java.awt.GridBagConstraints.*;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.List;

/**
 * User: huojun
 * Date: 11-Oct-2006
 * Time: 6:58:32 PM
 */
public class FindUsagesAction extends PortableAction {
    public FindUsagesAction() {
        super("Find Usages");
    }

    public void actionPerformed(ActionEvent e) {
        DataCollector dc = SingletonProvider.getProvider().getDataCollector();
        List<ISearchable> searchables = dc.go(getUserData(), ISearchable.class);
        Frame frame = (Frame) Utils.getFrame();
        SearchOptions sod = new SearchOptions(frame, searchables);
        sod.setVisible(true);
        if (sod.isOk()) {
            searchables = sod.getSearchables();
            if (searchables.size() > 0) {
                Runnable runnable = new searcher(searchables, getUserData(), OutputPane.getOutputPane());
                Thread t = new Thread(runnable);
                t.start();
                Config cfg = Config.getConfig();
                if (cfg.isShowSearchTip()) {
                    Object[] options = {"OK", "Don't tell me again"};
                    int ret = JOptionPane.showOptionDialog(frame, "A search process is started, the result will be displayed in the DBOutput window when search is finished!", "Message",
                            JOptionPane.DEFAULT_OPTION, JOptionPane.PLAIN_MESSAGE, null, options, options[0]);

                    cfg.setShowSearchTip(ret == 0);
                }
            }
        }
    }

    public static class searcher implements Runnable {
        private List<ISearchable> _searchables;
        private Object _userData;
        private ISearchResultListener _listener;

        public searcher(List<ISearchable> searchables, Object userData, ISearchResultListener listener) {
            _searchables = searchables;
            _userData = userData;
            _listener = listener;
        }

        public void run() {
            try {
                List<SearchResult> results = new ArrayList<SearchResult>();
                for (ISearchable s : _searchables) {
                    List<SearchResult> sr = s.find(_userData);
                    if (null != sr) {
                        results.addAll(sr);
                    }
                }
                SearchResults results1 = new SearchResults(_userData, results);
                if( null != _listener ) {
                    _listener.notifyResult(results1);
                }
            } catch (final Exception e1) {
                SwingUtilities.invokeLater(new Runnable() {
                    public void run() {
                        Utils.error(e1);
                    }
                });
            }
        }
    }


    private static class SearchOptions extends JDialog {
        protected boolean _ok;
        private CheckListModel<ISearchable> _model = new CheckListModel<ISearchable>();

        public SearchOptions(Frame owner, List<ISearchable> a) throws HeadlessException {
            super(owner);
            Container cp = getContentPane();
            cp.setLayout(new GridBagLayout());
            GridBagConstraints c;
            setTitle("Available Search Options");
            c = new GridBagConstraints(0, 0, 2, 1, 1, 1, NORTH, BOTH, new Insets(2, 2, 2, 2), 1, 1);
            CheckList list = new CheckList(_model);
            _model.setData(a);
            JScrollPane jsp = new JScrollPane(list);
            cp.add(jsp, c);
            c = new GridBagConstraints(0, 1, 1, 1, 0, 0, WEST, NONE, new Insets(2, 2, 2, 2), 1, 1);
            cp.add(new JButton(new ActionSearch()), c);
            c = new GridBagConstraints(1, 1, 1, 1, 0, 0, EAST, NONE, new Insets(2, 2, 2, 2), 1, 1);
            cp.add(new JButton(new ActionCancel()), c);
            setSize(400, 280);
            setModal(true);
            setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
            setLocationRelativeTo(null);

        }


        private class ActionSearch extends AbstractAction {

            public ActionSearch() {
                super("Search");
                putValue(AbstractAction.MNEMONIC_KEY, KeyEvent.VK_S);
            }

            public void actionPerformed(ActionEvent e) {
                _ok = true;
                setVisible(false);
            }
        }


        public boolean isOk() {
            return _ok;
        }

        public List<ISearchable> getSearchables() {
            return _model.getData();
        }

        private class ActionCancel extends AbstractAction {

            public ActionCancel() {
                super("Cancel");
                putValue(AbstractAction.MNEMONIC_KEY, KeyEvent.VK_C);
            }

            public void actionPerformed(ActionEvent e) {
                _ok = false;
                setVisible(false);
            }
        }

    }
}
