package dbhelp.plugin.action.portable;

import dbhelp.db.Database;
import dbhelp.db.ForeignKey;
import dbhelp.db.Table;
import dbhelp.db.Procedure;
import dbhelp.db.model.DBFilterObject;
import dbhelp.db.model.DatabasesObject;
import dbhelp.db.model.IDBNode;
import dbhelp.db.ui.DBTree;
import diagram.shape.AbstractShape;
import template.TemplateManager;
import utils.SingletonProvider;

import javax.swing.*;
import java.awt.event.MouseEvent;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: David Huo
 * Date: Sep 21, 2006
 * Time: 8:31:13 PM
 */
public class PopupMenuManager {
    private HashMap<Object, Set<PortableAction>> _actions = new HashMap<Object, Set<PortableAction>>();
    private static MyComparator _actionSorter = new MyComparator();

    public PopupMenuManager() {
        // All portable actions
        addAction(new ViewTableData(), Table.class);
        addAction(new FindUsagesAction(), Table.class);
        addAction(new FindUsagesAction(), Procedure.class);
        addAction(new DiagramObjectProperties(), Table.class, ForeignKey.class);
        addAction(new RefreshTreeFolder(), DBTree.class);
        addAction(new DatabaseConnect(), Database.class);
        addAction(new ActionComplexView(), Table.class);
        addAction(new ActionComplexView(), Procedure.class);
        addAction(new DatabaseDisconnect(), Database.class);
        addAction(new EditDatabase(), Database.class);
        addAction(new SetCatalogFilterAction(), Database.class);
        addAction(new RemoveDatabaseAction(), Database.class);
        addAction(new SwitchToTreeView(), IDBNode.class);
        addAction(new CopyCell(), JTable.class);
        addAction(new ActionSetCellNull(), JTable.class);
        addAction(new TabClose(), JTabbedPane.class);
        addAction(new TabCloseAll(), JTabbedPane.class);
        addAction(new TabCloseOthers(), JTabbedPane.class);
        addAction(new SetFilterAction(), DBFilterObject.class);
        addAction(new EditDatabasesAction(), DatabasesObject.class);
        addAction(new AddDatabaseAction(), DatabasesObject.class);
        ActionGroup ag = new ActionGroup("Shapes");
        ag.addAction(new ShapeMovetoFront());
        ag.addAction(new ShapeMovetoBack());
        ag.addAction(new ShapeRemove());
        addAction(ag, AbstractShape.class);

    }

    public static PopupMenuManager getInstance() {
        return SingletonProvider.getProvider().getPopupMenuManager();
    }

    public void addAction(PortableAction a, Class... keys) {
        for (Class key : keys) {
            Set<PortableAction> actionSet = getActionSet(key);
            actionSet.add(a);
        }
    }

    public Set<PortableAction> getActions(Class key, Object userData) {
//        Utils.debug("key = " + key);
        Set<PortableAction> actions = getActions(key);
        for (PortableAction a : actions) {
            a.setUserData(userData);
        }
        return actions;
    }

    private Set<PortableAction> getActions(Class key) {
//        Utils.debug("key = " + key);
        Set<PortableAction> actions = new HashSet<PortableAction>();
        Set<Object> keys = _actions.keySet();
        for (Object key1 : keys) {
            Class aClass = (Class) key1;
//            Utils.debug("Registered = " + aClass);
            if (aClass.isAssignableFrom(key)) {
//                Utils.debug("fired action key = " + aClass);
                actions.addAll(_actions.get(aClass));
            }
        }
        return actions;
    }

    public boolean removeAction(Class key, PortableAction a) {
        Set<PortableAction> actions = getActions(key);
        return null != actions && actions.remove(a);
    }

    private Set<PortableAction> getActionSet(Class key) {
        Set<PortableAction> actionSet = _actions.get(key);
        if (null == actionSet) {
            actionSet = new HashSet<PortableAction>();
            _actions.put(key, actionSet);
        }
        return actionSet;
    }

    public static JPopupMenu createMenu(MouseEvent e, Set<Object> keys) {
        List<PortableAction> actions = new ArrayList<PortableAction>();
        for (Object key : keys) {
            Set<PortableAction> actionSet = getInstance().getActions(key.getClass(), key);
            actions.addAll(actionSet);
        }
        Set<PortableAction> templateActions = TemplateManager.getActions(keys);
        if (templateActions.size() > 0) {
            ActionGroup ag = new ActionGroup("Templates");
            ag.addAll(templateActions);
            actions.add(ag);
        }
        return buildMenu(e, actions);
    }

    private static JPopupMenu buildMenu(MouseEvent e, List<PortableAction> actions) {
        PortableAction[] portableActions = actions.toArray(new PortableAction[actions.size()]);
        Arrays.sort(portableActions, _actionSorter);
        if (actions.size() > 0) {
            JPopupMenu m = new JPopupMenu();
            for (PortableAction action : portableActions) {
                action.setMouseEvent(e);
                if (action instanceof ActionGroup) {
                    JMenu menu = new JMenu(action);
                    m.add(menu);
                    addActionGroup((ActionGroup) action, e, menu);
                } else {
                    JMenuItem menuItem = new JMenuItem(action);
                    m.add(menuItem);
                }
            }
            return m;
        }
        return null;
    }

    private static void addActionGroup(ActionGroup ag, MouseEvent e, JMenuItem menuItem) {
        PortableAction[] actions = ag.getActions();
        Arrays.sort(actions, _actionSorter);
        for (PortableAction action : actions) {
            if (action instanceof ActionGroup) {
                ActionGroup actionGroup = (ActionGroup) action;
                JMenu menu = new JMenu(actionGroup);
                menuItem.add(menu);
                actionGroup.setUserData(ag.getUserData());
                addActionGroup(actionGroup, e, menu);
            } else {
                JMenuItem mi = new JMenuItem(action);
                menuItem.add(mi);
                action.setUserData(ag.getUserData());
                action.setMouseEvent(e);
            }
        }
    }

    private static class MyComparator implements Comparator<PortableAction> {
        public int compare(PortableAction o1, PortableAction o2) {
            return o1.toString().compareTo(o2.toString());
        }
    }
}
