package dbhelp.plugin.action.portable;

import dbhelp.db.Column;
import dbhelp.db.Database;
import dbhelp.db.SQLCommander;
import dbhelp.db.Table;
import dbhelp.db.model.ISQLResultListener;
import dbhelp.plugin.OutputPane;
import utils.Config;
import utils.Utils;

import java.awt.event.ActionEvent;

/**
 * Created by IntelliJ IDEA.
 * User: David Huo
 * Date: Sep 22, 2006
 * Time: 12:46:20 PM
 */
public class ViewTableData extends TableAction {
    public ViewTableData() {
        super("View Data");
    }

    public void actionPerformed(ActionEvent e) {
        Table t = getTable();
        Database db = t.getDatabase();
        if (null != db) {
            Database[] databases = Config.getConfig().getDatabases();
            // try to replace the table's database with the defined database instance
            // If a table is restored from a diagram file, it has a new database object.
            // We need to replace it with the current working database instance.
            for (int i = 0; null != databases && i < databases.length; i++) {
                Database definedDatabase = databases[i];
                if (definedDatabase.isSameDatabase(db)) {
                    if (!definedDatabase.equals(db)) {
                        t.setDatabase(definedDatabase);
                        Utils.debug("Reset the database for table:" + t);
                    }
                }
            }
            viewTableData(t, OutputPane.getOutputPane());
        } else {
            Utils.message("Connect the database first!");
        }
    }

    public static void viewTableData(Table t, ISQLResultListener listener) {
        try {
            StringBuilder sb = new StringBuilder();
            Database db = t.getDatabase();
            if (!db.isConnected()) {
                db.connect();
            }
            Column[] columns = t.getColumns();
            for (Column column : columns) {
                String name = column.getName();
                if (sb.length() > 0) {
                    sb.append(", ");
                }
                sb.append(db.quote(name));
            }
            sb.insert(0, "SELECT ").append(" FROM ").append( db.quote(t) );
            SQLCommander.runSQLCommand(db, sb.toString(), t, listener);
        } catch (Exception e1) {
            Utils.error(e1);
        }
    }
}
