package dbhelp.plugin.complete;

import bean.PropertyChangeMultiCaster;
import dbhelp.plugin.complete.utils.CompleterItem;
import dbhelp.plugin.complete.utils.CompleterModel;
import utils.Icons;
import utils.TablePacker;
import utils.Utils;

import javax.swing.*;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableModel;
import java.awt.*;
import java.awt.event.*;
import java.util.Collection;

/**
 * User: huojun
 * Date: 5-Oct-2006
 * Time: 4:17:02 PM
 */
public abstract class AbstractCompleteWindow extends JPanel {
    protected CompleterModel _model = new CompleterModel();
    protected MyTable _table = new MyTable(_model);
    protected CompleterItem _lastItem;
    protected JLabel _dbLabel = new JLabel();

    protected AbstractCompleteWindow() {
        setLayout(new BorderLayout());
        JScrollPane jsp = new JScrollPane(_table);
        jsp.getViewport().setBackground(Color.white);
        add(jsp, BorderLayout.CENTER);
        _table.setTableHeader(null);
        _table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        _table.setShowGrid(false);
        _table.setIntercellSpacing(new Dimension(0, 2));
        _table.setColumnSelectionAllowed(false);
        _table.setRowSelectionAllowed(true);
        _table.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() == 2) {
                    completeWord();
//                    getEditor().getComponent().requestFocus();
                    setVisible(false);
                }
            }
        });
        add(_dbLabel, BorderLayout.SOUTH);
//        _dbLabel.setToolTipText("Click to refresh");
        _dbLabel.addMouseListener(new MyMouseAdapter(this));
        _dbLabel.setEnabled(true);
        setBorder(BorderFactory.createEtchedBorder());
        setSize(300, 200);
        setVisible(false);
    }

    public void setItems(Collection<CompleterItem> list) {
//        Utils.debug("CompleteWindow.setItems");
        _model.setList(list);
        setDefaultSelection();
        layoutList();
    }

    public void setText(String text) {
        _dbLabel.setText(text);
    }

    private void layoutList() {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                int maxWidth = TablePacker.packTable(_table, 2);
                maxWidth += 24;
                maxWidth = maxWidth < 150 ? 150 : maxWidth;
                maxWidth = maxWidth > 640 ? 640 : maxWidth;
                setSize(maxWidth, getHeight());
                validate();
                setLocation(getCompleterLocation());
            }
        });
    }

    public void setFilter(String prefix) {
//        Utils.debug("prefix = " + prefix);
        _model.setFilter(prefix);
        setDefaultSelection();
        layoutList();
    }

    private void setDefaultSelection() {
        if (_model.getRowCount() > 0) {
            _table.getSelectionModel().setSelectionInterval(0, 0);
        }
    }

    public void up() {
        int row = _table.getSelectedRow();
        if ( row == 0 ) {
            postCtrlKeyToTable(KeyEvent.KEY_PRESSED, KeyEvent.VK_END, KeyEvent.CTRL_DOWN_MASK);
            postCtrlKeyToTable(KeyEvent.KEY_RELEASED, KeyEvent.VK_END, KeyEvent.CTRL_DOWN_MASK);
        } else {
            postCtrlKeyToTable(KeyEvent.KEY_PRESSED, KeyEvent.VK_UP);
            postCtrlKeyToTable(KeyEvent.KEY_RELEASED, KeyEvent.VK_UP);
        }
    }

    private void postCtrlKeyToTable(int keyStatus, int key) {
        postCtrlKeyToTable(keyStatus, key, 0);
    }

    private void postCtrlKeyToTable(int keyStatus, int key, int modifiers) {
        KeyEvent keyEvent = new KeyEvent(_table, keyStatus, System.currentTimeMillis(), modifiers, key, KeyEvent.CHAR_UNDEFINED);
        _table.processKeyEvent(keyEvent);
    }

    public abstract void showCompleter();

    public abstract void closeCompleter();

    protected abstract Point getCompleterLocation();

    public void down() {
        int row = _table.getSelectedRow();
        if ( row == _table.getRowCount() - 1 ) {
            postCtrlKeyToTable(KeyEvent.KEY_PRESSED, KeyEvent.VK_HOME, KeyEvent.CTRL_DOWN_MASK);
            postCtrlKeyToTable(KeyEvent.KEY_RELEASED, KeyEvent.VK_HOME, KeyEvent.CTRL_DOWN_MASK);
        } else {
            postCtrlKeyToTable(KeyEvent.KEY_PRESSED, KeyEvent.VK_DOWN);
            postCtrlKeyToTable(KeyEvent.KEY_RELEASED, KeyEvent.VK_DOWN);
        }
    }

    public CompleterItem[] getSelectedItems() {
        int[] rows = _table.getSelectedRows();
        CompleterItem[] items = new CompleterItem[null == rows ? 0 : rows.length];
        for (int i = 0; i < items.length; i++) {
            items[i] = _model.getItem(rows[i]);
        }
        return items;
    }

    public Object getLastItem() {
        return _lastItem == null ? null : _lastItem.getObject();
    }

    public void itemSelected() {
        CompleterItem[] items = getSelectedItems();
        if (null != items && items.length > 0) {
            if (items[0].isKeepLastItem()) {
                _lastItem = items[0];
            }
        }
    }

    public String getFilter() {
        return _model.getFilter();
    }

    public void setFilter(CharSequence completingWord) {
        Utils.debug("setFilter : completingWord = " + completingWord);
        if (completingWord == null) {
            setFilter(null);
        } else {
            String s = completingWord.toString();
            if (s.endsWith(".")) {
                s = null;
            } else if( s.startsWith("\"") || s.startsWith("`") ) {
                s = s.substring(1);
            }
            setFilter(s);
        }
    }

    public void pageUp() {
        postCtrlKeyToTable(KeyEvent.KEY_PRESSED, KeyEvent.VK_PAGE_UP);
        postCtrlKeyToTable(KeyEvent.KEY_RELEASED, KeyEvent.VK_PAGE_UP);
    }

    public void pageDown() {
        postCtrlKeyToTable(KeyEvent.KEY_PRESSED, KeyEvent.VK_PAGE_DOWN);
        postCtrlKeyToTable(KeyEvent.KEY_RELEASED, KeyEvent.VK_PAGE_DOWN);
    }

    public void selectAll() {
        _table.selectAll();
    }

    public abstract void completeWord();

    protected JLayeredPane getLayeredPane() {
        Component c = Utils.getFrame();
        if (c instanceof JFrame) {
            return ((JFrame) c).getLayeredPane();
        } else if (c instanceof JDialog) {
            return ((JDialog) c).getLayeredPane();
        }
        return null;
    }

    protected class MyTable extends JTable {
        private MyCellRenderer _iconRenderer = new MyCellRenderer();

        public MyTable(TableModel dm) {
            super(dm);
        }

        public void processKeyEvent(KeyEvent keyEvent) {
            char keyChar = keyEvent.getKeyChar();
            if (keyChar == KeyEvent.VK_ENTER) {
                completeWord();
                closeCompleter();
//                getEditor().getComponent().requestFocus();
                return;
            } else if (keyChar == KeyEvent.VK_ESCAPE) {
                closeCompleter();
//                getEditor().getComponent().requestFocus();
                return;
            }
            super.processKeyEvent(keyEvent);
        }


        public TableCellRenderer getCellRenderer(int row, int column) {
            // Description column right alignment
            TableCellRenderer cellRenderer = super.getCellRenderer(row, column);
            Component c = cellRenderer.getTableCellRendererComponent(this, getValueAt(row, column), false, false, row, column);
            if (c instanceof JLabel) {
                JLabel label = (JLabel) c;
                label.setHorizontalAlignment(column == 1 ? JLabel.LEFT : JLabel.RIGHT);
            }
            return column == 0 ? _iconRenderer : cellRenderer;
        }
    }

    private static class MyCellRenderer extends DefaultTableCellRenderer {
        private static Icons _icons = Icons.getInstance();

        public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
            Integer iconIndex = (Integer) value;
            if (iconIndex >= 0 && iconIndex < _icons.size()) {
                Icon icon = _icons.get(iconIndex);
                setIcon(icon);
            }
            return this;
        }
    }

    private static class MyMouseAdapter extends MouseAdapter {
        private AbstractCompleteWindow _window;

        public MyMouseAdapter(AbstractCompleteWindow window) {
            _window = window;
        }

        public void mouseClicked(MouseEvent e) {
            PropertyChangeMultiCaster.getInstance().fireChangeEvent(_window, "refresh");
        }
    }
}
