package dbhelp.plugin.complete;

import dbhelp.db.*;
import dbhelp.plugin.complete.oracle.PackageScanner;
import dbhelp.plugin.complete.utils.CompleterItem;
import dbhelp.plugin.complete.utils.CompleterItem.CatalogItem;
import dbhelp.plugin.complete.utils.CompleterItem.ColumnItem;
import dbhelp.plugin.complete.utils.CompleterItem.KeywordItem;
import utils.Utils;
import utils.text.ITokenEnumeration;
import utils.text.IWordEnumeration;
import utils.text.WordScanner;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * User: huojun
 * Date: 15-Sep-2006
 * Time: 11:33:41 AM
 */
public class DefaultSQLResolver implements ISQLResolver {
    protected List<CompleterItem> _items = new ArrayList<CompleterItem>();
    private KeywordItem ITEM_STAR = new KeywordItem("*");
    protected Database _database;

    public static void main(String[] args) throws Exception {
//        String text = Utils.getFileText("C:\\temp\\procedures.sql");
////        String text = Utils.getFileText("C:\\Documents and Settings\\huojun\\IdeaProjects\\DBHelper\\src\\resources\\test.sql");
        DefaultSQLResolver s = new DefaultSQLResolver();
////        Collection<CompleterItem> completerItems = s.scanVars(text, text.indexOf("@emp_role = 'BU'"));
////        Collection<CompleterItem> completerItems = s.scanVars(text, text.indexOf("@new_emp_role = 'CA'"));
//        Collection<CompleterItem> completerItems = s.scanVars(text, text.indexOf("^"));
//        for (CompleterItem completerItem : completerItems) {
//            Utils.debug("completerItem = " + completerItem + ", " + completerItem.getDescription());
//        }
//        s.resolveCorrelation(text, "t", text.indexOf("^"));
        String text = Utils.getFileText("c:\\temp\\bug.sql");
        CharSequence ss = s.getSelectStatement(text, text.indexOf('^'));
        System.out.println("ss = " + ss);
    }

    public Collection<CompleterItem> resolve(Database db, CharSequence text, int offset) {
        _database = db;
        List<CompleterItem> items = new ArrayList<CompleterItem>();
        CharSequence completingWord = WordScanner.getCompletingWord(text, offset);
        try {
            Utils.debug("completingWord = " + completingWord);
            if (null != completingWord && db.isConnected() && completingWord.toString().endsWith(".")) {
                if (isValidNaming(completingWord)) {
                    String x = completingWord.toString();
                    resolve(x, items, text, offset, db, false);
                    if (items.size() == 0) {
                        if (isOracle(db)) {
                            Utils.debug("Resolving Oracle package");
                            List<CompleterItem> list = PackageScanner.findPackages(text);
                            String pn = x.substring(0, x.length() - 1);
                            for (CompleterItem c : list) {
                                PackageScanner.OraclePackage p = (PackageScanner.OraclePackage) c.getObject();
                                if (p.getName().equalsIgnoreCase(pn)) {
                                    for (Object o : p.getMembers()) {
                                        items.add(CompleterItem.create(o));
                                    }
                                    break;
                                }
                            }
                        }
                    }
                }
            } else {
                return getAllItems(db, text, offset);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (items.size() == 0) {
            items.add(CompleterItem.EMPTY);
        }
        return items;
    }

    private boolean isOracle(Database db) {
        String dbVender = db.getDatabaseProductName();
        return null != dbVender && dbVender.toLowerCase().indexOf("oracle") >= 0;
    }

    private void resolve(String x, List<CompleterItem> items, CharSequence text, int offset, Database database, boolean selfCall) throws Exception {
        Utils.debug("DefaultSQLResolver.resolving: " + x);
        List<Table> tables;
        TempTable tempTable;
        Object t;
        String[] tokens = x.split("\\.");
        int tlen = tokens.length;
        Schema sh;
        if (tlen == 1) {
//                String w = unQuote(tokens[0]);
            String w = tokens[0];
            if (matchCatalog(database, w)) {
                Utils.debug(w + " is a catalog");
                items.addAll(schemas(database));
                sh = new Schema();
                sh.setCatalog(database.unQuote(w));
                sh.setName(null);
                items.addAll(resolveTableAndProc(database, sh));
            } else if ((sh = matchSchema(database, w)) != null) {
                Utils.debug(w + " is a schema");
                items.addAll(resolveTableAndProc(database, sh));
            } else if ((tempTable = matchTempTable(text, offset, w)) != null) {
                items.add(ITEM_STAR);
                items.addAll(CompleterItem.listToItems(tempTable.getColumns(), CompleterItem.TempTableColumnItem.class));
            } else if ((tables = matchTables(database, w)).size() > 0) {
                Table table = tables.get(0);
                String tt = table.getTableType();
                Utils.debug(w + " is a " + tt);
                Column[] columns = table.getColumns();
                if (columns == null || columns.length == 0) {
                    if (tt.equals("SYNONYM")) {
                        // oracle package names
                        items.addAll(resolvePackageMembers(database, table.getName()));
                        return;
                    }
                }
                items.add(ITEM_STAR);
                items.addAll(resolveColumns(tables));
            } else if (!selfCall && (t = resolveCorrelation(database, text, w, offset)) != null) {
                Utils.debug(w + " is a correlation name");
                if (t instanceof String) {
                    resolve(t.toString(), items, text, offset, database, true);
                } else if (t instanceof DerivedTable) {
                    DerivedTable dt = (DerivedTable) t;
//                        Utils.debug("dt = " + dt + ", " + dt.getColumns());
                    items.addAll(CompleterItem.listToItems(dt.getColumns(), CompleterItem.DerivedColumn.class));
                    items.add(ITEM_STAR);
                }
            }
        }
    }

    private TempTable matchTempTable(CharSequence text, int offset, String w) throws Exception {
        Collection<CompleterItem> items = scanVars(text, offset);
        for (CompleterItem item : items) {
            if (item instanceof CompleterItem.TempTableItem) {
                CompleterItem.TempTableItem tt = (CompleterItem.TempTableItem) item;
                TempTable t = (TempTable) tt.getObject();
                if (t.getName().equals(w)) {
                    return t;
                }
            }
        }
        return null;
    }

    public void refresh() {
        Utils.debug("DefaultSQLResolver.refresh");
        _items.clear();
    }

    public Collection<CompleterItem> getAllItems(Database database, CharSequence text, int offset) {
        if (_items.size() == 0) {
            try {
                _items.addAll(catalogs(database));
                _items.addAll(schemas(database));
                _items.addAll(keywords(database));
                _items.addAll(tables(database));
                _items.addAll(procudures(database));
                _items.addAll(functions(database));
                if (isOracle(database)) {
                    _items.addAll(PackageScanner.findPackages(text));
                }

            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        Collection<CompleterItem> vars = null;
        try {
            vars = scanVars(text, offset);
        } catch (Exception e) {
            e.printStackTrace();
        }
        ArrayList<CompleterItem> list = new ArrayList<CompleterItem>(_items);
        if (null != vars) {
//            Utils.debug("vars = " + vars);
            list.addAll(vars);
        }
        return list;
    }

    private List<CompleterItem> resolveColumns(Table t) throws IllegalAccessException, InstantiationException {
        return CompleterItem.listToItems(t.getColumns(), CompleterItem.ColumnItem.class);
    }

    private boolean isValidNaming(CharSequence completingWord) {
        int n = 0;
        for (int i = 0; i < completingWord.length(); i++) {
            if (completingWord.charAt(i) == '.') {
                n++;
                if (n > 3) {
                    return false;
                }
            }
        }
        return !"...".equals(completingWord);
    }
//
//    private Table resolveTable(Database db, String catalog, String schema, String tableName) throws Exception {
////        Utils.debug("DefaultSQLResolver.resolveTable");
////        Utils.debug("catalog = " + catalog + ", schema = " + schema + ", table = " + tableName);
//        List<Table> tables = db.getMetaData().getTables(catalog, schema, null, tableName);
//        return tables.size() == 1 ? tables.get(0) : null;
//    }

    private Object resolveCorrelation(Database database, CharSequence text, String w, int offset) {
        String x = database.unQuote(w);
        Utils.debug("DefaultSQLResolver.resolveCorrelation: x = " + x);
        CharSequence selectStatement = getSelectStatement(text, offset);
        if (null != selectStatement) {
            Utils.debug(selectStatement);
            ITokenEnumeration e = WordScanner.tokenScan(selectStatement, 0);
            while (e.hasMoreElements()) {
                CharSequence token = e.nextElement();
                if (null != token) {
                    String L1 = e.lookahead(1);
                    String L2 = e.lookahead(2);
                    String L3 = e.lookahead(3);
                    String L4 = e.lookahead(4);
//                    IWordEnumeration.TYPE type = e.getType();
                    Utils.debug("token = [" + token + "], L1 = [" + L1 + "]");
//                    IWordEnumeration.TYPE type = e.getType();
                    if (token.toString().equals(x) && "(".equals(L1)) {
//                        Utils.debug(token + " is derived table name");
                        DerivedTable dt = new DerivedTable(token.toString());
                        if (e.consumeUntil("(") != null) {
                            while (e.hasMoreElements()) {
                                CharSequence col = e.nextElement();
                                if (null != col) {
                                    dt.addColumn(col.toString());
                                    if (")".equals(e.consumeUntil(",", ")"))) {
                                        return dt;
                                    }
                                }
                            }
                        }
                        break;
                    } else if (x.equals(L1) && !".".equals(L2)) {
                        Utils.debug(x + " is correlation for " + token);
                        return token.toString();
                    } else if ("AS".equalsIgnoreCase(L2) && x.equals(L3)
                            && !"(".equals(L4) && !".".equals(L4)) {
                        Utils.debug(x + " is correlation for " + L1);
                        return L1;
                    }
                }
            }
        }
        return null;
    }

    private CharSequence matchSelectStatement(CharSequence text, ITokenEnumeration e, int offset) {
        int beginIndex = e.getOffset();
        while (e.hasMoreElements()) {
            CharSequence t = e.nextElement();
            if (null != t) {
                String n = t.toString();
                if ("(".equalsIgnoreCase(n)) {
                    String L1 = e.lookahead(1);
                    if ("SELECT".equalsIgnoreCase(L1)) {
                        if (e.getOffset() < offset) {
                            CharSequence stmt = matchSelectStatement(text, e, offset);
                            if (stmt != null) {
                                return stmt;
                            }
                        }
                    } else {
                        if (!")".equals(e.consumeUntil(")"))) {
                            return null;
//                            if (e.getOffset() > offset) {
//                                return text.subSequence(beginIndex, e.getOffset());
//                            }
                        }
//                        else {
//                            return null;
//                        }
                    }
                }
                if (Utils.in(n, "END", ";", ")", "INSERT", "DELETE", "UPDATE", "BEGIN")) {
                    // discard this SELECT
                    if (e.getOffset() < offset) {
                        return null;
                    } else if (")".equals(n)) {
                        // include the subquery and continue to match the query
                        return null;
                    } else {
                        // found the SELECT
                        return text.subSequence(beginIndex, e.getOffset());
                    }
                }
                String L1 = e.lookahead(1);
                if ("SELECT".equalsIgnoreCase(L1)) {
                    // another select
                    return null;
                }
            }
        }
        return null;
    }

    private CharSequence getSelectStatement(CharSequence text, int offset) {
        ITokenEnumeration e = WordScanner.tokenScan(text, 0);
        int lastSelectIndex = -1;
        while (e.hasMoreElements()) {
            CharSequence t = e.nextElement();
            if (null != t) {
                String token = t.toString();
                if ("SELECT".equalsIgnoreCase(token)) {
                    if (e.getOffset() < offset) {
                        lastSelectIndex = e.getOffset();
                        CharSequence stmt = matchSelectStatement(text, e, offset);
                        if (null != stmt) {
                            return stmt;
                        }
                    }
                }
            }
        }
        return lastSelectIndex < 0 ? null : text.subSequence(lastSelectIndex, text.length());
    }

    private List<Table> matchTables(Database database, String w) throws Exception {
        Utils.debug("DefaultSQLResolver.matchTables");
        String t = database.unQuote(w);
        return database.getMetaData().getTables(null, null, null, t);
    }

    private List<CompleterItem> resolvePackageMembers(Database db, String packageName) throws Exception {
        ArrayList<CompleterItem> items = new ArrayList<CompleterItem>();
        List<Procedure> members = db.getMetaData().getProcedures(packageName, null, null);
        items.addAll(CompleterItem.listToItems(members, CompleterItem.ProcedureItem.class));
        return items;
    }

    private List<CompleterItem> resolveTableAndProc(Database database, Schema sh) throws Exception {
        ArrayList<CompleterItem> items = new ArrayList<CompleterItem>();
        IDbMeta metaData = database.getMetaData();
        List<Table> tables = metaData.getTables(sh.getCatalog(), sh.getName(), null, null);
        items.addAll(CompleterItem.listToItems(tables, CompleterItem.TableItem.class));
        List<Procedure> procedures = metaData.getProcedures(null, sh.getName(), null);
        items.addAll(CompleterItem.listToItems(procedures, CompleterItem.ProcedureItem.class));
        return items;
    }

    private Schema matchSchema(Database database, String w) throws Exception {
        Utils.debug("DefaultSQLResolver.matchSchema: " + w);
        String s = database.unQuote(w);
        List<Schema> schemas = database.getMetaData().getSchemas();
        for (Schema schema : schemas) {
            if (schema.getName().equals(s)) {
                return schema;
            }
        }
        return null;
    }

    private boolean matchCatalog(Database database, String w) throws Exception {
        Utils.debug("DefaultSQLResolver.matchCatalog: " + w);
        String cat = database.unQuote(w);
        List<Catalog> catalogs = database.getMetaData().getCatalogs();
        for (Catalog catalog : catalogs) {
            if (catalog.getName().equalsIgnoreCase(cat)) {

                return true;
            }
        }
        return false;
    }

    private Collection<CompleterItem> catalogs(Database db) throws Exception {
        List<Catalog> schemas = db.getMetaData().getCatalogs();
        return CompleterItem.listToItems(schemas, CatalogItem.class);
    }

    private List<CompleterItem> schemas(Database db) throws Exception {
        List<Schema> schemas = db.getMetaData().getSchemas();
        return CompleterItem.listToItems(schemas, CompleterItem.SchemaItem.class);
    }

    private String scanParameters(ITokenEnumeration e, List<CallParameter> vars, String procedureName) {
//        Utils.debug("procedureName = " + procedureName);
        while (e.hasMoreElements()) {
            CallParameter cp = new CallParameter();
            cp.setProcedureName(procedureName);
            String x = e.nextElement().toString();
            if (")".equals(x)) {
                return x;
            } else if ("IN".equalsIgnoreCase(x)
                    || "OUT".equalsIgnoreCase(x) || "INOUT".equalsIgnoreCase(x)) {
                cp.setType(x);
                if (e.hasMoreElements()) {
                    cp.setName(e.nextElement().toString());
                }
            } else {
                cp.setName(x);
            }
            StringBuilder sb = new StringBuilder();
            while (e.hasMoreElements()) {
                if (sb.length() > 0) {
                    sb.append(' ');
                }
                CharSequence typeToken = e.nextElement();
                if (null != typeToken) {
                    sb.append(typeToken);
                    if ("(".equals(typeToken.toString())) {
                        toMatchingParentheses(e, sb);
                    }
                    String L1 = e.lookahead(1);
                    if (Utils.in(L1, "AS", "DEFAULT", "NULL", "NOT", "OUTPUT", ",", ")", "WITH", "=")) {
                        String nt1 = e.consumeUntil("AS", "DEFAULT", "NULL", "NOT", "OUTPUT", ",", ")", "WITH", "=");

                        if ("AS".equalsIgnoreCase(nt1) || ")".equalsIgnoreCase(nt1) || "WITH".equalsIgnoreCase(nt1)) {
                            cp.setDataType(sb.toString());
                            vars.add(cp);
                            return nt1;
                        } else if (",".equalsIgnoreCase(nt1)
                                || "DEFAULT".equalsIgnoreCase(nt1)
                                || "NOT".equalsIgnoreCase(nt1)
                                || "NULL".equalsIgnoreCase(nt1)) {
                            cp.setDataType(sb.toString());
                            vars.add(cp);
                            if (",".equals(nt1)) {
                                break;
                            } else {
                                nt1 = e.consumeUntil(",", ")");
//                                cp.setDataType(sb.toString());
//                                vars.add(cp);
                                if (")".equals(nt1)) {
                                    return nt1;
                                } else {
                                    break;
                                }
                            }
                        } else if ("=".equalsIgnoreCase(nt1)) {
                            nt1 = e.consumeUntil("OUTPUT", ")", "AS", ",", "WITH");
                            if ("OUTPUT".equalsIgnoreCase(nt1)) {
                                cp.setType("OUTPUT");
                                cp.setDataType(sb.toString());
                                vars.add(cp);
                                nt1 = e.consumeUntil(")", "AS", ",", "WITH");
                                if (",".equals(nt1)) {
                                    break;
                                } else {
                                    return nt1;
                                }
                            } else if (",".equals(nt1)) {
                                cp.setDataType(sb.toString());
                                vars.add(cp);
                                break;
                            } else {
                                cp.setDataType(sb.toString());
                                vars.add(cp);
                                return nt1;
                            }
                        } else if ("OUTPUT".equalsIgnoreCase(nt1)) {
                            cp.setType("OUTPUT");
                            cp.setDataType(sb.toString());
                            vars.add(cp);
                            String term = e.consumeUntil(",", ")", "AS", "WITH");
                            if (",".equals(term)) {
                                break;
                            } else {
                                return term;
                            }
                        } else if (null == nt1) {
                            return null;
                        }
                    }
                } else {
                    return null;
                }
            }
        }
        return null;
    }

    private void toMatchingParentheses(ITokenEnumeration e, StringBuilder sb) {
        int n = 1;
        while (e.hasMoreElements()) {
            CharSequence nt = e.nextElement();
            if (null != nt) {
                String s = nt.toString();
                if ("(".equals(s)) {
                    n++;
                }
                sb.append(nt);
                if (")".equals(s)) {
                    n--;
                    if (n == 0) {
                        return;
                    }
                }
            }
        }
    }

    private Collection<CompleterItem> scanVars(CharSequence text, int offset) throws IllegalAccessException, InstantiationException {
        ITokenEnumeration e = WordScanner.tokenScan(text, 0);
        List<CompleterItem> vars = new ArrayList<CompleterItem>();
        scanLocalVars(e, offset, vars);
//        System.out.println("vars = " + vars);
        return vars;
    }

    private int scanLocalVars(ITokenEnumeration e, int offset, List<CompleterItem> vars) throws IllegalAccessException, InstantiationException {
//        Utils.debug("DefaultSQLResolver.scanLocalVars");
        List<CompleterItem> localVars = new ArrayList<CompleterItem>();
        List<CallParameter> parms = new ArrayList<CallParameter>();
        boolean ignoreEnd = false;
        while (e.hasMoreElements()) {
            CharSequence token = e.nextElement();
            if (e.getOffset() >= offset) {
                break;
            }
            if (null != token) {
                String t = token.toString();
                if ("CREATE".equalsIgnoreCase(t)) {
                    String L1 = e.lookahead(1);
                    String procName;
                    if ("PROCEDURE".equalsIgnoreCase(L1) || "FUNCTION".equalsIgnoreCase(L1)) {
                        localVars.clear();
                        // consume PROCEDURE || Function
                        e.nextElement();
                        // match owner
                        procName = matchProcName(e);
                        if ("(".equalsIgnoreCase(e.lookahead(1))) {
                            // skip "("
                            e.nextElement();
                        }
                        String term = scanParameters(e, parms, procName);
                        if ( null != term ) {
                            if ("AS".equalsIgnoreCase(term)) {
                                // T-SQL, no BEGIN END;
                                int found = scanLocalVars(e, offset, vars);
                                if (found != -1) {
                                    vars.addAll(CompleterItem.listToItems(parms, CompleterItem.CallParamItem.class));
                                }
                            } else {
                                // regular begin end
//                                continue;
                            }
                        }
                    }
                } else if ("BEGIN".equalsIgnoreCase(t)) {
                    String tokenAfterBegin = e.lookahead(1);
                    if (!"TRAN".equalsIgnoreCase(tokenAfterBegin)
                            && !"TRANSACTION".equalsIgnoreCase(tokenAfterBegin)) {
                        if (e.hasMoreElements()) {
                            if( scanLocalVars(e, offset, vars) == -1 ) {
//                                System.out.println("Clear parms = " + parms);
                                parms.clear();
                            }
                        }
                    }
                } else if ("END".equalsIgnoreCase(t)) {
                    if (ignoreEnd) {
                        ignoreEnd = false;
                        continue;
                    }
                    int endOffset = e.getOffset() + t.length();
                    String tokenAfterEnd = e.lookahead(1);
                    if (!"IF".equalsIgnoreCase(tokenAfterEnd)
                            && !"LOOP".equalsIgnoreCase(tokenAfterEnd)) {
                        if (endOffset < offset) {
                            // leaving a block of begin .. end, clear the local vars
//                            System.out.println("Clear vars = " + localVars);
                            localVars.clear();
                            return -1;
                        }
                    }
                } else if ("DECLARE".equalsIgnoreCase(t)) {
                    String nt1 = e.lookahead(1);
                    String nt2 = e.lookahead(2);
                    String nt3 = e.lookahead(3);
                    if ("LOCAL".equalsIgnoreCase(nt1)
                            && "TEMPORARY".equalsIgnoreCase(nt2) && "TABLE".equalsIgnoreCase(nt3)) {
                        TempTable tt = matchTempTable(e);
                        if (null != tt) {
                            localVars.add(new CompleterItem.TempTableItem(tt));
                        }
                    } else if ("CURSOR".equalsIgnoreCase(nt2) || "CURSOR".equalsIgnoreCase(nt3)) {
                        Cursor c = new Cursor(e.nextElement().toString());
                        localVars.add(new CompleterItem.CursorItem(c));
                    } else if ("EXCEPTION".equalsIgnoreCase(nt2)) {
                        DeclaredException de = new DeclaredException(e.nextElement().toString());
                        localVars.add(new CompleterItem.ExceptionItem(de));
                    } else {
                        Variable var = matchVar(e);
                        localVars.add(new CompleterItem.VarItem(var));
                    }
                } else if ("CASE".equalsIgnoreCase(t)) {
                    ignoreEnd = true;
                }
            }

        }
        if (localVars.size() > 0) {
            vars.addAll(localVars);
        }
        if( parms.size() > 0 ) {
            vars.addAll( CompleterItem.listToItems(parms, CompleterItem.CallParamItem.class) );
        }
        return localVars.size();
    }

    private String matchProcName(ITokenEnumeration e) {
        String L2 = e.lookahead(2);
        String L3 = e.lookahead(3);
        String L4 = e.lookahead(4);
        int n = 1;
        if (".".equals(L4)) {
            n = 5;
        } else if (".".equals(L3)) {
            n = 4;
        } else if (".".equals(L2)) {
            n = 3;
        }
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < n; i++) {
            sb.append(e.nextElement());
        }
        return sb.toString();
    }

    private TempTable matchTempTable(ITokenEnumeration e) {
        Utils.debug("DefaultSQLResolver.matchTempTable");
        if (e.consumeUntil("TABLE") != null) {
            if (e.hasMoreElements()) {
                TempTable t = new TempTable(e.nextElement().toString());
                if (e.consumeUntil("(") != null) {
                    while (e.hasMoreElements()) {
                        String nt = e.lookahead(1);
                        // not table constraint
                        if (!Utils.in(nt, "UNIQUE", "CONSTRAINT", "PRIMARY",
                                "CHECK", "NOT", "FOREIGN", "REFERENCES")) {
                            TempTableColumn c = new TempTableColumn();
                            c.setName(e.nextElement().toString());
                            StringBuilder type = new StringBuilder();
                            while (e.hasMoreElements()) {
                                CharSequence ne = e.nextElement();
                                if (null != ne) {
                                    String token = ne.toString();
                                    if (type.length() > 0) {
                                        type.append(' ');
                                    }
                                    type.append(token);
                                    if ("(".equals(token)) {
                                        toMatchingParentheses(e, type);
                                    }
                                    String nt1 = e.lookahead(1);
                                    if (Utils.in(nt1, "DEFAULT", "NOT", "NULL", "CONSTRAINT", "UNIQUE"
                                            , "PRIMARY", "REFERENCES", "COMPUTE")) {
                                        // data type finished
                                        c.setType(type.toString());
                                        t.addColumn(c);
                                        e.consumeUntil(",", ")");
                                        break;
                                    }
                                    if (",".equals(token)) {
                                        c.setType(type.toString());
                                        t.addColumn(c);
                                        // next column
                                        break;
                                    } else if (")".equals(token)) {
                                        // definition finished
                                        return t;
                                    }
                                }
                            }
                        } else {
                            e.nextElement();
                        }
                    }
                }
                return t;
            }
        }
        return null;
    }

    private Variable matchVar(ITokenEnumeration e) {
        String name = null;
        String type = null;
        if (e.hasMoreElements()) {
            name = e.nextElement().toString();
            StringBuilder sb = new StringBuilder();
            while (e.hasMoreElements()) {
                if (sb.length() > 0) {
                    sb.append(' ');
                }
                sb.append(e.nextElement());
                String L1 = e.lookahead(1);
                if (";".equals(L1)
                        || (_database != null && _database.isKeyword(L1))
                        || e.getType() == IWordEnumeration.TYPE.MULTILINE_COMMENT
                        || e.getType() == IWordEnumeration.TYPE.SINGLELINE_COMMENT
                        || "DECLARE".equalsIgnoreCase(L1)
                        ) {
                    type = sb.toString();
                    break;
                }
            }
        }
        return new Variable(name, type);
    }

    private Collection<CompleterItem> functions(Database db) throws Exception {
        return CompleterItem.listToItems(db.getMetaData().getFunctions(), CompleterItem.FunctionItem.class);
    }

    private Collection<CompleterItem> procudures(Database db) throws Exception {
        return CompleterItem.listToItems(db.getMetaData().getProcedures(null, null, null), CompleterItem.ProcedureItem.class);
    }

    private Collection<? extends CompleterItem> keywords(Database db) throws Exception {
        return CompleterItem.listToItems(db.getKeywords(), CompleterItem.KeywordItem.class);
    }

    private Collection<? extends CompleterItem> tables(Database db) throws Exception {
        return CompleterItem.listToItems(db.getMetaData().getAllTables(), CompleterItem.TableItem.class);
    }

    private List<CompleterItem> resolveColumns(List<Table> t) throws IllegalAccessException, InstantiationException {
        ArrayList<CompleterItem> items = new ArrayList<CompleterItem>();
        for (Table table : t) {
            items.addAll(CompleterItem.listToItems(table.getColumns(), ColumnItem.class));
        }
        return items;
    }

    public Collection<CompleterItem> resolveChildren(Database db, Object parent) {
        try {
            if (parent instanceof Table) {
                List<CompleterItem> items = resolveColumns((Table) parent);
                items.add(ITEM_STAR);
                return items;
            } else if (parent instanceof Catalog) {
                return schemas(db);
            } else if (parent instanceof Schema) {
                return resolveTableAndProc(db, (Schema) parent);
            }
        } catch (Exception e) {
            Utils.debug("DefaultSQLResolver.resolveChildren: " + e);
        }
        return new ArrayList<CompleterItem>();
    }

    public String toString() {
        return "Default fuzzy resolver";
    }
}
