package dbhelp.plugin.complete.oracle;

import dbhelp.db.Procedure;
import dbhelp.db.Variable;
import dbhelp.plugin.complete.utils.CompleterItem;
import utils.Utils;
import utils.text.ForwardsEnumerator;
import static utils.text.IWordEnumeration.TYPE;
import utils.text.TokenIterator;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * User: huojun
 * Date: 24-Oct-2006
 * Time: 2:00:36 PM
 */
public class PackageScanner {
    private static ForwardsEnumerator _source = new ForwardsEnumerator();
    private static TokenIterator _tokens = new TokenIterator(_source);


    public static void main(String[] args) throws IOException {
        String text = Utils.getFileText("c:\\temp\\ora.sql");
        List<CompleterItem> list = findPackages(text);
        for(CompleterItem c: list) {
            System.out.println("c = " + c);
            System.out.println(c.getObject());
        }
    }

    public static List<CompleterItem> findPackages(CharSequence text) {
        List<CompleterItem> items = new ArrayList<CompleterItem>();
        _source.setText(text);
        while( _tokens.hasMoreElements() ) {
            CharSequence token = _tokens.nextElement();
            if( null != token ) {
                String t = token.toString();
                if( "PACKAGE".equalsIgnoreCase(t) ) {
                    items.add( new CompleterItem.Package(consumePackage(_tokens)) );
                }
            }
        }
        return items;
    }

    private static OraclePackage consumePackage(TokenIterator tokens) {
        OraclePackage p = new OraclePackage();
        while( tokens.hasMoreElements() ) {
            CharSequence token = tokens.nextElement();
            if( null != token ) {
                String t = token.toString();
                if( "BODY".equalsIgnoreCase(t) ) {
                    // SKIP BODY keyword
                    continue;
                }
                p.setName(t);
                while( tokens.hasMoreElements() ) {
                    CharSequence tt = tokens.nextElement();
                    TYPE type = tokens.getType();
                    String L1 = tokens.lookahead(1);
                    String L2 = tokens.lookahead(2);
                    String L3 = tokens.lookahead(3);
                    if( tt != null ) {
                        String w = tt.toString();
                        if( "END".equalsIgnoreCase(w) ) {
                            return p;
                        } else if( "PROCEDURE".equalsIgnoreCase(w) ) {
                            if( null != L1  ) {
                                p.addMember(new Procedure(L1));
                                String term = tokens.consumeUntil(";", "BEGIN");
                                if( "BEGIN".equals(term) ) {
                                    consumeBlock(tokens);
                                } else if( null == term ) {
                                    return p;
                                }
                            }
                        } else if( type == TYPE.WORD ) {
                            if( ":".equals(L2) && "=".equals(L3) ) {
                                Variable v = new Variable(w);
                                v.setType(L1);
                                p.addMember(v);
                            }
                        }
                    }
                }
            }
        }
        return p;
    }

    private static void consumeBlock(TokenIterator tokens) {
        int n=1;
        while(tokens.hasMoreElements()) {
            CharSequence t = tokens.nextElement();
            if( null != t ) {
                String token = t.toString();
                if( "BEGIN".equalsIgnoreCase(token) ) {
                    n++;
                } else if( "END".equalsIgnoreCase(token) ) {
                    n--;
                    if( n==0 ) {
                        return;
                    }
                }
            }
        }
    }

    public static class OraclePackage {
        private String _name;
        private List<Object> _members = new ArrayList<Object>();

        public OraclePackage(String name) {
            _name = name;
        }

        public OraclePackage() {
            setName("Unknown");
        }


        public void setName(String name) {
            _name = name;
        }

        public void addMember(Object o) {
            _members.add(o);
        }

        public String getName() {
            return _name;
        }

        public List<Object> getMembers() {
            return _members;
        }


        public String toString() {
            return getName();
        }
    }
}
