package dbhelp.plugin.complete.utils;

import dbhelp.db.*;

import java.util.List;
import java.util.Collection;
import java.util.ArrayList;
import java.util.Arrays;


/**
 * User: huojun
 * Date: 14-Sep-2006
 * Time: 10:36:18 AM
 */
public class CompleterItem {

    protected Object _object;
    public static Empty EMPTY = new Empty();

    public CompleterItem() {
    }

    public CompleterItem(Object object) {
        _object = object;
    }

    public int getIconIndex() {
        return 0;
    }

    public void setObject(Object object) {
        _object = object;
    }

    public boolean isKeepLastItem() {
        return false;
    }

    public Object getObject() {
        return _object;
    }

    public String toCompleteString() {
        return toString();
    }

    public String toString() {
        return _object == null ? "" : _object.toString();
    }

    public String getDescription() {
        return toString();
    }

    public static List<CompleterItem> listToItems(Object[] list, Class<? extends CompleterItem> c) throws IllegalAccessException, InstantiationException {
        return listToItems(null == list ? null : Arrays.asList(list), c);
    }

    public static List<CompleterItem> listToItems(Collection<?> list, Class<? extends CompleterItem> c) throws IllegalAccessException, InstantiationException {
        List<CompleterItem> items = new ArrayList<CompleterItem>();
        if ( null != list ) {
            for( Object o : list ) {
                CompleterItem ci = c.newInstance();
                ci.setObject(o);
                items.add( ci );
            }
        }
        return items;
    }

    public static CompleterItem create(Object o) {
        if( o instanceof Procedure ) {
            return new ProcedureItem(o);
        } else if( o instanceof Variable ) {
            return new VarItem(o);
        } else if( o instanceof Table ) {
            return new TableItem(o);
        }
        return new CompleterItem(o);
    }

    public static class Empty extends CompleterItem {
        public Empty() {
            super("No suggestion");
        }

        public String getDescription() {
            return "";
        }
    }

    public static class TableItem extends CompleterItem {

        public TableItem() {
        }

        public TableItem(Object o) {
            super(o);
        }

        public int getIconIndex() {
            return 5;
        }

        public boolean isKeepLastItem() {
            return true;
        }


        public String toString() {
            return getTable().getName();
        }

        public String toCompleteString() {
            Table t = getTable();
            Database db = t.getDatabase();
            return db == null ? t.getName() : db.quote(t.getName());
        }

        private Table getTable() {
            return ((Table)getObject());
        }

        public String getDescription() {
            Table t = getTable();
            StringBuilder sb = new StringBuilder();
            if( t.getCatalog() != null ) {
                sb.append(t.getCatalog());
            }
            if( t.getSchema() != null ) {
                if( sb.length() > 0 ) {
                    sb.append('.');
                }
                sb.append(t.getSchema());
            }
            return t.getTableType() + ", " + sb.toString();
        }
    }

    public static class ColumnItem extends CompleterItem {
        public String getDescription() {
            Column c = getColumn();
            return c.getTableName() + ", " + c.getTypeName();
        }

        public String toCompleteString() {
            Column c = getColumn();
            Table table = c.getTable();
            Database db = null == table ? null : table.getDatabase();
            return db == null ? c.getName() : db.quote(c.getName());
        }


        public String toString() {
            return getColumn().getName();
        }

        private Column getColumn() {
            return ((Column)getObject());
        }

        public int getIconIndex() {
            Column c = getColumn();
            return c.isPrimaryKey() && c.isForeignKey() ? 6
                    : ( c.isPrimaryKey() ? 2 : (c.isForeignKey() ? 4 : 3));
        }
    }

    public static class KeywordItem extends CompleterItem {
        // find an icon for it

        public KeywordItem() {
        }

        public KeywordItem(Object object) {
            super(object);
        }

        public String getDescription() {
            return Database.SQL92_KEYWORD_LIST.contains(getObject().toString()) ? "SQL92 keyword" : "Keyword";
        }
    }

    public static class FunctionItem extends CompleterItem {
        // find an icon for it
        public String getDescription() {
            return ((Function) getObject()).getType();
        }
    }

    public static class ProcedureItem extends CompleterItem {

        public ProcedureItem() {
        }

        public ProcedureItem(Object object) {
            super(object);
        }

        public int getIconIndex() {
            return 0;
        }

        public String getDescription() {
            StringBuilder sb = new StringBuilder();
            Procedure t = getProcedure();
            if( t.getCatalog() != null ) {
                sb.append(t.getCatalog()).append('.');
            }
            if( t.getSchema() != null ) {
                sb.append(t.getSchema());
            }
            return "Procedure, " + sb.toString();
        }

        private Procedure getProcedure() {
            return (Procedure) getObject();
        }

        public String toCompleteString() {
            Procedure p = getProcedure();
            Database db = p.getDatabase();
            return db == null ? p.getName() : db.quote(p.getName());
        }

        public String toString() {
            return getProcedure().getName();
        }
    }

    public static class SchemaItem extends CompleterItem {
        public int getIconIndex() {
            return 11;
        }

        public String getDescription() {
            Schema schema = getSchema();
            return schema.getVendorTerm();
        }

        private Schema getSchema() {
            return (Schema) getObject();
        }

        public String toCompleteString() {
            Database db = getSchema().getDatabase();
            return db == null ? super.toString() : db.quote(getSchema().toString());
        }
    }
    
    public static class CatalogItem extends CompleterItem {
        public int getIconIndex() {
            return 1;
        }

        public String getDescription() {
            return getCatalog().getVendorTerm();
        }

        private Catalog getCatalog() {
            return (Catalog) getObject();
        }


        public String toCompleteString() {
            Database db = getCatalog().getDatabase();
            return db == null ? super.toString() : db.quote(getCatalog().toString());
        }
    }

    public static class VarItem extends CompleterItem {
        public VarItem() {
        }

        public VarItem(Object object) {
            super(object);
        }

        public String getDescription() {
            return ((Variable) getObject()).getType() + ", Variable";
        }
    }

    public static class CursorItem extends CompleterItem {
        public CursorItem() {
        }

        public CursorItem(Object object) {
            super(object);
        }

        public String getDescription() {
            return "Declared cursor";
        }
    }

    public static class ExceptionItem extends CompleterItem {
        public ExceptionItem() {
        }

        public ExceptionItem(DeclaredException de) {
            super(de);
        }

        public String getDescription() {
            return "Declared exception";
        }
    }

    public static class TempTableItem extends CompleterItem {
        public TempTableItem() {
        }

        public TempTableItem(TempTable tt) {
            super(tt);
        }

        public String getDescription() {
            return "Temporary Table";
        }
    }

    public static class CallParamItem extends CompleterItem {
        public CallParamItem() {
        }

        public int getIconIndex() {
            return 7;
        }

        public CallParamItem(Object object) {
            super(object);
        }

        public String getDescription() {
            CallParameter p = ((CallParameter) getObject());
            return "Parameter, " + p.getDataType() + (null == p.getType() ? ", INPUT" : ", " + p.getType());
        }
    }

    public static class TempTableColumnItem extends CompleterItem {

        public TempTableColumnItem() {
        }

        public TempTableColumnItem(Object object) {
            super(object);
        }

        public String toString() {
            TempTableColumn c = (TempTableColumn) getObject();
            return c.getName() + " " + c.getType();
        }

        public String getDescription() {
            return "Local Temp Table Column";
        }
    }

    public static class DerivedColumn extends CompleterItem {
        public DerivedColumn() {
        }

        public DerivedColumn(Object object) {
            super(object);
        }

        public String getDescription() {
            return "Derived table column";
        }
    }

    public static class Package extends CompleterItem {

        public Package() {
        }

        public Package(Object object) {
            super(object);
        }

        public String getDescription() {
            return "Package";
        }
    }


}
