package dbhelp.plugin.idea.action;

import com.intellij.codeInsight.completion.actions.CodeCompletionAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataConstants;
import com.intellij.openapi.actionSystem.DataContext;
import com.intellij.openapi.editor.Document;
import com.intellij.openapi.editor.Editor;
import dbhelp.db.Database;
import dbhelp.plugin.complete.AbstractCompleteWindow;
import dbhelp.plugin.complete.ISQLResolver;
import dbhelp.plugin.complete.utils.CompleterItem;
import dbhelp.plugin.idea.complete.CompleteWindow;
import dbhelp.plugin.idea.utils.IDEAUtils;
import utils.Utils;

import java.util.Collection;

/**
 * User: huojun
 * Date: 5-Oct-2006
 * Time: 10:52:08 AM
 */
public class CompleteAction extends CodeCompletionAction {

    public void actionPerformed(AnActionEvent anActionEvent) {
        if ( BaseAction.isSQLFile(anActionEvent)) {
            handleEvent(anActionEvent);
        } else {
            super.actionPerformed(anActionEvent);
        }
    }

    protected static void handleEvent(AnActionEvent e) {
        DataContext dc = e.getDataContext();
        Editor editor = (Editor) dc.getData(DataConstants.EDITOR);
        Document document = editor.getDocument();
        if (document.isWritable()) {
//                CharSequence text = document.getCharsSequence();
            int offset = editor.getCaretModel().getOffset();
//                if (!WordScanner.inLiteralOrComment(text, offset)) {
            AbstractCompleteWindow cw = CompleteWindow.getInstance();
            Database database = IDEAUtils.getDatabase(document);
            try {
                if (null != database) {
                    CharSequence text = document.getCharsSequence();
                    ISQLResolver resolver = database.getResolver();
                    Collection<CompleterItem> completerItems = resolver.resolve(database, text, offset - 1);
                    cw.setText(database.toString());
                    cw.setItems(completerItems);
                    cw.setFilter(IDEAUtils.getCompletingWord(editor));
                    cw.showCompleter();
                } else {
                    Utils.message("Select the database first!");
                }
            } catch (IllegalStateException e1) {
                Utils.debug(e1);
                // no suggestion in ignored token
            } catch (Exception e1) {
                Utils.debug("CompleteAction.handleEvent: " + e1);
            }
        }
    }
}
