package dbhelp.plugin.idea.action;

import com.intellij.openapi.actionSystem.DataContext;
import com.intellij.openapi.editor.Document;
import com.intellij.openapi.editor.Editor;
import dbhelp.db.Database;
import dbhelp.plugin.complete.AbstractCompleteWindow;
import dbhelp.plugin.complete.ISQLResolver;
import dbhelp.plugin.complete.utils.CompleterItem;
import dbhelp.plugin.idea.utils.IDEAUtils;
import utils.Utils;

import java.util.Collection;

/**
 * Created by IntelliJ IDEA.
 * User: David Huo
 * Date: Sep 19, 2006
 * Time: 12:17:19 PM
 */
public abstract class CompleterCtrlAction extends AbstractEditorAction {

    public static CloseCompleter createCloser() {
        return new CloseCompleter();
    }

    public void execute(Editor editor, DataContext dataContext) {
        if (getCompleter().isVisible()) {
            handleAction(editor, dataContext);
        } else {
            super.execute(editor, dataContext);
        }
    }

    protected abstract void handleAction(Editor editor, DataContext dataContext);

    public static class CloseCompleter extends CompleterCtrlAction {
        protected void handleAction(Editor editor, DataContext dataContext) {
            getCompleter().closeCompleter();
        }
    }

    public static class EADown extends CompleterCtrlAction {
        protected void handleAction(Editor editor, DataContext dataContext) {
            getCompleter().down();
        }
    }

    public static class EAUp extends CompleterCtrlAction {
        protected void handleAction(Editor editor, DataContext dataContext) {
            getCompleter().up();
        }
    }

    public static class EAPageDown extends CompleterCtrlAction {
        protected void handleAction(Editor editor, DataContext dataContext) {
            getCompleter().pageDown();
        }
    }

    public static class EAPageUp extends CompleterCtrlAction {
        protected void handleAction(Editor editor, DataContext dataContext) {
            getCompleter().pageUp();
        }
    }

    public static class EAEnter extends CompleterCtrlAction {
        protected void handleAction(Editor editor, DataContext dataContext) {
            AbstractCompleteWindow cw = getCompleter();
            CompleterItem[] items = cw.getSelectedItems();
            if (items.length > 0) {
                IDEAUtils.completeWord(items);
            }
            cw.closeCompleter();
        }
    }

    public static class EATab extends CompleterCtrlAction {
        protected void handleAction(Editor editor, DataContext dataContext) {
            Document doc = editor.getDocument();
            Database db = IDEAUtils.getDatabase(doc);
            if (null != db && db.isConnected()) {
                AbstractCompleteWindow cw = getCompleter();
                CompleterItem[] items = cw.getSelectedItems();
                if (null != items && items.length > 0) {
                    try {
                        ISQLResolver r = db.getResolver();
                        Collection<CompleterItem> list = r.resolveChildren(db, items[0].getObject());
                        if (list.size() > 0) {
                            cw.setItems(list);
                        }
                    } catch (Exception e) {
                        Utils.debug("CompleterCtrlAction$EATab.handleAction: " + e);
                    }
                }
            }
        }
    }

}
