package dbhelp.plugin.idea.complete;

import static com.intellij.openapi.actionSystem.IdeActions.*;
import com.intellij.openapi.editor.CaretModel;
import com.intellij.openapi.editor.Editor;
import com.intellij.openapi.editor.LogicalPosition;
import com.intellij.openapi.editor.actionSystem.EditorActionHandler;
import com.intellij.openapi.editor.actionSystem.EditorActionManager;
import dbhelp.plugin.idea.action.AbstractEditorAction;
import dbhelp.plugin.idea.action.CompleterCtrlAction;
import static dbhelp.plugin.idea.action.CompleterCtrlAction.createCloser;
import dbhelp.plugin.complete.AbstractCompleteWindow;
import utils.SingletonProvider;
import dbhelp.plugin.idea.utils.IDEAUtils;

import javax.swing.FocusManager;
import javax.swing.*;
import java.awt.*;

/**
 * Created by IntelliJ IDEA.
 * User: David Huo
 * Date: Sep 13, 2006
 * Time: 10:28:56 PM
 */
public class CompleteWindow extends AbstractCompleteWindow {
//    private static CompleteWindow _instance = new CompleteWindow();
    private CompleterCtrlAction.EAEnter _editorEnter = new CompleterCtrlAction.EAEnter();
    private CompleterCtrlAction.CloseCompleter _editorEscape = createCloser();
    private JFrame _frame;
//
//    public static void main(String[] args) throws Exception {
//        JFrame f = new JFrame();
//        f.setLayout(new BorderLayout());
//        CompleteWindow cw = new CompleteWindow();
//        f.getContentPane().add(cw, BorderLayout.CENTER);
//        f.setSize(800, 600);
//        cw.setVisible(true);
//        f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
//        Config.restore();
//        java.util.List<Table> tables = Config.getConfig().getDatabases()[0].getMetaData().getAllTables();
//        cw.setItems(CompleterItem.listToItems(tables, CompleterItem.TableItem.class));
//        JPanel buttonPane = new JPanel();
//        buttonPane.add(new JButton(new actionUp(cw)));
//        buttonPane.add(new JButton(new actionDown(cw)));
//        f.getContentPane().add(buttonPane, BorderLayout.SOUTH);
//        f.setVisible(true);
//    }

    public static AbstractCompleteWindow getInstance() {
        return SingletonProvider.getProvider().getCompleteWindow();
    }

    public CompleteWindow() {
        super();
    }

    public void showCompleter() {
        hookCtrlKeys();
        if (_table.getRowCount() > 0) {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    setLocation(getCompleterLocation());
                    setVisible(true);
                }
            });
        }
    }

    private void hookCtrlKeys() {
        hookEditorAction(ACTION_EDITOR_ENTER, _editorEnter);
        hookEditorAction(ACTION_EDITOR_ESCAPE, _editorEscape);

    }

    public void closeCompleter() {
        unhookCtrlKeys();
        if (isVisible()) {
            setVisible(false);
        }
    }

    private void unhookCtrlKeys() {
        unhookEditorAction(ACTION_EDITOR_ENTER, _editorEnter);
        unhookEditorAction(ACTION_EDITOR_ESCAPE, _editorEscape);
    }

    private void unhookEditorAction(String id, AbstractEditorAction action) {
        EditorActionManager eam = EditorActionManager.getInstance();
        if (action.equals(eam.getActionHandler(id))) {
            eam.setActionHandler(id, action.getOriginalAction());
        }
    }

    protected Point getCompleterLocation() {
        Editor editor = IDEAUtils.getActiveEditor();
//        Utils.debug("editor = " + editor);
        CaretModel caretModel = editor.getCaretModel();
        LogicalPosition lp = caretModel.getLogicalPosition();
        JComponent cc = editor.getContentComponent();
        Point point = editor.logicalPositionToXY(lp);
        Point xy = SwingUtilities.convertPoint(cc, point, editor.getComponent());
//        Utils.debug("xy = " + xy);
        Dimension size = getSize();
        Point p;
        Rectangle vr = editor.getComponent().getVisibleRect();
//        Utils.debug("vr = " + vr);
//        Utils.debug("size = " + size);
//        Utils.debug("point = " + point);
        if (xy.y + size.height > vr.y + vr.getHeight()) {
            p = editor.logicalPositionToXY(new LogicalPosition(lp.line, lp.column));
            p.y = point.y - size.height;
        } else {
            p = editor.logicalPositionToXY(new LogicalPosition(lp.line + 1, lp.column));
        }

        if (point.x + size.width > vr.getWidth()) {
            p.x = (int) (vr.getWidth() - size.width);
        }

        return SwingUtilities.convertPoint(cc, p, getFrame().getLayeredPane());
    }

    private void hookEditorAction(String actionId, AbstractEditorAction newAction) {
        EditorActionManager am = EditorActionManager.getInstance();
        if (!newAction.equals(am.getActionHandler(actionId))) {
            EditorActionHandler orgAction = am.setActionHandler(actionId, newAction);
            newAction.setOriginalAction(orgAction);
        }
    }

    private JFrame getFrame() {
        if (null == _frame) {
            _frame = (JFrame) FocusManager.getCurrentManager().getActiveWindow();
            _frame.getLayeredPane().add(this, JLayeredPane.POPUP_LAYER);
            hookEditorAction(ACTION_EDITOR_MOVE_CARET_LEFT, createCloser());
            hookEditorAction(ACTION_EDITOR_TAB, new CompleterCtrlAction.EATab());
            hookEditorAction(ACTION_EDITOR_MOVE_CARET_RIGHT, createCloser());
            hookEditorAction(ACTION_EDITOR_MOVE_CARET_UP, new CompleterCtrlAction.EAUp());
            hookEditorAction(ACTION_EDITOR_MOVE_CARET_DOWN, new CompleterCtrlAction.EADown());
            hookEditorAction(ACTION_EDITOR_MOVE_CARET_PAGE_UP, new CompleterCtrlAction.EAPageUp());
            hookEditorAction(ACTION_EDITOR_MOVE_CARET_PAGE_DOWN, new CompleterCtrlAction.EAPageDown());
        }
        return _frame;
    }

    public void completeWord() {
        IDEAUtils.completeWord(getSelectedItems());
    }

}
