package dbhelp.plugin.idea.complete;

import com.intellij.openapi.editor.Document;
import com.intellij.openapi.editor.event.DocumentAdapter;
import com.intellij.openapi.editor.event.DocumentEvent;
import dbhelp.db.Database;
import dbhelp.plugin.complete.AbstractCompleteWindow;
import dbhelp.plugin.complete.utils.CompleterItem;
import dbhelp.plugin.idea.utils.IDEAUtils;
import utils.Utils;
import utils.text.WordScanner;

import java.util.Collection;


/**
 * User: huojun
 * Date: 14-Sep-2006
 * Time: 1:26:47 PM
 */
public class DocListener extends DocumentAdapter {

    public void documentChanged(DocumentEvent de) {
//        Utils.debug("DocListener.documentChanged, " + de.getNewFragment());
        Document doc = de.getDocument();
        AbstractCompleteWindow cw = CompleteWindow.getInstance();
        Database db = IDEAUtils.getDatabase(doc);
        cw.setText(db.toString());
        CharSequence text = doc.getCharsSequence();
        int adjust = de.getNewLength() - de.getOldLength() == -1 ? -1 : 0;
        int offset = de.getOffset() + adjust;
//        Utils.debug("x = " + x);
        CharSequence newChar = de.getNewFragment();
        if (newChar != null && newChar.length() > 0 && newChar.charAt(0) == '.' ) {
            // resolve table name or correlation
            try {
                Collection<CompleterItem> completerItems = db.getResolver().resolve(db, text, offset);
                cw.setItems(completerItems);
                cw.showCompleter();
            } catch (IllegalStateException e) {
                // ignore
            } catch (Exception e) {
                Utils.error(e);
            }
        } else if (cw.isVisible()) {
            if( isWhitespaceTyped(de) ) {
                cw.setFilter( null );
                cw.closeCompleter();
                return;
            }
            try {
                CharSequence x = WordScanner.getCompletingWord(text, offset);
//            Utils.debug("cw visible, completing = " + x);
                // filter current list
                // de.getNewLength() - de.getOldLength() == -1 to identify DEL or BACKSPACE
                // offset is +1 then regular char insert
                if (adjust < 0) {
                    // only refresh the list when dot was deleted
                    CharSequence of = de.getOldFragment();
                    if (deletedDot(of) || afterDot(x)) {
                        try {
                            Collection<CompleterItem> completerItems = db.getResolver().resolve(db, text, offset);
                            cw.setItems(completerItems);
                        } catch (Exception e) {
                            Utils.error(e);
                        }
                    } else if( shouldClose(of) ) {
                        cw.closeCompleter();
                    }
                }
                cw.setFilter(x);
            } catch (IllegalStateException e) {
                // ignore the no suggestion
            }
        }
    }

    private boolean shouldClose(CharSequence of) {
        return of != null && of.length() == 1 && ( of.charAt(0) == ' ' || of.charAt(0) == '\n' );
    }

    private boolean afterDot(CharSequence x) {
        return null != x && x.charAt(x.length() - 1) == '.';        
    }

    private boolean isWhitespaceTyped(DocumentEvent de) {
        CharSequence newChars = de.getNewFragment();
        return newChars != null && newChars.length() > 0 && Character.isWhitespace(newChars.charAt(0));
    }

    private boolean deletedDot(CharSequence of) {
        return of != null && of.length() > 0 && of.charAt(0) == '.';
    }
}
