package dbhelp.plugin.idea.filetype;

import com.intellij.openapi.fileEditor.*;
import com.intellij.openapi.util.UserDataHolderBase;
import com.intellij.codeHighlighting.BackgroundEditorHighlighter;
import com.intellij.ide.structureView.StructureViewBuilder;

import javax.swing.*;

import org.jetbrains.annotations.NonNls;

import java.beans.PropertyChangeListener;
import java.io.Serializable;
import java.io.File;
import java.awt.*;

import dbhelp.plugin.idea.ui.IDEADiagram;
import dbhelp.plugin.ui.DatabaseDiagram;
import dbhelp.db.Database;
import dbhelp.db.Table;
import utils.Config;
import utils.Utils;
import diagram.shape.AbstractShape;

/**
 * Created by IntelliJ IDEA.
 * User: David Huo
 * Date: Sep 20, 2006
 * Time: 9:16:36 PM
 */
public class DiagramEditor extends UserDataHolderBase implements FileEditor {
    private FileEditorState _state = new DiagramFileEditorState();
    private IDEADiagram _diagram;

    public static void main(String[] args) throws Exception {
        JFrame f = new JFrame();
        Container cp = f.getContentPane();
        cp.setLayout( new BorderLayout() );
        DatabaseDiagram diagram = new IDEADiagram();
        cp.add( diagram, BorderLayout.CENTER );
        f.setSize(800, 600);
        f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        f.setVisible(true);
        Config cfg = Config.restore();
        Database db = cfg.getDatabases()[0];
        db.connect();
        java.util.List<Table> t1 = db.getMetaData().getTables(null, null, null, "Customer");
        java.util.List<Table> t2 = db.getMetaData().getTables(null, null, null, "sales_order");
        java.util.List<Table> t3 = db.getMetaData().getTables(null, null, null, "sales_order_items");
        AbstractShape as = diagram.addTable(t1.get(0));
        Utils.store(new File("c:\\temp\\diagram.dia"), as);
    }

    public DiagramEditor() {
    }

    public JComponent getComponent() {
        if( null == _diagram ) {
            _diagram = new IDEADiagram();
        }
        return _diagram;
    }

    public IDEADiagram getDiagram() {
        return _diagram;
    }

    public Database getDatabase() {
        return _diagram.getDatabase();
    }

    public JComponent getPreferredFocusedComponent() {
        return getComponent();
    }

    @NonNls
    public String getName() {
        return "Database Diagram Viewer";
    }

    public FileEditorState getState(FileEditorStateLevel level) {
        if(level == FileEditorStateLevel.FULL) {

        }
        return _state;
    }

    public void setState(FileEditorState fileEditorState) {
//        Utils.debug("DiagramEditor.setState");
        _state = fileEditorState;
    }

    public boolean isModified() {
        return true;
    }

    public boolean isValid() {
        return true;
    }

    public void selectNotify() {
    }

    public void deselectNotify() {
    }

    public void addPropertyChangeListener(PropertyChangeListener propertyChangeListener) {
    }

    public void removePropertyChangeListener(PropertyChangeListener propertyChangeListener) {
    }

    public BackgroundEditorHighlighter getBackgroundHighlighter() {
        return null;
    }

    public FileEditorLocation getCurrentLocation() {
        return null;
    }

    public StructureViewBuilder getStructureViewBuilder() {
        return null;
    }

    private static class DiagramFileEditorState implements FileEditorState, Serializable {

        public boolean canBeMergedWith(FileEditorState otherState, FileEditorStateLevel level) {
            return otherState instanceof DiagramFileEditorState;
        }
    }

}
