package dbhelp.plugin.idea.ui;

import com.intellij.openapi.fileEditor.FileEditorManager;
import com.intellij.openapi.vfs.LocalFileSystem;
import com.intellij.openapi.vfs.VirtualFile;
import dbhelp.plugin.idea.utils.IDEAUtils;
import dbhelp.plugin.ui.DatabaseDiagram;
import utils.Utils;

import javax.swing.*;
import static javax.swing.JOptionPane.YES_OPTION;
import static javax.swing.JOptionPane.showConfirmDialog;
import javax.swing.filechooser.FileFilter;
import java.awt.*;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;


/**
 * Created by IntelliJ IDEA.
 * User: huojun
 * Date: 24-Oct-2005
 * Time: 1:32:35 PM
 */
public class IDEADiagram extends DatabaseDiagram {

    public void load() throws Exception {
        FileReader xmlIn = null;
        try {
            xmlIn = new FileReader(getFile());
            _diagram.loadDiagramFromXml(xmlIn);
            fixReferences();
        } catch (Exception e) {
            throw e;
        } finally {
            if (null != xmlIn) {
                try {
                    xmlIn.close();
                } catch (IOException e) {
                    //
                }
            }
        }
    }

    public void save() throws Exception {
        File file = getFile();
        if (null != file) {
            saveToFile(file);
        }
    }

    private VirtualFile saveToFile(File file) throws Exception {
        Utils.debug("Saving to file: " + file);
        VirtualFile virtualFile = IDEAUtils.createVirtualFile(file, null);
        FileWriter fw = new FileWriter(file);
        _diagram.saveDiagramToXml(fw);
        fw.close();
        return virtualFile;
    }

    public void saveAs() throws Exception {
        JFileChooser c = new JFileChooser();
        c.setFileFilter(new FileFilter() {
            public boolean accept(File f) {
                return (f.getName().endsWith(".dia")) || f.isDirectory();
            }

            public String getDescription() {
                return "Diagram file";
            }
        });
        c.setFileSelectionMode(JFileChooser.FILES_ONLY);
        c.setDialogTitle("Choose file to save");
        c.setDialogType(JFileChooser.SAVE_DIALOG);
        Component frame = Utils.getFrame();
        if (c.showSaveDialog(frame) == JFileChooser.APPROVE_OPTION) {
            File f = c.getSelectedFile();
            if (!f.getName().toLowerCase().endsWith(".dia")) {
                // make sure the extension is .dia
                f = new File(f.getParentFile(), f.getName() + ".dia");
            }
            if (!f.exists() || showConfirmDialog(frame, "Overwrite existing file?") == YES_OPTION) {
                VirtualFile vf = saveToFile(f);
                loadVirtualFile(vf);
            }
        }
    }

    private void loadVirtualFile(VirtualFile vf) {
        if (null != vf) {
            FileEditorManager fem = FileEditorManager.getInstance(IDEAUtils.getProject());
            File currentFile = getFile();
            LocalFileSystem fm = LocalFileSystem.getInstance();
            if (currentFile != null) {
                VirtualFile cvf = fm.findFileByIoFile(currentFile);
                fem.closeFile(cvf);
            }
            fem.openFile(vf, true);
        }
    }


}
