package dbhelp.plugin.idea.utils;

import bean.PropertyChangeMultiCaster;
import com.intellij.ide.DataManager;
import com.intellij.openapi.actionSystem.DataConstants;
import com.intellij.openapi.actionSystem.DataContext;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.command.CommandProcessor;
import com.intellij.openapi.editor.CaretModel;
import com.intellij.openapi.editor.Document;
import com.intellij.openapi.editor.Editor;
import com.intellij.openapi.fileEditor.FileDocumentManager;
import com.intellij.openapi.fileEditor.FileEditor;
import com.intellij.openapi.fileEditor.FileEditorManager;
import com.intellij.openapi.fileTypes.FileType;
import com.intellij.openapi.fileTypes.SyntaxHighlighter;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.project.ProjectManager;
import com.intellij.openapi.vfs.LocalFileSystem;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.wm.ToolWindow;
import com.intellij.openapi.wm.ToolWindowManager;
import com.intellij.openapi.wm.WindowManager;
import dbhelp.db.Database;
import dbhelp.plugin.complete.utils.CompleterItem;
import dbhelp.plugin.idea.SQLFileTypeSupport;
import dbhelp.plugin.idea.complete.CompleteWindow;
import dbhelp.plugin.ui.ChooseDatabaseDialog;
import utils.Config;
import utils.Utils;
import utils.text.WordScanner;

import javax.swing.*;
import java.awt.*;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Reader;
import java.lang.reflect.Method;
import java.util.concurrent.CountDownLatch;

/**
 * User: huojun
 * Date: 9-Oct-2006
 * Time: 6:04:48 PM
 */
public class IDEAUtils {
    private static Project getCurrentProject() {
        WindowManager wm = WindowManager.getInstance();
        Project[] projects = ProjectManager.getInstance().getOpenProjects();
        for (Project p : projects) {
            Window aw = javax.swing.FocusManager.getCurrentManager().getActiveWindow();
            Window b = wm.suggestParentWindow(p);
            if (SwingUtilities.isDescendingFrom(aw, b)
                    || aw.equals(b)) {
                return p;
            }
        }
        return null;
    }

    public static void showToolWindow(final String id) {
        Runnable runnable = new Runnable() {
            public void run() {
                ToolWindowManager twm = ToolWindowManager.getInstance(getProject());
                ToolWindow me = twm.getToolWindow(id);
                me.show(new Runnable() {
                    public void run() {
                    }
                });
            }
        };
        SwingUtilities.invokeLater(runnable);
    }

    public static Project getProject() {
        DataContext dc = DataManager.getInstance().getDataContext();
        Project project = (Project) dc.getData(DataConstants.PROJECT);
        if (null != project) {
            return project;
        } else {
            project = getCurrentProject();
            if (null == project) {
                Utils.debug("Can't find the current project!");
            }
        }
        return project;
    }

    public static void completeWord(final CompleterItem[] items) {
        final Editor editor = getActiveEditor();
        final Document document = editor.getDocument();
        //final String w = CompleteWindow.getInstance().getFilter();
        CharSequence word = getCompletingWord(editor);
        String w = null == word ? null : word.toString();
        final int deleteLen = null == w ? 0 : w.length();
        final String prefix = null != w && w.endsWith(".") ? w : "";
        CompleteWindow.getInstance().itemSelected();
        ApplicationManager.getApplication().runWriteAction(new Runnable() {
            public void run() {
                CommandProcessor.getInstance().executeCommand(editor.getProject(), new Runnable() {
                    public void run() {
                        CaretModel caretModel = editor.getCaretModel();
                        int offset = caretModel.getOffset();
                        CharSequence cs = toCompleteString(items);
                        document.replaceString(offset - deleteLen, offset, cs);
                        caretModel.moveToOffset(offset + cs.length() - deleteLen);
                    }

                    private CharSequence toCompleteString(CompleterItem[] items) {
                        StringBuilder sb = new StringBuilder();
                        for (int i = 0; null != items && i < items.length; i++) {
                            if (items.length > 1) {
                                // skip star if there are multiple selections
                                if ("*".equals(items[i].getObject())) {
                                    continue;
                                }
                            }
                            if (sb.length() > 0) {
                                sb.append(", ");
                            }
                            sb.append(prefix).append(items[i].toCompleteString());
                        }
                        return sb.toString();
                    }
                }, "complete", this);

            }
        });
    }

    public static Editor getActiveEditor() {
        FileEditorManager instance = FileEditorManager.getInstance(getProject());
//        Utils.debug("selectedTextEditor = " + selectedTextEditor);
        return instance.getSelectedTextEditor();
    }

    public static Database getDatabase(Document doc) {
        Database database = doc.getUserData(Keys.KEY_DATABASE);
        if (null == database) {
            Database[] databases = Config.getConfig().getDatabases();
            if (null != databases && databases.length > 0) {
                database = databases[0];
                doc.putUserData(Keys.KEY_DATABASE, database);
            }
        }
        return database;
    }

    public static File getTempFolder() {
        String homeDir = System.getProperty("user.home");
        File f = new File(homeDir, "dbhelp");
        f = new File(f, "temporary");
        if (!f.exists()) {
            f.mkdirs();
        }
        return f;
    }

    public static VirtualFile createTempVirtualFile(String name, Reader data) throws Exception {
        return createVirtualFile(new File(getTempFolder(), name), data);
    }

    public static VirtualFile createVirtualFile(File name, Reader data) throws InterruptedException {
        CountDownLatch c = new CountDownLatch(1);
        MyRunnable myRunnable = new MyRunnable(name, data, c);
        ApplicationManager.getApplication().runWriteAction(myRunnable);
        c.await();
        return myRunnable.getVirtualFile();
    }

    public static FileEditor[] openEditor(VirtualFile vf) {
        Project project = getProject();
        FileEditorManager fem = FileEditorManager.getInstance(project);
        return fem.openFile(vf, true);
    }

    public static CharSequence getCompletingWord(Editor e) {
        Utils.debug("WordScanner.getCompletingWord( editor )");
        int offset = e.getCaretModel().getOffset();
        return WordScanner.getCompletingWord(e.getDocument().getCharsSequence(), offset - 1);
    }

    public static void setDatabase(Database db, Document document) {
        Database database = getDatabase(document);
        document.putUserData(Keys.KEY_DATABASE, db);
        PropertyChangeMultiCaster.getInstance().fireChangeEvent(document, "database");
        FileDocumentManager fdm = FileDocumentManager.getInstance();
        VirtualFile file = fdm.getFile(document);
        FileType fileType = file.getFileType();
        SyntaxHighlighter h = null;
        try {
            Method getHighlighterMethod = FileType.class.getMethod("getHighlighter", Project.class, VirtualFile.class);
            Utils.debug("IDEAUtils.setDatabase under IDEA 6.x");
            h = (SyntaxHighlighter) getHighlighterMethod.invoke(fileType, getProject(), file);
        } catch (Exception e) {
            Utils.debug("IDEAUtils.setDatabase under IDEA 5.x");
            try {
                Method getHighlighterMethod = FileType.class.getMethod("getHighlighter", Project.class);
                h = (SyntaxHighlighter) getHighlighterMethod.invoke(fileType, getProject());
            } catch (Exception e1) {
                Utils.debug("IDEAUtils.setDatabase: " + e);
            }
        }
        if (null != h) {
            SQLFileTypeSupport.SQLLexer lexer = (SQLFileTypeSupport.SQLLexer) h.getHighlightingLexer();
            lexer.setDatabase(database);
            toggleDocument(document);
        }
    }

    private static void toggleDocument(final Document document) {
        ApplicationManager.getApplication().runWriteAction(new Runnable() {
            public void run() {
                CommandProcessor.getInstance().executeCommand(getProject(), new Runnable() {
                    public void run() {
                        CharSequence text = document.getCharsSequence();
                        document.replaceString(0, text.length(), "");
                        document.replaceString(0, 0, text);
                    }
                }, "toggle", this);
            }
        });
    }

    public static boolean chooseDatabase(Document doc) {
        Database[] databases = Config.getConfig().getDatabases();
        if (databases != null && databases.length > 0) {
            ChooseDatabaseDialog d = new ChooseDatabaseDialog((Frame) Utils.getFrame());
            d.setVisible(true);
            if (d.isConfirmed()) {
                setDatabase(d.getDatabase(), doc);
                return true;
            }
        } else {
            Utils.message("No database has been defined!");
            return false;
        }
        return false;
    }

    private static class MyRunnable implements Runnable {
        private Reader _data;
        private File _file;
        private VirtualFile _virtualFile;
        private CountDownLatch _lock;

        public MyRunnable(File file, Reader data, CountDownLatch lock) {
            _file = file;
            _data = data;
            _lock = lock;
        }

        public MyRunnable(File folder, String name, Reader data, CountDownLatch lock) {
            _lock = lock;
            _data = data;
            _file = new File(folder, name);
        }

        public File getFile() {
            return _file;
        }

        public VirtualFile getVirtualFile() {
            return _virtualFile;
        }

        public void run() {
            try {
                FileWriter fw = new FileWriter(_file);
                if (_data != null) {
                    char[] buf = new char[1024];
                    int chars = _data.read(buf);
                    while (chars != -1) {
                        fw.write(buf, 0, chars);
                        chars = _data.read(buf);
                    }
                }
                fw.close();
//                Utils.debug("_file = " + _file);
                LocalFileSystem fs = LocalFileSystem.getInstance();
                _virtualFile = fs.refreshAndFindFileByIoFile(_file);
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                _lock.countDown();
            }
        }
    }
}
