package dbhelp.plugin.ui;

import dbhelp.db.Database;
import utils.text.IWordEnumeration;
import static utils.text.IWordEnumeration.TYPE.*;
import utils.text.WordScanner;
import utils.text.editor.TokenLine;
import utils.text.editor.Token;
import utils.text.editor.RenderAttribute;

import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.*;
import java.util.List;

/**
 * User: huojun
 * Date: 14-Oct-2006
 * Time: 9:21:19 AM
 */
public class ColorTextViewer extends JComponent {
    protected List<TokenLine> _lines = new ArrayList<TokenLine>();
    protected Set<Integer> _markedLines = new HashSet<Integer>();
    protected static BufferedImage DUMMY_IMAGE = new BufferedImage(2, 2, BufferedImage.TYPE_INT_ARGB);
    protected int _lineHeight;
    protected Color _markColor;
    protected Font _markFont;
    protected Font _font = new Font( null, Font.PLAIN, 12 );
    protected int _tabSize = 4;
    protected Set<String> _keywords = new HashSet<String>();
    protected Map<IWordEnumeration.TYPE, RenderAttribute> _attributes = new HashMap<IWordEnumeration.TYPE, RenderAttribute>();

    public static void main(String[] args) throws IOException {
        uiTest();
//        String s = "\nabcdef\nxyz\n";
//        String[] strings = s.split("\\n");
//        for (int i = 0; null != strings && i < strings.length; i++) {
//            String string = strings[i];
//            System.out.println("string = " + string);
//        }
    }

    private static void uiTest() throws IOException {
        JFrame f = new JFrame();
        Container c = f.getContentPane();
        c.setLayout(new BorderLayout());
        ColorTextViewer viewer = new ColorTextViewer();
        JScrollPane jsp = new JScrollPane(viewer);
        c.add(jsp, BorderLayout.CENTER);
        jsp.getViewport().setBackground(Color.white);
        f.setSize(800, 600);
        f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        f.setVisible(true);
        String text = "/*SELECT * FROM \n" +
                "     employee */\n " +
                "where \"employee\".emp_id > 400";
        viewer.setText(text);
//        viewer.setText(Utils.getFileText("c:\\temp\\reload.sql"));
        viewer.markLines(new int[]{2, 10, 15, 20});
        viewer.addKeywords( Database.SQL92_KEYWORD_LIST );
    }

    public void addKeywords(Set<String> keywords) {
        _keywords.addAll(keywords);
    }

    public ColorTextViewer() {
        Font f = getFont();
        addRenderAttribute( new RenderAttribute(KEYWORD, f.deriveFont(Font.BOLD), Color.blue) );
        addRenderAttribute( new RenderAttribute(STRING, f.deriveFont(Font.BOLD), new Color(92,151,89)) );
        addRenderAttribute( new RenderAttribute(MULTILINE_COMMENT, f.deriveFont(Font.BOLD), Color.gray) );
        addRenderAttribute( new RenderAttribute(SINGLELINE_COMMENT, f.deriveFont(Font.BOLD), Color.gray) );
        addRenderAttribute( new RenderAttribute(OBJECT, f.deriveFont(Font.BOLD), Color.red) );
    }


    public int getTabSize() {
        return _tabSize;
    }

    public void setTabSize(int tabSize) {
        _tabSize = tabSize;
    }

    public Font getFont() {
        return _font;
    }

    public void setFont(Font font) {
        _font = font;
    }

    public void addRenderAttribute(RenderAttribute a) {
        _attributes.put( a.getType(), a );
    }

    public void addKeywords(String[] keywords) {
        _keywords.addAll(Arrays.asList(keywords));
        repaint();
    }

    public Font getMarkFont() {
        return _markFont;
    }

    public void setMarkFont(Font markFont) {
        _markFont = markFont;
    }

    public Color getMarkColor() {
        return _markColor == null ? Color.magenta : _markColor;
    }

    public void setMarkColor(Color markColor) {
        _markColor = markColor;
    }

    public void markLines(int[] lines) {
        for (int i = 0; null != lines && i < lines.length; i++) {
            int line = lines[i];
            markLine(line);
        }
    }

    public void clearMarks() {
        _markedLines.clear();
    }

    public void markLine(int line) {
        if (line >= 0 && line < _lines.size()) {
            _markedLines.add(line);
        }
    }

    public void setText(String text) {
        Graphics g = DUMMY_IMAGE.createGraphics();
        FontMetrics fm = g.getFontMetrics(getFont());
        _lines.clear();
        int maxWidth = parse(text, fm);
        _lineHeight = fm.getHeight();
        Dimension size = new Dimension(maxWidth, _lineHeight * _lines.size());
        setPreferredSize(size);
        revalidate();
        clearMarks();
        repaint();
    }

    private int parse(String text, FontMetrics fm) {
        int maxWidth = 0;
        WordScanner ws = new WordScanner(text);
        IWordEnumeration e = ws.scan(0);
        List<Token> line = new ArrayList<Token>();
        while (e.hasMoreElements()) {
            CharSequence tokenText = e.nextElement();
            IWordEnumeration.TYPE type = e.getType();            
            if (type == NEWLINE) {
                maxWidth = Math.max(addParsedLine(line, fm), maxWidth);
            } else if (type == MULTILINE_COMMENT || type == STRING) {
                String literal = tokenText.toString();
                String[] strings = literal.split("\\n");
                if (strings.length > 0) {
                    line.add(new Token(strings[0], type));
                    if ( strings.length > 1 ) {
                        maxWidth = Math.max(addParsedLine(line, fm), maxWidth);
                        for (int i = 1; i < strings.length; i++) {
                            line.add(new Token(strings[i], type));
                            if (i + 1 < strings.length) {
                                maxWidth = Math.max(addParsedLine(line, fm), maxWidth);
                            }
                        }
                    }
                }
            } else {
                Token t = new Token(tokenText, type);
                line.add(t);
            }
        }
        if( line.size() > 0 ) {
            maxWidth = Math.max(addParsedLine( line, fm), maxWidth);
        }
        return maxWidth;
    }

    private int addParsedLine(List<Token> line, FontMetrics fm) {
        TokenLine tl = new TokenLine(line.toArray(new Token[line.size()]));
        StringBuilder sb = new StringBuilder();
        for(Token t : tl.tokens) {
            sb.append( t.text );
        }
        _lines.add(tl);
        line.clear();
        return fm.stringWidth(sb.toString());
    }

    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        Rectangle vr = getVisibleRect();
        _markFont = _markFont == null ? getFont().deriveFont(Font.BOLD | Font.ITALIC) : _markFont;
        for (int line = 0; line < _lines.size(); line++) {
            int y = line * _lineHeight;
            if (y + _lineHeight > vr.getY() && y < vr.getY() + vr.getHeight()) {
                boolean b = _markedLines.contains(line);
                Color c = b ? getMarkColor() : Color.black;
                Font f = b ? _markFont : getFont();
                TokenLine tokenLine = _lines.get(line);
                int lineY = y + _lineHeight;
                int x = 0;
                for (int i = 0; null != tokenLine.tokens && i < tokenLine.tokens.length; i++) {
                    Token token = tokenLine.tokens[i];
                    String str = token.text.toString();
                    if (token.type.equals(WHITESPACES)) {
                        str = expandTab(str, _tabSize);
                    }
                    if( !b ) {
                        if( token.type == WORD && _keywords.contains(str.toUpperCase() ) ) {
                            token.type = KEYWORD;
                        }
                        RenderAttribute attribute = _attributes.get(token.type);
                        if ( null != attribute ) {
                            c = attribute.getColor();
                            f = attribute.getFont();
                        } else {
                            c = Color.black;
                            f = getFont();
                        }
                    }
                    g.setFont(f);
                    g.setColor(c);
                    g.drawString(str, x, lineY);
                    x += g.getFontMetrics().stringWidth(str);
                }
            }
        }
    }

    private String expandTab(CharSequence str, int tabSize) {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < str.length(); i++) {
            if (str.charAt(i) == '\t') {
                for (int n = 0; n < tabSize; n++) {
                    sb.append(' ');
                }
            } else {
                sb.append(' ');
            }
        }
        return sb.toString();
    }


}
