package dbhelp.plugin.ui;

import dbhelp.db.Database;
import dbhelp.db.ForeignKey;
import dbhelp.db.IDbMeta;
import dbhelp.db.Table;
import diagram.DiagramViewer;
import diagram.shape.*;
import diagram.shape.database.Reference;
import diagram.shape.database.TableBox;
import diagram.shape.database.TableBox2;
import utils.Utils;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.List;


/**
 * User: huojun
 * Date: 10-Oct-2006
 * Time: 5:02:57 PM
 */
public class DatabaseDiagram extends JScrollPane {
    private Database _database;
    protected DBDiagramViewer _diagram = new DBDiagramViewer(this);
    private File _file;

    public DatabaseDiagram() {
        setViewportView(_diagram);
        getVerticalScrollBar().setUnitIncrement(20);
    }

    public DBDiagramViewer getDiagram() {
        return _diagram;
    }

    public Database getDatabase() {
        return _database;
    }

    public File getFile() {
        return _file;
    }

    public void setFile(File file) {
        _file = file;
    }

    public void load() throws Exception {
    }

    public AbstractShape addTable(Table t) {
        if (!_diagram.containsKey(t)) {
            TableBox value = new TableBox2(t);
            _diagram.addShape(value);
            try {
                addReferences(value, t);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return getTableShape(t);
    }

    private AbstractShape getTableShape(Table t) {
        return (AbstractShape) _diagram.get(t);
    }

    private void addReferences(AbstractShape s, Table t) throws Exception {
        Database db = t.getDatabase();
        IDbMeta m = db.getMetaData();
        ForeignKey[] foreignKeys = t.getForeignKeys();
        for (ForeignKey fk : foreignKeys) {
            List<Table> tables = m.getTables(fk.getPkTableCatalog(), fk.getPkTableSchema(), null, fk.getPkTableName());
            Table t1 = tables.get(0);
            if (_diagram.containsKey(t1)) {
                AbstractShape ts = getTableShape(t1);
                addArrowForReference(s, ts, fk, Arrow.END_ARROW);
            }
        }
        Table[] existingsTables = getExistingsTables();
        for (Table table : existingsTables) {
            if (!table.equals(t)) {
                ForeignKey[] fks = table.getForeignKeys();
                for (ForeignKey fk : fks) {
                    String pkTableName = fk.getPkTableName();
                    if (pkTableName.equals(t.getName())) {
                        List<Table> tables = m.getTables(null, null, null, pkTableName);
                        Table pkTable = tables.get(0);
                        if (_diagram.containsKey(pkTable)) {
                            AbstractShape s1 = getTableShape(table);
                            addArrowForReference(s, s1, fk, Arrow.BEGIN_ARROW);
                        }
                    }
                }
            }
        }
    }

    private Table[] getExistingsTables() {
        ArrayList<Table> a = new ArrayList<Table>();
        for (Object o : _diagram.keySet()) {
            if (o instanceof Table) {
                Table t = (Table) o;
                a.add(t);
            }
        }
        return a.toArray(new Table[a.size()]);
    }

    public void fixReferences() {
        DBDiagramViewer d = getDiagram();
        List<AbstractShape> arrows = d.getShape(Reference.class);
        for (AbstractShape s : arrows) {
            Reference a = (Reference) s;
            ForeignKey fk = (ForeignKey) a.getObject();
            Table t1 = new Table(fk.getPkTableCatalog(), fk.getPkTableSchema(), fk.getPkTableName());
            Table t2 = new Table(fk.getFkTableCatalog(), fk.getFkTableSchema(), fk.getFkTableName());
            AbstractShape tb1 = d.getShape(t1);
            AbstractShape tb2 = d.getShape(t2);
            TextLabel label = (TextLabel) d.getShape(fk.toString());
            Link link = new FuzzyLink(a, tb1, tb2);
            link.setText(label);
            _diagram.put(fk, link);
        }
    }

    private void addArrowForReference(AbstractShape s, AbstractShape s1, ForeignKey fk, byte style) {
        Reference a = new Reference();
        TextLabel b = new TextLabel(fk.toString());
        a.setEndStyle(style);
        _diagram.addShape(a);
        _diagram.addShape(b);
        Link link = new FuzzyLink(a, s, s1);
        link.setText(b);
        a.setObject(fk);
        _diagram.put(fk, link);
        a.moveToBack();
    }

    public void sureVisible(Object table) {
        AbstractShape[] shapes = getDiagram().getShapes();
        for (AbstractShape shape : shapes) {
            Object object = shape.getObject();
            if (table.equals(object)) {
                _diagram.scrollRectToVisible(shape.getBounds());
                shape.setSelected(true);
                return;
            }
        }
    }

    public void setDatabase(Database database) {
        _database = database;
    }

    public static class DBDiagramViewer extends DiagramViewer {
        private DatabaseDiagram _dbView;

        public DBDiagramViewer(DatabaseDiagram dbview) {
            super();
            _dbView = dbview;
        }


        protected void setupMenu() {
            super.setupMenu();
            JMenuItem miSave = new JMenuItem(new ActionSave());
            JMenuItem miSaveAs = new JMenuItem(new ActionSaveAs());
            _popupMenu.add(miSave);
            _popupMenu.add(miSaveAs);
        }

//        public void removeShape(AbstractShape s) {
//            super.removeShape(s);
////            Utils.debug("DBDiagram$MyDiagramViewer.removeShape: " + s);
//            Map<Object, Object> objects = _dbView.getObjects();
//            objects.remove(s.getObject());
//            Object[] keys = objects.keySet().toArray();
//            for (Object key : keys) {
//                if (key instanceof ForeignKey) {
//                    ForeignKey fk = (ForeignKey) key;
//                    Link l = (Link) objects.get(fk);
//                    if (l.getBeginShape().equals(s) || l.getEndShape().equals(s)) {
//                        super.removeShape(l.getArrow());
//                        super.removeShape(l.getText());
//                        objects.remove(key);
//                    }
//                }
//            }
//        }

        public void find() {
            FindShapeDialog.show(this);
        }

        private class ActionSave extends AbstractAction {
            public ActionSave() {
                super("Save diagram");
            }

            public void actionPerformed(ActionEvent e) {
                try {
                    _dbView.save();
                } catch (Exception e1) {
                    Utils.error(e1);
                }
            }
        }

        private class ActionSaveAs extends AbstractAction {

            public ActionSaveAs() {
                super("Save diagram As...");
            }

            public void actionPerformed(ActionEvent e) {
                try {
                    _dbView.saveAs();
                } catch (Exception e1) {
                    Utils.error(e1);
                }
            }
        }
    }

    public void saveAs() throws Exception {

    }

    public void save() throws Exception {
    }
}
