package dbhelp.plugin.ui;

import dbhelp.db.Database;
import dbhelp.db.Table;
import dbhelp.db.model.TablePositioner;
import diagram.shape.AbstractShape;
import diagram.shape.database.TableBox2;
import utils.Config;

import javax.swing.FocusManager;
import javax.swing.*;
import java.awt.*;
import java.util.ArrayList;
import java.util.Arrays;

/**
 * User: huojun
 * Date: 10-Oct-2006
 * Time: 9:16:33 PM
 */
public abstract class DatabaseDiagramPainter {
    protected Table[] _tables;
    protected DatabaseDiagram _diagram;
    protected boolean _autoAddRefTables;

    public DatabaseDiagramPainter(Table[] tables) {
        if (tables != null && tables.length > 0) {
            _tables = tables;
            try {
                _diagram = getDiagram(tables[0].getDatabase());
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
    }

    public static Boolean askPaintOptions() {
        Config cfg = Config.getConfig();
        boolean autoAddRefTables;

        if (cfg.isAskAutoAddRefTables()) {
            Window aw = FocusManager.getCurrentManager().getActiveWindow();
            Object[] options = {"Yes", "Yes and don't ask again", "No", "No and don't ask again"};
            int index = cfg.isAutoAddRefTables() ? 0 : 2;
            Object retObj = JOptionPane.showInputDialog(aw, "Automatically add reference tables?", "Confirm",
                    JOptionPane.PLAIN_MESSAGE, null, options, options[index]);
            if( null == retObj ) {
                return null;
            }
            int ret = Arrays.asList(options).indexOf(retObj);
            autoAddRefTables = ret == 0 || ret == 1;
            if (ret == 1 || ret == 3) {
                cfg.setAskAutoAddRefTables(false);
                cfg.setAutoAddRefTables(autoAddRefTables);
            }
        } else {
            autoAddRefTables = cfg.isAutoAddRefTables();
        }
        return autoAddRefTables;
    }

    public boolean isAutoAddRefTables() {
        return _autoAddRefTables;
    }

    public void setAutoAddRefTables(boolean autoAddRefTables) {
        _autoAddRefTables = autoAddRefTables;
    }

    public DatabaseDiagram getDiagram() {
        return _diagram;
    }

    protected abstract DatabaseDiagram getDiagram(Database database) throws Exception;

    public void layoutTables() {
        final DatabaseDiagram.DBDiagramViewer viewer = getDiagram().getDiagram();
        final Runnable runnable = new Runnable() {
            public void run() {
                try {
                    ProgressMonitor pm = new ProgressMonitor();
                    IProgressReporter reporter = pm.getReporter();
                    pm.start();
                    reporter.setText("Reversing tables...");
                    java.util.List<Table> tables = new ArrayList<Table>();
                    tables.addAll(Arrays.asList(_tables));
                    java.util.List<AbstractShape> list = viewer.getShape(TableBox2.class);
                    for (AbstractShape s : list) {
                        tables.add((Table) s.getObject());
                    }
                    TablePositioner pos = new TablePositioner(tables.toArray(new Table[tables.size()]));
                    pos.setAddRefTables(isAutoAddRefTables());
                    pos.setProgressReporter(reporter);
                    pos.layoutShapes(_diagram);
                } catch (Exception e) {
                    if (!(e instanceof AbortException)) {
                        e.printStackTrace();
                    }
                }
            }
        };
        viewer.removeAllShapes();
        final Runnable runnable1 = new Runnable() {
            public void run() {
                Thread t = new Thread(runnable);
                t.start();
            }
        };
        SwingUtilities.invokeLater(runnable1);
    }
}
