/*
 * DatabaseProfilePanel.java
 *
 * Created on October 27, 2006, 2:30 PM
 */

package dbhelp.plugin.ui;

import bean.editor.*;
import bean.PropertyChangeMultiCaster;
import dbhelp.db.Attribute;
import dbhelp.db.Database;
import dbhelp.db.JDBCDriver;
import utils.Config;
import utils.Utils;
import utils.Icons;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import java.awt.*;
import static java.awt.GridBagConstraints.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.io.File;
import java.sql.Driver;
import java.sql.Connection;
import java.util.ArrayList;

/**
 * @author huojun
 */
public class DatabaseProfilePanel extends javax.swing.JPanel {
    private static String SUN_ODBC_DRIVER = "sun.jdbc.odbc.JdbcOdbcDriver";
    protected JarListModel _jarList;
    private Database _database;
    private JLabel jlURL;
    protected JTextField jtURL;
    protected JDialog _dialog;
    protected DefaultComboBoxModel _classNameModel;
    private ArrayPropertyPanel _extraEditor = new ArrayPropertyPanel();
    protected MyFocusAdapter _selectAllListener = new MyFocusAdapter();
    private ActionAddJars _actionAddJars = new ActionAddJars();
    protected JCheckBox _jcbUseFullName;

    public static void main(String[] args) {
        Database db = new Database(SUN_ODBC_DRIVER, "jdbc:odbc:ASA 9.0 Sample", "dba", "sql");
        Attribute[] attributes = new Attribute[]{new Attribute("charset", "iso1")};
        db.setProperties(attributes);
        db = showDialog(null, db);
        System.out.println("db = " + db);
    }

    public static Database showDialog(Frame w, Database db) {
        JDialog d = new JDialog(w);
        d.setTitle("Database Profile");
        Container cp = d.getContentPane();
        cp.setLayout(new BorderLayout());
        DatabaseProfilePanel pane = new DatabaseProfilePanel();
        pane.setDialog(d);
        pane.setDatabase(db);
        cp.add(pane, BorderLayout.CENTER);
        d.setSize(500, 480);
        d.setLocationRelativeTo(null);
        d.setDefaultCloseOperation(JDialog.EXIT_ON_CLOSE);
        d.setModal(true);
        Utils.addEscClose(d);
        d.setVisible(true);
        d.dispose();
        return pane.getDatabase();
    }

    public Database getDatabase() {
        return _database;
    }


    public JDialog getDialog() {
        return _dialog;
    }

    public void setDialog(JDialog dialog) {
        _dialog = dialog;
    }

    public void setDatabase(Database db) {
        _database = db == null ? new Database() : db;
        jtName.setText(_database.getName());
        jtPassword.setText(_database.getPassword());
        jtUser.setText(_database.getUser());
        jcbAutoCommit.setSelected(_database.isAutoCommit());
        _jcbUseFullName.setSelected(_database.isUseFullObjectName());
        _jarList.removeAllElements();
        FileHolder[] jars = _database.getJars();
        for (int i = 0; null != jars && i < jars.length; i++) {
            FileHolder holder = jars[i];
            File file = holder.getFile();
            if (file != null) {
                _jarList.addElement(file);
            }
        }
        _extraEditor.setValue(_database.getProperties());
        _extraEditor.setReadOnly(false);
        refreshClassNames();
        jcombDriver.setSelectedItem(_database.getDriver());
        jtURL.setText(_database.getUrl());
    }

    /**
     * Creates new form NewDatabaseProfilePanel
     */
    public DatabaseProfilePanel() {
        initComponents();
    }

    /**
     * This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">
    private void initComponents() {
        GridBagConstraints gridBagConstraints;

        jTabs = new JTabbedPane();
        pBasic = new JPanel();
        jlName = new JLabel();
        jtName = new JTextField();
        jlDriver = new JLabel("Driver: ");
        jcombDriver = new JComboBox();
        jlUser = new JLabel();
        jlURL = new JLabel();
        jtUser = new JTextField();
        jlPassword = new JLabel();
        jtPassword = new JPasswordField();
        jcbAutoCommit = new JCheckBox();
        jbConnect = new JButton();
        jPanel3 = new JPanel();
        jlJarList = new JLabel();
        jScrollPane1 = new JScrollPane();
        jlJars = new JList();
        jbAdd = new JButton();
        jbDelete = new JButton();
        jPanel4 = new JPanel();
//        jspExtra = new JScrollPane(_extraEditor);
        jPanel2 = new JPanel();
        jbCancel = new JButton();
        jbSave = new JButton();
        jtURL = new JTextField();

        setLayout(new BorderLayout());

        pBasic.setLayout(new GridBagLayout());

        jlName.setText("Name:");
        jlName.setDisplayedMnemonic('m');
        jlName.setLabelFor(jtName);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = WEST;
        gridBagConstraints.insets = new Insets(2, 2, 2, 2);
        pBasic.add(jlName, gridBagConstraints);

        jtName.setMargin(new Insets(2, 2, 2, 2));
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = HORIZONTAL;
        gridBagConstraints.anchor = WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new Insets(2, 2, 8, 2);
        jtName.addFocusListener(_selectAllListener);
        pBasic.add(jtName, gridBagConstraints);

        jlDriver.setDisplayedMnemonic('r');
        jlDriver.setLabelFor(jcombDriver);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = WEST;
        gridBagConstraints.insets = new Insets(2, 2, 2, 2);
        pBasic.add(jlDriver, gridBagConstraints);

        _classNameModel = new DefaultComboBoxModel();
        jcombDriver.setModel(_classNameModel);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = HORIZONTAL;
        gridBagConstraints.anchor = WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new Insets(2, 2, 8, 2);
        JPanel jpBrowse = new JPanel(new BorderLayout());
        jpBrowse.add(jcombDriver, BorderLayout.CENTER);
        JButton jbAddJars = new ToolbarButton(Icons.getInstance().get(27));
        jbAddJars.setToolTipText("Click to browse JDBC jar files");
        jpBrowse.add(jbAddJars, BorderLayout.EAST);
        jbAddJars.addActionListener(_actionAddJars);

        pBasic.add(jpBrowse, gridBagConstraints);
        jcombDriver.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent actionEvent) {
                Object o = jcombDriver.getSelectedItem();
                if (null != o) {
                    JDBCDriver driver = JDBCDriver.getDriver(o.toString());
                    if (null != driver) {
                        jtURL.setText(driver.getUrlPattern()[0]);
                    } else {
                        jtURL.setText("jdbc:");
                    }
                }
            }
        });

        jlURL.setText("URL:");
        jlURL.setDisplayedMnemonic('L');
        jlURL.setLabelFor(jtURL);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.anchor = WEST;
        gridBagConstraints.insets = new Insets(2, 2, 2, 2);
        pBasic.add(jlURL, gridBagConstraints);

        jtURL.addFocusListener(_selectAllListener);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.fill = HORIZONTAL;
        gridBagConstraints.anchor = WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new Insets(2, 2, 2, 2);
        jtURL.setMargin(new Insets(2, 2, 2, 2));
        pBasic.add(jtURL, gridBagConstraints);

        jlUser.setText("User:");
        jlUser.setDisplayedMnemonic('U');
        jlUser.setLabelFor(jtUser);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.anchor = WEST;
        gridBagConstraints.insets = new Insets(2, 2, 2, 2);
        pBasic.add(jlUser, gridBagConstraints);

        jtUser.setMargin(new Insets(2, 2, 2, 2));
        jtUser.addFocusListener(_selectAllListener);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 7;
        gridBagConstraints.fill = HORIZONTAL;
        gridBagConstraints.anchor = WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new Insets(2, 2, 8, 2);
        pBasic.add(jtUser, gridBagConstraints);

        jlPassword.setText("Password:");
        jlPassword.setDisplayedMnemonic('P');
        jlPassword.setLabelFor(jtPassword);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 8;
        gridBagConstraints.anchor = WEST;
        gridBagConstraints.insets = new Insets(2, 2, 2, 2);
        pBasic.add(jlPassword, gridBagConstraints);

        jtPassword.setMargin(new Insets(2, 2, 2, 2));
        jtPassword.addFocusListener(_selectAllListener);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 9;
        gridBagConstraints.fill = HORIZONTAL;
        gridBagConstraints.anchor = WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new Insets(2, 2, 8, 2);
        pBasic.add(jtPassword, gridBagConstraints);

        jcbAutoCommit.setText("Auto Commit");
        jcbAutoCommit.setMnemonic('o');
        JPanel pBooleans = new JPanel(new GridBagLayout());
        _jcbUseFullName = new JCheckBox("Use Full Object Name");
        _jcbUseFullName.setMnemonic('b');
        gridBagConstraints = new GridBagConstraints(0, 0, 1, 1, 1, 0, WEST, NONE, new Insets(2, 2, 2, 2), 1, 1);
        pBooleans.add(jcbAutoCommit, gridBagConstraints);
        gridBagConstraints = new GridBagConstraints(1, 0, 1, 1, 1, 0, EAST, NONE, new Insets(2, 2, 2, 2), 1, 1);
        pBooleans.add(_jcbUseFullName, gridBagConstraints);
        _jcbUseFullName.setToolTipText("Whether DBHelper uses catalog.schema.object or object as table/stored procdure name references" +
                "Some database doesn't support using catalog in SQL statememnt or you defined the connection URL with database name, " +
                "you can checkRead this option to turn if the full name reference");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 10;
        gridBagConstraints.fill = HORIZONTAL;
        gridBagConstraints.anchor = WEST;
        gridBagConstraints.insets = new Insets(2, 2, 2, 2);
        pBasic.add(pBooleans, gridBagConstraints);

        jbConnect.setText("Test Connection");
        jbConnect.setMnemonic('i');
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 11;
        gridBagConstraints.anchor = EAST;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new Insets(2, 2, 2, 2);
        pBasic.add(jbConnect, gridBagConstraints);
        jbConnect.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (verifyInfo()) {
                    try {
                        Connection connection = _database.getConnection();
                        Utils.message("Connection successful");
                        connection.close();
                    } catch (Exception e1) {
                        Utils.error(e1);
                    }
                }
            }
        });

        jTabs.addTab("Connection Settings", pBasic);

        jPanel3.setLayout(new GridBagLayout());

        jlJarList.setDisplayedMnemonic('J');
        jlJarList.setLabelFor(jlJars);
        jlJarList.setText("JDBC Driver Jar Files:");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = WEST;
        gridBagConstraints.insets = new Insets(2, 2, 2, 2);
        jPanel3.add(jlJarList, gridBagConstraints);

        _jarList = new JarListModel();
        jlJars.setModel(_jarList);
        jScrollPane1.setViewportView(jlJars);

        pBasic.setBorder(BorderFactory.createTitledBorder("Basic Settings"));
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.gridheight = 2;
        gridBagConstraints.fill = BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new Insets(2, 2, 2, 2);
        jPanel3.add(jScrollPane1, gridBagConstraints);

        jbAdd.setText("Add");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = HORIZONTAL;
        gridBagConstraints.anchor = WEST;
        gridBagConstraints.insets = new Insets(2, 2, 2, 2);
        jPanel3.add(jbAdd, gridBagConstraints);
        jbAdd.setMnemonic('d');
        jbAdd.addActionListener(_actionAddJars);

        jbDelete.setText("Delete");
        jbDelete.setMnemonic('t');
        jbDelete.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                jbDeleteActionPerformed();
            }
        });

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = HORIZONTAL;
        gridBagConstraints.anchor = NORTHEAST;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new Insets(2, 2, 2, 2);
        jPanel3.add(jbDelete, gridBagConstraints);

        jPanel4.setLayout(new BorderLayout());

        jPanel4.setBorder(BorderFactory.createTitledBorder("Extra Connection Options"));
        jPanel4.add(_extraEditor, BorderLayout.CENTER);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanel3.add(jPanel4, gridBagConstraints);

        jTabs.addTab("Additional", jPanel3);

        add(jTabs, BorderLayout.CENTER);

        jPanel2.setLayout(new GridBagLayout());

        jbCancel.setMnemonic('C');
        jbCancel.setText("Cancel");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new Insets(2, 2, 2, 2);
        jPanel2.add(jbCancel, gridBagConstraints);
        jbCancel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                _database = null;
                getDialog().setVisible(false);
            }
        });

        jbSave.setMnemonic('S');
        jbSave.setText("Save");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = EAST;
        jPanel2.add(jbSave, gridBagConstraints);

        jbSave.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (verifyInfo()) {
                    getDialog().setVisible(false);
                }
            }
        });

        add(jPanel2, BorderLayout.SOUTH);
        _extraEditor.setComponentType(Attribute.class);
        _extraEditor.getTable().setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN);
    }// </editor-fold>

    private void addJars() {
        JFileChooser f = new JFileChooser();
        f.setFileFilter(new FileFilter() {
            public boolean accept(File f) {
                return f.isDirectory() || f.getName().toLowerCase().endsWith(".jar");
            }

            public String getDescription() {
                return "Jar files";
            }
        });
        f.setMultiSelectionEnabled(true);
        if (f.showOpenDialog(getParent()) == JFileChooser.APPROVE_OPTION) {
            File[] files = f.getSelectedFiles();
            for (File i : files) {
                _jarList.addElement(i);
            }
            _database.setJars(_jarList.getJars());
            refreshClassNames();
        }
    }

    private boolean verifyInfo() {
        _database.setUrl(jtURL.getText().trim());
        _database.setUser(jtUser.getText().trim());
        _database.setPassword(String.valueOf(jtPassword.getPassword()));
        _database.setAutoCommit(jcbAutoCommit.isSelected());
        _database.setUseFullObjectName(_jcbUseFullName.isSelected());
        try {
            PropertyChangeMultiCaster.getInstance().set(_database, "name", jtName.getText().trim());
        } catch (Exception e) {
            Utils.debug("DatabaseProfilePanel.verifyInfo: " + e);
        }
        Object o = _classNameModel.getSelectedItem();
        Attribute[] properties = (Attribute[]) _extraEditor.getValue();
        _database.setProperties(properties);
        _database.setDriver(null == o ? null : o.toString());
        try {
            Utils.sureValue("Name", _database.getName());
            Utils.sureValue("User", _database.getUser());
            Utils.sureValue("URL", _database.getUrl());
            Utils.sureValue("Driver", _database.getDriver());
        } catch (Exception e) {
            Utils.error(e);
            return false;
        }
        return true;
    }

    private void refreshClassNames() {
        String[] classes;
        FileHolder[] list = Utils.getJarList(_database.getJars(), Config.getConfig().getExtensionJars());
        classes = Utils.searchClassInJars(list, Driver.class);
        _classNameModel.removeAllElements();
        JDBCDriver driver = null;
        for (String s : classes) {
            _classNameModel.addElement(s);
            JDBCDriver jdbcDriver = JDBCDriver.getDriver(s);
            if (driver == null) {
                driver = jdbcDriver;
            }
        }
        _classNameModel.addElement(SUN_ODBC_DRIVER);
        driver = driver == null ? JDBCDriver.getDriver(SUN_ODBC_DRIVER) : driver;
        jcombDriver.setSelectedItem(driver.getClassName());
        jtURL.setText(driver.getUrlPattern()[0]);
    }


    private void jbDeleteActionPerformed() {
        Object[] objects = jlJars.getSelectedValues();
        for (int i = 0; null != objects && i < objects.length; i++) {
            Object object = objects[i];
            _jarList.removeElement(object);
        }
        _database.setJars(_jarList.getJars());
        refreshClassNames();
    }


    // Variables declaration - do not modify
    private JButton jbConnect;
    private JLabel jlJarList;
    private JPanel pBasic;
    private JPanel jPanel2;
    private JPanel jPanel3;
    private JPanel jPanel4;
    private JScrollPane jScrollPane1;
    private JTabbedPane jTabs;
    private JTextField jtUser;
    private JButton jbAdd;
    private JButton jbCancel;
    private JButton jbDelete;
    private JButton jbSave;
    private JCheckBox jcbAutoCommit;
    private JComboBox jcombDriver;
    private JLabel jlDriver;
    private JList jlJars;
    private JLabel jlName;
    private JLabel jlPassword;
    private JLabel jlUser;
    //    private JScrollPane jspExtra;
    private JTextField jtName;
    private JPasswordField jtPassword;
    // End of variables declaration

    private static class JarListModel extends DefaultListModel {
        public FileHolder[] getJars() {
            java.util.List<FileHolder> files = new ArrayList<FileHolder>();
            for (int i = 0; i < getSize(); i++) {
                FileHolder f = new FileHolder(getElementAt(i).toString());
                files.add(f);
            }
            return files.toArray(new FileHolder[getSize()]);
        }
    }

    private static class MyFocusAdapter extends FocusAdapter {
        public void focusGained(FocusEvent e) {
            JTextField f = (JTextField) e.getSource();
            f.selectAll();
        }
    }

    private class ActionAddJars implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            addJars();
        }
    }
}
