package dbhelp.plugin.ui;

import diagram.DiagramViewer;
import diagram.shape.AbstractShape;
import diagram.shape.FuzzyLink;

import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.*;

import utils.Utils;
import dbhelp.db.IDBObject;

/**
 * User: huojun
 * Date: 10-Oct-2006
 * Time: 9:55:44 AM
 */
public class FindShapeDialog extends QuickCloseDialog {
    private ShapeModel _model = new ShapeModel();
    private DiagramViewer _diagram;

    public FindShapeDialog(Frame owner) throws HeadlessException {
        super(owner);
        setTitle("Search object in diagram");
        Container cp = getContentPane();
        cp.setLayout(new BorderLayout());
        SearchListPanel searchPane = new SearchListPanel();
        searchPane.setModel(_model);
        final JTable table = searchPane.getTable();
        table.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        table.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() == 2) {
                    int row = table.getSelectedRow();
                    if (row >= 0) {
                        Object valueAt = table.getValueAt(row, 0);
                        Object o = _diagram.get(valueAt);
                        AbstractShape s = null;
                        if (o instanceof AbstractShape) {
                            s = ((AbstractShape) o);
                        } else if( o instanceof FuzzyLink ) {
                            s = ((FuzzyLink)o).getText();
                        }
                        if (null != s) {
                            s.setSelected(true);
                            _diagram.scrollRectToVisible(s.getBounds());
                            dispose();
                        }
                    }
                }
            }
        });
        cp.add(searchPane, BorderLayout.CENTER);
        setModal(true);
        setSize(640, 480);
        setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        setLocationRelativeTo(null);
    }

    public static void show(DiagramViewer diagram) {
        FindShapeDialog fd = new FindShapeDialog((Frame) Utils.getFrame());
        fd.setDiagram(diagram);
        fd.setVisible(true);
    }

    private void setDiagram(DiagramViewer diagram) {
        _diagram = diagram;
        _model.setShapes(diagram.getShapes());
    }

    private static class ShapeModel extends AbstractTableModel {
        private java.util.List<Object> _data = new ArrayList<Object>();

        public void setShapes(AbstractShape[] shapes) {
            _data.clear();
            for (AbstractShape shape : shapes) {
                Object object = shape.getObject();
                if (object instanceof IDBObject) {
                    _data.add(object);
                }
            }
        }

        public int getRowCount() {
            return _data.size();
        }

        public int getColumnCount() {
            return 2;
        }

        public String getColumnName(int column) {
            return column == 0 ? "Object" : "Description";
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            Object o = _data.get(rowIndex);
            return columnIndex == 0 ? o : o.getClass().getSimpleName();
        }
    }
}
