package dbhelp.plugin.ui;

import javax.swing.FocusManager;
import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Timer;
import java.util.TimerTask;

/**
 * Created by IntelliJ IDEA.
 * User: huojun
 * Date: 1-Mar-2006
 * Time: 1:28:30 PM
 */
public class ProgressMonitor {
    private Timer _timer;
    private String _title = "Please wait";
    private IProgressReporter _reporter;
    private int _initDelay = 100;

    public String getTitle() {
        return _title;
    }

    public void setTitle(String title) {
        _title = title;
    }

    public IProgressReporter getReporter() {
        if (null == _reporter) {
            Window aw = FocusManager.getCurrentManager().getActiveWindow();
            _reporter = new MyProgressReporter((Frame) aw, _title);
        }
        return _reporter;
    }

    public void setReporter(IProgressReporter reporter) {
        _reporter = reporter;
    }

    public int getInitDelay() {
        return _initDelay;
    }

    public void setInitDelay(int initDelay) {
        _initDelay = initDelay;
    }

    public void start() {
        if (null != _timer) {
            _timer.cancel();
            _timer = null;
        }
        _timer = new Timer();
        _timer.schedule(new TimerTask() {
            public void run() {
                getReporter().setVisible(true);
            }
        }, getInitDelay());
    }

    private static class MyProgressReporter extends JDialog implements IProgressReporter {
        protected JLabel _statusText = new JLabel();
        protected JProgressBar _bar = new JProgressBar();
        protected JButton _cbCancel = new JButton("Cancel");
        protected IProgressor _progressor;
        protected boolean _cancelled = false;
        protected boolean _done = false;

        public MyProgressReporter(Frame owner, String title) throws HeadlessException {
            super(owner, title);
            setLayout(new GridBagLayout());
            GridBagConstraints g;
            Insets insets = new Insets(6, 8, 6, 8);
            g = new GridBagConstraints(0, 0, 1, 1, 1, 0, GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL, insets, 0, 0);
            add(_statusText, g);
            g = new GridBagConstraints(0, 1, 1, 1, 1, 0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, insets, 0, 0);
            add(_bar, g);
            g = new GridBagConstraints(0, 2, 1, 1, 0, 0, GridBagConstraints.SOUTH, GridBagConstraints.NONE, new Insets(10, 8, 4, 8), 0, 0);
            add(_cbCancel, g);
            setSize(250, 120);
            setModal(true);
            setLocationRelativeTo(null);
            _cbCancel.setMnemonic('C');
            _cbCancel.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    _cancelled = !_done;
                    setVisible(false);
                    dispose();
                }
            });
        }

        public void setMaxValue(int value) {
            _bar.setMaximum(value);
        }

        public void setMinValue(int value) {
            _bar.setMinimum(value);
        }

        public void setStatusObject(Object object) {
        }

        public boolean isCancelled() {
            return _cancelled;
        }

        public void setText(String text) {
            _statusText.setText(text);
        }

        public void setValue(int value) {
            _bar.setValue(value);
        }

        public void done(final boolean keepDialog) {
            _done = true;
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    _bar.setValue(_bar.getMaximum());
                    _cbCancel.setText("Close");
                    if (!keepDialog) {                        
                        setVisible(false);
                        dispose();
                    }
                }
            });
        }
    }

}
