package dbhelp.plugin.ui;

import bean.PropertyChangeMultiCaster;
import bean.editor.ToolbarButton;
import dbhelp.db.Database;
import dbhelp.db.Query;
import dbhelp.db.SQLCommander;
import dbhelp.plugin.OutputPane;
import dbhelp.plugin.QueryEditor;
import utils.Config;
import utils.Utils;
import utils.text.editor.TokenEditor;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableModel;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.IOException;
import java.io.StringReader;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.List;

/**
 * User: huojun
 * Date: 28-Sep-2006
 * Time: 4:27:49 PM
 */
public class QueryManagerPane extends JPanel implements IStayTab {
    protected TokenEditor _textView;
    protected SearchListPanel _searchPane;
    protected JSplitPane _jsp;
    protected TableModel _model;
    protected JPanel _view;
    protected JScrollPane _editorPane;
    protected static final String EDIT_QUERY_ACTION = "edit-query";
    protected JToolBar _toolbar;

    public QueryManagerPane() {
        _model = new MyTableModel();
        initComponents();
        _toolbar = new JToolBar(JToolBar.VERTICAL);
        add(_toolbar, BorderLayout.WEST);

        ToolbarButton b;
        b = new ToolbarButton(18, new ActionNewQuery());
        _toolbar.add(b);
        b = new ToolbarButton(19, new ActionDelete(_searchPane));
        _toolbar.add(b);
        b = new ToolbarButton(28, new ActionRun());
        _toolbar.add(b);        
        ActionEdit actionEdit = new ActionEdit();
        b = new ToolbarButton(15, "Edit the selected query", EDIT_QUERY_ACTION);
        _toolbar.add(b);

        _toolbar.getActionMap().put(EDIT_QUERY_ACTION, actionEdit);
        b = new ToolbarButton(22, new ActionSave());
        _toolbar.add(b);

        JTable table = _searchPane.getTable();
        _textView.setEditable(false);
        table.setDefaultRenderer(Long.class, new TimeRenderer());
        ListSelectionModel sm = table.getSelectionModel();
        sm.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        sm.addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                if (!e.getValueIsAdjusting()) {
                    Query query = getSelectedQuery();
                    if (null != query) {
                        _textView.setText(query.getSQLScript());
                    }
                }
            }
        });
    }

    protected void initComponents() {
        setLayout(new BorderLayout());
        _jsp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        _jsp.setOneTouchExpandable(true);
        add(_jsp, BorderLayout.CENTER);
        _jsp.setDividerLocation(.5);
        _view = new JPanel(new GridBagLayout());
        Dimension zeroSize = new Dimension(0, 0);
        _view.setMinimumSize(zeroSize);
        _view.setPreferredSize(new Dimension(460, 200));

        _searchPane = new SearchListPanel();
        _searchPane.setModel(_model);
        _jsp.setLeftComponent(_searchPane);
        _textView = new TokenEditor();
        _editorPane = new JScrollPane(_textView);
        _editorPane.setMinimumSize(zeroSize);
        _editorPane.setPreferredSize(new Dimension(400, 200));
        _jsp.setRightComponent(_editorPane);
        _textView.addKeywords(Database.SQL92_KEYWORD_LIST);
    }

    private Query getSelectedQuery() {
        JTable table = _searchPane.getTable();
        int rowIndex = table.getSelectedRow();
        if (rowIndex >= 0) {
            return (Query) table.getValueAt(rowIndex, 0);
        } else {
            return null;
        }
    }

    private class TimeRenderer extends DefaultTableCellRenderer {
        private Date _date = new Date();
        private SimpleDateFormat _format = new SimpleDateFormat("yy/MM/dd HH:mm:ss");

        public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
            _date.setTime((Long) value);
            this.setText(_format.format(_date));
            return this;
        }
    }

    private class MyTableModel extends AbstractTableModel {
        private List<Query> _data = new ArrayList<Query>();

        public MyTableModel() {
            Query[] savedQueries = Config.getConfig().getSavedQueries();
            setData(savedQueries);
            PropertyChangeMultiCaster.getInstance().addListener(Config.class, new PropertyChangeListener() {
                public void propertyChange(PropertyChangeEvent evt) {
                    if (evt.getPropertyName().equals("savedQueries")) {
                        Query query = getSelectedQuery();
                        Config c = (Config) evt.getSource();
                        Query[] queries = c.getSavedQueries();
                        setData(queries);
                        for (int i = 0; null != queries && i < queries.length; i++) {
                            Query q = queries[i];
                            if( q.equals(query) ) {
                                _textView.setText(q.getSQLScript());
                                _searchPane.getTable().getSelectionModel().setSelectionInterval(i,i);
                                return;
                            }
                        }
                        _textView.setText("");
                    }
                }
            });
        }

        public void setData(Query[] data) {
            List<Query> l = new ArrayList<Query>();
            if (null != data) {
                l.addAll(Arrays.asList(data));
            }
            setData(l);
        }

        public void setData(List<Query> data) {
            _data = data;
            fireTableStructureChanged();
        }

        public String getColumnName(int column) {
            return column == 0 ? "Name" : "Created Time";
        }

        public boolean isCellEditable(int rowIndex, int columnIndex) {
            return false;
        }

        public int getRowCount() {
            return _data.size();
        }

        public int getColumnCount() {
            return 2;
        }

        public Class<?> getColumnClass(int columnIndex) {
            return columnIndex == 1 ? Long.class : super.getColumnClass(columnIndex);
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            Query query = _data.get(rowIndex);
            return columnIndex == 0 ? query : query.getTimestamp();
        }

    }

    private static class ActionDelete extends AbstractAction {
        private SearchListPanel _searchPane;

        public ActionDelete(SearchListPanel searchPane) {
            super("Delete the selected query");
            _searchPane = searchPane;
        }

        public void actionPerformed(ActionEvent e) {
            JTable table = _searchPane.getTable();
            int row = table.getSelectedRow();
            if (row >= 0) {
                Query query = (Query) table.getValueAt(row, 0);
                int ret = JOptionPane.showConfirmDialog(Utils.getFrame(), "Are you sure to delete query: " + query, "Confirm", JOptionPane.YES_NO_OPTION);
                if (ret == 0) {
                    Config cfg = Config.getConfig();
                    Query[] savedQueries = cfg.getSavedQueries();
                    List<Query> list = new ArrayList<Query>();
                    for (int i = 0; null != savedQueries && i < savedQueries.length; i++) {
                        Query savedQuery = savedQueries[i];
                        if (!savedQuery.equals(query)) {
                            list.add(savedQuery);
                        }
                    }
                    try {
                        PropertyChangeMultiCaster.getInstance().set(cfg, "savedQueries", list.toArray(new Query[list.size()]));
                    } catch (Exception e1) {
                        Utils.error(e1);
                    }
                }
            } else {
                Utils.message("Select a query first!");
            }
        }
    }

    private class ActionEdit extends AbstractAction {

        public void actionPerformed(ActionEvent e) {
            Query query = (Query) (e.getSource() instanceof Query ? e.getSource() : getSelectedQuery());
            if (null != query) {
                OutputPane outputPane = OutputPane.getOutputPane();
                Component[] components = outputPane.getComponents();
                for( Component c : components ) {
                    if( c instanceof QueryEditor ) {
                        QueryEditor qe = (QueryEditor) c;
                        if( qe.getQuery().equals(query) ) {
                            outputPane.setSelectedComponent(c);
                            return;
                        }
                    }
                }
                QueryEditor c = new QueryEditor();
                c.setQuery(query);
                outputPane.addTab(c.getTitle(), c);
                outputPane.setSelectedComponent(c);
            } else {
                Utils.message("Select a query first!");
            }
        }
    }

    private class ActionNewQuery extends AbstractAction {

        public ActionNewQuery() {
            super("Create a new query");
        }

        public void actionPerformed(ActionEvent e) {
            String name = JOptionPane.showInputDialog("Query Name");
            if (Utils.hasValue(name)) {
                Config cfg = Config.getConfig();
                Query[] savedQueries = cfg.getSavedQueries();
                Collection<Query> queries = new ArrayList<Query>();
                if (null != savedQueries) {
                    queries.addAll(Arrays.asList(savedQueries));
                }
                Query newQuery = new Query();
                newQuery.setName(name);
                queries.add(newQuery);
                Query[] value = queries.toArray(new Query[queries.size()]);
                try {
                    PropertyChangeMultiCaster.getInstance().set(cfg, "savedQueries", value);
                    e.setSource(newQuery);
                    _toolbar.getActionMap().get(EDIT_QUERY_ACTION).actionPerformed(e);
                } catch (Exception e1) {
                    Utils.error(e1);
                }
            }
        }
    }

    private class ActionSave extends AbstractAction {

        public ActionSave() {
            super("Export the selected query");
        }

        public void actionPerformed(ActionEvent e) {
            Query query = getSelectedQuery();
            if (null != query) {
                JFileChooser f = new JFileChooser();
                f.setDialogTitle("Choose File");
                f.setSelectedFile(new File(query.getName() + ".sql"));
                if (f.showSaveDialog(Utils.getFrame()) == JFileChooser.APPROVE_OPTION) {
                    try {
                        Utils.createFile(f.getSelectedFile(), new StringReader(query.getSQLScript()));
                    } catch (IOException e1) {
                        Utils.error(e1);
                    }
                }
            } else {
                Utils.message("Select a query first!");
            }
        }
    }

    private class ActionRun extends AbstractAction {
        public ActionRun() {
            super("Run this query");
        }

        public void actionPerformed(ActionEvent e) {
            Database[] databases = Config.getConfig().getDatabases();
            Query q = getSelectedQuery();
            if (null != q) {
                for (int i = 0; null != databases && i < databases.length; i++) {
                    if (databases[i].getName().equals(q.getDatabaseName())) {
                        if (databases[i].isConnected()) {
                            SQLCommander.runSQLCommand(databases[i], q.getSQLScript(), null, OutputPane.getOutputPane());
                            return;
                        } else {
                            Utils.message("Please connect database: " + q.getDatabaseName());
                            return;
                        }
                    }
                }
                Utils.message("This query has an invalid database name or it isn't specified!");
            } else {
                Utils.message("Select the saved query first!");
            }
        }
    }
}
