package dbhelp.plugin.ui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

/**
 * Created by IntelliJ IDEA.
 * User: David Huo
 * Date: Sep 19, 2006
 * Time: 8:22:51 PM
 */
public class QuickCloseDialog extends JDialog {
    //  Handle escape key to close the dialog
    protected KeyStroke escapeKeyStroke = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0, false);
    protected KeyStroke enterKeyStroke = KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0, false);
    private static final String STRING_ESC_ACTION = "ESCAPE";
    private static final String STRING_ENTER_ACTION = "enter";
    protected JButton _defaultButton;
    protected boolean _confirmed;

    public QuickCloseDialog(Frame owner) throws HeadlessException {
        super(owner);
        initComponents();
    }

    public QuickCloseDialog(Dialog owner) throws HeadlessException {
        super(owner);
        initComponents();
    }

    private void initComponents() {
        int whenInFocusedWindow = JComponent.WHEN_IN_FOCUSED_WINDOW;
        JRootPane rootPane = getRootPane();
        rootPane.getInputMap(whenInFocusedWindow).put(escapeKeyStroke, STRING_ESC_ACTION);
        rootPane.getActionMap().put(STRING_ESC_ACTION, new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                escape();
            }
        });
        rootPane.getInputMap(whenInFocusedWindow).put(enterKeyStroke, STRING_ENTER_ACTION);
        rootPane.getActionMap().put(STRING_ENTER_ACTION, new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                enter();
            }
        });
    }

    protected void escape() {
        _confirmed = false;
        setVisible(false);
    }

    public boolean isConfirmed() {
        return _confirmed;
    }

    public void setConfirmed(boolean confirmed) {
        _confirmed = confirmed;
    }

    public JButton getDefaultButton() {
        return _defaultButton;
    }

    public void setDefaultButton(JButton defaultButton) {
        _defaultButton = defaultButton;
        // Although setDefaultButton doesn't seem to work,
        // we still set it so we have the display indicator
        // we use getActionMap instead
        if (null != defaultButton ) {
            getRootPane().setDefaultButton(_defaultButton);
        }
    }

    protected void enter() {
        if (null != _defaultButton) {
            ActionListener[] actionListeners = _defaultButton.getActionListeners();
            for (ActionListener listener : actionListeners) {
                if (null != listener) {
                    listener.actionPerformed(new ActionEvent(this, 0, "defaultButton"));
                }
            }
        }
        _confirmed = true;
    }

    public static QuickCloseDialog newInstance(Component frame) {
        if( frame instanceof Frame ) {
            return new QuickCloseDialog((Frame)frame );
        } else if( frame instanceof Dialog ) {
            return new QuickCloseDialog((Dialog) frame);
        }
        return null;
    }
}