package dbhelp.plugin.ui;

import bean.editor.TableSorter;
import dbhelp.db.Table;
import dbhelp.db.model.FilterColumnModel;
import dbhelp.db.model.ResultModel;
import dbhelp.plugin.action.portable.PopupMouseAdapter;
import utils.Config;
import utils.TablePacker;

import javax.swing.*;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableModel;
import java.awt.*;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseEvent;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

/**
 * Created by IntelliJ IDEA.
 * User: huojun
 * Date: 6-Sep-2005
 * Time: 10:19:38 PM
 */
public class QuickEditTable extends JTable {
    private final MyFocusAdapter _focustListener = new MyFocusAdapter();
    private FilterColumnModel _filter;
    private TableSorter _sorter;
    private TableModel _metaData;
    private boolean _packTable = true;

    public QuickEditTable() {
        setRowHeight(20);
        setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        addMouseListener(new MyPopupMouseAdapter());
        TableSorter sorter = getTableSorter();
        sorter.setTableHeader(getTableHeader());
    }

    public ResultModel getMetaData() {
        TableModel tm = _metaData;
        return (ResultModel) (tm instanceof ResultModel ? tm : null);
    }

    public TableModel getDataModel() {
        return _metaData;
    }

    protected TableSorter getTableSorter() {
        if( null == _sorter ) {
            _sorter = new TableSorter(getFilterColumnModel());
        }
        return _sorter;
    }


    public boolean isPackTable() {
        return _packTable;
    }

    public void setPackTable(boolean packTable) {
        _packTable = packTable;
    }

    public FilterColumnModel getFilterColumnModel() {
        if( null == _filter ) {
            _filter = new FilterColumnModel();
        }
        return _filter;
    }

    public void setModel(TableModel dataModel) {
        super.setModel(getTableSorter());
        if ( null != dataModel ) {
            _metaData = dataModel;
            getFilterColumnModel().setDelegate( dataModel );
            if ( Config.getConfig().isPackResultTable() && isPackTable() ) {
                TablePacker.packAsync(this, 2);
            }
        }        
    }

    public void refresh() throws Exception {
        TableModel m = getMetaData();
        if (m instanceof ResultModel) {
            ResultModel rm = (ResultModel) m;
            Table t = rm.getTable();
            rm = rm.refresh();
            rm.setTable(t);
            setModel(rm);
        }
    }

    public Component prepareEditor(TableCellEditor editor, int row, int column) {
        Component component = super.prepareEditor(editor, row, column);
        if (component instanceof JTextField) {
            JTextField t = (JTextField) component;
            FocusListener[] fls = t.getFocusListeners();
            if (!Arrays.asList(fls).contains(_focustListener)) {
                t.addFocusListener(_focustListener);
            }
        }
        return component;
    }

    public void changeSelection(int rowIndex, int columnIndex, boolean toggle, boolean extend) {
        super.changeSelection(rowIndex, columnIndex, toggle, extend);
        if (!extend && !toggle) {
            editCellAt(rowIndex, columnIndex);
            TableCellEditor cellEditor = getCellEditor();
            if (cellEditor instanceof DefaultCellEditor) {
                DefaultCellEditor dce = (DefaultCellEditor) cellEditor;
                dce.getComponent().requestFocus();
            }
        }
    }

    private static class MyFocusAdapter extends FocusAdapter {
        public void focusGained(FocusEvent e) {
            ((JTextField) e.getSource()).selectAll();
        }
    }

    private static class MyPopupMouseAdapter extends PopupMouseAdapter {

        protected Set<Object> getActionKeys(MouseEvent e) {
            Set<Object> keys = new HashSet<Object>();
            keys.add(e.getSource());
            return keys;
        }
    }
}
