package dbhelp.plugin.ui;

import bean.PropertyChangeMultiCaster;
import bean.editor.ToolbarButton;
import bean.editor.utils.Item;
import dbhelp.db.*;
import dbhelp.db.model.*;
import dbhelp.plugin.OutputPane;
import dbhelp.plugin.action.portable.PopupMouseAdapter;
import utils.Config;
import utils.Utils;
import utils.text.editor.TokenEditor;
import utils.ui.TableFilterDialog;

import javax.swing.*;
import static javax.swing.KeyStroke.getKeyStroke;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;
import java.awt.*;
import static java.awt.Event.CTRL_MASK;
import static java.awt.GridBagConstraints.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.sql.ResultSetMetaData;
import java.util.*;
import java.util.List;

/**
 * User: huojun
 * Date: 20-Oct-2006
 * Time: 12:24:42 PM
 */
public class ResultTablePanel extends JPanel {
    protected JTabbedPane _tabs = new JTabbedPane(JTabbedPane.BOTTOM);
    protected TokenEditor _editor = new TokenEditor();
    protected MyPopupMouseAdapter _tableMouseAdaptor = new MyPopupMouseAdapter();
    protected JButton _jbSync;
    protected JButton _jbAdd;
    protected JButton _jbRemove;
    protected JButton _jbRefresh;
    protected JButton _jbEdit;
    protected JComboBox _databaseCombo;
    //    protected JCheckBox _jcbAutoCommit;
    protected String _title;
    protected static int _id = 1;
    protected JSplitPane _splitPane;
    protected JScrollPane _editorView;
    protected int _dividerPosition = -1;
    protected JButton _lbResultInfo = new JButton();
    protected JToolBar _toolbar1;
    protected Database _database;
    protected DefaultComboBoxModel _databaseList = new DefaultComboBoxModel();
    protected ActionSelectDatabase _actionSelectDatabase = new ActionSelectDatabase();

    public ResultTablePanel() {
        initComponent();
        PropertyChangeMultiCaster.getInstance().addListener(Config.class, new DatabaseChangeListener());
    }

    public void addResults(SQLResultHolder results) {
        String cmd = results.getSqlCmd();
        _database = results.getDatabase();
        _tabs.removeAll();
        if (!Utils.hasValue(_editor.getText())) {
            _editor.setText(cmd);
        }
        Table table = results.getTable();
        for (ISQLResult r : results.getResult()) {
            if (r instanceof ResultModel) {
                ResultModel m = (ResultModel) r;
                m.setTable(table);
                QuickEditTable t = new QuickEditTable();
                t.setModel(m);
                JScrollPane jsp = new JScrollPane(t);
                _tabs.add(m.getName(), jsp);
            }
            OutputPane.getOutputPane().getLogWriter().println(r);
        }
        _title = null != table ? table.getName() : "Result " + _id++;
        _databaseCombo.setSelectedItem(_database);
        _editor.setDatabase(_database);
        enableButtons();
    }


    public String getTitle() {
        return _title;
    }

    public void setTitle(String title) {
        _title = title;
    }

    protected Database getDatabase() {
        Object o = _databaseCombo.getSelectedItem();
        if (o instanceof Database) {
            return (Database) o;
        }
        Utils.message("Define a database profile first!");
        return null;
    }

    protected void getDatabaseList(Database db) {
        _databaseList.removeAllElements();
        Database[] databases = Config.getConfig().getDatabases();
        for (int i = 0; null != databases && i < databases.length; i++) {
            Database database = databases[i];
            _databaseList.addElement(database);
        }
        if (null != db) {
            _databaseCombo.setSelectedItem(db);
        }
    }

    protected void initComponent() {
        setLayout(new BorderLayout());
        _splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        _splitPane.setOneTouchExpandable(true);
        add(_splitPane, BorderLayout.CENTER);
        Dimension zeroSize = new Dimension(0, 0);
        _tabs.setMinimumSize(zeroSize);
        _splitPane.setLeftComponent(_tabs);
        Dimension ps = new Dimension(400, 200);
        _tabs.setPreferredSize(ps);
        _editorView = new JScrollPane(_editor);
        _editorView.setMinimumSize(zeroSize);
        _editorView.setPreferredSize(ps);
        _splitPane.setRightComponent(_editorView);
        _editorView.setVisible(false);

        _splitPane.setDividerLocation(1.0);
        _toolbar1 = new JToolBar(JToolBar.VERTICAL);
//        bar.setFloatable(false);
        AbstractAction action;

        action = new ActionSync();
        action.putValue("object", this);
        _jbSync = new ToolbarButton(17, action);
        _toolbar1.add(_jbSync);

        action = new ActionAdd();
        action.putValue("object", this);
        _jbAdd = new ToolbarButton(18, action);
        _toolbar1.add(_jbAdd);

        action = new ActionDelete();
        action.putValue("object", this);
        _jbRemove = new ToolbarButton(19, action);
        _toolbar1.add(_jbRemove);

        action = new ActionRefresh();
        action.putValue("object", this);
        _jbRefresh = new ToolbarButton(20, action);
        _toolbar1.add(_jbRefresh);

        action = new actionScript(_splitPane);
        JButton jbScript = new ToolbarButton(21, action);
        _toolbar1.add(jbScript);

        action = new ActionEdit();
        action.putValue("object", this);
        _jbEdit = new ToolbarButton(15, action);
        _toolbar1.add(_jbEdit);

        action = new ActionSave();
        action.putValue("object", this);
        JButton jbSave = new ToolbarButton(22, "Save Query", "save-query");
        _toolbar1.add(jbSave);
        _toolbar1.getActionMap().put("save-query", action);

        add(_toolbar1, BorderLayout.WEST);

        _tabs.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                QuickEditTable t = getTable();
                _lbResultInfo.setVisible(null != t);
                if (t != null) {
                    _lbResultInfo.setText("Resultset: " + t.getRowCount() + " row(s)");
                }
                enableButtons();
            }
        });

        toolbar2();
    }

    private void toolbar2() {
        JToolBar toolbar = new JToolBar(JToolBar.HORIZONTAL);
        toolbar.setFloatable(false);
        getDatabaseList( null );
        _databaseCombo = new JComboBox(_databaseList);
        JLabel lbDatabase = new JLabel("Database: ");
        toolbar.add(lbDatabase);
        toolbar.add(_databaseCombo);
        lbDatabase.setDisplayedMnemonic('D');
        lbDatabase.setLabelFor(_databaseCombo);
//        _jcbAutoCommit = new JCheckBox("Auto Commit");
//        toolbar.addSeparator(new Dimension(8, 8));
//        toolbar.add(_jcbAutoCommit);
        toolbar.addSeparator(new Dimension(4, 4));
        ActionRun actionRun = new ActionRun();
        toolbar.add(new ToolbarButton(28, actionRun));
        toolbar.addSeparator(new Dimension(10, 10));
        toolbar.add(new JLabel("Ctrl+Space to complete"));
        toolbar.addSeparator(new Dimension(4, 4));
        toolbar.add(_lbResultInfo);
        _lbResultInfo.setVisible(false);
        _lbResultInfo.setToolTipText("Click to see more information about the result");
        _lbResultInfo.addActionListener(new ActionMoreInfo(this));
        toolbar.add(Box.createHorizontalStrut(1000));
        _databaseCombo.addActionListener(_actionSelectDatabase);
//        _jcbAutoCommit.addActionListener(new ActionListener() {
//            public void actionPerformed(ActionEvent e) {
//                Database o = getDatabase();
//                if (o != null) {
//                    o.setAutoCommit(_jcbAutoCommit.isEnabled());
//                }
//            }
//        });
//        _jcbAutoCommit.setEnabled(false);
        add(toolbar, BorderLayout.PAGE_START);
        _editor.addHotKey("runsql", getKeyStroke(KeyEvent.VK_ENTER, CTRL_MASK), actionRun);
        _databaseCombo.setEnabled(false);
    }

    public QuickEditTable getTable() {
        JScrollPane jsp = (JScrollPane) _tabs.getSelectedComponent();
        if (null != jsp) {
            Component[] components = jsp.getViewport().getComponents();
            for (Component c : components) {
                if (c instanceof JTable) {
                    return (QuickEditTable) c;
                }
            }
        }
        return null;
    }

    private class MyPopupMouseAdapter extends PopupMouseAdapter {
        protected Set<Object> getActionKeys(MouseEvent e) {
            Set<Object> list = new HashSet<Object>();
            list.add(e.getSource());
            return list;
        }
    }

    private static abstract class tableAction extends AbstractAction {
        protected tableAction() {
        }

        protected tableAction(String name) {
            super(name);
        }

        public void actionPerformed(ActionEvent e) {
            ResultTablePanel panel = getPanel();
            QuickEditTable table = panel.getTable();
            if (null != table) {
                TableModel m = table.getMetaData();
                if (m instanceof ResultModel) {
                    ResultModel rm = (ResultModel) m;
                    try {
                        handleAction(table, rm);
                    } catch (Exception e1) {
                        Utils.error(e1);
                    }
                }
            } else {
                Utils.message("There isn't any result table!");
            }
        }

        protected ResultTablePanel getPanel() {
            return (ResultTablePanel) getValue("object");
        }

        protected abstract void handleAction(QuickEditTable table, ResultModel rm) throws Exception;
    }

    private static class ActionEdit extends tableAction {
        public ActionEdit() {
            super("Edit");
        }

        protected void handleAction(QuickEditTable table, ResultModel rm) {
            if (rm.getTable() != null) {
                rm.setEditable(true);
                getPanel().enableButtons();
            } else {
                try {
                    Database db = rm.getDatabase();
                    showResultTableDialog(db, rm);
                } catch (Exception e) {
                    Utils.error(e);
                }
            }
        }

        private void showResultTableDialog(Database db, ResultModel rm) throws Exception {
            if (!db.isConnected()) {
                db.connect();
            }
            List<Table> list = db.getMetaData().getTables(null, null, null, null);
            QuickCloseDialog d = QuickCloseDialog.newInstance(Utils.getFrame());
            if (null != d) {
                Container c = d.getContentPane();
                c.setLayout(new GridBagLayout());
                SearchListPanel searchPanel = new SearchListPanel();
                GridBagConstraints g;
                g = new GridBagConstraints(0, 0, 1, 1, 1, 1, NORTH, BOTH, new Insets(2, 2, 2, 2), 1, 1);
                c.add(searchPanel, g);
                searchPanel.setModel(new ListTableModel(list));
                searchPanel.setSearchText(rm.getTableName());
                g = new GridBagConstraints(0, 1, 1, 1, 0, 0, EAST, NONE, new Insets(2, 2, 2, 2), 1, 1);
                c.add(new JButton(new ActionOk(d, rm, searchPanel.getTable())), g);
                d.setTitle("Select Table");
                d.setSize(360, 300);
                d.setModal(true);
                d.setLocationRelativeTo(null);
                d.setVisible(true);
                if (d.isConfirmed()) {
                    getPanel().enableButtons();
                }
                d.dispose();
            }
        }

        private static class ListTableModel extends AbstractTableModel {
            private List _list;

            public ListTableModel(List list) {
                _list = list;
            }

            public int getRowCount() {
                return _list.size();
            }


            public String getColumnName(int column) {
                return column == 0 ? "Name" : "Full Name";
            }

            public int getColumnCount() {
                return 2;
            }

            public Object getValueAt(int rowIndex, int columnIndex) {
                Table t = (Table) _list.get(rowIndex);
                return columnIndex == 0 ? t.getName() : t;
            }
        }

        private static class ActionOk extends AbstractAction {
            private JTable _table;
            private ResultModel _model;
            private QuickCloseDialog _dialog;

            public ActionOk(QuickCloseDialog d, ResultModel rm, JTable table) {
                super("Enable Edit");
                putValue(AbstractAction.MNEMONIC_KEY, KeyEvent.VK_E);
                _table = table;
                _model = rm;
                _dialog = d;
            }

            public void actionPerformed(ActionEvent e) {
                int row = _table.getSelectedRow();
                if (row >= 0) {
                    Table t = (Table) _table.getValueAt(row, 1);
                    Column[] columns = t.getColumns();
                    for (Column c : columns) {
                        Column column = _model.getColumn(c.getName());
                        if (null == column) {
                            if (c.getNullable() == ResultSetMetaData.columnNoNulls) {
                                Utils.message("Unullable Column " + c + " is not included in the result!");
                                return;
                            }
                        } else {
                            column.setTable(t);
                        }
                    }
                    for (Column c : _model.getColumns()) {
                        if (!t.equals(c.getTable())) {
                            Utils.message("Column " + c + " is not in table " + t);
                            return;
                        }
                    }
                    _model.setTable(t);
                    _model.setEditable(true);
                    _dialog.setConfirmed(true);
                    _dialog.setVisible(false);
                } else {
                    Utils.message("Select a table please!");
                }
            }
        }
    }

    private void enableButtons() {
        QuickEditTable t = getTable();
        if (null != t) {
            ResultModel model = t.getMetaData();
            boolean editable = model.isEditable() && model.isSyncable();
            _jbSync.setEnabled(editable);
            _jbAdd.setEnabled(editable);
            _jbRemove.setEnabled(editable);
            _jbEdit.setEnabled(_tabs.getTabCount() == 1);
            _jbRefresh.setEnabled(editable);
        }
    }

    private static class ActionSync extends tableAction {
        public ActionSync() {
            super("Synchronize");
        }

        protected void handleAction(QuickEditTable table, ResultModel rm) throws Exception {
            rm.synchronize();
            table.refresh();
        }
    }

    private static class ActionDelete extends tableAction {
        public ActionDelete() {
            super("Delete Row");
        }

        protected void handleAction(QuickEditTable table, ResultModel m) throws Exception {
            int index = table.getSelectedRow();
            if (index > 0) {
                m.deleteRow(index);
                m.fireTableRowsDeleted(index, index);
            }
        }
    }

    private static class ActionSave extends tableAction {

        protected void handleAction(QuickEditTable table, ResultModel m) throws Exception {
            String name = JOptionPane.showInputDialog("Query Name");
            if (Utils.hasValue(name)) {
                Database database = m.getDatabase();
                Config cfg = Config.getConfig();
                Query[] savedQueries = cfg.getSavedQueries();
                Collection<Query> queries = new ArrayList<Query>();
                if (null != savedQueries) {
                    queries.addAll(Arrays.asList(savedQueries));
                }
                Query newQuery = new Query();
                newQuery.setName(name);
                newQuery.setDatabaseName(database.getName());
                newQuery.setSQLScript(m.getStatement());
                queries.add(newQuery);
                Query[] value = queries.toArray(new Query[queries.size()]);
                PropertyChangeMultiCaster.getInstance().set(cfg, "savedQueries", value);
            }
        }
    }

    private static class ActionAdd extends tableAction {
        public ActionAdd() {
            super("Add Row");
        }

        protected void handleAction(QuickEditTable table, ResultModel m) throws Exception {
            m.newRow();
            int row = m.getRowCount() - 1;
            m.fireTableRowsInserted(row, row);
            table.scrollRectToVisible(table.getCellRect(row, 0, true));
            ListSelectionModel sm = table.getSelectionModel();
            sm.setSelectionInterval(row, row);
            table.editCellAt(row, 0);
        }

    }

    private static class ActionRefresh extends tableAction {
        public ActionRefresh() {
            super("Refresh");
        }

        protected void handleAction(QuickEditTable table, ResultModel m) throws Exception {
            try {
                table.refresh();
            } catch (Exception e1) {
                Utils.error(e1);
            }
        }
    }

    private class actionScript extends AbstractAction {
        private JSplitPane _split;

        public actionScript(JSplitPane split) {
            super("Show SQL statement");
            _split = split;
        }

        public void actionPerformed(ActionEvent e) {
            Component rc = _split.getRightComponent();
            boolean visible = rc.isVisible();
            rc.setVisible(!visible);
            if (visible) {
                _dividerPosition = _split.getDividerLocation();
                _split.setDividerLocation(1.0);
            } else {
                if (_dividerPosition != -1) {
                    _split.setDividerLocation(_dividerPosition);
                } else {
                    _split.setDividerLocation(.6);
                }
            }
        }
    }

    private class ActionRun extends AbstractAction {

        public ActionRun() {
            super("Run SQL (Ctrl+Enter)");
        }

        public void actionPerformed(ActionEvent e) {
            String selectedText = _editor.getSelectedText();
            String text = selectedText == null ? _editor.getText() : selectedText;
            if (Utils.hasValue(text)) {
                Database db = getDatabase();
                if (null != db) {
                    SQLCommander.runSQLCommand(db, text, null, new MyISQLResultListener());
                    _editor.requestFocus();
                }
            } else {
                Toolkit.getDefaultToolkit().beep();
            }
        }

        private class MyISQLResultListener implements ISQLResultListener {
            public void notifyResult(final SQLResultHolder results) {
                Runnable runnable = new Runnable() {
                    public void run() {
                        addResults(results);
                        Rectangle tabsRect = _tabs.getVisibleRect();
//                        Rectangle editorRect = _editorView.getVisibleRect();
                        if (tabsRect.getWidth() == 0 || tabsRect.getHeight() == 0) {
                            _splitPane.setDividerLocation(.6);
                        }
                    }
                };
                SwingUtilities.invokeLater(runnable);
            }
        }
    }

    private class ActionMoreInfo implements ActionListener {
        private ResultTablePanel _panel;

        public ActionMoreInfo(ResultTablePanel resultTablePanel) {
            _panel = resultTablePanel;
        }

        public void actionPerformed(ActionEvent e) {
            ResultModel model = _panel.getTable().getMetaData();
            ResultColumnModel rm = new ResultColumnModel(model.getColumns());
            TableFilterDialog.showFilterDialog(rm, "Resultset MetaData", getTable());
        }
    }

    private class ResultColumnModel extends AbstractTableModel {
        private List<Item<Column>> _items = new ArrayList<Item<Column>>();

        public ResultColumnModel(Column[] columns) {
            FilterColumnModel fcm = getTable().getFilterColumnModel();
            for (int i = 0; null != columns && i < columns.length; i++) {
                Column column = columns[i];
                Item<Column> o = new Item<Column>(column);
                o.setSelected(fcm.isColumnVisible(i));
                _items.add(o);
            }
        }

        public int getRowCount() {
            return _items.size();
        }

        public int getColumnCount() {
            return 4;
        }

        public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
            Boolean b = (Boolean) aValue;
            _items.get(rowIndex).setSelected(b);
            FilterColumnModel model = getTable().getFilterColumnModel();
            model.setVisible(rowIndex, b);
        }

        public boolean isCellEditable(int rowIndex, int columnIndex) {
            return columnIndex == 0;
        }

        public Class<?> getColumnClass(int columnIndex) {
            return columnIndex == 0 ? Boolean.class : super.getColumnClass(columnIndex);
        }

        public String getColumnName(int column) {
            if (column == 0) {
                return "Visible";
            } else if (column == 1) {
                return "Name";
            } else if (column == 2) {
                return "Type";
            } else {
                return "Table";
            }
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            Item<Column> item = _items.get(rowIndex);
            Column value = item.getValue();
            if (columnIndex == 0) {
                return item.isSelected();
            } else if (columnIndex == 1) {
                return value.getName();
            } else if (columnIndex == 2) {
                return value.getTypeString();
            } else {
                return value.getTable() == null ? "N/A" : value.getTable().getName();
            }
        }
    }

    private class DatabaseChangeListener implements PropertyChangeListener {

        public void propertyChange(PropertyChangeEvent evt) {
            if ("databases".equals(evt.getPropertyName())) {
                _databaseCombo.removeActionListener( _actionSelectDatabase );
                getDatabaseList( _database );
                _databaseCombo.addActionListener(_actionSelectDatabase);
            }
        }
    }

    private class ActionSelectDatabase implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            Database db = getDatabase();
            if (null != db) {
//                    _jcbAutoCommit.setSelected(db.isAutoCommit());
                _editor.setDatabase(db);
                _editor.requestFocus();
            }
        }
    }
}
