package dbhelp.plugin.ui;

import bean.editor.TableSorter;
import dbhelp.db.model.FilteredTableModel;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import java.awt.*;
import static java.awt.GridBagConstraints.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * User: huojun
 * Date: 29-Sep-2006
 * Time: 3:42:02 PM
 */
public class SearchListPanel extends JPanel {
    private FilteredTableModel _filterModel = new FilteredTableModel(new DefaultTableModel());
    private TableSorter _sorterModel = new TableSorter(_filterModel);
    private JTable _table = new JTable(_sorterModel);
    private JCheckBox _regSearch = new JCheckBox("Regexp");
    private JCheckBox _caseSensitive = new JCheckBox("Case Sensitive");
    private JTextField _jtSearch = new JTextField();

    public SearchListPanel() {
        super(new GridBagLayout());
        GridBagConstraints g;
        g = new GridBagConstraints(0, 2, 1, 1, 0, 0, EAST, NONE, new Insets(2, 2, 2, 2), 0, 0);
        JLabel jlSearch = new JLabel("Search:");
        jlSearch.setDisplayedMnemonic('a');
        jlSearch.setLabelFor(_jtSearch);
        _jtSearch.setMargin(new Insets(1,1,1,1));
        add(jlSearch, g);
        g = new GridBagConstraints(1, 2, 1, 1, 1, 0, EAST, HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0);
        add(_jtSearch, g);
        g = new GridBagConstraints(0, 3, 2, 1, 1, 0, NORTH, HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0);
        add(ctrlPanel(), g);

        _regSearch.setMnemonic('x');
        _caseSensitive.setMnemonic('i');
        JScrollPane view = new JScrollPane(_table);
        g = new GridBagConstraints(0, 4, 2, 1, 1, 1, NORTH, BOTH, new Insets(2, 2, 2, 2), 0, 0);
        add(view, g);
        Dimension minimumSize = new Dimension(0, 0);
        view.setPreferredSize(new Dimension(300, 300));
        view.setMinimumSize(minimumSize);
        _table.setRowHeight(20);
        _table.setGridColor(Color.white);
        view.getViewport().setBackground(Color.white);
        _jtSearch.getDocument().addDocumentListener(new MyDocumentListener());
        _sorterModel.setTableHeader(_table.getTableHeader());
    }

    public JTable getTable() {
        return _table;
    }

    public void setModel(TableModel m) {
        _filterModel.setModel(m);
    }

    private void updateFilter(String filter) {
        _filterModel.reguarExpSearch(_regSearch.isSelected());
        _filterModel.setCaseSensitive(_caseSensitive.isSelected());
        _filterModel.setFilter(filter.length() == 0 ? null : filter);
    }

    private Component ctrlPanel() {
        JPanel p = new JPanel();
        p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
//        p.add(new JButton(new refreshAction("Refresh")));
//        p.add(Box.createHorizontalStrut(20));
        p.add(_regSearch);
        p.add(Box.createHorizontalStrut(20));
        p.add(_caseSensitive);
        checkOptionAction a = new checkOptionAction();
        _regSearch.addActionListener(a);
        _caseSensitive.addActionListener(a);
        return p;
    }

    public void setSearchText(String text) {
        _jtSearch.setText(text);
    }

    private class checkOptionAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            updateFilter(_jtSearch.getText());
        }
    }

    private class MyDocumentListener implements DocumentListener {
        public void insertUpdate(DocumentEvent e) {
            updateFilter(_jtSearch.getText());
        }

        public void removeUpdate(DocumentEvent e) {
            updateFilter(_jtSearch.getText());
        }

        public void changedUpdate(DocumentEvent e) {
            updateFilter(_jtSearch.getText());
        }
    }
}
