package dbhelp.plugin.ui;

import bean.editor.ToolbarButton;
import dbhelp.db.model.search.SearchResult;
import dbhelp.plugin.action.portable.PopupMouseAdapter;
import utils.TablePacker;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * User: huojun
 * Date: 12-Oct-2006
 * Time: 8:49:37 PM
 */
public class SearchResultPane extends QueryManagerPane {

    public SearchResultPane() {
        _model = new SearchResultModel();
        initComponents();
        JToolBar bar = new JToolBar(JToolBar.VERTICAL);
        add(bar, BorderLayout.WEST);
        ToolbarButton b = new ToolbarButton(21, new ActionShowSQL(_editorPane));
        bar.add(b);
        _jsp.setDividerLocation(.7);
        _searchPane.setPreferredSize(new Dimension(500, 200));
        _editorPane.setPreferredSize(new Dimension(200, 200));
        final JTable table = _searchPane.getTable();
        table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        table.addMouseListener( new PopupMouseAdapter() {
            protected Set<Object> getActionKeys(MouseEvent e) {
                Set<Object> actions = new HashSet<Object>();
                int row = table.getSelectedRow();
                if( row >= 0 ) {
                    Object o = table.getValueAt(row, 0);
                    actions.add(((SearchResult) o).getObject());
                }
                return actions;
            }
        });
        _textView.setEditable(false);
        table.getSelectionModel().addListSelectionListener( new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                if( !e.getValueIsAdjusting() ) {
                    int row = table.getSelectedRow();
                    if( row >= 0 ) {
                        SearchResult sr = (SearchResult) table.getValueAt(row, 0);
                        _textView.clearMarkedLines();
                        if (sr instanceof SearchResult.TextSearchResult) {
                            SearchResult.TextSearchResult tsr = (SearchResult.TextSearchResult) sr;
                            _textView.setText( tsr.getFullText() );
                            _textView.markLine( tsr.getLine() );
                        } else {
                            _textView.setText(sr.getDescription());
                        }
                    }
                }
            }
        });

    }

    public void setSearchResult(List<SearchResult> result) {
        _textView.setText("");
        ((SearchResultModel) _model).setData( result );
        TablePacker.packAsync(_searchPane.getTable(), 2);
    }

    private class ActionShowSQL extends AbstractAction {
        private Component _viewer;

        public ActionShowSQL(Component textViewer) {
            super("Show Syntax");
            _viewer = textViewer;
        }

        public void actionPerformed(ActionEvent e) {
            boolean visible = _viewer.isVisible();
            _viewer.setVisible( !visible);
            _jsp.setDividerLocation( visible ? 1.0 : 0.6 );
        }
    }

    private class SearchResultModel extends AbstractTableModel {
        private List<SearchResult> _list = new ArrayList<SearchResult>();

        public int getRowCount() {
            return _list.size();
        }

        public void setData(List<SearchResult> data) {
            _list.clear();
            if (null != data) {
                _list.addAll(data);
            }
            fireTableDataChanged();
        }

        public String getColumnName(int column) {
            if( column == 0 ) {
                return "Name";
            } else if( column == 2 ) {
                return "Line";
            } else if( column == 1 ) {
                return "Type";
            } else {
                return "Description";
            }
        }

        public int getColumnCount() {
            return 4;
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            SearchResult sr = _list.get(rowIndex);
            if (columnIndex == 0) {
                return sr;
            } else if( columnIndex == 2 ) {
                if (sr instanceof SearchResult.TextSearchResult) {
                    SearchResult.TextSearchResult tsr = (SearchResult.TextSearchResult) sr;
                    return "" + tsr.getLine();
                } else {
                    return "n/a";
                }
            } else if( columnIndex == 1 ) {
                return sr.getObject().getClass().getSimpleName();
            } else {
                return sr.getDescription();
            }
        }
    }
}
