package dbhelp.plugin.ui;

import utils.Utils;
import utils.text.IWordEnumeration;

import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.HashSet;
import java.util.Set;

/**
 * User: huojun
 * Date: 14-Oct-2006
 * Time: 9:21:19 AM
 */
public class TextViewer extends JComponent {
    protected String[] _lines = new String[0];
    protected Set<Integer> _markedLines = new HashSet<Integer>();
    protected static BufferedImage DUMMY_IMAGE = new BufferedImage(2,2, BufferedImage.TYPE_INT_ARGB);
    protected int _lineHeight;
    protected Color _markColor;
    protected Font _markFont;
    protected FontMetrics _fm;
//    protected List<Token> _tokens = new ArrayList<Token>();

    public static void main(String[] args) throws IOException {
        JFrame f = new JFrame();
        Container c = f.getContentPane();
        c.setLayout( new BorderLayout() );
        TextViewer viewer = new TextViewer();
        JScrollPane jsp = new JScrollPane(viewer);
        c.add(jsp, BorderLayout.CENTER );
        jsp.getViewport().setBackground(Color.white);
        f.setSize(800, 600);
        f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        f.setVisible(true);
        viewer.setText(Utils.getFileText("c:\\temp\\procedures.sql"));
        viewer.markLines( new int[] {2, 10, 15, 20} );

    }

    public TextViewer() {
    }

    private void getLineHight() {
        Graphics g = DUMMY_IMAGE.createGraphics();
        _fm = g.getFontMetrics(getFont());
        _lineHeight = _fm.getHeight();
    }


    public Font getMarkFont() {
        return _markFont;
    }

    public void setMarkFont(Font markFont) {
        _markFont = markFont;
    }

    public Color getMarkColor() {
        return _markColor == null ? Color.red : _markColor;
    }

    public void setMarkColor(Color markColor) {
        _markColor = markColor;
    }

    public void markLines(int[] lines) {
        for (int i = 0; null != lines && i < lines.length; i++) {
            int line = lines[i];
            markLine( line );
        }
    }

    public void clearMarks() {
        _markedLines.clear();
    }

    public void markLine(int line) {
        if( line >= 0 && line < _lines.length ) {
            _markedLines.add( line );
        }
    }

    public void setText(String text) {
        int width = 0;
//        _tokens.clear();
        _lines = null == text ? new String[0] : text.split("\\n");
//        parse(text);
        getLineHight();
        for(String line: _lines) {
            int strWidth = _fm.stringWidth(line);
            width = strWidth > width ? strWidth : width;
        }
        Dimension size = new Dimension(width, _lineHeight * _lines.length );
        setPreferredSize( size );
        revalidate();
        clearMarks();
        Runnable runnable = new Runnable() {
            public void run() {
                repaint();
            }
        };
        SwingUtilities.invokeLater(runnable);
    }
//
//    private void parse(String text) {
//        WordScanner ws = new WordScanner(text);
//        IWordEnumeration e = ws.scan(0);
//        while( e.hasMoreElements() ) {
//            Token t = new Token(e.nextElement(), e.getType(), e.getOffset());
//            _tokens.add(t);
//        }
//    }

    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        Rectangle vr = getVisibleRect();
        _markFont = _markFont == null ? getFont().deriveFont(Font.BOLD | Font.ITALIC) : _markFont;
        for(int line = 0; line < _lines.length; line++ ) {
            int y = line * _lineHeight;
            if( y + _lineHeight > vr.getY() && y < vr.getY() + vr.getHeight() ) {
                boolean b = _markedLines.contains(line);
                Color c = b ? getMarkColor() : Color.black;
                g.setColor( c );
                Font f = b ?  _markFont : getFont();
                g.setFont(f);
                String str = _lines[line];
                g.drawString(str, 0, y + _lineHeight);
//                Token[] tokens = getLineTokens(line);
            }
        }
    }

//    private Token[] getLineTokens(int line) {
//        int l = 0;
//        List<Token> tokens = new ArrayList<Token>();
//        for(Token t : _tokens) {
//            if( l == line ) {
//
//            }
//        }
//    }

    private static class Token {
        CharSequence text;
        IWordEnumeration.TYPE type;
        int offset;

        public Token(CharSequence text, IWordEnumeration.TYPE type, int offset) {
            this.text = text;
            this.type = type;
            this.offset = offset;
        }
    }
}
