package diagram;


import bean.editor.AnnotationPropertyEditorPane;
import bean.editor.ArrayHolder;
import bean.serialization.BeanXmlSerializer;
import diagram.shape.*;

import javax.swing.*;
import java.awt.*;
import java.awt.datatransfer.Clipboard;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.io.Reader;
import java.io.Writer;
import java.util.*;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: huojun
 * Date: 11-Aug-2005
 * Time: 6:17:13 PM
 */
public class DiagramViewer extends MyLayeredPane {
    protected MouseMonitor _monitor = new MouseMonitor(this);
    protected Lasso _lasso = new Lasso();
    protected ArrayList<AbstractShape> _selectedShapes = new ArrayList<AbstractShape>();
    protected JPopupMenu _popupMenu = new JPopupMenu();
    protected MyMouseAdapter _lassoMonitor = new MyMouseAdapter(this);
    protected DragImage _dragImage = new DragImage();
    protected Map<Object, Object> _objects = new HashMap<Object, Object>();
    protected Vector<IDragListener> _dragListeners = new Vector<IDragListener>();

    public static void main(String[] args) {
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (Throwable e) {
            // ignore, use default look & feel
        }
        JFrame f = new JFrame("Diagram Framework");
        Container contentPane = f.getContentPane();
        contentPane.setLayout(new BorderLayout());
        DiagramViewer sd = new DiagramViewer();
        JScrollPane jsp = new JScrollPane(sd);
        jsp.setPreferredSize(new Dimension(600, 600));
        jsp.setMinimumSize(new Dimension(600, 600));
        JSplitPane spliter = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, jsp, new AnnotationPropertyEditorPane());
        spliter.setResizeWeight(1.0);
        spliter.setOneTouchExpandable(true);
        contentPane.add(spliter, BorderLayout.CENTER);

        Rect rect = new Rect();
        sd.addShape(rect);
        Line line = new Line();
        sd.addShape(line);
        line.setLocation(300, 200);
        sd.addShape(new Circle());
        Arrow arrow = new Arrow();
        sd.addShape(arrow);
        TextBox tb = new TextBox("Apple\nBanana\nOrange\nPears\nCoconut");
        sd.addShape(tb);

        Triangle triangle = new Triangle();
        sd.addShape(triangle);

        TextLabel label = new TextLabel("Link Demo");
        sd.addShape(label);

        TextLabel gl = new TextLabel("Attach this arrow");
        sd.addShape(gl);

        PolygonShape ps = new PolygonShape();
        ps.addPoint(100, 100);
        ps.addPoint(50, 180);
        ps.addPoint(150, 220);
        ps.addPoint(250, 120);
        ps.addPoint(180, 80);
        ps.close();
        sd.addShape(ps);

        Link l = new Link(arrow, tb, ps);
        l.setText(label);

        Arrow ga = new Arrow();
        sd.addShape(ga);

        ArrowGlue arrowGlue = new ArrowGlue(sd, ga);
        arrowGlue.setShape(gl);

        f.setSize(800, 600);
        f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        f.setVisible(true);
    }

    public DiagramViewer() {
        setPreferredSize(new Dimension(DIAGRAM_WIDTH, DIAGRAM_HEIGHT));
        setBackground(Color.white);
        setOpaque(true);
        addMouseListener(_lassoMonitor);
        addMouseMotionListener(_lassoMonitor);
        add(_lasso, JLayeredPane.DEFAULT_LAYER);
        add(_dragImage, JLayeredPane.DEFAULT_LAYER);
        setupMenu();
    }

    public Object get(Object key) {
        return _objects.get(key);
    }

    public void put(Object key, Object value) {
        _objects.put(key, value);
    }

    public Set<Object> keySet() {
        return _objects.keySet();
    }

    public boolean containsKey(Object key) {
        return _objects.containsKey(key);
    }

    protected void setupMenu() {
        JMenuItem miFind = new JMenuItem("Find Object");
        JMenuItem miRemoveAll = new JMenuItem("Remove All Shapes");
        JMenuItem miSelectAll = new JMenuItem("Select All Shapes");
        JMenuItem miCopy = new JMenuItem("Copy Diagram as Image");
        JMenuItem miRemove = new JMenuItem("Remove shape(s)");
        _popupMenu.add(miFind);
        _popupMenu.add(miRemoveAll);
        _popupMenu.add(miCopy);
        _popupMenu.add(miRemove);
        _popupMenu.addSeparator();
        _popupMenu.add(miSelectAll);

        miFind.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                find();
            }
        });

        miSelectAll.addActionListener( new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                selectAll();
            }
        });

        miRemoveAll.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                removeAllShapes();
            }
        });

        miRemove.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                AbstractShape[] sss = getSelectedShapes();
                for (int i = 0; null != sss && i < sss.length; i++) {
                    AbstractShape ss = sss[i];
                    ss.remove();
                }
            }
        });
        miCopy.addActionListener(new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                copyDiagram();
            }
        });
    }

    public void find() {
    }

    public List<AbstractShape> getShape(Class<? extends AbstractShape> type) {
        List<AbstractShape> list = new ArrayList<AbstractShape>();
        AbstractShape[] shapes = getShapes();
        for (int i = 0; null != shapes && i < shapes.length; i++) {
            AbstractShape s = shapes[i];
            if (s.getClass().equals(type)) {
                list.add(s);
            }
        }
        return list;
    }

    public AbstractShape getShape(Object userObject) {
        return (AbstractShape) get(userObject);
    }

    public void removeAllShapes() {
        AbstractShape[] shapes = getShapes();
        for (int i = 0; null != shapes && i < shapes.length; i++) {
            AbstractShape shape = shapes[i];
            shape.remove();
        }
    }

    public AbstractShape[] getShapes() {
        ArrayList<AbstractShape> a = new ArrayList<AbstractShape>();
        Component[] components = getComponents();
        for (int i = 0; null != components && i < components.length; i++) {
            Component c = components[i];
            if (c instanceof ShapeContainer) {
                ShapeContainer sc = (ShapeContainer) c;
                a.add(sc.getShape());
            }
        }
        return a.toArray(new AbstractShape[a.size()]);
    }

    public void addDragListener(IDragListener l) {
        if (null != l && !_dragListeners.contains(l)) {
            _dragListeners.add(l);
        }
    }

    boolean askDragListeners(AbstractShape s, Handle dh, int x, int y) {
        IDragListener[] v = _dragListeners.toArray(new IDragListener[_dragListeners.size()]);
        for (IDragListener listener : v) {
            if (s.equals(listener.getMonitorShape())) {
                if (!listener.accept(dh, x, y)) {
                    return false;
                }
            }
        }
        return true;
    }

    boolean askDragListeners(AbstractShape s, int x, int y) {
        IDragListener[] v = _dragListeners.toArray(new IDragListener[_dragListeners.size()]);
        for (IDragListener listener : v) {
            if (s.equals(listener.getMonitorShape())) {
                if (!listener.accept(x, y)) {
                    return false;
                }
            }
        }
        return true;
    }

    public void removeDragListener(IDragListener l) {
        _dragListeners.remove(l);
    }


    public void copyDiagram() {
        Rectangle rect = null;
        Component[] components = getComponents();
        for (Component component : components) {
            if (component instanceof ShapeContainer) {
                rect = rect == null ? component.getBounds() : rect.union(component.getBounds());
            }
        }
        if (null != rect) {
            BufferedImage bi = new BufferedImage(rect.width, rect.height, BufferedImage.TYPE_3BYTE_BGR);
            Graphics g = bi.getGraphics();
            g.fillRect(0, 0, rect.width, rect.height);
            g.translate(-rect.x, -rect.y);
            for (Component component : components) {
                if (component instanceof ShapeContainer) {
                    ShapeContainer sc = (ShapeContainer) component;
                    sc.getShape().paint(g);
                }
            }
            ImageSelection is = new ImageSelection(bi);
            Clipboard cp = Toolkit.getDefaultToolkit().getSystemClipboard();
            cp.setContents(is, null);
        }

    }

    public ShapeContainer addShape(AbstractShape s) {
        return addShape(s, null);
    }

    public ShapeContainer addShape(AbstractShape s, Point pos) {
        ShapeContainer sc = new ShapeContainer(s);
        s.setDiagram(this);
        sc.addMouseListener(_monitor);
        sc.addMouseMotionListener(_monitor);
        add(sc, JLayeredPane.DEFAULT_LAYER);
        if (null != pos) {
            s.setLocation(pos);
        }
        if( null != s.getObject() ) {
            put(s.getObject(), s);
        }
        return sc;
    }

    public DragImage getDragImage() {
        return _dragImage;
    }

    public void selectAll() {
        setSelectAll(true);
    }

    public void removeShape(AbstractShape s) {
        if (null != s && this.equals(s.getDiagram()) && s.getContainer() != null) {
            remove(s.getContainer());
            _selectedShapes.remove(s);
            s.setDiagram(null);
            if( null != s.getObject() ) {
                _objects.remove(s.getObject());
            }
            repaint();
        }
    }


    public void deselectAll() {
        setSelectAll(false);
    }

    public MyMouseAdapter getLassoMonitor() {
        return _lassoMonitor;
    }

    public JPopupMenu getPopupMenu() {
        return _popupMenu;
    }

    protected void setSelectAll(boolean flag) {
        Component[] comps = getComponentsInLayer(JLayeredPane.DEFAULT_LAYER);
        for (Component comp : comps) {
            if (comp instanceof ShapeContainer) {
                ShapeContainer sc = (ShapeContainer) comp;
                AbstractShape s = sc.getShape();
                if (flag) {
                    selectShape(s);
                } else {
                    deselectShape(s);
                }
            }
        }
    }

    public void selectShape(AbstractShape s) {
        selectShape(s, true);
        if (!_selectedShapes.contains(s)) {
            _selectedShapes.add(s);
        }
//        try {
////            getPropertyEditor().setBean(s);
//        } catch (IntrospectionException e) {
//            e.printStackTrace();
//        }
    }

    public AbstractShape findShapeAtPoint(Point p) {
        return findShapeAtPoint(p, null);
    }

    public void selectShapes(Rectangle b) {
        _selectedShapes.clear();
        Component[] objs = getComponentsInLayer(JLayeredPane.DEFAULT_LAYER);
        for (Component obj : objs) {
            if (obj instanceof ShapeContainer) {
                ShapeContainer sc = (ShapeContainer) obj;
                AbstractShape shape = sc.getShape();
                if (b.contains(shape.getBounds())) {
                    selectShape(shape);
                }
            }
        }
    }

    public AbstractShape findShapeAtPoint(Point p, AbstractShape[] skips) {
        java.util.List list = null == skips ? null : Arrays.asList(skips);
        Component[] comps = getComponentsInLayer(JLayeredPane.DEFAULT_LAYER);
        for (Component comp : comps) {
            if (comp instanceof ShapeContainer) {
                ShapeContainer sc = (ShapeContainer) comp;
                AbstractShape shape = sc.getShape();
                if (shape.contains(p)
                        && (null == list || (!list.contains(shape)))) {
                    return shape;
                }
            }
        }
        return null;
    }

    public AbstractShape[] getSelectedShapes() {
        AbstractShape[] shapes = new AbstractShape[_selectedShapes.size()];
        _selectedShapes.toArray(shapes);
        return shapes;
    }

    public void fireMultiShapeChangeEvent() {
        AbstractShape[] ss = getSelectedShapes();
        for (int i = 0; null != ss && i < ss.length; i++) {
            AbstractShape s = ss[i];
            s.fireShapeChangeEvent();
        }
    }

    public void deselectShape(AbstractShape shape) {
        selectShape(shape, false);
        _selectedShapes.remove(shape);
    }

    protected void selectShape(AbstractShape shape, boolean selected) {
        if (this.equals(shape.getDiagram())) {
            shape.setSelected(selected);
            shape.getContainer().repaint();
        }
    }

    public void delete(Object key) {
        _objects.remove(key);
    }

    public List<Map.Entry> getValues(Class aClass) {
        List<Map.Entry> list = new ArrayList<Map.Entry>();
        Set<Map.Entry<Object, Object>> entries = _objects.entrySet();
        for (Map.Entry entry : entries) {
            if( entry.getKey().getClass().equals(aClass) ) {
                list.add(entry);
            }
        }
        return list;
    }

    protected class Lasso extends JComponent {
        public Lasso() {
            setVisible(false);
            setSize(50, 50);
        }

        protected void paintComponent(Graphics g) {
            g.drawRect(0, 0, getWidth() - 1, getHeight() - 1);
        }
    }

    protected class MyMouseAdapter implements MouseListener, MouseMotionListener {
        private DiagramViewer _diagram;
        private Point b;
        private Rectangle _lassRect = null;
        private boolean _canDrag;
        private Point _lassoPoint;

        public MyMouseAdapter(DiagramViewer diagram) {
            _diagram = diagram;
        }

        public void mousePressed(MouseEvent e) {
            if (Utilities.isActionButton(e)) {
                if (e.getSource().equals(_diagram)) {
                    deselectAll();
                }
                b = e.getPoint();
                _canDrag = true;
            } else {
                checkPopup(e);
            }
        }

        private void checkPopup(MouseEvent e) {
            if (e.isPopupTrigger()) {
                _diagram.getPopupMenu().show(_diagram, e.getX(), e.getY());
            }
        }

        public void mouseEntered(MouseEvent e) {
        }

        public void mouseExited(MouseEvent e) {
        }

        public void mouseReleased(MouseEvent e) {
            if (Utilities.isActionButton(e)) {
                if (_lasso.isVisible()) {
                    _lasso.setVisible(false);
                    if (null != _lassRect) {
                        for (AbstractShape s : _selectedShapes) {
                            int x = s.getX() - _lassRect.x + _lasso.getX();
                            int y = s.getY() - _lassRect.y + _lasso.getY();
                            s.setLocation(x, y);
                        }
                    }
                    _diagram.selectShapes(_lasso.getBounds());
                }
            } else {
                checkPopup(e);
            }
            _canDrag = false;
            _lassRect = null;
        }

        public void mouseDragged(MouseEvent e) {
            if (_canDrag) {
                Point p = e.getPoint();
                if (_selectedShapes.size() == 0) {
                    _lasso.setVisible(true);
                    _diagram.moveToFront(_lasso);
                    _lasso.setLocation(Math.min(b.x, p.x), Math.min(b.y, p.y));
                    _lasso.setSize(Math.abs(p.x - b.x), Math.abs(p.y - b.y));
                    _diagram.scrollRectToVisible(_lasso.getBounds());
                } else if (!e.getSource().equals(_diagram)) {
                    if (null == _lassRect) {
                        AbstractShape[] selectedShapes = _diagram.getSelectedShapes();
                        for (AbstractShape ss : selectedShapes) {
                            _lassRect = null == _lassRect ? ss.getBounds() : _lassRect.union(ss.getBounds());
                        }
                        _lassoPoint = _lassRect.getLocation();
                    }
                    _lasso.setSize(_lassRect.getSize());
                    _lasso.setVisible(true);
                    _lasso.setLocation(_lassoPoint.x + p.x - b.x, _lassoPoint.y + p.y - b.y);
                    _diagram.scrollRectToVisible(_lasso.getBounds());
                }
            }
        }

        public void mouseMoved(MouseEvent e) {
        }

        public void mouseClicked(MouseEvent e) {
            if (Utilities.isActionButton(e)) {
                deselectAll();
            } else {
                checkPopup(e);
            }
        }
    }

    public void loadDiagramFromXml(Reader xmlIn) throws Exception {
        ArrayHolder<Object> ah = (ArrayHolder<Object>) BeanXmlSerializer.fromXml(xmlIn);
        removeAllShapes();
        Rectangle rect = null;
        for (Object o : ah.getArray()) {
            AbstractShape shape = (AbstractShape) o;
            if (null == rect) {
                rect = shape.getBounds();
            } else {
                rect.add(shape.getBounds());
            }
            addShape(shape);
        }
        if (null != rect) {
            scrollRectToVisible(rect);
        }
    }

    public void saveDiagramToXml(Writer w) throws Exception {
        ArrayHolder<AbstractShape> ah = new ArrayHolder<AbstractShape>();
        AbstractShape[] shapes = getShapes();
        ah.setArray(shapes);
        BeanXmlSerializer.toXml(ah, w);
    }
}
