package diagram;

import diagram.shape.AbstractShape;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;


/**
 * Created by IntelliJ IDEA.
 * User: huojun
 * Date: 12-Aug-2005
 * Time: 10:34:26 AM
 */
public class MouseMonitor implements MouseListener, MouseMotionListener {
    private DiagramViewer _diagram;
    private AbstractShape _selectedShape;
    private int _bx;
    private int _by;
    private boolean _canDrag;

    public MouseMonitor(DiagramViewer diagram) {
        _diagram = diagram;
    }

    protected ShapeContainer getContainer(MouseEvent e) {
        return (ShapeContainer) e.getSource();
    }

    protected Point absPoint(MouseEvent e) {
        JComponent sc = (JComponent) e.getSource();
        Point p = e.getPoint();
        return new Point(p.x + sc.getX(), p.y + sc.getY());
    }

    public void mouseClicked(MouseEvent e) {
        _diagram.requestFocus();
    }

    private void selectShape(MouseEvent e) {
        checkRemovedShape();
        ShapeContainer sc = getContainer(e);
        Point p = absPoint(e);
        AbstractShape shape = sc.getShape();
//        if (shape.getHandleAtPoint(p) == null) {
        if (shape.getHandleAtPoint(p) != null || shape.contains(p)) {
            if (shape.isSelected()) {
                if (e.isControlDown()) {
                    shape.deselect();
                    _selectedShape = null;
                    return;
                }
            } else if (!e.isControlDown()) {
                _diagram.deselectAll();
            }
            _selectedShape = shape;
            _selectedShape.select();
        } else {
            AbstractShape sap = _diagram.findShapeAtPoint(p);
            _diagram.deselectAll();
            if (null != sap) {
                _selectedShape = sap;
                _selectedShape.select();
            } else {
                _selectedShape = null;
            }
        }
//        }
//        else if (!shape.equals(_selectedShape)) {
//            _selectedShape = shape;
//            if (!_selectedShape.isSelected()) {
//                _selectedShape.select();
//            }
//        }
        if (null != _selectedShape) {
            ShapeContainer c = _selectedShape.getContainer();
            if (e.getSource().equals(c)) {
                _bx = e.getX();
                _by = e.getY();
            } else {
                p = translate(e);
                _bx = p.x;
                _by = p.y;
            }
        }
    }

    private void checkRemovedShape() {
        if (null != _selectedShape && _selectedShape.getDiagram() == null) {
            _selectedShape = null;
        }
    }

    private Point translate(MouseEvent e) {
        Point p = absPoint(e);
        ShapeContainer c = _selectedShape.getContainer();
        p.x -= c.getX();
        p.y -= c.getY();
        return p;
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
        selectShape(e);
        if (multiSelected()) {
            _canDrag = true;
            forwardMouseEvent(e);
            return;
        }
        if (null != _selectedShape) {
            if (Utilities.isActionButton(e)) {
                _diagram.requestFocus();
                _canDrag = true;
                _diagram.getDragImage().setShape(_selectedShape);
            } else {
                checkPopup(e);
            }
        } else {
            // if there is not a shape selected,
            // forward this event to the diagram mouse monitor
            forwardMouseEvent(e);
        }
    }

    private void checkPopup(MouseEvent e) {
        if (e.isPopupTrigger()) {
            if (null != _selectedShape) {
//                JMenuItem[] menu = _selectedShape.getPopupMenu();
//                JMenuItem[] userMenu = _selectedShape.getUserMenu();
//                PopupMenuManager mgr = PopupMenuManager.getMgr();
//                JPopupMenu popup = mgr.getMenu(_selectedShape.getObject(), e);
//                if (null != popup) {
//                    popup.addSeparator();
//                }
//                if (null != menu || userMenu != null) {
//                    popup = popup == null ? new JPopupMenu() : popup;
//                    for (int i = 0; menu != null && i < menu.length; i++) {
//                        JMenuItem mi = menu[i];
//                        popup.add(mi);
//                    }
//                    for (int i = 0; userMenu != null && i < userMenu.length; i++) {
//                        JMenuItem mi = userMenu[i];
//                        popup.add(mi);
//                    }
//                    popup.show(_selectedShape.getContainer(), e.getX(), e.getY());
//                }
            } else {
                forwardMouseEvent(e);
            }
        }
    }

    private boolean multiSelected() {
        return _diagram.getSelectedShapes().length > 1;
    }

    private void forwardMouseEvent(MouseEvent e) {
        JComponent source = (JComponent) e.getSource();
        e.translatePoint(source.getX(), source.getY());
        _diagram.dispatchEvent(e);
    }

    public void mouseReleased(MouseEvent e) {
        if (_canDrag) {
            _canDrag = false;
            if (multiSelected()) {
                forwardMouseEvent(e);
                _diagram.fireMultiShapeChangeEvent();
                return;
            }
            if (_selectedShape.getDraggedHandle() == null) {
                DragImage di = _diagram.getDragImage();
                di.stopDrag();
            }
            _selectedShape.setDraggedHandle(null);
//            _diagram.repaint();
        } else if (e.isPopupTrigger()) {
            checkPopup(e);
        } else {
            forwardMouseEvent(e);
        }
    }

    public void mouseDragged(MouseEvent e) {
        if (_canDrag) {
            if (multiSelected()) {
                forwardMouseEvent(e);
                return;
            }
            Point p = e.getPoint();
            ShapeContainer c = getContainer(e);
            AbstractShape s = c.getShape();
            if (!s.equals(_selectedShape)) {
                // if the selected shape is not the one
                // receives drag event, we need to
                // translate the drag event coordination
                // to the selected shape
                p = translate(e);
            }
            Handle dh = _selectedShape.getDraggedHandle();
            if (dh == null) {
                DragImage di = _diagram.getDragImage();
                di.beginDrag();
                int dy = _selectedShape.getY() + p.y - _by;
                int dx = _selectedShape.getX() + p.x - _bx;
                if (_diagram.askDragListeners(_selectedShape, dx, dy)) {
                    if (dx>=0 && dx < DiagramViewer.DIAGRAM_WIDTH
                            && dy >=0 && dy < DiagramViewer.DIAGRAM_HEIGHT) {
                        di.setLocation(dx, dy);
                        _diagram.scrollRectToVisible(di.getBounds());
                    }
//                    else {
//                        Utils.debug("dx = " + dx + ", dy = " + dy);
//                    }
                }
            } else {
                if (!_selectedShape.isLocked() && _diagram.askDragListeners(_selectedShape, dh, p.x - Handle.SIZE, p.y - Handle.SIZE)) {
                    _selectedShape.handleMoved(dh, p.x - Handle.SIZE, p.y - Handle.SIZE);
                    _selectedShape.fireShapeChangeEvent();
                }
            }
        } else {
            forwardMouseEvent(e);
        }
    }

    public void mouseMoved(MouseEvent e) {
        ShapeContainer c = getContainer(e);
        if (c.getShape().isSelected()) {
            Point p = absPoint(e);
            Handle[] handles = c.getShape().getHandles();
            for (Handle handle : handles) {
                if (handle.pointInShape(p)) {
                    c.setCursor(Cursor.getPredefinedCursor(handle.getCursor()));
                    return;
                }
            }
            c.setCursor(Cursor.getDefaultCursor());
        }
    }
}
