package diagram.shape;

import bean.annotation.Property;
import diagram.*;

import java.awt.*;
import java.beans.*;
import java.util.Vector;


/**
 * Created by IntelliJ IDEA.
 * User: huojun
 * Date: 11-Aug-2005
 * Time: 6:30:28 PM
 */
public abstract class AbstractShape {
    protected DiagramViewer _diagram;
    protected boolean _selected;
    protected boolean _locked;
    protected boolean _opaque;
    protected Vector<ShapeListener> _listener = new Vector<ShapeListener>();
    protected ShapeContainer _container;
    protected Handle _draggedHandle;
    protected Object _object;
    protected PropertyChangeSupport _pcs = new PropertyChangeSupport(this);
    protected VetoableChangeSupport _vcs = new VetoableChangeSupport(this);
    private Font _font = new Font("Arial", Font.PLAIN, 10);
    private Color _background = Color.white;
    private Color _foreground = Color.black;

    protected AbstractShape() {
    }

    @Property(xmlAttr = false)
    public Color getForeground() {
        return _foreground;
    }

    public void setForeground(Color foreground) {
        _foreground = foreground;
        paintContainer();
    }

    @Property(xmlAttr = false)
    public Color getBackground() {
        return _background;
    }

    public void setBackground(Color background) {
        _background = background;
        paintContainer();
    }

    public boolean isOpaque() {
        return _opaque;
    }

    @Property(xmlAttr = false)
    public void setOpaque(boolean opaque) {
        _opaque = opaque;
        paintContainer();
    }

    @Property(xmlAttr = false)
    public Font getFont() {
        return _font;
    }

    public void setFont(Font font) {
        _font = font;
    }

    public DiagramViewer getDiagram() {
        return _diagram;
    }

    public Point getIntersection(Point p) {
        return getCenter();
    }

    public Point getIntersection(Point p1, Point p2) {
        Point p2d = Utilities.getIntersectionPoint(getBounds(), p1, p2);
        return p2d == null ? getCenter() : p2d;
    }

    public Object getObject() {
        return _object;
    }

    public void setObject(Object object) {
        _object = object;
    }

    public boolean isLocked() {
        return _locked;
    }

    public void setLocked(boolean locked) {
        _locked = locked;
    }

    @Property(xmlAttr = false)
    public Handle getDraggedHandle() {
        return _draggedHandle;
    }

    @Property(xmlAttr = false)
    public ShapeContainer getContainer() {
        return _container;
    }

    @Property(xmlAttr = false)
    public Point getCenter() {
        Rectangle b = getBounds();
        return new Point(b.x + b.width / 2, b.y + b.height / 2);
    }

    public void setContainer(ShapeContainer container) {
        _container = container;
    }

    @Property(xmlAttr = false)
    public void setDiagram(DiagramViewer diagram) {
        _diagram = diagram;
    }

    public void moveToFront() {
        if (null != _diagram && _container != null) {
            _diagram.moveToFront(_container);
        }
    }

    public void moveToBack() {
        if (null != _diagram && _container != null) {
            _diagram.moveToBack(_container);
        }
    }

    public void addChangeListener(ShapeListener l) {
        if (!_listener.contains(l)) {
            _listener.add(l);
        }
    }

    public void removeChangeListener(ShapeListener l) {
        _listener.remove(l);
    }

    public void fireShapeChangeEvent() {
        ShapeListener[] v = _listener.toArray(new ShapeListener[_listener.size()]);
        for (ShapeListener l : v) {
            l.shapeChanged(this);
        }
    }

    protected void notifyListener() {
//        Vector v = (Vector) _listener.clone();
//        for (Iterator getActionKeys = v.getActionKeys(); getActionKeys.hasNext();) {
//            ShapeListener l = (ShapeListener) getActionKeys.next();
//            l.shapeChanged(this);
//        }
    }

    public boolean isSelected() {
        return _selected;
    }

    public void remove() {
        if( null != _diagram ) {
            removeChangeListeners();
            _diagram.removeShape(this);
        }
    }

    public void select() {
        if( null != _diagram ) {
            _diagram.selectShape(this);
            this.moveToFront();
        }
    }

    public void deselect() {
        if( null != _diagram ) {
            _diagram.deselectShape(this);
        }
    }

    public void removeChangeListeners() {
        _listener.clear();
    }

    public void setSelected(boolean selected) {
        _selected = selected;
        paintContainer();
    }

    public int getX() {
        return getBounds().x;
    }

    public void setX(int x) {
        Point l = getLocation();
        setLocation(x, l.y);
    }

    public void setY(int y) {
        Point l = getLocation();
        setLocation(l.x, y);
    }

    public int getY() {
        return getBounds().y;
    }

    public boolean contains(Point p) {
        return getBounds().contains(p);
    }

    @Property(xmlAttr = false)
    public Handle[] getHandles() {
        return null;
    }

    @Property(xmlAttr = false)
    public void setDraggedHandle(Handle draggedHandle) {
        _draggedHandle = draggedHandle;
    }

    @Property(xmlAttr = false)
    public abstract Rectangle getBounds();

    public abstract void paint(Graphics g);

    public void paintDragImage(Graphics g) {
        paint(g);
    }

    public void setLocation(int x, int y) {
        setLocation( new Point( x, y) );
    }

    public abstract void setLocation(Point p);

    public void handleMoved(Handle h, int x, int y) {
    }

    public Handle getHandleAtPoint(Point p) {
        Handle[] handles = getHandles();
        for (int i = 0; null != handles && i < handles.length; i++) {
            Handle h = handles[i];
            if (h.pointInShape(p)) {
                return _draggedHandle = h;
            }
        }
        return _draggedHandle = null;
    }

    public int getHeight() {
        return getBounds().height;
    }

    public int getWidth() {
        return getBounds().width;
    }

    public Dimension getSize() {
        return new Dimension(getWidth(), getHeight());
    }

    public void sureVisible() {
        if( null != getDiagram() ) {
            getDiagram().scrollRectToVisible(getBounds());
        }
    }

    public Point getLocation() {
        Rectangle b = getBounds();
        return new Point(b.x, b.y);
    }

    protected boolean vote(String propertyName, int oldValue, int newValue) {
        try {
            _vcs.fireVetoableChange(propertyName, oldValue, newValue);
        } catch (PropertyVetoException e) {
            return false;
        }
        return true;
    }

    protected boolean vote(String propertyName, Object oldValue, Cloneable newValue) {
        try {
            _vcs.fireVetoableChange(propertyName, oldValue, newValue);
        } catch (PropertyVetoException e) {
            return false;
        }
        return true;
    }

    protected boolean vote(String propertyName, boolean oldValue, boolean newValue) {
        try {
            _vcs.fireVetoableChange(propertyName, oldValue, newValue);
        } catch (PropertyVetoException e) {
            return false;
        }
        return true;
    }

    protected void adjustContainer() {
        if (null != _container) {
            _container.adjust();
        }
    }

    public void addVetoableChangeListener(VetoableChangeListener listener) {
        _vcs.addVetoableChangeListener(listener);
    }

    public void removeVetoableChangeListener(VetoableChangeListener listener) {
        _vcs.removeVetoableChangeListener(listener);
    }

    public VetoableChangeListener[] getVetoableChangeListeners() {
        return _vcs.getVetoableChangeListeners();
    }

    public void fireVetoableChange(String propertyName, Object oldValue, Object newValue) throws PropertyVetoException {
        _vcs.fireVetoableChange(propertyName, oldValue, newValue);
    }

    public void fireVetoableChange(String propertyName, int oldValue, int newValue) throws PropertyVetoException {
        _vcs.fireVetoableChange(propertyName, oldValue, newValue);
    }

    public void fireVetoableChange(String propertyName, boolean oldValue, boolean newValue) throws PropertyVetoException {
        _vcs.fireVetoableChange(propertyName, oldValue, newValue);
    }

    public void firePropertyChange(PropertyChangeEvent evt) {
        _pcs.firePropertyChange(evt);
    }

    public PropertyChangeListener[] getPropertyChangeListeners() {
        return _pcs.getPropertyChangeListeners();
    }

    public void addPropertyChangeListener(PropertyChangeListener listener) {
        _pcs.addPropertyChangeListener(listener);
    }

    public void removePropertyChangeListener(PropertyChangeListener listener) {
        _pcs.removePropertyChangeListener(listener);
    }

    public boolean hasListeners(String propertyName) {
        return _pcs.hasListeners(propertyName);
    }

    public void firePropertyChange(String propertyName, int oldValue, int newValue) {
        _pcs.firePropertyChange(propertyName, oldValue, newValue);
    }

    public void firePropertyChange(String propertyName, boolean oldValue, boolean newValue) {
        _pcs.firePropertyChange(propertyName, oldValue, newValue);
    }

    public PropertyChangeListener[] getPropertyChangeListeners(String propertyName) {
        return _pcs.getPropertyChangeListeners(propertyName);
    }

    public void addPropertyChangeListener(String propertyName, PropertyChangeListener listener) {
        _pcs.addPropertyChangeListener(propertyName, listener);
    }

    public void removePropertyChangeListener(String propertyName, PropertyChangeListener listener) {
        _pcs.removePropertyChangeListener(propertyName, listener);
    }

    public void firePropertyChange(String propertyName, Object oldValue, Shape newValue) {
        _pcs.firePropertyChange(propertyName, oldValue, newValue);
    }

    protected void paintContainer() {
        if (null != _container) {
            _container.repaint();
        }
    }

}
