package diagram.shape;

import diagram.Handle;
import diagram.ICanDrawDragIcon;

import java.awt.*;
import java.awt.geom.Line2D;

/**
 * Created by IntelliJ IDEA.
 * User: huojun
 * Date: 11-Aug-2005
 * Time: 9:09:38 PM
 */
public class Line extends AbstractShape implements ICanDrawDragIcon {
    protected int _x1 = 10;
    protected int _x2 = 100;
    protected  int _y1 = 10;
    protected int _y2 = 100;
    protected static final BasicStroke DASH_STROKE = new BasicStroke(2f,
                BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND, 0, new float[]{1f, 3f}, 0);
    protected Line2D.Double _line = new Line2D.Double(_x1, _y1, _x2, _y2);
    protected Handle _h1 = new Handle(Cursor.HAND_CURSOR);
    protected Handle _h2 = new Handle(Cursor.HAND_CURSOR);
    protected Handle[] _handles = new Handle[]  { _h1, _h2 };

    public Line() {
        adjust();
    }

    public int getX1() {
        return _x1;
    }

    public void setX1(int x1) {
        if (vote("x1", _x1, x1)) {
            _x1 = x1;
            adjust();
        }
    }

    public int getY2() {
        return _y2;
    }

    public void setY2(int y2) {
        if (vote("y2", _y2, y2)) {
            _y2 = y2;
            adjust();
        }
    }

    public int getY1() {
        return _y1;
    }

    public void setY1(int y1) {
        if (vote("y1", _y1, y1)) {
            _y1 = y1;
            adjust();
        }
    }

    public int getX2() {
        return _x2;
    }

    public void setX2(int x2) {
        if (vote("x2", _x2, x2)) {
            _x2 = x2;
            adjust();
        }
    }

    public Handle[] getHandles() {
        _h1.setPoint( getX1(), getY1() );
        _h2.setPoint( getX2(), getY2() );
        return _handles;
    }

    public void handleMoved(Handle h, int x, int y) {
        if( h.equals(_h1) ) {
            setX1( getX() + x );
            setY1( getY() + y );
        } else if( h.equals(_h2) ) {
            setX2( getX() + x );
            setY2( getY() + y );
        }
        adjust();
    }

    public boolean contains(Point p) {
        double v = _line.ptLineDist(p);
        return v < 2;
    }

    protected void adjust() {
        Object b = _line.clone();
        _line.setLine( _x1, _y1, _x2, _y2 );
        adjustContainer();
        notifyListener();
        super.firePropertyChange("bounds", b, _line);
    }

    public Rectangle getBounds() {
        return _line.getBounds();
    }

    public void setLocation(Point p) {
        if (vote("location", getLocation(), p)) {
            Rectangle b = getBounds();
            _x1 += (p.x - b.x);
            _x2 += (p.x - b.x);
            _y1 += (p.y - b.y);
            _y2 += (p.y - b.y);
            adjust();
        }
    }

    public void paint(Graphics g) {
        g.setColor(getForeground());
        g.drawLine( _x1, _y1, _x2, _y2 );
    }
}
