package diagram.shape;

import diagram.ShapeListener;

import java.awt.*;


/**
 * Created by IntelliJ IDEA.
 * User: huojun
 * Date: 11-Aug-2005
 * Time: 2:31:33 PM
 */
public class Link implements ShapeListener {
    protected Arrow _arrow;
    protected TextBox _text;
    protected AbstractShape _beginShape;
    protected AbstractShape _endShape;

    public Link(Arrow arrow, AbstractShape beginShape, AbstractShape endShape) {
        _arrow = arrow;
        setBeginShape(beginShape);
        setEndShape(endShape);
        adjustArrowForShape();
    }

    public TextBox getText() {
        return _text;
    }

    public void setText(TextBox text) {
        _text = text;
        _arrow.addChangeListener(this);
    }

    public Link(Arrow arrow) {
        _arrow = arrow;
    }

    public Arrow getArrow() {
        return _arrow;
    }

    public void setArrow(Arrow arrow) {
        _arrow = arrow;
    }

    public AbstractShape getBeginShape() {
        return _beginShape;
    }

    public void setBeginShape(AbstractShape beginShape) {
        _beginShape = beginShape;
        _beginShape.addChangeListener(this);
        follow(beginShape);
    }

    public AbstractShape getEndShape() {
        return _endShape;
    }

    public void setEndShape(AbstractShape endShape) {
        _endShape = endShape;
        _endShape.addChangeListener(this);
        follow(endShape);
    }

    public void shapeChanged(AbstractShape s) {
        follow(s);
    }

    protected void follow(AbstractShape s) {
        if (s instanceof Arrow) {
            if( _text != null ) {
                Point c = getArrow().getCenter();
                _text.setLocation( c.x - _text.getWidth() / 2, c.y - _text.getHeight() / 2);
                _text.moveToFront();
            }
        } else {
            adjustArrowForShape();
        }
    }

    protected void adjustArrowForShape() {
        if (null != _endShape && null != _beginShape) {
            Point intersection = _beginShape.getIntersection(_endShape.getCenter());
            getArrow().setX1(intersection.x);
            getArrow().setY1(intersection.y);
            intersection = _endShape.getIntersection(_beginShape.getCenter());
            getArrow().setX2(intersection.x);
            getArrow().setY2(intersection.y);
            getArrow().fireShapeChangeEvent();
        }
    }

    public void shapeResized(AbstractShape s) {
        follow(s);
    }
}
