package diagram.shape;

import diagram.Handle;
import diagram.Utilities;

import java.awt.*;
import java.util.ArrayList;

/**
 * Created by IntelliJ IDEA.
 * User: huojun
 * Date: 12-Aug-2005
 * Time: 2:46:35 PM
 */
public class PolygonShape extends AbstractShape {
    protected Polygon _polygon = new Polygon();
    protected ArrayList<Handle> _handles = new ArrayList<Handle>();
    private Point[] _points;

    public void addPoint(int x, int y) {
        Handle h = new Handle(Cursor.MOVE_CURSOR);
        _polygon.addPoint(x, y);
        _handles.add(h);
    }

    public Point[] getPoints() {
        return _points;
    }

    public Point getPoint(int index) {
        return new Point(_polygon.xpoints[index], _polygon.ypoints[index]);
    }

    public void setPoint(int index, Point p) {
        _polygon.xpoints[index] = p.x;
        _polygon.ypoints[index] = p.y;
        adjust();
    }

    public void setPoints(Point[] points) {
        _polygon.reset();
        for (Point p : points) {
            _polygon.addPoint(p.x, p.y);
        }
        adjust();
    }

    public void close() {
        // close the shape
        _polygon.addPoint(_polygon.xpoints[0], _polygon.ypoints[0]);
        adjust();
    }

    public Point getIntersection(Point p) {
        Point p2d = Utilities.getIntersectionPoint(_polygon, p);
        return p2d == null ? getCenter() : p2d;
    }

    public boolean contains(Point p) {
        return _polygon.contains(p);
    }

    public Handle[] getHandles() {
        for (int i = 0; i < _handles.size(); i++) {
            Handle h = _handles.get(i);
            h.setPoint(_polygon.xpoints[i], _polygon.ypoints[i]);
        }
        Handle[] handles = new Handle[_handles.size()];
        _handles.toArray(handles);
        return handles;
    }

    public void handleMoved(Handle h, int x, int y) {
        int index = _handles.indexOf(h);
        Rectangle b = getBounds();
        _polygon.xpoints[index] = b.x + x;
        _polygon.ypoints[index] = b.y + y;
        adjust();
    }

    public Rectangle getBounds() {
        return _polygon.getBounds();
    }

    public void setLocation(Point r) {
        Rectangle b = getBounds();
        for (int i = 0; i < _polygon.npoints; i++) {
            _polygon.xpoints[i] += r.x - b.x;
            _polygon.ypoints[i] += r.y - b.y;
        }
        adjust();
    }

    protected void adjust() {
        updatePoints();
        adjustContainer();
        notifyListener();
    }

    private void updatePoints() {
        int n = _polygon.npoints;
        if (null == _points || _points.length != n) {
            _points = new Point[ n ];
        }
        for (int i = 0; i < _points.length-1; i++) {
            _points[i] = new Point( _polygon.xpoints[i], _polygon.ypoints[i] );
        }
        int last = _points.length - 1;
        _polygon.xpoints[last] = _polygon.xpoints[0];
        _polygon.ypoints[last] = _polygon.ypoints[0];
        _points[last] = new Point(_polygon.xpoints[0], _polygon.ypoints[0]);
        _polygon.invalidate();
    }

    public void paint(Graphics g) {
        g.setColor(getForeground());
        g.drawPolygon(_polygon);
    }
}
