package diagram.shape;

import bean.annotation.Property;
import diagram.Handle;
import diagram.Utilities;

import java.awt.*;

/**
 * Created by IntelliJ IDEA.
 * User: huojun
 * Date: 11-Aug-2005
 * Time: 6:40:14 PM
 */
public class Rect extends AbstractShape {
    private Rectangle _rect = new Rectangle(10, 10, 100, 100);
    private Handle _lt = new Handle(Cursor.NW_RESIZE_CURSOR);
    private Handle _rt = new Handle(Cursor.NE_RESIZE_CURSOR);
    private Handle _lb = new Handle(Cursor.SW_RESIZE_CURSOR);
    private Handle _rb = new Handle(Cursor.SE_RESIZE_CURSOR);
    private Handle _tm = new Handle(Cursor.N_RESIZE_CURSOR);
    private Handle _rm = new Handle(Cursor.E_RESIZE_CURSOR);
    private Handle _lm = new Handle(Cursor.W_RESIZE_CURSOR);
    private Handle _bm = new Handle(Cursor.S_RESIZE_CURSOR);
    private Handle[] _handles = new Handle[]{
            _lt, _rt, _lb, _rb, _tm, _rm, _lm, _bm
    };

    public Rect() {
    }

    @Property(xmlAttr = false)
    public Rectangle getBounds() {
        return _rect.getBounds();
    }

    @Property(xmlAttr = false)
    public Handle[] getHandles() {
        Rectangle b = getBounds();
        _lt.setPoint(b.x, b.y);
        _rt.setPoint(b.x + b.width, b.y);
        _lb.setPoint(b.x, b.y + b.height);
        _rb.setPoint(b.x + b.width, b.y + b.height);
        _tm.setPoint(b.x + b.width / 2, b.y);
        _lm.setPoint(b.x, b.y + b.height / 2);
        _rm.setPoint(b.x + b.width, b.y + b.height / 2);
        _bm.setPoint(b.x + b.width / 2, b.y + b.height);
        return _handles;
    }

    public void handleMoved(Handle h, int x, int y) {
        Rectangle b = getBounds();
        Rectangle r = new Rectangle();
        if (h.equals(_lt)) {
            r.setBounds(b.x + x, b.y + y, b.width - x, b.height - y);
        } else if (h.equals(_rt)) {
            r.setBounds(b.x, b.y + y, x, b.height - y);
        } else if (h.equals(_rb)) {
            r.setBounds(b.x, b.y, x, y);
        } else if (h.equals(_lb)) {
            r.setBounds(b.x + x, b.y, b.width - x, y);
        } else if (h.equals(_lm)) {
            r.setBounds(b.x + x, b.y, b.width - x, b.height);
        } else if (h.equals(_rm)) {
            r.setBounds(b.x, b.y, x, b.height);
        } else if (h.equals(_tm)) {
            r.setBounds(b.x, b.y + y, b.width, b.height - y);
        } else if (h.equals(_bm)) {
            r.setBounds(b.x, b.y, b.width, y);
        }
        if (r.width >= 4 && r.height >= 4) {
            setBounds(r);
        }
    }

    public void setLocation(Point p) {
        Rectangle b = getBounds();
        b = new Rectangle(p.x, p.y, b.width, b.height);
        setBounds(b);
    }

    public void setWidth(int w) {
        Rectangle b = getBounds();
        setBounds(new Rectangle(b.x, b.y, w, b.height));
    }

    public void setSize(int w, int h) {
        Rectangle b = getBounds();
        setBounds(new Rectangle(b.x, b.y, w, h));
    }

    public void setHeight(int h) {
        Rectangle b = getBounds();
        setBounds(new Rectangle(b.x, b.y, b.width, h));
    }

    public void setBounds(Rectangle r) {
        if (vote("bounds", _rect.clone(), r)) {
            Object b = getBounds().clone();
            _rect.setBounds(r);
            adjust();
            super.firePropertyChange("bounds", b, getBounds());
        }
    }

    protected void adjust() {
        adjustContainer();
        notifyListener();
    }

    public Point getIntersection(Point p) {
        Point p2d = Utilities.getIntersectionPoint(getBounds(), p);
        return p2d == null ? getCenter() : p2d;
    }

    public void paint(Graphics g) {
        if (isOpaque()) {
            g.setColor(getBackground());
            g.fillRect(_rect.x, _rect.y, _rect.width, _rect.height);
        }
        g.setColor(getForeground());
        g.drawRect(_rect.x, _rect.y, _rect.width, _rect.height);
    }
}
