package diagram.shape;

import bean.annotation.Property;

import java.awt.*;
import java.awt.image.BufferedImage;

/**
 * Created by IntelliJ IDEA.
 * User: huojun
 * Date: 9-Aug-2005
 * Time: 1:07:12 PM
 */
public class TextBox extends Rect {
    private String _text;
    private boolean _drawBorder = true;
    private static final int LINE_SPACE = 1;
    private static final int DELTA_X = 4;
    private static final int DELTA_Y = 4;

    public TextBox() {
        this("TextBox");
        setOpaque(true);
    }

    public TextBox(String text) {
        setText(text);
    }

    public void setBackground(Color background) {
        super.setBackground(background);
        paintContainer();
    }

    public boolean isDrawBorder() {
        return _drawBorder;
    }

    public void setDrawBorder(boolean drawBorder) {
        _drawBorder = drawBorder;
        super.paintContainer();
    }

    public void setFont(Font font) {
        super.setFont(font);
        calculateSize(null);
        paintContainer();
    }

    @Property(xmlCDATA = true)
    public String getText() {
        return _text;
    }

    public void setText(String text) {
        _text = text;
        calculateSize(null);
    }

    public void setForeground(Color foreground) {
        super.setForeground(foreground);
        paintContainer();
    }

    protected void calculateSize(Graphics g) {
        g = sureGraphic(g);
        if (null != g) {
            g.setFont(getFont());
            String[] lines = _text.split("\n");
            int maxWidth = 0;
            int h = 0;
            FontMetrics fm = g.getFontMetrics();
            for (String line : lines) {
                int w = fm.stringWidth(line);
                if (maxWidth < w) {
                    maxWidth = w;
                }
                h += fm.getHeight();
                h += LINE_SPACE;
            }
            setSize(maxWidth + 2 * DELTA_X, h + DELTA_Y );
        }
    }

    protected Graphics sureGraphic(Graphics g) {
        if (null == g) {
            BufferedImage i = new BufferedImage(1, 1,
                    BufferedImage.TYPE_INT_ARGB);
            g = i.getGraphics();
        }
        return g;
    }

    public void paint(Graphics g) {
        String[] lines = _text.split("\n");
        Rectangle b = getBounds();
        int y = DELTA_Y + b.y;
        g.setColor(getBackground());
        if (isOpaque()) {
            g.fillRect(b.x, b.y, b.width - 1, b.height - 1);
        }
        g.setColor(getForeground());
        g.setFont(getFont());
        FontMetrics fm = g.getFontMetrics();
        int height = fm.getHeight();
        if (isDrawBorder()) {
            g.drawRect(b.x, b.y, b.width - 1, b.height - 1);
        }
        Graphics g1 = g.create();
        g1.clipRect(b.x, b.y, b.width-1, b.height -1);
        for (int i = 0; y + height<b.height+b.y && i < lines.length ; i++) {
            String line = lines[i];
            g1.drawString(line, DELTA_X + b.x, y + fm.getAscent());
            y += (height + LINE_SPACE);
        }
    }
}
