package diagram.shape.database;

import dbhelp.db.Column;
import dbhelp.db.Table;
import dbhelp.db.PrimaryKey;
import diagram.ShapeContainer;
import utils.Utils;

import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import java.awt.*;
import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.sql.Types;
import java.util.*;

/**
 * User: huojun
 * Date: 21-Nov-2006
 * Time: 4:48:50 PM
 */
public class TableBox2 extends TableBox {
    private TableDataModel _model;
    private java.util.List<Integer> _columnWidths;
    protected static final int DELTA = 4;
//    protected static final Color TITLE_COLOR = new Color(240, 252, 160);
    protected static final Color TITLE_COLOR = Color.white;

    public static void main(String[] args) {
        JFrame f = new JFrame();
        f.setSize(800, 600);
        Table t = new Table();
        t.setName("Employee");
        Column columnId = new Column("id", Types.INTEGER);
        columnId.setPrimaryKeyInfo(new PrimaryKey());
        columnId.setColumnDef("autoincrement");
        t.setColumns(new Column[]{
                columnId,
                new Column("first_Name", Types.VARCHAR, 10),
                new Column("last_Name", Types.VARCHAR, 10),
                new Column("salary_per_month", Types.DECIMAL, 10)
        });
        TableBox2 box2 = new TableBox2(t);
        ShapeContainer container = new ShapeContainer(box2);
        box2.setContainer(container);
        f.getLayeredPane().add(container);
        f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        f.setVisible(true);
    }

    public TableBox2(Table t) {
        super(t);
    }

    public TableBox2() {
    }


    private java.util.List<Integer> getColumnWidths() {
        if (null == _columnWidths) {
            _columnWidths = new ArrayList<Integer>();
        }
        return _columnWidths;
    }

    private TableDataModel getTableModel() {
        if (null == _model) {
            _model = new TableDataModel();
        }
        return _model;
    }


    public Dimension calulateSize() {
        return getSize();
    }

    private void calculateSize() {
        Graphics g = sureGraphic(null);
        FontMetrics fm = g.getFontMetrics(getFont());
        getColumnWidths().clear();
        int width = 0;
        for (int i = 0; i < getTableModel().getColumnCount(); i++) {
            int columnMaxWidth = getColumnWidth(fm, i) + DELTA;
            width += columnMaxWidth;
            getColumnWidths().add(columnMaxWidth);
        }
        int maxWidth = Math.max(width, getTableTitleWidth(fm) + DELTA);
        Dimension dimension = new Dimension(maxWidth, (getTableModel().getRowCount() + 1) * fm.getHeight() + DELTA/2 );
        setSize(dimension.width, dimension.height);
    }

    private int getTableTitleWidth(FontMetrics fm) {
        return fm.stringWidth(getTable().getName());
    }

    private int getColumnWidth(FontMetrics fm, int columnIndex) {
        int width = 0;
        for (int i = 0; i < getTableModel().getRowCount(); i++) {
            Object o = getTableModel().getValueAt(i, columnIndex);
            int w = fm.stringWidth(o == null ? "" : o.toString());
            width = w > width ? w : width;
        }
        return width;
    }

    public void paint(Graphics g) {
        Rectangle b = getBounds();
        g.setFont(getFont());
        if (isOpaque()) {
            g.setColor(getBackground());
            g.fillRect(b.x, b.y, b.width - 1, b.height - 1);
        }
        if (isDrawBorder()) {
            g.setColor(getForeground());
            g.drawRect(b.x, b.y, b.width - 1, b.height - 1);
        }
        g.setColor(getForeground());
        FontMetrics fm = g.getFontMetrics(getFont());
        int tableTitleWidth = getTableTitleWidth(fm);
        int ascent = fm.getAscent();
        Graphics g1 = g.create();
        g1.clipRect(b.x, b.y, b.width-1, b.height -1);
        int offX = DELTA / 2;
        g1.setFont( getFont().deriveFont(Font.ITALIC) );
        g1.setColor(TITLE_COLOR);
        g1.fillRect(b.x + 1, b.y + 1, b.width - 2, fm.getHeight() - 1);
        g1.setColor(getForeground());
        g1.drawString(getTable().getName(), b.x + (b.width - tableTitleWidth) / 2 + offX, b.y + ascent);
        int y1 = b.y + fm.getHeight();
        g1.drawLine(b.x, y1, b.x + b.width - 1, y1);
        int y = y1 + 1;
        g1.setFont(getFont());
        for (int i = 0; i < _model.getRowCount(); i++) {
            int x = b.x + offX;
            for (int c = 0; c < _model.getColumnCount(); c++) {
                Object o = _model.getValueAt(i, c);
                g1.drawString(o == null ? "" : o.toString(), x, y + ascent);
                x += getColumnWidths().get(c);
            }
            y += fm.getHeight();
        }
    }

    public Table getTable() {
        return (Table) getObject();
    }

    public void setObject(Object object) {
        super.setObject(object);
        getTableModel().setTable((Table) object);
        calculateSize();
    }

    private static class TableDataModel extends AbstractTableModel {
        private Table _table;
        private String[] _columnNames = new String[]{"name", "typeString", "key", "nullable"};
        protected PropertyDescriptor[] _columnPds;


        public TableDataModel() {
            try {
                BeanInfo info = Introspector.getBeanInfo(Column.class);
                _columnPds = info.getPropertyDescriptors();
            } catch (IntrospectionException e) {
                throw new RuntimeException(e);
            }
        }

        public TableDataModel(Table table) {
            this();
            setTable(table);
        }


        public String getColumnName(int column) {
            return _columnNames[column];
        }

        public Table getTable() {
            return _table;
        }

        public void setTable(Table table) {
            _table = table;
//            fireTableDataChanged();
        }

        public int getRowCount() {
            Column[] columns = _table.getColumns();
            return columns == null ? 0 : columns.length;
        }

        public int getColumnCount() {
            return _columnNames.length;
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            Column column = _table.getColumns()[rowIndex];
            String columnName = getColumnName(columnIndex);
            if (columnName.equals("key")) {
                boolean pk = column.isPrimaryKey();
                boolean fk = column.isForeignKey();
                return pk && fk ? "PF" : (pk ? "PK" : (fk ? "FK" : ""));
            }
            for (PropertyDescriptor pd : _columnPds) {
                if (pd.getName().equals(columnName)) {
                    try {
                        Object o = pd.getReadMethod().invoke(column);
                        String value = o == null ? null : o.toString();
                        if (columnName.equals("nullable")) {
                            return "0".equals(value) ? "N" : "Y";
                        } else if (columnName.equals("columnDef")) {
                            return null == value || "NULL".equalsIgnoreCase(value) ? "" : value;
                        }
                        return o;
                    } catch (Exception e) {
                        Utils.debug("TableBox2$TableDataModel.getValueAt: " + e);
                    }
                }
            }
            return null;
        }
    }
}
