package template;

import org.apache.velocity.Template;
import org.apache.velocity.context.Context;
import utils.Config;
import utils.Utils;

import java.io.File;
import java.io.IOException;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Created by IntelliJ IDEA.
 * User: huojun
 * Date: 12-Oct-2005
 * Time: 5:25:34 PM
 */
public class TemplateDescriptor {
    private String _name;
    private String _target;
    private String _databaseName;
    private long _timestamp;
    private String _fileName;
    private File _file;
    private Template _template;
    private static File _root = Config.getConfig().getTemplateFileFolder().getFile();

    public String getDatabaseName() {
        return _databaseName;
    }

    public void setDatabaseName(String driver) {
        _databaseName = driver;
    }

    public String getName() {
        return _name;
    }

    public void setName(String name) {
        _name = name;
    }

    public String getFileName() {
        return _fileName;
    }

    public void setFileName(String fileName) {
        String[] tokens = fileName.split("/");
        File f = null;
        for(String t : tokens) {
            f = new File(f == null ? _root : f, t);
        }
        Utils.debug("fileName = " + f);
        if( null != f && f.exists() ) {
            _file = f;
            setTimestamp(f.lastModified());
        } else {
            Utils.debug("Invalid template file name: " + fileName);
        }
        _fileName = fileName;
    }

    public boolean isDirty() {
        return null == _file || null == _template || _file.lastModified() != getTimestamp();
    }

    public long getTimestamp() {
        return _timestamp;
    }

    private void setTimestamp(long timestamp) {
        _timestamp = timestamp;
    }

    public String getTarget() {
        return _target;
    }

    public void setTarget(String target) {
        _target = target;
    }

    public List<TemplateParameter> getRuntimeParams() throws IOException {
        String text = Utils.getFileText(getFile());
//        Utils.debug("text = " + text);
        Pattern pattern = Pattern.compile("\\$runtime_([a-zA-Z_\\d]+)(\\((.+)\\))?");
        Matcher matcher = pattern.matcher(text);
        List<TemplateParameter> a = new ArrayList<TemplateParameter>();
        while (matcher.find()) {
            String name = matcher.group(1);
            String defValue = matcher.group(3);
            TemplateParameter p = new TemplateParameter(name, defValue);
//            Utils.debug("o = " + o);
            if ( !a.contains(p) ) {
                a.add(p);
            }
        }
//        Utils.debug("a = " + a);
        return a;
    }

    public String toString() {
        return getName();
    }

    public String execute(Context context) throws Exception {
        if( isDirty() ) {
            _template = TemplateManager.getTemplate(getFileName());
            _timestamp = _file.lastModified();
        }
        StringWriter w = new StringWriter();
        _template.merge(context, w);
        return w.toString();
    }

    public File getFile() {
        return _file;
    }

    public static class TemplateParameter {
        private String _name;
        private String _defaultValue;

        public TemplateParameter(String name, String defaultValue) {
            _name = name;
            _defaultValue = defaultValue;
        }


        public String getName() {
            return _name;
        }

        public void setName(String name) {
            _name = name;
        }

        public String getDefaultValue() {
            return _defaultValue;
        }

        public void setDefaultValue(String defaultValue) {
            _defaultValue = defaultValue;
        }

        public TemplateParameter() {
            setName("");
        }


        public String toString() {
            return _name + "(" + _defaultValue + ")";
        }

        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            TemplateParameter that = (TemplateParameter) o;

            return _name.equals(that._name);

        }

        public int hashCode() {
            return _name.hashCode();
        }
    }
}
