package template;

import bean.PropertyChangeMultiCaster;
import dbhelp.db.Column;
import dbhelp.db.Database;
import dbhelp.db.Procedure;
import dbhelp.db.Table;
import dbhelp.plugin.action.portable.PortableAction;
import dbhelp.plugin.action.portable.RunTemplateAction;
import org.apache.velocity.Template;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.context.Context;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import utils.Config;
import utils.Utils;

import javax.swing.*;
import javax.xml.parsers.DocumentBuilderFactory;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.IOException;
import java.io.Reader;
import java.io.Writer;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: huojun
 * Date: 13-Oct-2005
 * Time: 1:48:29 PM
 */
public class TemplateManager extends HashSet<TemplateDescriptor> {
    private static TemplateManager _mgr = new TemplateManager();
    private static final String KEY_FILE_RESOURCE_LOADER_PATH = "file.resource.loader.path";
    protected VelocityEngine _engine;

    public static void main(String[] args) throws IOException {
        initialize();
        TemplateDescriptor td = getTemplateDescriptor("Parameter Test");
        List<TemplateDescriptor.TemplateParameter> runtimeParams = td.getRuntimeParams();
        System.out.println("runtimeParams = " + runtimeParams);
    }

    private static List<TemplateDescriptor> getDescriptors() {
        List<TemplateDescriptor> a = new ArrayList<TemplateDescriptor>();
        try {
            File root = Config.getConfig().getTemplateFileFolder().getFile();
            if ( null != root ) {
                File metaFile = new File(root, "templates.xml");
                Utils.debug("metaFile = " + metaFile);
                if (metaFile.exists()) {
                    DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
                    Document document = dbf.newDocumentBuilder().parse(metaFile);
                    NodeList templates = document.getElementsByTagName("template");
                    for (int i = 0; null != templates && i < templates.getLength(); i++) {
                        Element te = (Element) templates.item(i);
                        TemplateDescriptor td = new TemplateDescriptor();
                        td.setTarget(te.getAttribute("target"));
                        td.setName(te.getAttribute("name"));
                        td.setDatabaseName(te.getAttribute("database"));
                        td.setFileName(te.getAttribute("file"));
                        Utils.debug("Template: " + td + ", at: " + td.getFile());
                        a.add(td);
                    }
                } else {
                    Utils.debug(metaFile + " doesn't exist, no template will be collected!");
                }
            }  else {
                Utils.debug("Template folder doesn't exist, no template will be collected!");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return a;
    }

    private VelocityEngine getEngine() {
        if( null == _engine ) {
            throw new IllegalStateException("Call initialize first!");
        }
        return _engine;
    }

    private void setEngine(VelocityEngine engine) {
        _engine = engine;
    }

    public static void initialize() {
        Utils.debug("TemplateManager.initialize");
        try {
            Properties p = new Properties();
            File f = Config.getConfig().getTemplateFileFolder().getFile();
            String value = f.toURL().getFile();
            int index = value.indexOf(':');
            value = index > 0 ? value.substring(index + 1) : value;
            VelocityEngine velocityEngine = new VelocityEngine();
            p.setProperty(KEY_FILE_RESOURCE_LOADER_PATH, value);
            p.setProperty("directive.foreach.counter.name", "velocityCount");
            p.setProperty("directive.foreach.counter.initial.value", "0");
            velocityEngine.init(p);
            getMgr().setEngine(velocityEngine);
            loadTemplates();
            PropertyChangeMultiCaster.getInstance().addListener(Config.class, new PropertyChangeListener() {
                public void propertyChange(PropertyChangeEvent evt) {
                    if( evt.getPropertyName().equals("templateFileFolder") ) {
                        loadTemplates();
                    }
                }
            });
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    private static void loadTemplates() {
        TemplateManager mgr = getMgr();
        mgr.clear();
        mgr.addAll(getDescriptors());
    }

    public static TemplateManager getMgr() {
        return _mgr;
    }

    private TemplateManager() {
    }

    public static Set<PortableAction> getActions(Set<Object> keys) {
        Set<PortableAction> actions = new HashSet<PortableAction>();
        for(Object key : keys) {
            String target = keyToTargetName(key);
            if( null != target ) {
                actions.addAll(getActions(target, key));
            }
        }
        return actions;
    }

    public static TemplateDescriptor getTemplateDescriptor(String name) {
        for (TemplateDescriptor td : _mgr) {
            if( td.getName().equals(name) ) {
                return td;
            }
        }
        return null;
    }

    private static String keyToTargetName(Object key) {
        if( key instanceof Table ) {
            return "table";
        } else if( key instanceof Table[] ) {
            return "tables";
        } else if( key instanceof JTable ) {
            return "resultset";
        } else if( key instanceof Column ) {
            return "column";
        } else if( key instanceof Column[] ) {
            return "columns";
        } else if( key instanceof Procedure ) {
            return "procedure";}
        else if( key instanceof Procedure[] ) {
            return "procedures";
        } else if( key instanceof Database ) {
            return "database";
        }
        return null;
    }

    public static List<PortableAction> getActions(String target, Object key) {
        List<PortableAction> actions = new ArrayList<PortableAction>();
//        Database db = getDatabaseFromKeyObject(key);
//        Utils.debug("key:" + key + ", key to database = " + db);
//        String driver = null == db ? null : db.getDriver();
        for(TemplateDescriptor td : getMgr()) {
            if( td.getTarget().equals(target) ) {
//                String dbName = td.getDatabaseName();
//                if ( null == driver || !Utils.hasValue(dbName) || driver.indexOf(dbName) >= 0) {
                    // checkRead database name to make sure use property templates
                    RunTemplateAction o = new RunTemplateAction(td);
                    o.setTarget(key);
                    actions.add( o );
//                }
            }
        }
        return actions;
    }

//    private static Database getDatabaseFromKeyObject(Object key) {
//        Database db = null;
//        if( key instanceof Database ) {
//            db = (Database) key;
//        } else if( key instanceof Table) {
//            db = ((Table)key).getDatabase();
//        } else if (key instanceof Procedure) {
//            Procedure procedure = (Procedure) key;
//            db = procedure.getDatabase();
//        } else if (key instanceof Table[]) {
//            Table[] ts = (Table[])key;
//            db = ts.length > 0 ? ts[0].getDatabase() : null;
//        } else if (key instanceof Procedure[]) {
//            Procedure[] ps = (Procedure[]) key;
//            db = ps.length > 0 ? ps[0].getDatabase() : null;
//        } else if (key instanceof Column) {
//            Column column = (Column) key;
//            db = column.getTable().getDatabase();
//        } else if (key instanceof Column[]) {
//            Column[] columns = (Column[]) key;
//            db = columns.length > 0 ? columns[0].getTable().getDatabase() : null;
//        }
//        return db;
//    }

    public boolean runTemplate(Context c, Writer out, Reader in) throws Exception {
        return getMgr().getEngine().evaluate(c, out, "template run", in);
    }

    public static Template getTemplate(String fileName) throws Exception {
        return getMgr().getEngine().getTemplate(fileName);
    }

    public String runTemplate(String templateName, Context c) {
        for (TemplateDescriptor d : getMgr()) {
            if (d.getName().equals(templateName)) {
                RunTemplateAction a = new RunTemplateAction(d);
                try {
                    return a.runTemplate(c);
                } catch (Exception e) {
                    Utils.debug(e);
                }
            }
        }
        return null;
    }
}
