package utils;

import bean.annotation.Property;
import bean.editor.FileHolder;
import dbhelp.db.Database;
import dbhelp.db.Query;

import java.beans.XMLDecoder;
import java.io.*;

/**
 * User: huojun
 * Date: 12-Sep-2006
 * Time: 12:20:32 PM
 */
public class Config {
    private static Config _instance;
    private Database[] _databases = new Database[]{};
    private FileHolder _templateFileFolder;
    private boolean _showTemplateInClipboardMessage = true;
    private Query[] _savedQueries;
    private FileHolder[] _extensionJars;
    private boolean _packResultTable = true;
    private long _lastmodified;
    private boolean _debug;
    private boolean _showSearchTip;
    private boolean _useInternalTemplates = true;
    private boolean _askAutoAddRefTables = true;
    private boolean _autoAddRefTables = true;

    public Config() {
    }

    public static Config getConfig() {
        if( null == _instance ) {
            _instance = new Config();
        }
        return _instance;
    }

    public boolean isDebug() {
        return _debug;
    }

    public void setDebug(boolean debug) {
        _debug = debug;
        Utils.setDebug(_debug);
    }


    public boolean isAskAutoAddRefTables() {
        return _askAutoAddRefTables;
    }

    public void setAskAutoAddRefTables(boolean askAutoAddRefTables) {
        _askAutoAddRefTables = askAutoAddRefTables;
    }

    public boolean isAutoAddRefTables() {
        return _autoAddRefTables;
    }

    public void setAutoAddRefTables(boolean autoAddRefTables) {
        _autoAddRefTables = autoAddRefTables;
    }

    public Query[] getSavedQueries() {
        return _savedQueries;
    }

    @Property(desc = "Uncheck this option if you modified the templates and add your own templates. If there are new " +
            "templates included in the new build, DBHelper will not add the new templates. You need to extract the templates" +
            "from the jar file and add them manually")
    public boolean isUseInternalTemplates() {
        return _useInternalTemplates;
    }

    public void setUseInternalTemplates(boolean useInternalTemplates) {
        _useInternalTemplates = useInternalTemplates;
    }

    public boolean isShowSearchTip() {
        return _showSearchTip;
    }

    public void setShowSearchTip(boolean showSearchTip) {
        _showSearchTip = showSearchTip;
    }

    public void setSavedQueries(Query[] savedQueries) {
        _savedQueries = savedQueries;
    }

    public long getLastmodified() {
        return _lastmodified;
    }

    public FileHolder[] getExtensionJars() {
        return _extensionJars;
    }

    public void setExtensionJars(FileHolder[] extensionJars) {
        _extensionJars = extensionJars;
    }

    public void setLastmodified(long lastmodified) {
        _lastmodified = lastmodified;
    }

    public void store(File f) throws IOException {
        OutputStream o = new FileOutputStream(f);
        Utils.store(o, this);
        o.close();
    }

    public boolean isPackResultTable() {
        return _packResultTable;
    }

    public void setPackResultTable(boolean packResultTable) {
        _packResultTable = packResultTable;
    }

    public FileHolder getTemplateFileFolder() {        
        return _templateFileFolder;
    }

    @Property(editor = "bean.editor.FileEditor")
    public void setTemplateFileFolder(FileHolder templateFileFolder) {
        _templateFileFolder = templateFileFolder;
    }

    public static Config restore(File f) throws Exception {
        _instance = restore(f, Config.class.getClassLoader());
        _instance.setLastmodified(f.lastModified());
        return _instance;
    }

    public static Config restore(File f, ClassLoader classLoader) throws IOException {
        InputStream in = new FileInputStream(f);
        Config cb = restore(in, classLoader);
        in.close();
        return cb;
    }

    public static Config restore(InputStream in) {
        return restore(in, Config.class.getClassLoader());
    }

    private static Config restore(InputStream in, ClassLoader cl) {
        XMLDecoder x = cl == null ? new XMLDecoder(in) : new XMLDecoder(in, null, null, cl);
        return (Config) x.readObject();
    }

    public boolean isShowTemplateInClipboardMessage() {
        return _showTemplateInClipboardMessage;
    }

    public void setShowTemplateInClipboardMessage(boolean b) {
        _showTemplateInClipboardMessage = b;
    }

    public String getVersion() {
        return "1.1.0030";
    }

    public Database[] getDatabases() {
        return _databases;
    }

    public void setDatabases(Database[] databases) {
        _databases = databases;
    }

    public static Config restore() throws Exception {
        return _instance = restore(Utils.getConfigFile());
    }
}
