package utils;

import dbhelp.plugin.DBBrowserPane;
import dbhelp.plugin.OutputPane;
import dbhelp.plugin.action.portable.PopupMenuManager;
import dbhelp.plugin.complete.AbstractCompleteWindow;

/**
 * User: huojun
 * Date: 5-Oct-2006
 * Time: 2:04:53 PM
 */
public abstract class SingletonProvider {
    private static SingletonProvider _provider = null;

    abstract public PopupMenuManager getPopupMenuManager();

    abstract public DBBrowserPane getDbBrowserPane();

    abstract public OutputPane getOutputPane();

    abstract public Config getConfig();

    abstract public DataCollector getDataCollector();

    abstract public AbstractCompleteWindow getCompleteWindow();

    public synchronized static void createProvider(String className) throws Throwable {
        _provider = (SingletonProvider) Class.forName(className).newInstance();
    }

    public synchronized static SingletonProvider getProvider() {
        if( null == _provider ) {
            Utils.debug("Using default Singleton provider!");
            _provider = new DefaultProvider();
        }
        return _provider;
    }

    public static class DefaultProvider extends SingletonProvider {
        private PopupMenuManager _popupMenuManager = null;
        private DBBrowserPane _dbBrowserPane = null;
        private OutputPane _outputPane = null;
        private Config _config = Config.getConfig();
        private DataCollector _dataCollector = null;
        private AbstractCompleteWindow _completeWindow = null;

        public PopupMenuManager getPopupMenuManager() {
            if (null == _popupMenuManager) {
                _popupMenuManager = new PopupMenuManager();
            }
            return _popupMenuManager;
        }

        public DBBrowserPane getDbBrowserPane() {
            if (null == _dbBrowserPane) {
                _dbBrowserPane = new DBBrowserPane();
            }
            return _dbBrowserPane;
        }

        public OutputPane getOutputPane() {
            if (null == _outputPane) {
                _outputPane = new OutputPane();
            }
            return _outputPane;
        }

        public Config getConfig() {
            return _config;
        }

        public DataCollector getDataCollector() {
            if( null == _dataCollector ) {
                _dataCollector = new DataCollector();
            }
            return _dataCollector;
        }

        public AbstractCompleteWindow getCompleteWindow() {
            return _completeWindow;
        }
    }
}
