package utils;

import javax.swing.*;
import javax.swing.table.*;
import java.awt.*;

/**
 * User: huojun
 * Date: 14-Sep-2006
 * Time: 12:21:30 PM
 */

public final class TablePacker {

    public static void packAsync(JTable table, int margin) {
        Thread t = new Thread(new PackingRunnable(table, margin));
        t.start();
//        SwingUtilities.invokeLater(new PackingRunnable(table, margin));
    }


    public static int packTable(final JTable t, final int margin) {
        int width = 0;
        for (int c = 0; c < t.getColumnCount(); c++) {
            width += packColumn(t, c, margin);
        }
        return width;
    }

    public static int packColumn(JTable table, int vColIndex, int margin) {
        DefaultTableColumnModel colModel = (DefaultTableColumnModel) table.getColumnModel();
        TableColumn col = colModel.getColumn(vColIndex);
        int pcw = getPreferredColumnWidth(table, vColIndex, margin);
        if (pcw >= 0) {
            col.setPreferredWidth(pcw);
        }
        return pcw;
    }
// Sets the preferred width of the visible column specified by vColIndex. The column

    // will be just wide enough to show the column head and the widest cell in the column.
    // margin pixels are added to the left and right
    // (resulting in an additional width of 2*margin pixels).
    public static int getPreferredColumnWidth(JTable table, int vColIndex, int margin) {
        DefaultTableColumnModel colModel = (DefaultTableColumnModel) table.getColumnModel();
        TableColumn col = colModel.getColumn(vColIndex);
        int width;

        // Get width of column header
        TableCellRenderer renderer = col.getHeaderRenderer();
        if (renderer == null) {
            JTableHeader header = table.getTableHeader();
            if (null != header) {
                renderer = header.getDefaultRenderer();
            }
        }
        Component comp;
        if (renderer != null) {
            comp = renderer.getTableCellRendererComponent(
                    table, col.getHeaderValue(), false, false, 0, 0);
            width = comp.getPreferredSize().width;
        } else {
            width = 0;
        }

        // Get maximum width of column data
        for (int r = 0; r < table.getRowCount(); r++) {
            renderer = table.getCellRenderer(r, vColIndex);
            comp = renderer.getTableCellRendererComponent(
                    table, table.getValueAt(r, vColIndex), false, false, r, vColIndex);
            width = Math.max(width, comp.getPreferredSize().width);
        }

        // Add margin
        width += 2 * margin;

        // Set the width
//        col.setPreferredWidth(width);
        return width;
    }

    private static class PackingRunnable implements Runnable {
        private final JTable _table;
        private final int _margin;

        public PackingRunnable(JTable table, int margin) {
            _table = table;
            _margin = margin;
        }

        public void run() {
            try {
                Thread.sleep(500);
            } catch (InterruptedException e) {
                //
            }
            final TableColumnModel cm = _table.getColumnModel();
            final int columnCount = cm.getColumnCount();
            final int[] widths = new int[columnCount];
            for (int c = 0; c < columnCount; c++) {
                widths[c] = getPreferredColumnWidth(_table, c, _margin);
                final int c1 = c;
                SwingUtilities.invokeLater(new Runnable() {
                    public void run() {
                        cm.getColumn(c1).setPreferredWidth(widths[c1]);
                    }
                });
            }
        }
    }
}
