package utils;

import bean.editor.FileHolder;
import utils.ui.ErrorDialog;

import javax.swing.FocusManager;
import javax.swing.*;
import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.ActionEvent;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.beans.XMLEncoder;
import java.io.*;
import java.lang.reflect.Array;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.*;
import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

/**
 * User: huojun
 * Date: 12-Sep-2006
 * Time: 1:06:19 PM
 */
public class Utils {
    private static boolean _debug;

    public static boolean isDebug() {
        return _debug;
    }

    public static void setDebug(boolean debug) {
        _debug = debug;
    }

    public static File getConfigFile() {
        String homeDir = System.getProperty("user.home");
        return new File(homeDir, "dbhelper.xml");
    }


    public static boolean hasValue(Object o) {
        return o != null && o.toString().trim().length() > 0;
    }

    public static void error(Throwable e) {
        ErrorDialog.showDialog("Error", e);
    }

    public static void errorMsg(Throwable e) {
        e.printStackTrace();
        String message = e.getMessage();
        Window aw = FocusManager.getCurrentManager().getActiveWindow();
        Object text = message == null ? e : message;
        JOptionPane.showMessageDialog(aw, text);
    }

    public static void message(Object o) {
        JOptionPane.showMessageDialog(FocusManager.getCurrentManager().getActiveWindow(), o == null ? "[Null]" : o);
    }

    public static Object activate(String className, FileHolder[] jars) throws MalformedURLException, ClassNotFoundException, InstantiationException, IllegalAccessException {
        if (null != jars && jars.length > 0) {
            URL[] urls = new URL[jars.length];
            for (int i = 0; i < jars.length; i++) {
                urls[i] = jars[i].getFile().toURL();
            }
            URLClassLoader l = new URLClassLoader(urls, Utils.class.getClassLoader());
            Class<?> aClass = l.loadClass(className);
            return aClass.newInstance();
        } else {
            return Class.forName(className).newInstance();
        }
    }

    public static String[] searchClassInJars(FileHolder[] jars, Class classToSearch) {
        List<URL> a = new ArrayList<URL>();
        for (int i = 0; null != jars && i < jars.length; i++) {
            try {
                a.add(jars[i].getFile().toURL());
            } catch (MalformedURLException e) {
                Utils.debug(e.getMessage());
            }
        }
        URL[] urls = a.toArray(new URL[a.size()]);
        URLClassLoader cl = new URLClassLoader(urls, Utils.class.getClassLoader());
        String suffix = ".class";
        List<String> classes = new ArrayList<String>();
        for (int i = 0; null != jars && i < jars.length; i++) {
            try {
                JarFile jf = new JarFile(jars[i].getFile());
                Enumeration<JarEntry> enumeration = jf.entries();
                while (enumeration.hasMoreElements()) {
                    JarEntry zf = enumeration.nextElement();
                    String name = zf.getName();
                    if (name.endsWith(suffix)) {
                        String className = name.substring(0, name.length() - suffix.length()).replace('/', '.');
                        try {
                            Class<?> cls = cl.loadClass(className);
                            if (!cls.isInterface() && classToSearch.isAssignableFrom(cls)) {
                                classes.add(className);
                            }
                        } catch (Throwable e) {
                            Utils.debug("e = " + e);
                        }
                    }
                }
            } catch (IOException e) {
                Utils.debug("e = " + e);
            }
        }
        return classes.toArray(new String[classes.size()]);
    }

    public static void dumpArray(Object[] a) {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; null != a && i < a.length; i++) {
            Object o = a[i];
            if (sb.length() > 0) {
                sb.append(", ");
            }
            if (o instanceof String || o instanceof Character || o instanceof Date) {
                sb.append('\'').append(o).append('\'');
            } else {
                sb.append(o == null ? "null" : o);
            }
        }
        Log.message("With parameter(s): %1$s", sb);

    }

    public static boolean in(String token, String... tokens) {
        for (String t : tokens) {
            if (t.equalsIgnoreCase(token)) {
                return true;
            }
        }
        return false;
    }

    public static String getFileText(String name) throws IOException {
        return getFileText(new File(name));
    }

    public static File createFile(File file, Reader data) throws IOException {
        FileWriter f = null;
        try {
            f = new FileWriter(file);
            if (data != null) {
                char[] buf = new char[1024];
                int chars = data.read(buf);
                while (chars != -1) {
                    f.write(buf, 0, chars);
                    chars = data.read(buf);
                }
            }
        } finally {
            if (null != f) {
                f.close();
            }
        }         
        return file;
    }

    public static FileHolder getDefaultTemplateFolder() {
        String homeDir = System.getProperty("user.home");
        File f = new File(homeDir, "dbhelp");
        f = new File(f, "templates");
        if (!f.exists()) {
            f.mkdirs();
        }
        return new FileHolder(f);
    }

    public static void copyToClipboard(String text) {
        StringSelection ss = new StringSelection(text);
        Clipboard cp = Toolkit.getDefaultToolkit().getSystemClipboard();
        cp.setContents(ss, null);
    }

    public static String getFileText(File file) throws IOException {
        FileInputStream f = new FileInputStream(file);
        StringWriter sw = new StringWriter();
        BufferedReader r = new BufferedReader(new InputStreamReader(f));
        String line = r.readLine();
        while (null != line) {
            sw.write(line);
            sw.write('\n');
            line = r.readLine();
        }
        f.close();
        return sw.toString();
    }

    public static Object collectionToTypeArray(List<Object> values, Class<? extends Object> aClass) {
        if (null != values) {
            Object array = Array.newInstance(aClass, values.size());
            for (int i = 0; i < values.size(); i++) {
                Array.set(array, i, values.get(i));
            }
            return array;
        }
        return null;
    }

    public static void store(File f, Object object) throws IOException {
        FileOutputStream out = new FileOutputStream(f);
        store(out, object);
        out.close();
    }

    public static void store(OutputStream out, Object object) {
        Thread thread = Thread.currentThread();
        ClassLoader ccl = thread.getContextClassLoader();
        thread.setContextClassLoader(object.getClass().getClassLoader());
        XMLEncoder e = new XMLEncoder(out);
        e.writeObject(object);
        e.flush();
        e.close();
        thread.setContextClassLoader(ccl);
    }

    public static Component getFrame() {
        return FocusManager.getCurrentManager().getActiveWindow();
    }

    public static void debug(Object o) {
        if (_debug) {
            System.out.println(o);
        }
    }

    public static void debug(String fmt, Object... args) {
        if(_debug) {
            System.out.printf(fmt, args);
        }
    }

    public static String arrayToString(Object[] a) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; null != a && i < a.length; i++) {
            if (sb.length() > 0) {
                sb.append(", ");
            }
            sb.append(a[i]);
        }
        return sb.toString();
    }

    public static void sureValue(String name, String value) throws Exception {
        if( !hasValue(value) ) {
            throw new IllegalArgumentException(name + " is a required field!");
        }
    }

    public static FileHolder[] getJarList(FileHolder[] ...jars) {
        List<FileHolder> list = new ArrayList<FileHolder>();
        if ( null != jars ) {
            for (FileHolder[] fh : jars) {
                if (null != fh) {
                    list.addAll(Arrays.asList(fh));
                }
            }
        }
        return list.toArray(new FileHolder[list.size()]);
    }

    public static File createDBHelperFolder(String name) {
        File f = new File(System.getProperty("user.home"));
        f = new File(f, "DBHelper");
        f = new File(f, name);
        f.mkdirs();
        return f;
    }

    public static File mkdirs(File templateFolder, String path) {
        File f = templateFolder;
        if ( hasValue(path) ) {
            String[] folders = path.split("/");
            for (String folder : folders) {
                f = new File(f, folder);
            }
            f.mkdirs();
        }
        return f;
    }

    public static void addEscClose(final JDialog dialog, ActionESCClose action) {
        int whenInFocusedWindow = JComponent.WHEN_IN_FOCUSED_WINDOW;
        KeyStroke escapeKeyStroke = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0, false);
        JRootPane rootPane = dialog.getRootPane();
        rootPane.getInputMap(whenInFocusedWindow).put(escapeKeyStroke, "ESCAPE");
        action = action == null ? new ActionESCClose(dialog) : action;
        rootPane.getActionMap().put("ESCAPE", action);
    }

    public static void addEscClose(JDialog dialog) {
        addEscClose(dialog, null);
    }

    private static class ActionESCClose extends AbstractAction {
        private final JDialog _dialog;

        public ActionESCClose(JDialog dialog) {
            _dialog = dialog;
        }

        public void actionPerformed(ActionEvent e) {
            _dialog.dispose();
        }
    }
}
