package utils.text;

/**
 * Created by IntelliJ IDEA.
 * User: David Huo
 * Date: Sep 19, 2006
 * Time: 3:11:18 PM
 * To change this template use File | Settings | File Templates.
 */
public class BackwardsEnumerator implements IWordEnumeration {
    private int _index;
    private CharSequence _text;
    private TYPE _type = TYPE.NULL;
    private String _operators = WordScanner.DEFAULT_OPERATORS;

    public BackwardsEnumerator(CharSequence text, int index) {
        _index = index;
        _text = text;
    }

    public String getOperators() {
        return _operators;
    }

    public void setOperators(String operators) {
        _operators = operators;
    }

    public void setIndex(int offset) {
        _index = offset;
    }

    public int getIndex() {
        return _index;
    }

    public int getLine() {
        int line = 0;
        String text = _text == null ? "" : _text.toString();
        int index = text.indexOf('\n');
        while( index >= 0 ) {
            if( index >= _index ) {
                return line;
            }
            line++;
            index = text.indexOf('\n', index+1);
        }
        return line;
    }

    public boolean hasMoreElements() {
        return null != _text && _index >= 0;
    }

    private char peek(int offset) {
        return offset >= 0 ? _text.charAt(offset) : 0;
    }

    public CharSequence nextElement() {
        char c = _text.charAt(_index);
        if (c == '\n') {
            return consumeNewLine(_index);
        } else if (Character.isWhitespace(c)) {
            return consumeWS(_index);
        } else if (isCommentEnd(_index)) {
            return consumeComment(_index);
        } else if (isOperator(_index)) {
            return consumeOperator(_index);
        } else {
            return consumeChars(_index);
        }
    }


    private CharSequence consumeNewLine(int index) {
        _type = TYPE.NEWLINE;
        _index = index - 1;
        return _text.subSequence(index, index + 1);
    }

    private CharSequence consumeChars(int index) {
        int p = index;
        while (p >= 0) {
            char c = peek(p);
            if (Character.isWhitespace(c)
                    || isOperator(p)
                    || isCommentEnd(p)
                    ) {
                break;
            } else {
                p--;
            }
        }
        _index = p;
        CharSequence cs = _text.subSequence(p + 1, index + 1);
        _type = WordScanner.isNumber(cs) ? TYPE.NUMBER : TYPE.WORD;
        return cs;
    }

    private CharSequence consumeComment(int index) {
        int p = index;
        while (p >= 0) {
            char c = peek(p);
            char c1 = peek(p - 1);
            if (c == '*' && c1 == '/') {
                p = p - 2;
                break;
            }
            p--;
        }
        _index = p;
        _type = TYPE.MULTILINE_COMMENT;
        return _text.subSequence(p < 0 ? 0 : p, index + 1);
    }

    private boolean isCommentEnd(int index) {
        char c = peek(index);
        char c1 = peek(index - 1);
        return c == '/' && c1 == '*';
    }

    private CharSequence consumeOperator(int index) {
        char c = peek(index);
        char c1 = peek(index - 1);
        int n = ((c == '|' && c1 == '|')
                || (c == '&' && c1 == '&')
                || (c == '+' && c1 == '+')
                || (c == '=' && c1 == '+')
                || (c == '=' && c1 == '-')
                || (c == '=' && c1 == '%')
                || (c == '=' && c1 == '*')
                || (c == '=' && c1 == '/')
                || (c == '*' && c1 == '/')
                || (c == '/' && c1 == '*')
                || (c == '-' && c1 == '-')
        ) ? 1 : 0;
        _index = _index - (n + 1);
        _type = TYPE.OPERATOR;
        return _text.subSequence(index - n, index + 1);
    }


    private boolean isOperator(int index) {
        char c = peek(index);
        return _operators.indexOf(c) >= 0;
    }

    private CharSequence consumeWS(int index) {
        int p = index;
        while (p >= 0) {
            char c = peek(p);
            if (!Character.isWhitespace(c) || c == '\n') {
                break;
            } else {
                p--;
            }
        }
        _index = p;
        _type = TYPE.WHITESPACES;
        return _text.subSequence(p + 1, index + 1);
    }

    public TYPE getType() {
        return _type;
    }

    public int getOffset() {
        return _index + 1;
    }
}
