package utils.text;

import java.util.List;
import java.util.Arrays;

/**
 * Created by IntelliJ IDEA.
 * User: David Huo
 * Date: Sep 19, 2006
 * Time: 3:15:32 PM
 * To change this template use File | Settings | File Templates.
 */
public class TokenIterator implements ITokenEnumeration {
    private IWordEnumeration _source;

    public TokenIterator(IWordEnumeration source) {
        _source = source;
    }

    public TYPE getType() {
        return _source.getType();
    }

    public String lookahead(int n) {
        int offset = _source.getIndex();
        int count = 0;
        CharSequence ret = null;
        while (hasMoreElements()) {
            CharSequence o = nextElement();
            count++;
            if (count == n) {
                ret = o;
                break;
            }
        }
        _source.setIndex(offset);
        return ret == null ? null : ret.toString();


    }

    public int getOffset() {
        return _source.getOffset();
    }

    public boolean hasMoreElements() {
        return _source.hasMoreElements();
    }

    public CharSequence nextElement() {
        CharSequence o = _source.nextElement();
        TYPE t = getType();
        while (t.equals(TYPE.MULTILINE_COMMENT)
                || t.equals(TYPE.SINGLELINE_COMMENT)
                || t.equals(TYPE.WHITESPACES)
                || t.equals(TYPE.NEWLINE)) {
            if (_source.hasMoreElements()) {
                o = _source.nextElement();
                t = getType();
            } else {
                return null;
            }
        }
        return o;
    }

    public void setIndex(int offset) {
        _source.setIndex(offset);
    }

    public String consumeUntil(String... tokens) {
        List<String> strings = Arrays.asList(tokens);
        int n=0;
        while (hasMoreElements()) {
            CharSequence t = nextElement();
            if (null != t) {
                String s = t.toString();
                if ("(".equals(s)) {
                    n++;
                }
                if (strings.contains(s.toUpperCase())) {
                    if( ")".equals(s) ) {
                        n--;
                        if ( n > 0 ) {
                            continue;
                        }
                    }
                    return s;
                }
            }
        }
        return null;
    }

    public String lookahead(String... tokens) {
        List<String> strings = Arrays.asList(tokens);
        int offset = _source.getIndex();
        while (hasMoreElements()) {
            CharSequence t = nextElement();
            if (null != t) {
                if (strings.contains(t.toString().toUpperCase())) {
                    _source.setIndex(offset);
                    return t.toString();
                }
            }
        }
        _source.setIndex(offset);
        return null;
    }

    public String nextElement(int i) {
        int n = 0;
        while (hasMoreElements()) {
            CharSequence t = nextElement();
            n++;
            if (n == i) {
                return null == t ? null : t.toString();
            }
        }
        return null;
    }

    public int getIndex() {
        return _source.getIndex();
    }

    public int getLine() {
        return _source.getLine();
    }
}
