package utils.text;

import utils.Utils;
import static utils.text.IWordEnumeration.TYPE;
import static utils.text.IWordEnumeration.TYPE.*;

import java.io.IOException;

/**
 * User: huojun
 * Date: 14-Sep-2006
 * Time: 3:32:50 PM
 */
public class WordScanner {
    public static final String DEFAULT_OPERATORS = "{}[]?/-+()<>&*^%;:'=|,.~";
    protected CharSequence _text;
    protected String _operators = DEFAULT_OPERATORS;

    public static void main(String[] args) throws IOException {
        String txt = "'a'; test */.06;. 'asademo.dba.table'exxx this2 && /* " +
                "multiline \n" +
                "comments " +
                "ents */ \t \n{is} \"a b\".cxx    \t ' a ''''stringcomment \n/*" +
                "wh ''', '''\n" +
                "--";
//        WordScanner ws = new WordScanner(txt);
//        ITokenEnumeration tokens = WordScanner.tokenScan(txt, 0);
//        while( tokens.hasMoreElements() ) {
//            Object o = tokens.nextElement();
//            Utils.debug("o = " + o + ", type = " + tokens.getType());
//        }
        CharSequence completingWord = getCompletingWord(txt, 2);
        Utils.debug("completingWord = " + completingWord);
//        IWordEnumeration<CharSequence> e = ws.backScan(txt.indexOf('^'));
//        while (e.hasMoreElements()) {
//            CharSequence sequence = e.nextElement();
//            Utils.debug("token = <" + sequence + ">, type = " + e.getType() + ", offset = " + e.getOffset());
//        }
//
//        String sw = Utils.getFileText("c:\\share\\sync9.sql");
//

//
////        IWordEnumeration<CharSequence> e1 = ws.scan(txt.indexOf('^'));
//        WordScanner ws = new WordScanner(sw.toString());
//        IWordEnumeration<CharSequence> e1 = ws.scan(0);
//        long begin = System.currentTimeMillis();
//        while (e1.hasMoreElements()) {
//            CharSequence s = e1.nextElement();
////            Utils.debug("s = <" + s + ">, type = " + e1.getType() + ", offset = " + e1.getOffset());
//        }
//        Utils.debug("time = " + (System.currentTimeMillis() - begin));
//        Runtime r = Runtime.getRuntime();
//        Utils.debug("Using memory = " + (r.totalMemory() - r.freeMemory()) / 1024);
    }

    public static ITokenEnumeration tokenScan(CharSequence text, int offset) {
        WordScanner s = new WordScanner(text);
        return new TokenIterator(s.scan(offset));
    }

    public static CharSequence getCompletingWord(CharSequence sc, int offset) {
        if (offset >= 0) {
            WordScanner ws = new WordScanner(sc);
            IWordEnumeration b = ws.scan(0);
//            Utils.debug("sc.charAt(offset) = " + sc.charAt(offset) + ", offset = " + offset);
            CharSequence preToken = "";
            TYPE preType = NULL;
            while (b.hasMoreElements()) {
                CharSequence s = b.nextElement();
//                Utils.debug("s = " + s);
                int pos = b.getOffset();
                if (pos <= offset) {
                    TYPE t = b.getType();
                    int len = s.length();
                    if (pos + len > offset) {
                        CharSequence token = s.subSequence(0, offset - pos + 1);
                        if (t == MULTILINE_COMMENT || t== NUMBER || t == SINGLELINE_COMMENT || t == STRING) {
                            if (offset + 1 == pos + len) {
                                // end of the token
                                return null;
                            } else {
                                throw new IllegalStateException("No suggestion in the ignored token");
                            }
                        } else if (t == WHITESPACES || t == NEWLINE) {
                            return null;
                        } else if (t == OPERATOR) {
                            // if the current word is dot
                            // we go back one word to see if it is a object: "xxx"
                            // or an identifier
                            Utils.debug("token = " + token);
                            if (token.charAt(0) == '.') {
                                if (OBJECT == preType || WORD == preType) {
                                    return preToken.toString() + ".";
                                }
                            } else {
                                return null;
                            }
                        }
                        return token;
                    }
                    preType = t;
                    preToken = s;
                } else {
                    break;
                }
            }
        }
        return null;
    }

    public WordScanner() {
    }

    public WordScanner(CharSequence text) {
        _text = text;
    }

    public CharSequence getText() {
        return _text;
    }

    public void setText(CharSequence text) {
        _text = text;
    }

    public String getOperators() {
        return _operators;
    }

    public void setOperators(String operators) {
        _operators = operators;
    }

    public IWordEnumeration scan(int offset) {
        sureRange(offset);
        ForwardsEnumerator fe = new ForwardsEnumerator(_text, offset);
        fe.setOperators(getOperators());
        return fe;
    }

    public IWordEnumeration backScan(int offset) {
        sureRange(offset);
        BackwardsEnumerator be = new BackwardsEnumerator(_text, offset);
        be.setOperators(getOperators());
        return be;
    }

    public static boolean isNumber(CharSequence cs) {
        for (int i = 0; null != cs && i < cs.length(); i++) {
            char c = cs.charAt(i);
            if (!Character.isDigit(c) && c != '.'
                    && (cs.charAt(0) != '-' || cs.charAt(0) != '+')) {
                return false;
            }
        }
        return true;
    }

    private void sureRange(int offset) {
        if (offset < 0 || offset > _text.length()) {
            throw new IllegalArgumentException("Offset: " + offset + " out of range 0 - " + _text.length());
        }
    }


}

