package utils.text.editor;

import static utils.text.IWordEnumeration.TYPE.*;

import javax.swing.text.*;
import java.awt.*;

/**
 * User: huojun
 * Date: 18-Oct-2006
 * Time: 11:38:22 AM
 */
public class TokenViewFactory implements ViewFactory {
    public View create(Element elem) {
        return new TokenView(elem);
    }

    public static class TokenView extends WrappedPlainView {
        public TokenView(Element elem) {
            super(elem);
        }

        protected int drawUnselectedText(Graphics g, int x, int y, int p0, int p1) throws BadLocationException {
            Element e = getElement();
            TokenDocument doc = (TokenDocument) e.getDocument();
            int line = e.getElementIndex(p0);
            Element lineElem = e.getElement(line);
            if (doc.isParserReady()) {
                int lineOffset = lineElem.getStartOffset();
                TokenLine tokenLine = doc.getTokenLine(line);
                int ret = drawTokenLine(g, x, y, p0, p1, tokenLine, doc, line, lineOffset);
                TokenLine nextLine = doc.getTokenLine(line + 1);
                Token lastToken = null != tokenLine && tokenLine.tokens!=null && tokenLine.tokens.length > 0
                        ? tokenLine.tokens[tokenLine.tokens.length-1] : null;
                if( null != nextLine && null != nextLine.tokens && nextLine.tokens.length > 0
                        && nextLine.tokens[0].included
                        && null != lastToken
                        && ( lastToken.type == STRING || lastToken.type == MULTILINE_COMMENT )
                        ) {
                    // try to detect if repaint for multi-line comment or string is necessary
                    super.getContainer().repaint();
                }
                return ret;
            } else {
                return super.drawUnselectedText(g, x, y, p0, p1);
            }
        }

        private int drawTokenLine(Graphics g, int x, int y, int p0, int p1, TokenLine tokenLine, TokenDocument doc, int line, int lineOffset) {
            Color c;
//            Font f;
            if (null != tokenLine) {
                int tokenOffset = lineOffset;
                for (int i = 0; null != tokenLine.tokens && i < tokenLine.tokens.length; i++) {
                    Token token = tokenLine.tokens[i];
                    String str = token.text.toString();
                    if (token.type == WORD && doc.isKeyword(str.toUpperCase())) {
                        token.type = KEYWORD;
                    }
                    if ( !doc.isMarkedLine(line) ) {
                        RenderAttribute attribute = doc.getRenderAttribute(token.type);
                        if (null != attribute) {
                            c = attribute.getColor();
    //                        f = attribute.getFont();
                        } else {
                            c = Color.black;
    //                        f = doc.getFont();
                        }
                    } else {
                        c = doc.getMarkedLineColor();
                    }
//                    g.setFont(f);
                    g.setColor(c);
                    int tokenLen = str.length();
                    if( tokenOffset >= p1 ) {
                        return x;
                    } else if( tokenOffset + tokenLen <= p0 ) {
                        tokenOffset += tokenLen;
                        continue;
                    } else if( tokenOffset < p0 && tokenOffset + tokenLen > p0 ) {
                        str = str.substring(p0 - tokenOffset, str.length());
//                        System.out.println("cross p0 = " + str);
                    } else if( tokenOffset < p1 && tokenOffset + tokenLen > p1 ) {
                        str = str.substring(0, p1 - tokenOffset);
//                        System.out.println("cross p1 = " + str);
                    }
                    Segment s = new Segment(str.toCharArray(), 0, str.length());
                    x = Utilities.drawTabbedText(s, x, y, g, this, 0);
                    tokenOffset += tokenLen;
                    if( tokenOffset > p1 ) {
                        return x;
                    }
                }
            }
            return x;
        }
    }
}
