package utils.text.editor.complete;

import dbhelp.plugin.complete.AbstractCompleteWindow;
import dbhelp.plugin.complete.utils.CompleterItem;
import utils.Utils;
import utils.text.WordScanner;
import utils.text.editor.TokenEditor;

import javax.swing.*;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.awt.font.FontRenderContext;
import java.awt.geom.Rectangle2D;

/**
 * User: huojun
 * Date: 20-Oct-2006
 * Time: 4:55:55 PM
 */
public class TokenCompleter extends AbstractCompleteWindow {
    protected TokenEditor _editor;
    protected Point _caretPoint = new Point(0, 0);
    protected static BufferedImage DUMMY_IMAGE = new BufferedImage(2, 2, BufferedImage.TYPE_INT_ARGB);
    protected Graphics2D _g2d = (Graphics2D) DUMMY_IMAGE.getGraphics();

    public void showCompleter() {
        addPopup();
        setVisible(true);
    }

    private void addPopup() {
        JLayeredPane f = getLayeredPane();
        if (null != f) {
            if (JLayeredPane.getLayer(this) != JLayeredPane.POPUP_LAYER) {
                f.add(this, JLayeredPane.POPUP_LAYER);
            }
        }
    }

    public TokenEditor getEditor() {
        return _editor;
    }

    public void setEditor(TokenEditor editor) {
        _editor = editor;
    }

    public void closeCompleter() {
        setVisible(false);
        getLayeredPane().remove(this);
        this.putClientProperty(JLayeredPane.LAYER_PROPERTY, null);
    }

    protected Point getCompleterLocation() {
        TokenEditor editor = getEditor();
        Rectangle vr = editor.getVisibleRect();
        try {
            Rectangle rect = editor.modelToView(editor.getCaretPosition());
            Point p = rect.getLocation();
            FontRenderContext frc = _g2d.getFontRenderContext();
            Rectangle2D mcb = editor.getFont().getMaxCharBounds(frc);
            int charHight = (int) (mcb.getHeight());
            if (p.x + getWidth() > vr.getWidth()) {
                p.x = (int) (p.x - (p.x + getWidth() - vr.getWidth()));
            }
            if (p.y + charHight + getHeight() > vr.getHeight()) {
                p.y = p.y - getHeight();
            } else {
                p.y = p.y + charHight;
            }
            JLayeredPane pane = getLayeredPane();
            return SwingUtilities.convertPoint(editor, p, pane);
        } catch (BadLocationException e) {
            return new Point(0, 3);
        }
    }

    private CharSequence toCompleteString(CompleterItem[] items, String prefix) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; null != items && i < items.length; i++) {
            if (items.length > 1) {
                // skip star if there are multiple selections
                if ("*".equals(items[i].getObject())) {
                    continue;
                }
            }
            if (sb.length() > 0) {
                sb.append(", ");
            }
            sb.append(prefix).append(items[i].toCompleteString());
        }
        return sb.toString();
    }

    public void completeWord() {
        try {
            TokenEditor ta = getEditor();
            int offset = ta.getCaretPosition();
            Document doc = ta.getDocument();
            String text = doc.getText(0, doc.getLength());
            CharSequence word = WordScanner.getCompletingWord(text, offset - 1);
//            Utils.debug("TokenCompleter.completeWord: " + word);
            String w = null == word ? null : word.toString();
            final int deleteLen = null == w ? 0 : w.length();
            final String prefix = null != w && w.endsWith(".") ? w : "";
            itemSelected();
            CharSequence cs = toCompleteString(getSelectedItems(), prefix);
            ta.setSelectionStart(offset - deleteLen);
            ta.setSelectionEnd(offset);
            ta.replaceSelection(cs.toString());
            ta.setCaretPosition(offset + cs.length() - deleteLen);
        } catch (BadLocationException e) {
            Utils.debug("TokenCompleter.completeWord: " + e);
        }
    }
}
