package utils.ui;

import bean.editor.utils.Item;
import dbhelp.db.model.FilterColumnModel;
import dbhelp.plugin.ui.QuickCloseDialog;
import dbhelp.plugin.ui.QuickEditTable;
import utils.TablePacker;
import utils.Utils;

import javax.swing.*;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;
import java.awt.*;
import static java.awt.GridBagConstraints.*;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.util.ArrayList;

/**
 * User: huojun
 * Date: 2-Nov-2006
 * Time: 2:18:16 PM
 */
public class TableFilterDialog {
    public static void showFilterDialog(FilterColumnModel fcm, String title, final JTable targetTable) {
        showFilterDialog(new DefaultFilterModel(fcm), title, targetTable);
    }

    public static void showFilterDialog(TableModel filterModel, String title, final JTable targetTable) {
        JTable table = new JTable();
        table.setRowHeight(20);
        table.setShowVerticalLines(false);
        table.setModel(filterModel);
        int width = TablePacker.packTable(table, 2);
        QuickCloseDialog d = QuickCloseDialog.newInstance(Utils.getFrame());
        if (null != d) {
            Container c = d.getContentPane();
            c.setLayout(new GridBagLayout());
            final JCheckBox jcbPack = new JCheckBox("Auto Pack Table");
            d.setSize(Math.max(width + 50, 280), 340);
            d.setModal(true);
            JScrollPane pane = new JScrollPane(table);
            pane.setBorder(BorderFactory.createTitledBorder("Column Count: " + filterModel.getRowCount()));
            GridBagConstraints g;
            g = new GridBagConstraints(0, 0, 2, 1, 1, 1, NORTH, BOTH, new Insets(2, 2, 2, 2), 1, 1);
            c.add(pane, g);
            g = new GridBagConstraints(0, 1, 1, 1, 0, 0, WEST, NONE, new Insets(2, 2, 2, 2), 1, 1);
            c.add(jcbPack, g);
            jcbPack.setMnemonic('P');
            g = new GridBagConstraints(1, 1, 1, 1, 0, 0, EAST, NONE, new Insets(2, 2, 2, 2), 1, 1);
            c.add(new JButton(new ActionSelectAll(table)), g);
            d.setLocationRelativeTo(null);
            targetTable.getModel().addTableModelListener(new TableModelListener() {
                public void tableChanged(TableModelEvent e) {
                    if (e.getColumn() == TableModelEvent.ALL_COLUMNS && jcbPack.isSelected()) {
                        Runnable runnable = new Runnable() {
                            public void run() {
                                TablePacker.packTable(targetTable, 2);
                            }
                        };
                        SwingUtilities.invokeLater(runnable);
                    }
                }
            });
            d.setTitle(title);
            d.setVisible(true);
            d.dispose();
        }
    }

    public static void showFilterDialog(QuickEditTable table, String title) {
        showFilterDialog(table.getFilterColumnModel(), title, table);
    }

    private static class ActionSelectAll extends AbstractAction {
        private JTable _table;

        public ActionSelectAll(JTable table) {
            super("Select All");
            putValue(AbstractAction.MNEMONIC_KEY, KeyEvent.VK_A);
            _table = table;
        }

        public void actionPerformed(ActionEvent e) {
            for (int i = 0; i < _table.getRowCount(); i++) {
                Boolean b = (Boolean) _table.getValueAt(i, 0);
                if (!b) {
                    _table.setValueAt(true, i, 0);
                }
            }
            _table.repaint();
        }
    }

    public static class DefaultFilterModel extends AbstractTableModel {
        protected java.util.List<Item<String>> _items = new ArrayList<Item<String>>();
        protected FilterColumnModel _filter;

        public DefaultFilterModel(FilterColumnModel fcm) {
            _filter = fcm;
            TableModel table = fcm.getDelegate();
            for (int i = 0; i < table.getColumnCount(); i++) {
                Item<String> o = new Item<String>(table.getColumnName(i));
                o.setSelected(fcm.isColumnVisible(i));
                _items.add(o);
            }
        }

        public int getRowCount() {
            return _items.size();
        }

        public int getColumnCount() {
            return 2;
        }

        public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
            Boolean b = (Boolean) aValue;
            _items.get(rowIndex).setSelected(b);
            _filter.setVisible(rowIndex, b);
        }

        public boolean isCellEditable(int rowIndex, int columnIndex) {
            return columnIndex == 0;
        }

        public Class<?> getColumnClass(int columnIndex) {
            return columnIndex == 0 ? Boolean.class : super.getColumnClass(columnIndex);
        }

        public String getColumnName(int column) {
            if (column == 0) {
                return "Visible";
            } else {
                return "Column Name";
            }
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            Item<String> item = _items.get(rowIndex);
            String value = item.getValue();
            if (columnIndex == 0) {
                return item.isSelected();
            } else {
                return value;
            }
        }
    }


}
