package diagram.shape;

import diagram.IDragListener;
import diagram.Handle;
import diagram.DiagramViewer;

import java.awt.*;

/**
 * Created by IntelliJ IDEA.
 * User: huojun
 * Date: 15-Feb-2006
 * Time: 4:58:58 PM
 */
public class FuzzyLink extends Link {
    protected int _ex;
    protected int _ey;
    protected int _bx;
    protected int _by;
    protected boolean _selfAdjusting;

    public FuzzyLink(WindowArrow arrow, AbstractShape beginShape, AbstractShape endShape) {
        super(arrow, beginShape, endShape);
        _bx = _beginShape.getWidth() / 2;
        _by = _beginShape.getHeight() / 2;
        _ex = _endShape.getWidth() / 2;
        _ey = _endShape.getHeight() / 2;
        DiagramViewer diagram = arrow.getDiagram();
        if( null == diagram ) {
            throw new IllegalArgumentException("Please add the shapes to the diagram then create this object!");
        }
        diagram.addDragListener(new MyIDragListener(arrow));
    }

    public void shapeResized(AbstractShape s) {
        shapeChanged(s);
    }

    public void shapeChanged(AbstractShape s) {
        if (_beginShape != null && _endShape != null) {
            if (s.equals(_arrow)) {
                if (!_selfAdjusting) {
                    _bx = _arrow.getX1() - _beginShape.getX();
                    _by = _arrow.getY1() - _beginShape.getY();
                    _ex = _arrow.getX2() - _endShape.getX();
                    _ey = _arrow.getY2() - _endShape.getY();
                    Point p1 = new Point(_arrow.getX1(), _arrow.getY1());
                    Point p2 = new Point(_arrow.getX2(), _arrow.getY2());
                    Point ins1 = _beginShape.getIntersection(p1, p2);
                    Point ins2 = _endShape.getIntersection(p1, p2);
                    ((WindowArrow) _arrow).setWindow(ins1, ins2);
                }
                if (null != _text) {
                    WindowArrow wa = (WindowArrow) _arrow;
                    int x = (wa.getWx1() + wa.getWx2()) / 2;
                    int y = (wa.getWy1() + wa.getWy2()) / 2;
                    _text.setLocation(x - _text.getWidth() / 2, y - _text.getHeight() / 2);
                    _text.moveToFront();
                }
            } else {
                _selfAdjusting = true;
                Point p1 = new Point(_beginShape.getX() + _bx, _beginShape.getY() + _by);
                Point p2 = new Point(_endShape.getX() + _ex, _endShape.getY() + _ey);
                Point ins1 = _beginShape.getIntersection(p1, p2);
                Point ins2 = _endShape.getIntersection(p1, p2);
                ((WindowArrow) _arrow).setWindow(ins1, ins2);
                _arrow.setX1(p1.x);
                _arrow.setY1(p1.y);
                _arrow.setX2(p2.x);
                _arrow.setY2(p2.y);
                _arrow.fireShapeChangeEvent();
                _selfAdjusting = false;
            }
        }
    }

    /// user can't move the handle or the arrow out of the tables
    private class MyIDragListener implements IDragListener {
        private final WindowArrow _arrow;

        public MyIDragListener(WindowArrow arrow) {
            _arrow = arrow;
        }

        public AbstractShape getMonitorShape() {
            return _arrow;
        }

        public boolean accept(Handle dh, int x, int y) {
            Handle[] handles = _arrow.getHandles();
            if (dh.equals(handles[0])) {
                return getBeginShape().contains( new Point(_arrow.getX() + x, _arrow.getY() + y)) ;
            } else if (dh.equals(handles[1])) {
                return getEndShape().contains( new Point(_arrow.getX() + x, _arrow.getY() + y)) ;
            }
            return true;
        }

        public boolean accept(int x, int y) {
            Rectangle b = _arrow.getBounds();
            int x1 = _arrow.getX1() + x - b.x;
            int x2 = _arrow.getX2() + x - b.x;
            int y1 = _arrow.getY1() + y - b.y;
            int y2 = _arrow.getY2() + y - b.y;
            return getBeginShape().contains(new Point(x1, y1))
                    && getEndShape().contains(new Point(x2, y2));
        }
    }
}
